﻿/**
* Grid控件命名空间定义
*/
RO = {};
RO.BasicFun = {};
RO.Enum = {};
RO.Enum.Event = {};
RO.Enum.Container = {};
RO.Enum.Container.ClassName = {};

///////枚举数据源扩展 //////
RO.Enum.EnumNodesCache = {};


/**************私有方法****************************************************************************************/
/**
* 判断obj是否为数组数据类型
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isArray = function (obj) {
    return ((typeof obj === 'object') && obj.constructor === Array) || ($.isArray(obj));
};

/**
* 判断obj是否为字符串数据类型
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isString = function (str) {
    return (typeof str === 'string') && str.constructor === String;
};

/**
* 判断obj是否为数字数据类型
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isNumber = function (obj) {
    if (obj === 0) return true;
    if (obj == "") return false;
    return !isNaN(obj);
};

/**
* 判断obj是否为整型数据类型
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isInt = function (obj) {
    var reg = /^(-|\+)?\d+$/;
    return RO.BasicFun.isNumber(obj) && reg.test(obj);
};

/**
* 判断obj是否为浮点型数据类型
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isFloat = function (obj) {
    if (RO.BasicFun.isInt(obj)) return true;
    var reg = /^(-|\+)?\d+\.\d*$/;
    return RO.BasicFun.isNumber(obj) && reg.test(obj);
};

/**
* 判断obj是否为日期数据类型
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isDate = function (obj) {
    return (typeof obj === 'object') && obj.constructor === Date;
};

/**
* 判断obj是否为函数数据类型
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isFunction = function (obj) {
    return typeof obj === 'function';
};

/**
* 判断obj是否为对象数据类型
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isObject = function (obj) {
    if (obj === null || typeof obj === 'undefined') {
        return false;
    }
    return typeof obj === 'object';
};

/**
* 判断obj是否为数组数据类型
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isBoolean = function (obj) {
    return typeof obj === 'boolean';
};

/**
* 判断obj是否为undefined
* @param obj {} 待判定参数
* @return {Boolean}
* @private
*/
RO.BasicFun.isNullOrUndefined = function (obj) {
    return typeof obj === 'undefined' || obj === null;
};

/**
* @param paramter {String} 参数
* @param funName {String} 函数名
* @return {}
* @private
*/
RO.BasicFun.alertParamCheck = function (paramter, funName) {
    if (alert(funName + "函数：参数" + paramter + "不合法！")) return;
};

RO.BasicFun.alertNullGrid = function (grid) {
    if (!grid) {
        //if (alert("尚未加载Grid，请确认！")) return false;
        return false;
    }
    return true;
};

//判断对象是否为空
RO.BasicFun.isEmptyObject = function (obj) {
    if (!obj) {
        return true;
    }
    for (var element in obj) {
        return false;
    }
    return true;
};

/**********常量定义************************************************************************************************/
//操作模式
RO.Enum.OperationMode = {
    ROW: true, //行方向
    COL: false //列方向
};
//回车键方向
RO.Enum.EnterDirection = {
    UP: "up",
    DOWN: "down",
    RIGHT: "right",
    LEFT: "left"
};
//表单区域类型
RO.Enum.FormRegionKind = {
    ROW: "row", //行浮动表
    COL: "col", //列浮动表
    Fixed: "fixed" //固定表
};

//边框样式枚举
RO.Enum.BorderStyle = {
    AUTO: -1, //默认边框样式
    NONE: 0,  //无边框
    SOLID: 1, //1px实线
    DASH: 2,  //虚线
    BOLD: 4,  //2px实线
    DOUBLE: 8,//双边实线
    DOTTED: 16//暂无
};
//背景样式
RO.Enum.BackStyle = {
    FILL: 1, //填充
    NONE: 0  //无
};
//数据链接编辑模式
RO.Enum.DataLinkEditMode = {
    dl_None: 0,            // 不允许直接录入    
    dl_Default: 1,         // 默认编辑方式，文本等    
    dl_DropDown: 2,        // 下拉选择    
    dl_DropDownEdit: 3,    // 下拉选择或手工输入
    dl_PopupDialog: 4,     // 弹出窗口选择    
    dl_CheckBox: 5,        // 复选框，只适合逻辑型   
    dl_ReadOnly: 6         // 数据只读
};
//枚举类型
RO.Enum.EnumType = {
    List: 0,  // 列表结构
    Tree: 1   // 树形结构
};
RO.Enum.ExtendAttrType = {
    Classify: 1 //  明细行与合计行合并时使用
}

/*表格事件定义*/
RO.Enum.Event.beforeDataChange = "beforeDataChange";
RO.Enum.Event.beforeOnPaste = "beforeOnPaste";
RO.Enum.Event.datachange = "datachange";
RO.Enum.Event.valuechange = "valuechange";
RO.Enum.Event.dataLoadByLazy = "dataLoadByLazy";
RO.Enum.Event.cellfocus = "cellfocus";
RO.Enum.Event.onFormDataLoaded = "onFormDataLoaded";
RO.Enum.Event.selectionChange = "gridSelectionChange";
RO.Enum.Event.tryEditReadOnlyCell = "tryEditReadOnlyCell";
RO.Enum.Event.gridColResize = "gridColResize";
RO.Enum.Event.gridRowResize = "gridRowResize";
RO.Enum.Event.crossSelectionChange = "crossSelectionChange";
RO.Enum.Event.onDeletePressed = "onDeletePressed";
RO.Enum.Event.beginChangeCell = "beginChangeCell";
RO.Enum.Event.pasteDataPartLost = "pasteDataPartLost";
RO.Enum.Event.beforeAppendRow = "beforeAppendRow";
RO.Enum.Event.beforeAppendCol = "beforeAppendCol";
RO.Enum.Event.deleteEnum = "deleteEnum";
RO.Enum.Event.cellDblClick = "celldblclick";
RO.Enum.Event.cellMouseIn = "cellMouseIn";
RO.Enum.Event.cellMouseOut = "cellMouseOut";
RO.Enum.Event.cellMouseClick = "cellMouseClick";
RO.Enum.Event.onScrolled = "onScrolled";
RO.Enum.Event.gridAsynLoad = "gridAsynLoad";
RO.Enum.Event.gridBeginEditing = "gridBeginEditing";
RO.Enum.Event.cellExpand = "cellExpand";
RO.Enum.Event.cellCollapse = "cellCollapse";
RO.Enum.Event.EditCell = "editCell";
RO.Enum.Event.CellChanging = "cellChanging";
RO.Enum.Event.AutoNORowChange = "aotonorowchange";
RO.Enum.Event.WorningFlag = "worningflag";
RO.Enum.Container.ClassName.grid_sheetPanel = "grid_sheetPanel";



/*表格常量定义*/
RO.Enum.DefaultBorderColor = "#d4d4d4";
RO.Enum.DefaultCellPadding = [0, 1, 0, 1];
RO.Enum.cellDataErrorFlag = "cellDataError";
RO.Enum.gridBrushCursor = "url('" + location.protocol + "//" + location.host + "/Content/base/css/icons/brush.cur'),auto";
//错误提示类型定义
RO.Enum.cellErrorCheckType = "cECP";//1为强制审核2为非强制性审核
// 数据行属性名称定义
RO.Enum.RegId = "reg";
RO.Enum.FloatRowKey = "fkey";
RO.Enum.State = "state";
RO.Enum.extendAttrs = "exA";//明细行扩展
//单元格属性名称定义
RO.Enum.DataType = "dt";//数据类型
RO.Enum.ReadOnlyCell = "roc";//是否只读
RO.Enum.EditMode = "im";//编辑模式
RO.Enum.EmptyCell = "emt";//空白单元格
RO.Enum.DataLinkId = "lid";//数据链接ID
RO.Enum.ToolTip = "tooltip";//提示信息
RO.Enum.Decimal = "dcm";//小数位
RO.Enum.ThoundsMark = "ths";//千分位
RO.Enum.ViewMode = "vm";//枚举单元格显示模式
RO.Enum.EnumID = "df";//枚举ID,之前用于存储视图ID
RO.Enum.EnumTypeID = "etid";//枚举类型ID,暂不使用
RO.Enum.DeleteEnumIDs = "deleeid";//删除枚举ID集合，用于修复建模删除枚举时界面不刷新导致下拉显示的问题
RO.Enum.EnumSrcType = "est"; //枚举数据来源类型，分成固定字典和动态枚举.
RO.Enum.EnumOperationType = "eot"; //枚举操作类型，eot=1输入单条数据，eot=2，可输入多条枚举数据。
RO.Enum.EnumDisplayMode = "edm"; //枚举的显示模式：下拉或者单元格内显示。
RO.Enum.EnumData = "enumdata";//枚举数据，用于建模设置单元格内显示时使用.
RO.Enum.EnumMulSelect = "ems";//枚举数据是否允许多选.
RO.Enum.EnumAllowUndefinedCode = "eau";//枚举数据是否允许自定义编码.
RO.Enum.EnumAllowNotLeafNode = "eann";//枚举数据是否允许选择非叶子节点.
RO.Enum.GridDesignMode = "gdm";//是否是表格设计模式，临时使用.
RO.Enum.GatherType = "gt";//单元格汇总类型
RO.Enum.AutoCalcLink = "acl";//自动运算类型
RO.Enum.EnumDataSourceType = "edt";//数据请求类型  

RO.Enum.TreeImgTextLeft = "titl";//单元格内树形文字左边的宽度
RO.Enum.ImgTextLeft = "itl";//单元格内树形文字左边的宽度
RO.Enum.AttachmentUploadUrl = "FileUploadUrl"; //上传附件地址
RO.Enum.ExtEditText = "rv";//单元格扩展值，区分editText

RO.Enum.AttachmentUploadUrl = "FileUploadUrl"; //上传附件地址
RO.Enum.ExtEditText = "rv";//单元格扩展值，区分editText

RO.Enum.DefaultCell = {
    showText: "",
    editText: "",
    selectable: true,
    editable: true,
    cellMode: COM.widget.Grid.Cell.Cell_MODE_NORMAL,
    //合并单元格相关
    rowSpan: 1,
    colSpan: 1,
    merged: false,
    //背景样式
    backStyle: COM.widget.Grid.Cell.BACK_STYLE.FILL,
    //背景颜色
    backColor: "white",
    //边框相关
    //[右，下，对角线，反对角线]
    border: [-1, -1, 0, 0],
    borderColor: [RO.Enum.DefaultBorderColor, RO.Enum.DefaultBorderColor, "black", "black"],
    fontName: "宋体",
    fontSize: 14,
    fontBold: false,
    fontItalic: false,
    fontColor: "#0",
    //描边
    //textStroke : textStroke || null,
    //阴影
    //textShadow : textShadow || null,
    wrapLine: true,
    indent: 0,
    horzAlign: 0,
    vertAlign: 0,
    vertText: false,
    fitFontSize: false,
    multiLine: true, //multiLine,
    //下划线和删除线，没有就设为null
    decoration: COM.widget.Grid.Cell.DECORATION.NONE,
    padding: RO.Enum.DefaultCellPadding, // 默认不设置padding，否则单元格大小和字体大小接近时文字会有被遮挡的情况
    //其他数据存储点
    clientData: {}
};

/*********************************************************************************************************************/
/*********************************************************************************************************************/
/*******全局缓存（window）**************************************************************************************************/
$_gridGlobalCache = {};
var GridGlobalCache = {};
GridGlobalCache.setCache = function (key, value) {
     $_gridGlobalCache[key] = value;
}
GridGlobalCache.getCache = function (key) {
    if ($_gridGlobalCache[key] != undefined) {
        return $_gridGlobalCache[key];
    } else {
        return "";
    }
}

/*********************************************************************************************************************/