﻿//MVC Model////////////////////////////////////////////////////////////
window._mvcModels = {};
window.failureAction = {};



$.Class("MvcBase", {

    action: function (path, data, callback, async, onError) {
        //执行action回调操作，data和callback都可以省略
        if (typeof data === "function") {
            callback = data;
            data = null;
        }
        var oriPath = path, oriData = data;
        path = $.doUrl(path);

        var self = this;

        var jsonData = data && JSON.stringify(data);
        var gzipData = jsonData && btoa(pako.gzip(jsonData, { to: "string" }));
        var useGZipData = gzipData && jsonData && gzipData.length < jsonData.length;

        jQuery.ajax({
            type: "post",
            url: path,
            contentType: "application/json",
            processData: false,
            data: useGZipData ? gzipData : jsonData,
            success: function (result) {
                if (callback) {
                    callback.call(self, result);
                }
            },
            error: function (xhr, status, data) {
                if (xhr.status == 401) {
                    $('.loading-indicator,.loading-indicator-overlay').remove();
                    window.failureAction = {
                        caller: self,
                        path: oriPath,
                        data: oriData,
                        callback: callback,
                        async: async,
                        onError: onError
                    };
                    $.doReLogin();
                }
                onError && onError.apply(self, arguments);
            },
            async: async == false ? false : true,
            headers: {
                ISGZIPDATA: useGZipData ? "1" : "0"
            }
        });
    }
    , call: function (action, controller, area, data, callback) {
        if (arguments.length == 3) {
            if (typeof area != "string") {
                if (typeof area == "function") {
                    callback = area;
                    area = null;
                } else {
                    data = area;
                    area = null;
                }
            }
        }
        else if (arguments.length == 4) {
            if (typeof data == "function") {
                callback = data;
                data = null;
            }
        }
        var path = $.RootPath ? $.RootPath : "/";
        if (area) {
            path += area;
        }
        path += "/" + controller + "/" + action;

        this.action(path, data, callback);
    }
    , newModel: function (entry, className, opt) {
        //动态创建新的模型
        opt.entry = $this;
        var model = $.New(eval(entry), opt);

        if (model["onEntry"]) {
            model.onEntry();
        }
        if (model.modelType == "mvc" && model["onLoaded"]) {
            model.onLoaded();
        }

    }

    , sendMessage: function (type, msg) {
        //发送消息，发送范围包括非MVC模型

        window.sendMessage(type, msg, this);
    }
    , addListener: function (type, fnc) {
        //添加监听

        window.addListener(type, fnc, this);
    }
    , removeListener: function (type, fnc) {
        //卸载消息监听，fnc可以省略

        window.removeListener(type, fnc, this);
    }
    , loadPlugins: function (plugins) {
        window.loadPlugins(plugins);
    }
    , unloadPlugin: function (name) {
        window.unloadPlugin(name);
    }
     , unloadAllPlugins: function () {
         window.unloadAllPlugins();
     }
    , m: function (modelName) {
        //获取模型

        return $.m(modelName);
    },
    openDlg: function (options) {
        window.openDlg(options);
    },
    closeDlg: function (name) {
        window.closeDlg(name);
    },
    jNoty: function (options) {
        window.jNoty(options);
    }
});

$.Class("MvcModel", MvcBase, {

    _plugins: []
    , construct: function () {
        //ModelName默认为类名
        //ModelName自定义设置方法：1、添加modelName属性，2、设置opt属性内容包含ModelName，
        //3、继承MvcModel的js类中添加ModelName属性，4、重载construct方法，自己设定
        if (!this.ModelName) {
            if (this.entry.attr("modelName")) {
                this.ModelName = this.entry.attr("modelName");
            } else {
                this.ModelName = this.ClassName || "DefaultModelName";
            }
        }
        if ($.m(this.ModelName)) {
            var prefix = this.ModelName + "_";
            for (var i = 1 ; i < 100; i++) {
                if (!$.m(prefix + i)) {
                    this.ModelName = prefix + i;
                    break;
                }
            }
        }

        window._mvcModels[this.ModelName] = this;

        if (this.entry) {
            this.entry.data("model", this);
            this.entry.bind("destroy", this, function (e) {
                if (this == e.target && e.data.destroy) {
                    e.data.destroy();
                }
                e.stopPropagation();
            });
        }

    },
    onEntry: function () {
    },
    onLoaded: function () {
        if (this._plugins && this._plugins.length > 0) {
            this.loadPlugins(this._plugins);
        }
    },
    onDestroy: function () {

    }
    , destroy: function () {
        //销毁事件

        delete window._mvcModels[this.ModelName];
        for (var i = 0; i < this._plugins.length; i++) {
            this.unloadPlugin(this._plugins[i].name);
        }
        if (this["onDestroy"]) {
            this["onDestroy"]();
        }
    }

});

$.Class("MvcPlugin", MvcBase, {
    state: "new"     /* new init load unload */
    , construct: function () {
        this.state = "init";
    }
    , load: function () {
        this.state = "load";
    }
    , unload: function () {
        this.state = "unload";
    }

});

//消息机制
window._listeners = {};
window.addListener = function (type, fnc, scope) {
    //添加监听

    var lis = window._listeners[type];
    if (!lis) {
        lis = [];
        window._listeners[type] = lis;
    }
    lis.push({ scope: scope, fnc: fnc });
};

window.removeListener = function (type, fnc, scope) {
    //卸载监听,fcn,scope都可省略
    var lis = window._listeners[type];
    if (lis) {
        for (var i = 0; i < lis.length; i++) {
            var li = lis[i];
            if (!fnc || (fnc == li.fnc && (!scope || scope == li.scope))) {
                lis.splice(i, 1);
                i--;
            }
        }
        if (lis.length == 0) {
            delete window._listeners[type];
        }
    }
};

window.sendMessage = function (type, msg, sender) {
    //发送消息

    window._sendMessage(type, msg, sender);
};

window._sendMessage = function (type, msg, sender) {
    //发送消息

    var lis = window._listeners[type];
    if (lis) {
        for (var i = 0; i < lis.length; i++) {
            var li = lis[i];
            //if ((li.scope && li.scope == sender) || !li.scope) {
            var r = li.fnc.call(li.scope, sender, msg);
            if (r !== undefined && r === false) {
                return false;
            }
            //}
        }
    }

    return true;
};

//插件机制
window.__plugins = {};
window.loadPlugins = function (plugins) {
    for (var i = 0 ; i < plugins.length; i++) {
        var p = plugins[i];
        if (!window.__plugins[p.name]) {
            var className = (p.className || p.name);
            var pClass = null;
            try {
                pClass = eval(className);
            } catch (e) {

            }
            if (pClass) {
                var pObj = $.New(pClass);
                if (pObj) {
                    window.__plugins[p.name] = pObj;
                    pObj.load();
                }
            } else {
                $.ajax({
                    type: "get",
                    url: p.url,
                    async: false,
                    success: function (data) {
                        var pObj = $.New(eval(data));
                        if (pObj) {
                            window.__plugins[p.name] = pObj;
                            pObj.load();
                        }
                    }
                });
            }

        } else {
            if (window.__plugins[p.name] && window.__plugins[p.name].state != "load") {
                window.__plugins[p.name].load();
            }
        }

    }
};




window.unloadPlugin = function (name) {
    var p = window.__plugins[name];
    if (p && p.state == "load") {
        p.unload();
    }
};

window.unloadAllPlugins = function () {
    for (var pName in window.__plugins) {
        var p = window.__plugins[pName];
        if (p.state == "load") {
            p.unload();
        }
    }
};


$.doUrl = function (url) {
    if (!url)
        return url;
    if ($._taskId_ && url.indexOf("_taskId_") === -1) {
        url += url.indexOf("?") !== -1 ? "&_taskId_=" + $._taskId_ : "?_taskId_=" + $._taskId_;
    }
    if ($._userId_ && url.indexOf("_userId_") === - 1) {
        url += url.indexOf("?") !== -1 ? "&_userId_=" + $._userId_ : "?_userId_=" + $._userId_;
    }
    if ($.shareToken && url.indexOf("shareToken") === -1) {
        url += url.indexOf("?") !== -1 ? "&shareToken=" + $.shareToken : "?shareToken=" + $.shareToken;
    }
    return url;
};

$.doReLogin = function () {
    var dom = $("<div style='position:absolute;top:50%;left:50%;transform:translateX(-50%) translateY(-50%);-ms-transform:translateX(-50%) translateY(-50%);-webkit-transform:translateX(-50%) translateY(-50%);'></div>").append(
        $("<form method='post' action='/Account/Login' id='reLoginForm'>").append(
            $("<p style='position:relative;margin-bottom:30px;'>").append(
                $("<label for='reLogin_Pwd' style='position:absolute;background-repeat:no-repeat;background-position:-21px -52px;height:20px;left:6px;overflow:hidden;padding:0 0 0 18px;top:8px;width:0;z-index:10;'>密码</label>").css("background-image", "url(" + $.masterSiteUrl + "Content/portal/image/login/icon.png" + ")"),
                $("<input id='reLogin_Pwd' type='password' name='pwd' placeholder='请输入当前用户密码' style='height:35px;width:268px;padding:0 0 0 30px;'>").focus(function () {
                    $(this).prev().css("background-position", "-21px -73px");
                }).blur(function () {
                    $(this).prev().css("background-position", "-21px -52px");
                })
            ),
            $("<p>").append(
                $("<button type='submit' class='gg-button-size-middle gg-button-color-green' style='width:267px;'>登录</button>").button()
            )
        ).on("submit", function () {
            var $this = $(this);
            $this.ajaxSubmit({
                beforeSend: function (xhr) {
                    xhr.setRequestHeader("jcpDoReLogin", true);
                },
                data: {
                    userId: $._userId_
                },
                success: function (result) {
                    if (!result) {
                        window.closeDlg();
                    } else {
                        if (result.Success) {
                            $.getJSON($.masterSiteUrl + "Account/LoginBySSB?userId=" + $._userId_ + "&password=" + $("#reLogin_Pwd").val() + "&callback=?");
                            jNoty("登录成功");
                            window.closeDlg();
                            if (window.failureAction && !$.isEmptyObject(window.failureAction)) {
                                window.failureAction.caller.action($.doUrl(window.failureAction.path), window.failureAction.data, window.failureAction.callback, window.failureAction.async, window.failureAction.onError);
                                window.failureAction = {};
                            }
                            if (window.failureDlg && !$.isEmptyObject(window.failureDlg)) {
                                $.ajax({
                                    url: $.doUrl(window.failureDlg.url),
                                    success: window.failureDlg.success
                                });
                                window.failureDlg = {};
                            }
                        } else {
                            if ($("#reLogin_Pwd").val())
                                jNoty("密码输入有误，请重新输入");
                            else
                                jNoty("请输入密码");
                        }
                    }
                }
            });
            return false;
        })
    );
    window.openDlg({
        title: "登录信息已过期，请输入密码重新登录",
        dom: dom,
        width: 365,
        height: 240,
        open: function () {
            $("[placeholder]", this).placeholder();
        }
    });
};