﻿/**
* 星评插件
* 依赖jQuery jQueryUI
*/
(function ($) {
    $.widget("ui.scores", {
        widgetName: "scores",
        version: 1.0,
        prefix: "scores_",

        options: {
            number: 5,  //星数
            size: 24,   //星大小
            iconClass: {"on": "startOn", "off": "startOff"},   //星样式
            title: [],  //星标题
            showTitle: true, //是否显示星标题
            readOnly: false,  //是否只读
            defaultScore: 0   //默认分值
        },

        _create: function () {
            var options = this.options,
                titleLen = options.title.length,
                style = "width:" + options.size + "px;height:" + options.size + "px;background-size:" + options.size + "px;",
                width = 0,
                height = 0;

            this._number = options.number;
            this._size = options.size;
            this._title = options.title;
            this._showTitle = options.showTitle;
            this._score = options.defaultScore;
            this.element.addClass("ui-scores");
            
            for (var i = 0; i < options.number; i++) {
                var itemClass = "scoreItem " + this.prefix + (i < this._score ? options.iconClass["on"] : options.iconClass["off"]),
                    title = this._showTitle ? (i < titleLen ? options.title[i] : (i + 1)) : "";
                    item = $("<li class='" + itemClass + "' title='" + title + "' index='" + (i + 1) + "' style='" + style + "'></li>");
                this.element.append(item);
                width += item.width();
                height = Math.max(height, item.height());
            }
            this.element.css({ "width": width, "height": height });

            !this.options.readOnly && this._addEvent();
        },

        /**
        * 设置分数
        * @param val {Int} 分值
        */
        setScore: function (val) {
            if (!this.options.readOnly) {
                this.update(val);
                this._score = val;
            }
        },

        /**
        * 获取分数
        * @returns {Int} 当前分值
        */
        getScore: function () {
            return this._score;
        },

        /**
        * 设置可编辑性
        * @param val {Bool} 是否可编辑
        */
        setCanEdit: function (val) {
            this.options.readOnly = !val;
            this._removeEvent();
            if (val) {
                this._addEvent();
            }
        },

        /**
        * 更新星状态
        * @param index {Int} 星坐标
        */
        update: function (index) {
            var offClass = this.prefix + this.options.iconClass["off"],
                onClass = this.prefix + this.options.iconClass["on"],
                number = this._number;

            if (this.options.readOnly) {
                return;
            }
            if (arguments.length == 0) {
                index = this._score;
            }
            var lis = $("li", this.element);
            for (var i = 0; i < index; i++) {
                $(lis[i]).removeClass(offClass).addClass(onClass);
            }
            for (var j = index; j < number; j++) {
                $(lis[j]).removeClass(onClass).addClass(offClass);
            }
        },

        _addEvent: function () {
            this.element.on("mouseenter", ".scoreItem", this._mouseenterHandler)
                .on("mouseleave", this._mouseleaveHandler)
                .on("click", ".scoreItem", this._clickHandler);
        },

        _removeEvent: function () {
            this.element.off("mouseenter", ".scoreItem", this._mouseenterHandler)
                .off("mouseleave", this._mouseleaveHandler)
                .off("click", ".scoreItem", this._clickHandler);
        },

        _mouseenterHandler: function (e) {
            var item = $(e.target), index = item.attr("index");
            item.parent().scores("update", index);
        },

        _mouseleaveHandler: function (e) {
            var item = $(e.target);

            if (item.hasClass("ui-scores")) {
                item.scores("update");
            } else {
                item.parent().scores("update");
            }
        },

        _clickHandler: function (e) {
            var item = $(e.target), index = item.attr("index"), element = item.parent();

            if (element.scores("getScore") == 1 && index == 1) {
                index = 0;
            }
            element.scores("setScore", index);
        },

        _destroy: function () {
            this._removeEvent();
            this.element.removeClass("ui-scores");
            this.element.children().remove();
        }

    })
})(jQuery);