﻿/*********************************************
 *           在线建模前台交互数据              *
 *                 Update Date：2014-8-13     *
**********************************************/
BM = {};
BM.Enum = {};

BM.BusinessList = function (listName) {
    var props = {
        Name: { value: listName == null ? "集合" : listName, enumerable: false, writable: true, configurable: false },
        Count: { value: 0, enumerable: false, writable: true, configurable: false },
        add: {
            value: function (key, value) {
                if (key == null)
                    throw new Error("null key is not available.");
                if (this[key]) {
                    this[key] = value;
                } else {
                    this[key] = value;
                    this.Count++;
                }
            }, enumerable: false, writable: true, configurable: false
        },
        get: {
            value: function (key) {
                return this[key];
            }, enumerable: false, writable: true, configurable: false
        },
        remove: {
            value: function (key) {
                if (this[key]) {
                    delete this[key];
                    this.Count--;
                }
            }, enumerable: false, writable: true, configurable: false
        },
        getCount: {
            value: function () {
                return this.Count;
            }, enumerable: false, writable: true, configurable: false
        },
        clear: {
            value: function () {
                for (var id in this) {
                    delete this[id];
                    this.Count--;
                }
            }, enumerable: false, writable: true, configurable: false
        },
        containsKey: {
            value: function (key) {
                return this[key] != null;
            }, enumerable: false, writable: true, configurable: false
        },
        toString: {
            value: function () {
                return this.Name + "，Count【" + this.Count + "】";
            }, enumerable: false, writable: true, configurable: false
        },
    };

    return Object.create(BM.BusinessList.prototype, props);
};

BM.FormManager = function (formObj) {
    this.FormObj = formObj;
};

BM.MultiLevelObject = function (multiLevelString) {
    this.parse(multiLevelString);
};

BM.MultiLevelObject.prototype.parse = function (multiLevelString) {
    if (!multiLevelString || multiLevelString == "") return null;

    this.LevelInfo = new BM.BusinessList();

    var levelInfoDealed = multiLevelString.replace('：', ':');//替换中文字符.
    levelInfoDealed = levelInfoDealed.replace('；', ';');//替换中文字符.

    var levelSet = levelInfoDealed.split('|');
    for (var sIdx = 0; sIdx < levelSet.length; sIdx++) {
        var levelItemSet = levelSet[sIdx].split(':');
        if (levelItemSet.length != 2)
            continue;
        var colIndex = parseInt(levelItemSet[0]);

        var tmp = levelItemSet[1].substring(1, levelItemSet[1].length - 1);
        var colItemSet = tmp.split(';');
        if (!this.LevelInfo.get(colIndex)) {
            var colset = [];
            this.LevelInfo.add(colIndex, colset);
        }

        for (var s1Index = 0; s1Index < colItemSet.length; s1Index++) {
            this.LevelInfo[colIndex].push(parseInt(colItemSet[s1Index]));
        }
    }

    return this;
};

BM.MultiLevelObject.prototype.addLevel = function (level, colRowIndex) {
    if (isNaN(level) || isNaN(colRowIndex)) return;

    var levelObj = this.LevelInfo[level];
    if (levelObj) {
        if ($.inArray(colRowIndex, levelObj) == -1)
            levelObj.push(colRowIndex);
    } else {
        var newLevelInfo = [];
        newLevelInfo.push(colRowIndex);
        this.LevelInfo.add(level, newLevelInfo);
    }
};

BM.MultiLevelObject.prototype.removeLevel = function (level) {
    this.LevelInfo.remove(level);
};

BM.MultiLevelObject.prototype.removeIndex = function (colRowIndex) {
    for (var level in this.LevelInfo) {
        var levelItem = this.LevelInfo[level];
        var pos = $.inArray(colRowIndex, levelItem);
        if (pos >= 0) {
            levelItem.splice(pos, 1);
        }
    }
};

BM.MultiLevelObject.prototype.getLevelCount = function () {
    return this.LevelInfo.getCount();
};

//获取多级浮动区域的虚拟表格行列数.
BM.MultiLevelObject.prototype.getVirtualRowCount = function () {
    return Math.ceil(Math.pow(2, this.LevelInfo.getCount()));
};

Date.prototype.format12H = function () {
    var hours = this.getHours();
    var minutes = this.getMinutes();
    var ampm = hours >= 12 ? 'PM' : 'AM';
    hours = hours % 12;
    hours = hours ? hours : 12;
    minutes = minutes < 10 ? '0' + minutes : minutes;
    var strTime = hours + ':' + minutes + ' ' + ampm;
    return strTime;
};