﻿//-------------------------------------------------------------------------------------------------------------------------------指标.
BM.FieldObject = function (fieldObj) {
    BM.UndoBase.call(this);

    this.ID = fieldObj.ID;
    this.Type = fieldObj.Type ? fieldObj.Type : BM.Enum.FieldType.ft_General;
    this.ValueType = fieldObj.ValueType ? fieldObj.ValueType : BM.Enum.FieldValueType.fv_Default;
    this.GatherType = fieldObj.GatherType ? fieldObj.GatherType : BM.Enum.FieldGatherType.fg_None;//汇总方式.
    this.Size = fieldObj.Size ? fieldObj.Size : 0;

    this.IsNew = fieldObj.IsNew;
    this.IsDirty = fieldObj.IsDirty;
    this.IsDeleted = fieldObj.IsDeleted;

    this.Title = fieldObj.Title ? fieldObj.Title : "";
    this.Code = fieldObj.Code ? fieldObj.Code : "";
    this.Nullable = (fieldObj.Nullable == null ? true : fieldObj.Nullable);//默认允许为空.
    this.FractionDigits = fieldObj.FractionDigits ? fieldObj.FractionDigits : 0;
    this.AllowUndefinedCode = (fieldObj.AllowUndefinedCode == null ? false : fieldObj.AllowUndefinedCode);
    this.AllowMultSelect = (fieldObj.AllowMultSelect == null ? false : fieldObj.AllowMultSelect);
    this.AllowNotLeafNodeRefer = (fieldObj.AllowNotLeafNodeRefer == null ? false : fieldObj.AllowNotLeafNodeRefer);
    this.ShowFormat = fieldObj.ShowFormat ? fieldObj.ShowFormat : "";
    if (fieldObj.DefaultValue && fieldObj.DefaultValue != "" && fieldObj.DefaultValue.length >= 1) {
        if (fieldObj.DefaultValue.substr(0, 1) == '"' && fieldObj.DefaultValue.substr(fieldObj.DefaultValue.length - 1, 1) == '"')
            this.DefaultValue = fieldObj.DefaultValue.substr(1, fieldObj.DefaultValue.length - 2);
        else
            this.DefaultValue = fieldObj.DefaultValue;
    } else {
        this.DefaultValue = null;
    }
    this.OwnerTableID = fieldObj.OwnerTableID ? fieldObj.OwnerTableID : 0;
    this.ReferField = fieldObj.ReferField ? fieldObj.ReferField : 0;
    //this.FieldName = fieldObj.FieldName ? fieldObj.FieldName : "";
    //this.TableName = fieldObj.TableName ? fieldObj.TableName : "";
};

BM.FieldObject.prototype = new BM.UndoBase();

//保存时，重新复制一份数据，以免原有数据状态被破坏.
BM.FieldObject.prototype.getCopyToSave = function () {
    var result = {};

    result.ID = this.ID;
    result.IsNew = this.IsNew;
    result.IsDirty = this.IsDirty;
    result.IsDeleted = this.IsDeleted;

    if (this.IsNew === true || this.IsDeleted === true || this.IsDirty === true) {
        result.Type = this.Type;
        result.ValueType = this.ValueType; 
        result.GatherType = this.GatherType;
        result.Size = this.Size;
        result.Title = this.Title;
        result.Code = this.Code;
        result.Nullable = this.Nullable;
        result.FractionDigits = this.FractionDigits;
        result.ShowFormat = this.ShowFormat;
        result.DefaultValue = this.DefaultValue;
        result.OwnerTableID = this.OwnerTableID;
        result.ReferField = this.ReferField;
        result.AllowMultSelect = this.AllowMultSelect;
        result.AllowUndefinedCode = this.AllowUndefinedCode;
        result.AllowNotLeafNodeRefer = this.AllowNotLeafNodeRefer;
    }

    return result;
};

BM.FieldObject.prototype.setType = function (newType) {
    if (newType == null || newType === "")
        return null;

    if (this.Type != newType) {
        this.beforePropertyChanged("Type", this.Type, newType);
        this.Type = newType;
        this.markDirty();
        this.propertyHasChanged("Type");
    }
    return this;
};

BM.FieldObject.prototype.setSize = function (newSize) {
    if (newSize == null || newSize === "")//note Size值为数字，不可用类似!Size来做判断。
        return null;

    var newValue = parseInt(newSize);
    if (this.Size != newValue) {
        this.beforePropertyChanged("Size", this.Size, newValue);
        this.Size = newValue;
        this.markDirty();
        this.propertyHasChanged("Size");
    }
    return this;
};

BM.FieldObject.prototype.setTitle = function (newTitle) {
    if (newTitle == null || newTitle.trim() === "")
        return null;

    newTitle = newTitle.trim();

    if (this.Title != newTitle) {
        this.beforePropertyChanged("Title", this.Title, newTitle);
        this.Title = newTitle;
        this.markDirty();
        this.propertyHasChanged("Title");
    }
    return this;
};

BM.FieldObject.prototype.setCode = function (newCode) {
    if (newCode == null || newCode === "")
        return null;

    newCode = newCode.trim();

    if (newCode.length > 50)
        newCode = newCode.substring(0, 50);//指标标识不可超过50.

    if (this.Code != newCode) {
        this.beforePropertyChanged("Code", this.Code, newCode);
        this.Code = newCode;
        this.markDirty();
        this.propertyHasChanged("Code");
    }
    return this;
};

BM.FieldObject.prototype.setFractionDigits = function (newFractionDigits) {
    if (newFractionDigits == null || newFractionDigits === "")
        return null;

    if (isNaN(newFractionDigits))
        return null;

    var newValue = parseInt(newFractionDigits);
    if (this.FractionDigits != newValue) {
        this.beforePropertyChanged("FractionDigits", this.FractionDigits, newValue);
        this.FractionDigits = newValue;
        this.markDirty();
        this.propertyHasChanged("FractionDigits");
    }
    return this;
};

BM.FieldObject.prototype.setOwnerTableID = function (newOwnerTableId) {
    if (newOwnerTableId == null || newOwnerTableId === "")
        return null;

    //note TableObject有可能是新建的，此时newOwnerTableId不是数字.【TableObject-!@#!$@%#^&&&^】
    //if (isNaN(newOwnerTableId))
    //    return null;

    if (this.OwnerTableID != newOwnerTableId) {
        this.beforePropertyChanged("OwnerTableID", this.OwnerTableID, newOwnerTableId);
        this.OwnerTableID = newOwnerTableId;
        this.markDirty();
        this.propertyHasChanged("OwnerTableID");
    }
    return this;
};

BM.FieldObject.prototype.setReferField = function (newReferField) {
    if (newReferField == null || newReferField === "")
        return null;

    if (isNaN(newReferField))
        return null;

    if (this.ReferField != newReferField) {
        this.beforePropertyChanged("ReferField", this.ReferField, newReferField);
        this.ReferField = newReferField;
        this.markDirty();
        this.propertyHasChanged("ReferField");
    }
    return this;
};

BM.FieldObject.prototype.setNullable = function (newNullable) {
    if (newNullable == null || newNullable === "")
        return null;

    if (this.Nullable != newNullable) {
        this.beforePropertyChanged("Nullable", this.Nullable, newNullable);
        this.Nullable = newNullable;
        this.markDirty();
        this.propertyHasChanged("Nullable");
    }
    return this;
};

BM.FieldObject.prototype.setShowFormat = function (newShowFormat) {
    if (newShowFormat == null)
        newShowFormat = "";

    if (this.ShowFormat != newShowFormat) {
        this.beforePropertyChanged("ShowFormat", this.ShowFormat, newShowFormat);
        this.ShowFormat = newShowFormat;
        this.markDirty();
        this.propertyHasChanged("ShowFormat");
    }
    return this;
};

BM.FieldObject.prototype.setDefaultValue = function (newDefaultValue) {
    if (newDefaultValue == "" || $.trim(newDefaultValue) == "")
        newDefaultValue = null;

    if (newDefaultValue != null)
        newDefaultValue = newDefaultValue.toString();

    if (this.DefaultValue != newDefaultValue) {
        this.beforePropertyChanged("DefaultValue", this.DefaultValue, newDefaultValue);
        this.DefaultValue = newDefaultValue;
        this.markDirty();
        this.propertyHasChanged("DefaultValue");
    }
    return this;
};

BM.FieldObject.prototype.setAllowUndefinedCode = function (newAllowUndefinedCode) {
    if (null == newAllowUndefinedCode || "" === newAllowUndefinedCode)
        return null;

    if (this.AllowUndefinedCode != newAllowUndefinedCode) {
        this.beforePropertyChanged("AllowUndefinedCode", this.AllowUndefinedCode, newAllowUndefinedCode);
        this.AllowUndefinedCode = newAllowUndefinedCode;
        this.markDirty();
        this.propertyHasChanged("AllowUndefinedCode");
    }
    return this;
};

BM.FieldObject.prototype.setAllowMultSelect = function (newAllowMultSelect) {
    if (null == newAllowMultSelect || "" === newAllowMultSelect)
        return null;

    if (this.AllowMultSelect != newAllowMultSelect) {
        this.beforePropertyChanged("AllowMultSelect", this.AllowMultSelect, newAllowMultSelect);
        this.AllowMultSelect = newAllowMultSelect;
        this.markDirty();
        this.propertyHasChanged("AllowMultSelect");
    }
    return this;
};

BM.FieldObject.prototype.setAllowNotLeafNodeRefer = function (newAllowNotLeafNodeRefer) {
    if (null == newAllowNotLeafNodeRefer || "" === newAllowNotLeafNodeRefer)
        return null;

    if (this.AllowNotLeafNodeRefer != newAllowNotLeafNodeRefer) {
        this.beforePropertyChanged("AllowNotLeafNodeRefer", this.AllowNotLeafNodeRefer, newAllowNotLeafNodeRefer);
        this.AllowNotLeafNodeRefer = newAllowNotLeafNodeRefer;
        this.markDirty();
        this.propertyHasChanged("AllowNotLeafNodeRefer");
    }
    return this;
};

//设置汇总方式.
BM.FieldObject.prototype.setGatherType = function (newGatherType) {
    if (newGatherType === "" || newGatherType == null)
        newGatherType = BM.Enum.FieldGatherType.fg_None;

    if (this.GatherType != newGatherType) {
        this.beforePropertyChanged("GatherType", this.GatherType, newGatherType);
        this.GatherType = newGatherType;
        this.markDirty();
        this.propertyHasChanged("GatherType");
    }
    return this;
};

BM.FieldObject.prototype.markDirty = function () {
    if (this.IsNew == false) {
        if (this.IsDirty != true) {
            this.beforePropertyChanged("IsDirty", this.IsDirty, true);
            this.IsDirty = true;
            this.propertyHasChanged("IsDirty");
        }
    }
    return this;
};

BM.FieldObject.prototype.markNew = function () {
    if (this.IsNew != true) {
        this.beforePropertyChanged("IsNew", this.IsNew, true);
        this.IsNew = true;
        this.propertyHasChanged("IsNew");
    }
    if (this.IsDirty != false) {
        this.beforePropertyChanged("IsDirty", this.IsDirty, false);
        this.IsDirty = false;
        this.propertyHasChanged("IsDirty");
    }
    if (this.IsDeleted != false) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, false);
        this.IsDeleted = false;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.FieldObject.prototype.markDeleted = function () {
    if (this.IsNew != false) {
        this.beforePropertyChanged("IsNew", this.IsNew, false);
        this.IsNew = false;
        this.propertyHasChanged("IsNew");
    }
    if (this.IsDirty != false) {
        this.beforePropertyChanged("IsDirty", this.IsDirty, false);
        this.IsDirty = false;
        this.propertyHasChanged("IsDirty");
    }
    if (this.IsDeleted != true) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, true);
        this.IsDeleted = true;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.FieldObject.prototype.markUnDeleted = function () {
    if (this.IsDeleted != false) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, false);
        this.IsDeleted = false;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.FieldObject.prototype.toString = function () {
    return "指标ID【" + this.ID + "】Code【" + this.Code + "】类型【" + this.Type + "】";
};

//从原指标复制信息到当前指标。除了主键ID.
BM.FieldObject.prototype.copyFrom = function (sourceField) {
    if (sourceField == null || !(sourceField instanceof BM.FieldObject))
        return;

    this.setType(sourceField.Type); 
    this.setGatherType(sourceField.GatherType);
    this.setSize(sourceField.Size);
    this.setTitle(sourceField.Title);
    this.setCode(sourceField.Code);
    this.setFractionDigits(sourceField.FractionDigits);
    this.setOwnerTableID(sourceField.OwnerTableID);
    this.setReferField(sourceField.ReferField);
    this.setNullable(sourceField.Nullable);
    this.setShowFormat(sourceField.ShowFormat);
    this.setDefaultValue(sourceField.DefaultValue);
    this.setAllowMultSelect(sourceField.AllowMultSelect);
};
