﻿//-------------------------------------------------------------------------------------------------------------------------------表单.
/*
* @class FormObject操作类
* @param id {String} 当前表单的ID
* @param title {String} 当前表单的标题
* @returns 返回一个FormObject操作类实例
*/
BM.FormObject = function (formObj) {
    BM.UndoBase.call(this);

    this.FormStyle = formObj.FormStyle;
    this.ID = formObj.ID;
    this.Title = formObj.Title;
    this.Order = formObj.Order;
    this.BigDataChanged = formObj.BigDataChanged;
    this.FillingGuide = formObj.FillingGuide;
    this.ActiveCondition = formObj.ActiveCondition;
    this.UnGather = formObj.UnGather;

    this.IsNew = formObj.IsNew;
    this.IsDeleted = formObj.IsDeleted;
    this.IsDirty = formObj.IsDirty;

    this.Published = formObj.Published;
    this.FormStatus = formObj.FormStatus;
    this.FormStyleChanged = false;

    this.OwnGroupId = formObj.OwnGroupId;
    this.Code = formObj.Code;
    this.TaskId = formObj.TaskId;
    this.FormType = formObj.FormType;
    this.Regions = new BM.UndoList("数据区域");
    this.ReservedFloatRegion = new BM.UndoList("备份浮动区域");
    this.RowExtensions = new BM.UndoList("区域扩展设置");
    for (var regId in formObj.Regions) {
        var tmp = formObj.Regions[regId];
        var regionObj = new BM.RegionObject(tmp);
        this.Regions.add(regionObj.ID, regionObj);

        if (regionObj.RegionKind != BM.Enum.DataRegionKind.dr_Simple) {
            var bkUpRegionObj = new BM.RegionObject(tmp);
            this.ReservedFloatRegion.add(bkUpRegionObj.ID, bkUpRegionObj);
        }

        //每一行都存储一个编号和小伙伴名称的设置，以便兼容新建表格.
        for (var row = regionObj.RegionTop; row <= regionObj.RegionBottom; row++) {
            var newRegionExtension = new BM.RegionExtension(regionObj.RegionExtension.toString());
            this.RowExtensions.add(row, newRegionExtension);
        }
    }
    this.DelRegions = new BM.UndoList("删除数据区域");

    this.Formulas = new BM.UndoList("公式");
    for (var fmId in formObj.Formulas) {
        var tempFormula = formObj.Formulas[fmId];
        var formulaObj = new BM.FormulaObject(tempFormula);
        this.Formulas.add(formulaObj.ID, formulaObj);
    }

    this.DelFormulas = new BM.UndoList("删除公式");

    //是否需要重新运用规则分析浮动行.
    this.NeedReAnalysis = false;
    this.onInitializingReservedCount = false;//是否正在初始化保留行相关信息.
    this.GlobalChanged = false;
};

BM.FormObject.prototype = new BM.UndoBase();

//属性更改事件，用于发送通知.
BM.FormObject.prototype.propertyHasChanged = function (propertyName) {
    //调用基类方法.
    BM.UndoBase.prototype.propertyHasChanged.call(this, propertyName);

    this.GlobalChanged = true;

    if (propertyName == "Title") {//note 表单标题修改后，同步更新页签的名称.
        var formTabModel = $.m("BM_FormTabBarModel");
        if (formTabModel != null)
            formTabModel.updateTabTextByFormId(this.ID, this.Title);
    }
};

//表单缓存是否已经修改.
BM.FormObject.prototype.isDataChanged = function () {
    if (this.IsNew === true)
        return true;

    if (this.GlobalChanged === false)
        return false;

    if (this.IsNew === true
        || this.IsDirty === true
        || this.IsDeleted === true
        || this.FormStyleChanged === true)
        return true;

    if (this.DelRegions.getCount() > 0)
        return true;

    if (this.DelFormulas.getCount() > 0)
        return true;

    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        var regionChanged = regionObj.isDataChanged();
        if (regionChanged == true)
            return true;
    }

    for (var formulaId in this.Formulas) {
        var formulaObj = this.Formulas.get(formulaId);
        var formulaChanged = formulaObj.isDataChanged();
        if (formulaChanged == true)
            return true;
    }
    return false;
};

//返回当前任务下是否有表单正在保存.
//note 正在保存的话，则不可以进行发布操作.
BM.FormObject.prototype.isBusySaving = function () {
    return this.FormStatus === BM.Enum.FormStatus.fs_Saving;
};

//保存时，重新复制一份数据，以免原有数据状态被破坏.
BM.FormObject.prototype.getCopyToSave = function () {
    var formHelper = $.m("BM_MainFrameModel").getService(BM_ServiceName_FormHelper);

    var newFormType = (this.getFloatRegionsCount() >= 1) ? BM.Enum.FormType.ft_FloatForm : BM.Enum.FormType.ft_FixForm;
    this.setFormType(newFormType);

    this.reAssignFieldsIfFormIsNew();

    var result = {};

    result.ID = this.ID;
    result.IsNew = this.IsNew;
    result.IsDeleted = this.IsDeleted;
    result.IsDirty = this.IsDirty;
    result.OwnGroupId = this.OwnGroupId;
    if (this.IsNew === true || this.IsDeleted === true || this.IsDirty === true) {
        if (this.FormStatus > BM.Enum.FormStatus.fs_Uninitialize) {
            result.FormStyle = formHelper.getCompressedStyle(this.FormStyle);
        } else {
            result.FormStyle = null;
        }
        result.Title = this.Title;
        result.Order = this.Order;
        result.BigDataChanged = this.BigDataChanged;
        result.OwnGroupId = this.OwnGroupId;
        result.Code = this.Code;
        result.TaskId = this.TaskId;
        result.FormType = this.FormType;
        result.FillingGuide = this.FillingGuide;
        result.UnGather = this.UnGather;
        result.ActiveCondition = this.ActiveCondition;
    }
    result.Regions = new BM.BusinessList("数据区域");
    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId).getCopyToSave();
        if (regionObj.RegionKind != BM.Enum.DataRegionKind.dr_Simple) {
            var rowExtension = this.RowExtensions.get(regionObj.RegionTop);
            if (rowExtension != null)
                regionObj.RegionExtension = rowExtension.getString();
            else
                regionObj.RegionExtension = "";
        } else {
            regionObj.RegionExtension = "";
        }
        result.Regions.add(regionObj.ID, regionObj);
    }

    for (var deletedRegionId in this.DelRegions) {
        var deletedRegionObj = this.DelRegions.get(deletedRegionId).getCopyToSave();
        result.Regions.add(deletedRegionObj.ID, deletedRegionObj);
    }

    result.Formulas = new BM.BusinessList("公式");
    for (var formulaId in this.Formulas) {
        var formulaObj = this.Formulas.get(formulaId).getCopyToSave();
        result.Formulas.add(formulaObj.ID, formulaObj);
    }

    for (var deletedFormulaId in this.DelFormulas) {
        var deletedFormulaObj = this.DelFormulas.get(deletedFormulaId).getCopyToSave();
        result.Formulas.add(deletedFormulaObj.ID, deletedFormulaObj);
    }

    return result;
};

BM.FormObject.prototype.getFormRegionKind = function () {
    for (var regId in this.Regions) {
        var regionObj = this.Regions.get(regId);
        if (regionObj.RegionKind !== BM.Enum.DataRegionKind.dr_Simple) {
            return regionObj.RegionKind;
        }
    }
    return BM.Enum.DataRegionKind.dr_Simple;
};

BM.FormObject.prototype.beforeSave = function () {
    //todo 保存表单之前，进行缓存参数的设置.
    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        regionObj.beforeSave();
    }

    //note 如果浮动区域的个数大于0的话，则将当前表单的类型设置为浮动表.
    var newFormType = (this.getFloatRegionsCount() >= 1) ? BM.Enum.FormType.ft_FloatForm : BM.Enum.FormType.ft_FloatForm.ft_FixForm;
    this.setFormType(newFormType);

    for (var deletedRegionId in this.DelRegions) {
        var deletedRegionObj = this.DelRegions.get(deletedRegionId);
        deletedRegionObj.beforeSave();

        this.Regions.add(deletedRegionObj.ID, deletedRegionObj);
        this.DelRegions.remove(deletedRegionId);
    }

    //处理公式
    for (var delFormulaId in this.DelFormulas) {
        var deletedFml = this.DelFormulas.get(delFormulaId);

        this.Formulas.add(deletedFml.ID, deletedFml);
        this.DelFormulas.remove(delFormulaId);
    }
};

//判断当前表单内是否包含参数，即链接.
BM.FormObject.prototype.isExistLink = function () {
    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        if (regionObj.IsDeleted == false) {
            for (var linkId in regionObj.Links) {
                var linkObj = regionObj.Links.get(linkId);
                if (linkObj.IsDeleted == false)
                    return true;
            }
        }
    }
    return false;
};

//判断是否存在重名存储表的检查，从区域内部改成表单内部检查.
BM.FormObject.prototype.isExistTableCode = function (tableCode) {
    if (tableCode == null || tableCode === "")
        return false;
    for (var regId in this.Regions) {
        var regionObj = this.Regions.get(regId);
        var existedTableByCodeInRegion = regionObj.isExistTableCode(tableCode);
        if (existedTableByCodeInRegion == true)
            return true;
    }

    for (var delRegId in this.DelRegions) {
        var delRegionObj = this.DelRegions.get(delRegId);
        var existedTableByCodeInDelRegion = delRegionObj.isExistTableCode(tableCode);
        if (existedTableByCodeInDelRegion == true)
            return true;
    }

    return false;
};

/**
* 获取当前表单的数据库ID
* @returns {String} 表单的ID
*/
BM.FormObject.prototype.getFormID = function () {
    return this.ID;
};

BM.FormObject.prototype.getRegions = function () {
    return this.Regions;
};

BM.FormObject.prototype.getRegionsCount = function () {
    return this.Regions.getCount();
};

//返回排序后的数据区域【按照坐标顺序】
BM.FormObject.prototype.getSortedRegions = function () {
    var result = [];
    var type = 0;
    for (var regId in this.Regions) {
        var regObj = this.Regions.get(regId);
        if (regObj.RegionKind == BM.Enum.DataRegionKind.dr_RowList) {
            type = 1;
        } else if (regObj.RegionKind == BM.Enum.DataRegionKind.dr_ColList) {
            type = 2;
        }
        result.push(regObj);
    }
    var orderedRegions = null;
    if (type == 0 || type == 1) {
        orderedRegions = result.sort(
            function (a, b) {
                if (a.RegionTop < b.RegionTop) return -1;
                if (a.RegionTop > b.RegionTop) return 1;
                return 0;
            }
        );
    }
    else if (type == 2) {
        orderedRegions = result.sort(
           function (a, b) {
               if (a.RegionLeft < b.RegionLeft) return -1;
               if (a.RegionLeft > b.RegionLeft) return 1;
               return 0;
           }
       );
    }
    return orderedRegions;
};

BM.FormObject.prototype.getFloatRegionsCount = function () {
    var floatRegionCount = 0;
    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        if (regionObj.RegionKind == BM.Enum.DataRegionKind.dr_RowList || regionObj.RegionKind == BM.Enum.DataRegionKind.dr_ColList)
            floatRegionCount += 1;
    }

    return floatRegionCount;
};

BM.FormObject.prototype.getRegionByID = function (regId) {
    return this.Regions.get(regId);
};

//默认优先选择浮动区域，后选择固定区域，因为大的固定区域会把浮动区域包含。
BM.FormObject.prototype.getRegionByPos = function (col, row) {
    for (var regFloat in this.Regions) {
        var regObjFloat = this.Regions.get(regFloat);
        if (regObjFloat.RegionKind != BM.Enum.DataRegionKind.dr_Simple
            && regObjFloat.contains(col, row))
            return regObjFloat;
    }

    for (var regFixed in this.Regions) {
        var regObjFixed = this.Regions.get(regFixed);
        if (regObjFixed.RegionKind == BM.Enum.DataRegionKind.dr_Simple
            && regObjFixed.contains(col, row))
            return regObjFixed;
    }
    return null;
};

//新方法支持从行列选择对应的浮动行【单行浮动】.
BM.FormObject.prototype.getFloatRegionByPos = function (col, row) {
    for (var regFloat in this.Regions) {
        var regObjFloat = this.Regions.get(regFloat);
        if (regObjFloat.RegionKind != BM.Enum.DataRegionKind.dr_Simple) {
            if (regObjFloat.RegionKind == BM.Enum.DataRegionKind.dr_RowList) {
                if (regObjFloat.RegionTop == row)
                    return regObjFloat;
            }
            else if (regObjFloat.RegionKind == BM.Enum.DataRegionKind.dr_ColList) {
                if (regObjFloat.RegionLeft == col)
                    return regObjFloat;
            }
        }
    }

    return null;
};

BM.FormObject.prototype.getFixedRegionByPos = function (col, row) {
    for (var regFixed in this.Regions) {
        var regObjFixed = this.Regions.get(regFixed);
        if (regObjFixed.RegionKind == BM.Enum.DataRegionKind.dr_Simple
            && regObjFixed.contains(col, row))
            return regObjFixed;
    }

    return null;
};

BM.FormObject.prototype.getRegionsByCol = function (col) {
    var result = new BM.BusinessList();
    for (var regId in this.Regions) {
        var regionObj = this.Regions.get(regId);
        if (regionObj.containsCol(col))
            result.add(regionObj.ID, regionObj);
    }

    return result;
};

BM.FormObject.prototype.getRegionsByRow = function (row) {
    var result = new BM.BusinessList();
    for (var regId in this.Regions) {
        var regionObj = this.Regions.get(regId);
        if (regionObj.containsRow(row))
            result.add(regionObj.ID, regionObj);
    }

    return result;
};

//根据指定行获取对应的浮动区域.
BM.FormObject.prototype.getFloatRegionByRegionTop = function (row) {
    for (var regId in this.Regions) {
        var regionObj = this.Regions.get(regId);
        if (regionObj.RegionTop == row
            && regionObj.RegionKind == BM.Enum.DataRegionKind.dr_RowList)
            return regionObj;
    }

    return null;
};

//根据指定行获取对应的浮动区域.
BM.FormObject.prototype._getDeletedFloatRegion = function () {
    for (var regId in this.DelRegions) {
        var deledRegionObj = this.DelRegions.get(regId);
        if (deledRegionObj.RegionKind == BM.Enum.DataRegionKind.dr_RowList)
            return deledRegionObj;
    }

    return null;
};

//删除数据区域（表样上通过按钮删除区域或者创建区域）
BM.FormObject.prototype.deleteRegion = function (regionId) {
    var regionObj = this.getRegionByID(regionId);
    if (regionObj) {

        if (isNaN(regionObj.ID) && regionObj.ID.indexOf("RegionObject") >= 0) {
            //若要删除的区域本身就是新建，则直接删除.2014-12-08.hcf
            this.Regions.remove(regionObj.ID);
        } else {
            regionObj.markDeleted();
            this.DelRegions.add(regionObj.ID, regionObj);
            this.Regions.remove(regionObj.ID);
        }

        var fixedRegionByPos = this.getFixedRegionByPos(regionObj.RegionLeft, regionObj.RegionTop);
        for (var linkId in regionObj.Links) { //将区域下的链接和指标移至表单的固定区域中.)
            var linkObj = regionObj.Links.get(linkId);
            linkObj.IsDeleted = false;
            linkObj.DataRegionID = fixedRegionByPos.ID;
            fixedRegionByPos.Links.add(linkId, linkObj);
            if (!isNaN(linkObj.LinkExpression)) {
                var fieldObj = regionObj.getFieldById(linkObj.LinkExpression);
                fieldObj.IsDeleted = false;
                var newOwnerTableObj = fixedRegionByPos.queryOrCreateTableObject();
                fieldObj.setOwnerTableID(newOwnerTableObj.ID);
                newOwnerTableObj.Fields.add(fieldObj.ID, fieldObj);
            }
        }
        for (var tableId in regionObj.Tables) {
            var tableCanDelete = regionObj.__checkTableCanDelete(tableId);
            if (tableCanDelete === true) {
                var tableObj = regionObj.Tables.get(tableId);
                if (isNaN(tableObj.ID) && tableObj.ID.indexOf("TableObject") >= 0) {
                    //note 循环体外进行清除.
                    //regionObj.Tables.remove(tableObj.ID);
                } else {
                    tableObj.markDeleted();
                    regionObj.DelTables.add(tableObj.ID, tableObj);
                    regionObj.Tables.remove(tableObj.ID);
                }
            }
        }

        regionObj.Links.clear();
        regionObj.Tables.clear();
    }
};

//删除公式
BM.FormObject.prototype.deleteFormula = function (formulaId) {
    if (formulaId < 0 || formulaId == null || formulaId === "")
        return;
    var tmpFormula = this.Formulas.get(formulaId);
    if (tmpFormula == null) {
        var delFormula = this.DelFormulas.get(formulaId);
        return;
    }
    if (isNaN(tmpFormula.ID) && tmpFormula.ID.indexOf("FormulaObject") >= 0) {
        this.Formulas.remove(formulaId);
    } else {
        tmpFormula.markDeleted();
        this.Formulas.remove(formulaId);
        this.DelFormulas.add(tmpFormula.ID, tmpFormula);
    }
};

BM.FormObject.prototype.deleteFormulaByPos = function (col, row) {
    var delFormulaIds = [];
    for (var formulaId in this.Formulas) {
        var tmpFormula = this.Formulas.get(formulaId);
        if (tmpFormula.PosX == col && tmpFormula.PosY == row) {
            delFormulaIds.push(formulaId);
        }
    }

    for (var idx in delFormulaIds) {
        var fmlId = delFormulaIds[idx];
        this.deleteFormula(fmlId);
    }
};

//创建一个与表单区域大小一样的固定区域.
BM.FormObject.prototype.createDefaultRegion = function (left, top, right, bottom) {
    if (left > right || top > bottom || left < 0 || top < 0 || right < 0 || bottom < 0) {
        //todo 非法参数.
        alert("invalid parameter: FormObject.createDefaultRegion(left, top, right, bottom)");
    }

    for (var regId in this.Regions) {//修复错误参数，工作表中只允许存在一个固定区域.
        var regionObj = this.Regions.get(regId);
        if (regionObj && regionObj.RegionKind == BM.Enum.DataRegionKind.dr_Simple) {
            if (regionObj.RegionLeft != left
                || regionObj.RegionTop != top
                || regionObj.RegionRight != right
                || regionObj.RegionBottom != bottom) {
                regionObj.setRegionLeft(left);
                regionObj.setRegionTop(top);
                regionObj.setRegionRight(right);
                regionObj.setRegionBottom(bottom);

                return regionObj;
            }
        }
    }


    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);

    var newRegionId = formHelper.JCPGuid("RegionObject-");
    var tmpRegion = {};
    tmpRegion.ID = newRegionId;
    tmpRegion.RegionLeft = left;
    tmpRegion.RegionTop = top;
    tmpRegion.RegionRight = right;
    tmpRegion.RegionBottom = bottom;
    tmpRegion.RegionKind = BM.Enum.DataRegionKind.dr_Simple;

    tmpRegion.IsNew = true;
    tmpRegion.IsDirty = false;
    tmpRegion.IsDeleted = false;
    tmpRegion.FormID = this.ID;
    tmpRegion.Title = this.Title + "默认固定区域";
    var newRegionObj = new BM.RegionObject(tmpRegion);

    this.Regions.add(newRegionObj.ID, newRegionObj);

    return newRegionObj;
};

//获取当前表单下的默认固定区域.
//note 如果表单为新建，且没有指标的话，则返回null，否则肯定会有区域，没有则为参数异常.
BM.FormObject.prototype.getDefaultRegion = function () {
    for (var regId in this.Regions) {
        var regObj = this.Regions.get(regId);
        if (regObj.RegionKind == BM.Enum.DataRegionKind.dr_Simple) {
            return regObj;
        }
    }
    return null;
};

//创建公式
BM.FormObject.prototype.createFormulaObject = function (col, row, formulaExp, checkFormula) {
    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);
    var curTask = mainFrameModel.getCurrentTask();
    var defaultScheme = curTask.getDefaultFormulaScheme();

    var formulaId = formHelper.JCPGuid("FormulaObject-");
    var fmlOrder = curTask.getNewOrder();
    var formulaObj = [];
    formulaObj.ID = formulaId;
    formulaObj.Expression = formulaExp;
    formulaObj.Description = "";
    formulaObj.Order = fmlOrder;
    formulaObj.Code = fmlOrder;
    formulaObj.AutoExecute = true;
    formulaObj.UseCalculate = !checkFormula;
    formulaObj.UseBalance = false;
    formulaObj.UseCheck = checkFormula;
    formulaObj.FormID = this.getFormID();
    formulaObj.SchemeID = defaultScheme.ID; //初始化任务创建的公式方案Id
    formulaObj.PosX = col;
    formulaObj.PosY = row;

    formulaObj.IsNew = true;
    formulaObj.IsDirty = false;
    formulaObj.IsDeleted = false;

    var formula = new BM.FormulaObject(formulaObj);
    return formula;
};

BM.FormObject.prototype.hasFormStyle = function () {
    if (!this.FormStyle) return false;
    if (this.FormStyle.rowCount > 0 || this.FormStyle.colCount > 0)
        return true;
    return false;
};

/*
* 根据坐标获取对应的链接.
* @param col {Int32} x坐标
* @param row {Int32} y坐标
* @returns 存在则返回一个LinkObject，不存在返回NULL.
*/
BM.FormObject.prototype.getLinkByPos = function (col, row) {
    for (var regId in this.Regions) {
        var linkObj = this.Regions.get(regId).getLinkByPos(col, row);
        if (linkObj)
            return linkObj;
    }

    return null;
};

/*
* 根据坐标获取对应的链接.
* @param col {Int32} x坐标
* @param row {Int32} y坐标
* @returns 存在则返回一个LinkObject，不存在返回NULL.
*/
BM.FormObject.prototype.getFieldByPos = function (col, row) {
    var linkObj = this.getLinkByPos(col, row);
    if (linkObj)
        return this.getFieldById(linkObj.LinkExpression);

    return null;
};

BM.FormObject.prototype.getTableById = function (tableId) {
    for (var regId in this.Regions) {
        var regionObj = this.Regions.get(regId);
        var tableObj = regionObj.getTableById(tableId);
        if (tableObj != null)
            return tableObj;
    }
    return null;
};

BM.FormObject.prototype.getFormulaByPos = function (col, row) {
    for (var fmId in this.Formulas) {
        var tempFormula = this.Formulas.get(fmId);
        if (tempFormula.PosX == col && tempFormula.PosY == row) {
            return tempFormula;
        }
    }
    return null;
};

BM.FormObject.prototype.getFormulaById = function (formulaId) {
    return this.Formulas.get(formulaId);
};

BM.FormObject.prototype.getFormulaExpressByPos = function (col, row) {

    var tmpFml = this.getFormulaByPos(col, row);
    if (tmpFml == null)
        return "";
    var result = tmpFml.Expression;
    var cellLink = this.getLinkByPos(col, row);
    var cellLinkGuid = "[" + cellLink.UniqueCode + "]";
    if (tmpFml && tmpFml.Expression && tmpFml.Expression.length > cellLinkGuid.length) {
        var leftCellCode = tmpFml.Expression.substring(0, cellLinkGuid.length);
        if (leftCellCode == cellLinkGuid) {
            result = tmpFml.Expression.substring(cellLinkGuid.length);
        }
    }
    return result;
};
/*
* 根据坐标判断当前坐标是否属于某个固定区域.
* @param col {Int32} x坐标
* @param row {Int32} y坐标

* @returns 存在则返回true，不存在返回false.
*/
BM.FormObject.prototype.cellInFloatRegion = function (col, row) {
    for (var regFloat in this.Regions) {
        var regObjFloat = this.Regions.get(regFloat);
        if (regObjFloat.RegionKind != BM.Enum.DataRegionKind.dr_Simple
            && regObjFloat.contains(col, row))
            return regObjFloat;
    }
    return null;
};

/*
* 根据选区创建对应的浮动区域.
* @param x {Int32} x坐标
* @param y {Int32} y坐标
* @param width {Int32} 宽度
* @param height {Int32} 高度
* @param rowList {Boolean} true为行浮动，否则为列浮动
* @returns 存在则返回一个LinkObject，不存在返回NULL.
*/
BM.FormObject.prototype.setFloatRegionByPos = function (x, width, y, height, rowList, colCount, rowCount) {
    //note 不进行冲突检查，如果区域已经存在的话，则直接返回该区域对象即可.
    //for (var col = x; col < x + width; col++) {
    //    for (var row = y; row < y + height; row++) {
    //        var cellInFloatRegion = this.cellInFloatRegion(col, row);
    //        if (cellInFloatRegion != null) {
    //            if (rowList == true && cellInFloatRegion == BM.Enum.DataRegionKind.dr_RowList) {
    //                jNoty("区域冲突");
    //                return null;
    //            }
    //            else if (rowList == false && cellInFloatRegion == BM.Enum.DataRegionKind.dr_ColList) {
    //                jNoty("区域冲突");
    //                return null;
    //            }
    //        }
    //    }
    //}

    var tmpTop = y;
    var tmpLeft = x;
    var tmpRight = x + width - 1;
    var tmpBottom = y + height - 1;
    var regKind;
    if (rowList) {
        tmpLeft = 1;
        tmpRight = colCount;
        regKind = BM.Enum.DataRegionKind.dr_RowList;
    } else {
        tmpTop = 1;
        tmpBottom = rowCount;
        regKind = BM.Enum.DataRegionKind.dr_ColList;
    }

    //var checkRegionAvailable = this.__checkSetFloatRegionAvailable(tmpLeft, tmpTop, tmpRight, tmpBottom, regKind);
    //if (checkRegionAvailable == false) {
    //    jNoty("区域冲突");
    //    return null;
    //}

    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);

    //var formDefaultRegion = null;
    //if (rowList) {
    //    for (var col1 = x; col1 < x + width; col1++) {
    //        for (var regId1 in this.Regions) {
    //            var regObj1 = this.Regions.get(regId1);
    //            if (regObj1.RegionKind == BM.Enum.DataRegionKind.dr_Simple
    //                && regObj1.RegionLeft <= col1
    //                && regObj1.RegionRight >= col1
    //                && regObj1.RegionTop <= y
    //                && regObj1.RegionBottom >= y + height - 1) {
    //                formDefaultRegion = regObj1;
    //                break;
    //            }
    //        }
    //    }
    //} else {
    //    for (var row1 = y; row1 < y + height; row1++) {
    //        for (var regId2 in this.Regions) {
    //            var regObj2 = this.Regions.get(regId2);
    //            if (regObj2.RegionKind == BM.Enum.DataRegionKind.dr_Simple
    //                && regObj2.RegionLeft <= x
    //                && regObj2.RegionRight >= x + width - 1
    //                && regObj2.RegionTop <= row1
    //                && regObj2.RegionBottom >= row1) {
    //                formDefaultRegion = regObj2;
    //                break;
    //            }
    //        }
    //    }
    //}

    var formDefaultRegion = this.getDefaultRegion();

    if (!formDefaultRegion) {
        formDefaultRegion = this.createDefaultRegion(1, 1, colCount - 1, rowCount - 1);
    }

    //todo 考虑到区域缩小或者区域扩大的情况.
    var existedRegion = this.getFloatRegionByPos(x, y);
    if (existedRegion != null) {
        //x, width, y, height
        var preLeft = existedRegion.RegionLeft;
        var preRight = existedRegion.RegionRight;
        var preTop = existedRegion.RegionTop;
        var preBottom = existedRegion.RegionBottom;
        var preWidth = preRight - preLeft + 1;
        var preHeight = preBottom - preTop + 1;

        if (rowList === true && existedRegion.RegionKind !== BM.Enum.DataRegionKind.dr_RowList) {
            throw new Error("区域冲突");
        } else if (rowList === false && existedRegion.RegionKind !== BM.Enum.DataRegionKind.dr_ColList) {
            throw new Error("区域冲突");
        }
        else if (rowList === true && existedRegion.RegionKind === BM.Enum.DataRegionKind.dr_RowList) {
            if (preHeight > height) {
                //note 区域高度变小.将原有属于该浮动区域的链接转移到默认固定区域中.
                var links = new BM.BusinessList();
                for (var col = 1; col < colCount; col++) {
                    for (var row = y + height; row <= preBottom; row++) {
                        var linkByPos = this.getLinkByPos(col, row);
                        if (linkByPos) {
                            links.add(linkByPos.ID, linkByPos);
                        }
                    }
                }

                for (var linkId in links) {
                    var linkObj = links[linkId];
                    var ownerRegion = this.getRegionByID(linkObj.DataRegionID);
                    if (ownerRegion.ID == formDefaultRegion.ID)
                        continue;
                    this.__moveLinkToNewRegion(linkObj, ownerRegion, formDefaultRegion);

                    var fieldObj = this.getFieldById(linkObj.LinkExpression);
                    var srcTable = ownerRegion.getTableById(fieldObj.OwnerTableID);
                    var targetTable = formDefaultRegion.queryOrCreateTableObject();
                    var newFieldObj = this.__copyFieldToNewTableNDelete(fieldObj, srcTable, targetTable, ownerRegion);
                    if (newFieldObj != null) {
                        linkObj.setLinkExpression(newFieldObj.ID);
                    }
                }
            }
            else if (preHeight < height) {
                //note 区域高度变大.
            }

            existedRegion.RegionLeft = 1;
            existedRegion.RegionTop = y;
            existedRegion.RegionRight = colCount - 1;
            existedRegion.RegionBottom = y + height - 1;
            existedRegion.RegionKind = BM.Enum.DataRegionKind.dr_RowList;
            existedRegion.Title = this.Title + "行浮动区域";
        }
        else if (rowList === false && existedRegion.RegionKind === BM.Enum.DataRegionKind.dr_ColList) {
            //note 列浮动暂不支持.
            if (preWidth > width) {
                //note 区域宽度变小.
            }

            existedRegion.RegionLeft = x;
            existedRegion.RegionTop = 1;
            existedRegion.RegionRight = x + width - 1;
            existedRegion.RegionBottom = rowCount - 1;
            existedRegion.RegionKind = BM.Enum.DataRegionKind.dr_ColList;
            existedRegion.Title = this.Title + "列浮动区域";
        }
    } else {
        var newRegionId = formHelper.JCPGuid("RegionObject-");
        var tmpRegion = {};
        tmpRegion.IsNew = true;
        tmpRegion.IsDirty = false;
        tmpRegion.IsDeleted = false;

        tmpRegion.ID = newRegionId;
        tmpRegion.FormID = this.ID;
        if (rowList) {
            tmpRegion.RegionLeft = 1;
            tmpRegion.RegionTop = y;
            tmpRegion.RegionRight = colCount - 1;
            tmpRegion.RegionBottom = y + height - 1;
            tmpRegion.RegionKind = BM.Enum.DataRegionKind.dr_RowList;
            tmpRegion.Title = this.Title + "行浮动区域";
        } else {
            tmpRegion.RegionLeft = x;
            tmpRegion.RegionTop = 1;
            tmpRegion.RegionRight = x + width - 1;
            tmpRegion.RegionBottom = rowCount - 1;
            tmpRegion.RegionKind = BM.Enum.DataRegionKind.dr_ColList;
            tmpRegion.Title = this.Title + "列浮动区域";
        }
        existedRegion = new BM.RegionObject(tmpRegion);
        this.Regions.add(existedRegion.ID, existedRegion);
    }
    var links = new BM.BusinessList();
    for (var row3 = existedRegion.RegionTop; row3 <= existedRegion.RegionBottom; row3++) {
        for (var col3 = existedRegion.RegionLeft; col3 <= existedRegion.RegionRight; col3++) {
            var linkByPos = this.getLinkByPos(col3, row3);
            if (linkByPos && linkByPos.DataRegionID != existedRegion.ID) {
                links.add(linkByPos.ID, linkByPos);
            }
        }
    }

    for (var linkId in links) {
        var linkObj = links[linkId];
        var ownerRegion = this.getRegionByID(linkObj.DataRegionID);
        this.__moveLinkToNewRegion(linkObj, ownerRegion, existedRegion);

        var fieldObj = this.getFieldById(linkObj.LinkExpression);
        var srcTable = ownerRegion.getTableById(fieldObj.OwnerTableID);
        var targetTable = existedRegion.queryOrCreateTableObject();
        var newFieldObj = this.__copyFieldToNewTableNDelete(fieldObj, srcTable, targetTable, ownerRegion);
        if (newFieldObj != null) {
            linkObj.setLinkExpression(newFieldObj.ID);
        }
    }

    for (var regId in this.Regions) {
        if (regId === existedRegion.ID)
            continue;
        var regObj = this.Regions.get(regId);
        if (regObj.RegionKind === BM.Enum.DataRegionKind.dr_Simple)
            continue;

        if (existedRegion.isIntersected(regObj)) {
            if (existedRegion.RegionKind === BM.Enum.DataRegionKind.dr_RowList) {
                if (existedRegion.RegionTop > regObj.RegionTop) {
                    regObj.setRegionBottom(existedRegion.RegionTop - 1);
                }
                else if (existedRegion.RegionTop < regObj.RegionTop) {
                    regObj.setRegionTop(existedRegion.RegionBottom + 1);
                }
            }
            else if (existedRegion.RegionKind === BM.Enum.DataRegionKind.dr_ColList) {
                if (existedRegion.RegionLeft > regObj.RegionLeft) {
                    regObj.setRegionRight(existedRegion.RegionLeft - 1);
                }
                else if (existedRegion.RegionLeft < regObj.RegionLeft) {
                    regObj.setRegionLeft(existedRegion.RegionRight + 1);
                }
            }
        }
    }

    return existedRegion;
};

//检测当前坐标是否有区域冲突.
BM.FormObject.prototype.__checkSetFloatRegionAvailable = function (left, top, right, bottom, dataRegionKind) {
    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);
    //如果列浮动，监测区域是否有行浮动
    if (dataRegionKind == BM.Enum.DataRegionKind.dr_ColList) {
        for (var regionId1 in this.Regions) {
            var regionObj1 = this.Regions.get(regionId1);
            if (regionObj1.RegionKind == BM.Enum.DataRegionKind.dr_RowList)
                return false;
            else {
                if (regionObj1.RegionKind == BM.Enum.DataRegionKind.dr_ColList) {
                    var regionRectangle1 = regionObj1.getDisplayRectangle();
                    if (formHelper.intersectRect(regionRectangle1.Left, regionRectangle1.Right, left, right))
                        return false;
                }
            }
        }
    }
        //如果行浮动，监测区域是否有列浮动
    else if (dataRegionKind == BM.Enum.DataRegionKind.dr_RowList) {
        for (var regionId2 in this.Regions) {
            var regionObj2 = this.Regions.get(regionId2);
            if (regionObj2.RegionKind == BM.Enum.DataRegionKind.dr_ColList)
                return false;
            else {
                if (regionObj2.RegionKind == BM.Enum.DataRegionKind.dr_RowList) {
                    var regionRectangle2 = regionObj2.getDisplayRectangle();
                    if (formHelper.intersectRect(regionRectangle2.Top, regionRectangle2.Bottom, top, bottom))
                        return false;
                }
            }
        }
    }
    return true;
};

//将数据链接转移到新的数据区域内
BM.FormObject.prototype.__moveLinkToNewRegion = function (linkObject, srcRegionObject, targetRegionObject) {
    if (linkObject.DataRegionID != srcRegionObject.ID
        || !(srcRegionObject.getLinkById(linkObject.ID))
        || linkObject.DataRegionID == targetRegionObject.ID
        || targetRegionObject.getLinkById(linkObject.ID))
        return;

    linkObject.setDataRegionID(targetRegionObject.ID);

    srcRegionObject.Links.remove(linkObject.ID);
    targetRegionObject.Links.add(linkObject.ID, linkObject);
};

//指标进行更换存储表操作时，进行复制新指标，并删除旧指标.
BM.FormObject.prototype.__copyFieldToNewTableNDelete = function (fieldObject, srcTableObject, targetTableObject, srcRegionObject) {
    if (fieldObject.OwnerTableID != srcTableObject.ID
       || !(srcTableObject.getFieldById(fieldObject.ID))
       || fieldObject.OwnerTableID == targetTableObject.ID
       || targetTableObject.getFieldById(fieldObject.ID))
        return null;

    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);

    var newFieldId = formHelper.JCPGuid("FieldObject-");
    var newFieldObj = new BM.FieldObject(fieldObject);
    newFieldObj.ID = newFieldId;
    newFieldObj.IsNew = true;
    newFieldObj.IsDirty = false;
    newFieldObj.IsDeleted = false;
    newFieldObj.OwnerTableID = targetTableObject.ID;
    newFieldObj.TableName = targetTableObject.Code;
    var newFieldCode = newFieldObj.Code;
    while (targetTableObject.isExistFieldCode(newFieldCode)) {//note 指标移到新表中，校验指标标识是否存在重码.
        newFieldCode = formHelper.generateNewStringByTemplet(newFieldCode);
    }
    newFieldObj.Code = newFieldCode;
    srcTableObject.delFieldById(fieldObject.ID);
    targetTableObject.addField(newFieldObj);

    var tableCanDelete = srcRegionObject.__checkTableCanDelete(srcTableObject.ID);
    if (tableCanDelete == true) {
        srcRegionObject.deleteTableByID(srcTableObject.ID);
    }

    return newFieldObj;
};

//根据坐标位置删除该坐标所在的浮动区域，并将区域内参数移至默认固定区域中.
BM.FormObject.prototype.deleteFloatRegionByPos = function (col, row) {
    var regionToDel = null;
    var formDefaultRegion = null;
    for (var regFloat in this.Regions) {
        var regObjFloat = this.Regions.get(regFloat);
        if (regObjFloat.RegionKind != BM.Enum.DataRegionKind.dr_Simple
            && regObjFloat.contains(col, row)) {
            regionToDel = regObjFloat;
        }
        else if (regObjFloat.RegionKind == BM.Enum.DataRegionKind.dr_Simple
            && regObjFloat.contains(col, row)) {
            formDefaultRegion = regObjFloat;
        }
    }

    if (regionToDel && formDefaultRegion) {//将区域中的指标，存储表移到表单的默认大区域中.
        var links = new BM.BusinessList();
        for (var col1 = regionToDel.RegionLeft; col1 <= regionToDel.RegionRight; col1++) {
            for (var row1 = regionToDel.RegionTop; row1 <= regionToDel.RegionBottom; row1++) {
                var linkByPos = regionToDel.getLinkByPos(col1, row1);
                if (linkByPos) {
                    links.add(linkByPos.ID, linkByPos);
                }
            }
        }

        for (var linkId in links) {
            var linkObj = links[linkId];
            this.__moveLinkToNewRegion(linkObj, regionToDel, formDefaultRegion);

            var fieldObj = regionToDel.getFieldById(linkObj.LinkExpression);
            var srcTable = regionToDel.getTableById(fieldObj.OwnerTableID);
            var targetTable = formDefaultRegion.queryOrCreateTableObject();
            var newFieldObj = this.__copyFieldToNewTableNDelete(fieldObj, srcTable, targetTable, regionToDel);
            if (newFieldObj != null) {
                linkObj.setLinkExpression(newFieldObj.ID);
            }
        }

        if (isNaN(regionToDel.ID) && regionToDel.ID.indexOf("RegionObject") >= 0) {
            //todo 删除新建的区域时，区域内链接或者存储表，指标等先移除到默认固定区域后，再进行移除.
            for (var linkId in regionToDel.Links) {
                if (!isNaN(linkId)) {
                    var linkObj = regionToDel.Links.get(linkId);
                    linkObj.markDeleted();
                    this.__moveLinkToDefaultRegionForDelete(linkObj, regionToDel, formDefaultRegion);
                }
            }
            regionToDel.Links.clear();
            for (var linkId in regionToDel.DelLinks) {
                if (!isNaN(linkId)) {
                    var linkObj = regionToDel.DelLinks.get(linkId);
                    linkObj.markDeleted();
                    this.__moveLinkToDefaultRegionForDelete(linkObj, regionToDel, formDefaultRegion);
                }
            }
            regionToDel.DelLinks.clear();
            for (var tableId in regionToDel.Tables) {
                if (!isNaN(tableId)) {
                    var tableObj = regionToDel.Tables.get(tableId);
                    tableObj.markDeleted();
                    this.__moveTableToDefaultRegionForDelete(tableObj, regionToDel, formDefaultRegion);
                }
            }
            regionToDel.Tables.clear();
            for (var tableId in regionToDel.DelTables) {
                if (!isNaN(tableId)) {
                    var tableObj = regionToDel.DelTables.get(tableId);
                    tableObj.markDeleted();
                    this.__moveTableToDefaultRegionForDelete(tableObj, regionToDel, formDefaultRegion);
                }
            }
            regionToDel.DelTables.clear();
            //如果要删除的区域本来就是新建的，则直接删除。   
            this.Regions.remove(regionToDel.ID);
            //end note.
        } else {
            regionToDel.markDeleted();
            this.Regions.remove(regionToDel.ID);
            this.DelRegions.add(regionToDel.ID, regionToDel);
        }

        return true;
    }

    return false;
};

//数据区域反删除，重复利用.
BM.FormObject.prototype.__unDeleteRegionByID = function (regionId) {
    var regionObj = this.DelRegions.get(regionId);
    if (regionObj) {
        regionObj.markUnDeleted();
        this.DelRegions.remove(regionId);
        this.Regions.add(regionId, regionObj);
    }
};

//将数据链接转移到默认数据区域内以便后台进行删除
BM.FormObject.prototype.__moveLinkToDefaultRegionForDelete = function (linkObject, srcRegionObject, targetRegionObject) {
    linkObject.markDeleted();
    linkObject.setDataRegionID(targetRegionObject.ID);

    srcRegionObject.Links.remove(linkObject.ID);
    srcRegionObject.DelLinks.remove(linkObject.ID);
    targetRegionObject.DelLinks.add(linkObject.ID, linkObject);
};

//将数据链接转移到默认数据区域内以便后台进行删除
BM.FormObject.prototype.__moveTableToDefaultRegionForDelete = function (tableObject, srcRegionObject, targetRegionObject) {
    tableObject.markDeleted();

    srcRegionObject.DelTables.remove(tableObject.ID);
    srcRegionObject.Tables.remove(tableObject.ID);
    targetRegionObject.DelTables.add(tableObject.ID, tableObject);
};

//设置指定单元格为指定指标类型和大小。
BM.FormObject.prototype.setFieldByPos = function (fieldType, precision, col, row, colCount, rowCount) {
    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);

    var fieldByCell = null;
    var regionByCell = this.getRegionByPos(col, row);
    if (!regionByCell) {
        regionByCell = this.createDefaultRegion(1, 1, colCount - 1, rowCount - 1);
    }
    var linkInCell = regionByCell.__getLinkByPos(col, row);
    if (linkInCell) {
        if (linkInCell.IsDeleted) {
            regionByCell.__unDeleteLinkByID(linkInCell.ID);
        }

        var fieldId = linkInCell.LinkExpression;
        var fieldObj = regionByCell.__getFieldById(fieldId);
        if (fieldObj) {
            var ownerTable = regionByCell.__getTableById(fieldObj.OwnerTableID);
            if (fieldObj.IsDeleted) {
                if (ownerTable) {
                    if (ownerTable.IsDeleted)
                        regionByCell.__unDeleteTableByID(ownerTable.ID);
                    ownerTable.unDeleteFieldByID(fieldObj.ID);
                }
            }

            //note 已存在指标的，根据操作参数来判断是否需要重建.//方法已废弃.
            var needCreateNewField = false;//this.needCreateNewField(fieldObj, fieldType, precision);

            this.changeFieldType(linkInCell, fieldObj, fieldType, precision);

            if (needCreateNewField == true) {
                var newFieldId = formHelper.JCPGuid("FieldObject-");
                var newFieldObj = new BM.FieldObject(fieldObj);
                newFieldObj.ID = newFieldId;
                newFieldObj.IsNew = true;
                newFieldObj.IsDirty = false;
                newFieldObj.IsDeleted = false;

                //note 移除指标之前，先改变指标的标识，以便标识在新指标中使用，以免数据丢失【数据库是根据指标标识来判断列是否一致】
                fieldObj.setCode(formHelper.generateNewStringByTemplet(fieldObj.Code));
                ownerTable.delFieldById(fieldObj.ID);
                //note end.

                var newFieldCode = newFieldObj.Code;
                while (ownerTable.isExistFieldCode(newFieldCode)) { //note 指标移到新表中，校验指标标识是否存在重码.
                    //note 新标识不再是加_1来处理。一直累加的话，会导致标识最后变长.
                    newFieldCode = formHelper.generateNewStringByTemplet(newFieldCode);
                }
                newFieldObj.Code = newFieldCode;
                ownerTable.addField(newFieldObj);

                linkInCell.setLinkExpression(newFieldObj.ID);
                fieldByCell = newFieldObj;
            } else {
                fieldByCell = fieldObj;
            }

        } else {
            //获取区域下的可用存储表.
            var ownerTablle1 = regionByCell.queryOrCreateTableObject();
            //创建新的指标.
            var newField1 = ownerTablle1.createFieldObject();
            var newFieldCode1 = formHelper.codeGenerate(col, row);
            while (ownerTablle1.isExistFieldCode(newFieldCode1)) {
                newFieldCode1 = formHelper.generateNewStringByTemplet(newFieldCode1);
            }
            newField1.setCode(newFieldCode1);
            this.changeFieldType(linkInCell, newField1, fieldType, precision);

            linkInCell.setLinkExpression(newField1.ID);
            fieldByCell = newField1;
        }
    } else {
        //获取区域下的可用存储表.
        var ownerTablle = regionByCell.queryOrCreateTableObject();
        //创建新的指标.
        var newField = ownerTablle.createFieldObject();
        var newFieldCode = formHelper.codeGenerate(col, row);
        while (ownerTablle.isExistFieldCode(newFieldCode)) {
            newFieldCode = formHelper.generateNewStringByTemplet(newFieldCode);
        }
        newField.setCode(newFieldCode);

        //创建新的数据链接.
        linkInCell = regionByCell.createLinkByPos(col, row);
        this.changeFieldType(linkInCell, newField, fieldType, precision);
        linkInCell.setLinkExpression(newField.ID);
        fieldByCell = newField;
    }

    this.setNeedReAnalysis(true);
    return [linkInCell, fieldByCell];
};

//判断当前指标是否需要新建.
//note 20150410 新的方案，不进行指标是否新建的校验，直接修改指标的类型.
BM.FormObject.prototype.needCreateNewField = function (fieldObj, newFieldType, newPrecision) {
    return false;

    if (fieldObj.Type == newFieldType
        && (newFieldType == BM.Enum.FieldType.ft_Decimal || newFieldType == BM.Enum.FieldType.ft_Float)         //数值型或者浮点型，精度变大不需要新建；
        && fieldObj.FractionDigits <= newPrecision)
        return false;

    if ((fieldObj.Type != BM.Enum.FieldType.ft_String && (newFieldType == BM.Enum.FieldType.ft_String || newFieldType == BM.Enum.FieldType.ft_General))           //任何类型转文本类型不用新建；
        || (fieldObj.Type == BM.Enum.FieldType.ft_String && (newFieldType == BM.Enum.FieldType.ft_String || newFieldType == BM.Enum.FieldType.ft_General))        //字符类型长度缩小变大都不需要新建；
        || (fieldObj.Type == newFieldType))                                                                     //源类型与目标类型一致不需要新建；
        return false;

    return true;
};

//设置当前单元格为公式单元格.
BM.FormObject.prototype.setFormulaByPos = function (formulaExpression, col, row, colCount, rowCount, checkFormula) {

    var cellLink = this.getLinkByPos(col, row);
    if (cellLink == null) {
        //note 设置公式时，如果该单元格不存在指标的话，则需要进行指标创建.(默认公式单元格类型为数值指标，长度18)
        var linkField = this.setFieldByPos(BM.Enum.FieldType.ft_Decimal, 2, col, row, colCount, rowCount);
        cellLink = linkField[0];
    }
    if (cellLink == null)
        throw new Error("设置单元格公式出错");

    var cellLinkGuid = "[" + cellLink.UniqueCode + "]";
    var fullFormulaExp = formulaExpression;

    if (!checkFormula) {
        fullFormulaExp = cellLinkGuid + formulaExpression;
    }

    var formulaExist = false;
    for (var fmlId in this.Formulas) {
        var tempFml = this.Formulas.get(fmlId);
        if (tempFml.PosX == col && tempFml.PosY == row) {
            formulaExist = true;
            break;
        }
    }

    var delFormula = false;
    if ("" == formulaExpression || "=" == formulaExpression) {
        delFormula = true;
    }

    if (!formulaExist) {
        //新增公式
        if (!formulaExpression)
            return;
        var formula = this.createFormulaObject(col, row, fullFormulaExp, checkFormula);
        this.Formulas.add(formula.ID, formula);
        var fieldByPos = this.getFieldById(cellLink.LinkExpression);
        fieldByPos.setGatherType(BM.Enum.FieldGatherType.fg_None);
    } else if (!delFormula) {
        //修改公式
        this.updateFormula(fullFormulaExp, col, row);
    } else {
        //删除公式
        var fml = this.getFormulaByPos(col, row);
        this.deleteFormula(fml.ID);
    }
    this.GlobalChanged = true;
};

//修改公式
BM.FormObject.prototype.updateFormula = function (formulaExp, col, row) {
    for (var fmId in this.Formulas) {
        var tempFormula = this.Formulas.get(fmId);
        if (tempFormula.PosX == col && tempFormula.PosY == row) {
            tempFormula.setExpression(formulaExp);
            break;
        }
    }
};

//修改指标的精度.
BM.FormObject.prototype.changeFieldPrecision = function (fieldType, precision, col, row) {
    var regionByCell = this.getRegionByPos(col, row);
    if (!regionByCell) {
        regionByCell = this.createDefaultRegion(1, 1, colCount - 1, rowCount - 1);
    }
    var linkInCell = regionByCell.getLinkByPos(col, row);
    if (linkInCell) {
        var fieldId = linkInCell.LinkExpression;
        var fieldObj = regionByCell.getFieldById(fieldId);
        if (fieldObj && fieldObj.Type == fieldType) {
            switch (fieldType) {
                case BM.Enum.FieldType.ft_String:
                    {
                        if (precision <= 500) {
                            fieldObj.setSize(precision);
                        }
                    }
                    break;
                case BM.Enum.FieldType.ft_Decimal:
                    {
                        if (precision <= 10) {
                            fieldObj.setFractionDigits(precision);
                        }
                    }
                    break;
                case BM.Enum.FieldType.ft_Float:
                    {
                        if (precision <= 10) {
                            fieldObj.setFractionDigits(precision);
                        }
                    }
                    break;
            }
        }
    }

};

//设置制定单元格的指标是否可空.
BM.FormObject.prototype.setFieldNullable = function (nullable, col, row, colCount, rowCount) {
    nullable = nullable || false;
    var linkObj = this.getLinkByPos(col, row);
    if (linkObj == null) {
        var linkFieldSet = this.setFieldByPos(BM.Enum.FieldType.ft_String, 500, col, row, colCount, rowCount);
        linkObj = linkFieldSet[0];
    }

    if (linkObj != null && (!isNaN(linkObj.LinkExpression) || linkObj.LinkExpression.indexOf("FieldObject") >= 0)) {
        var fieldObj = this.getFieldById(linkObj.LinkExpression);
        if (fieldObj) {
            fieldObj.setNullable(nullable);
        }
    }
    return fieldObj;
};

//设置指定单元格的指标枚举项是否多选.
BM.FormObject.prototype.setAllowMultSelect = function (allMultSelect, col, row, colCount, rowCount) {
    allMultSelect = allMultSelect || false;
    var linkObj = this.getLinkByPos(col, row);
    if (linkObj == null) {
        var linkFieldSet = this.setFieldByPos(BM.Enum.FieldType.ft_String, 500, col, row, colCount, rowCount);
        linkObj = linkFieldSet[0];
    }

    if (linkObj != null && (!isNaN(linkObj.LinkExpression) || linkObj.LinkExpression.indexOf("FieldObject") >= 0)) {
        var fieldObj = this.getFieldById(linkObj.LinkExpression);
        if (fieldObj && linkObj.hasEnumExpression()) {
            fieldObj.setAllowMultSelect(allMultSelect);
        }
    }
    return fieldObj;
};

//设置指定单元格的编辑模式.
BM.FormObject.prototype.setEditMode = function (editMode, col, row) {
    var linkObj = this.getLinkByPos(col, row);
    if (linkObj) {
        if (linkObj.IsDeleted) {
            var ownerRegionObj = this.getRegionByID(linkObj.DataRegionID);
            ownerRegionObj.__unDeleteLinkByID(linkObj.ID);
        }
        linkObj.setEditMode(editMode);
    }
};

//修改指标的类型为新类型，并设置默认属性.
BM.FormObject.prototype.changeFieldType = function (linkObj, fieldObj, fieldType, num) {
    switch (fieldType) {
        case BM.Enum.FieldType.ft_Logic:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_Logic);
                fieldObj.setSize(1);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_General:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_General);
                fieldObj.setSize(501);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_String:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_String);
                fieldObj.setSize(num);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_Date:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_Date);
                fieldObj.setSize(0);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_DateTime:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_DateTime);
                fieldObj.setSize(0);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_Decimal:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_Decimal);
                //note 小数位变大时，在可支持范围内增加指标的长度. http://10.2.12.123:8080/browse/GGG-657
                //if (fieldObj.Size == 0 || fieldObj.Size > 28) {
                //}
                //if (num > fieldObj.FractionDigits) {
                //    var difference = num - fieldObj.FractionDigits;
                //    var result = (fieldObj.Size + difference) > 28 ? 28 : fieldObj.Size + difference;
                //    fieldObj.setSize(result);
                //}
                //设置数值类型默认指标大小为28 
                fieldObj.setSize(28);
                fieldObj.setFractionDigits(num);
            } break;
        case BM.Enum.FieldType.ft_Float:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_Float);
                fieldObj.setSize(0);
                fieldObj.setFractionDigits(num);
            } break;
        case BM.Enum.FieldType.ft_Integer:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_Integer);
                fieldObj.setSize(0);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_Binary:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_Binary);
                fieldObj.setSize(0);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_Guid:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_Guid);
                fieldObj.setSize(0);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_Text:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_Text);
                fieldObj.setSize(0);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_Picture:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_Picture);
                fieldObj.setSize(0);
                fieldObj.setFractionDigits(0);
            } break;
        case BM.Enum.FieldType.ft_TimeStamp:
            {
                fieldObj.setType(BM.Enum.FieldType.ft_TimeStamp);
                fieldObj.setSize(0);
                fieldObj.setFractionDigits(0);
            } break;
        default:
            break;
    }

    if (fieldType == BM.Enum.FieldType.ft_Float
        || fieldType == BM.Enum.FieldType.ft_Decimal
        || fieldType == BM.Enum.FieldType.ft_Integer
        || fieldType == BM.Enum.FieldType.ft_General) {
        //note http://10.2.12.123:8080/browse/JQGG-1332
        //fieldObj.setGatherType(BM.Enum.FieldGatherType.fg_Sum);
        //note 去除此代码，因为保存也会运行此代码，会导致用户的“非默认”设置被改回去.
    } else {
        var gatherType = fieldObj.GatherType;
        if (fieldType == BM.Enum.FieldType.ft_Date || fieldType == BM.Enum.FieldType.ft_DateTime) {
            if (gatherType != BM.Enum.FieldGatherType.fg_Min && gatherType != BM.Enum.FieldGatherType.fg_Max && gatherType != BM.Enum.FieldGatherType.fg_Count && gatherType != BM.Enum.FieldGatherType.fg_None) {
                fieldObj.setGatherType(BM.Enum.FieldGatherType.fg_None);//note 不支持汇总的类型自动改成不汇总.
            }
        } else if (fieldType == BM.Enum.FieldType.ft_Logic) {
            if (gatherType != BM.Enum.FieldGatherType.fg_Sum && gatherType != BM.Enum.FieldGatherType.fg_None) {
                fieldObj.setGatherType(BM.Enum.FieldGatherType.fg_None);//note 不支持汇总的类型自动改成不汇总.
            }
        } else if (gatherType != BM.Enum.FieldGatherType.fg_None) {
            fieldObj.setGatherType(BM.Enum.FieldGatherType.fg_None);//note 不支持汇总的类型自动改成不汇总.
        }
    }
    switch (fieldType) {//设置成非字符型指标时，去除其枚举引用.
        case BM.Enum.FieldType.ft_String:
            {
                if (linkObj.hasEnumExpression()) {
                    linkObj.setEditMode(BM.Enum.DataLinkEditMode.dl_DropDown);
                    linkObj.setEnumStyle(BM.Enum.DataLinkEnumStyle.de_Title);
                } else {
                    linkObj.setEditMode(BM.Enum.DataLinkEditMode.dl_Default);
                    linkObj.setEnumStyle(BM.Enum.DataLinkEnumStyle.de_Code);
                }
            }
            break;
        case BM.Enum.FieldType.ft_General:
        case BM.Enum.FieldType.ft_Logic:
        case BM.Enum.FieldType.ft_Date:
        case BM.Enum.FieldType.ft_DateTime:
        case BM.Enum.FieldType.ft_Decimal:
        case BM.Enum.FieldType.ft_Float:
        case BM.Enum.FieldType.ft_Integer:
        case BM.Enum.FieldType.ft_Binary:
        case BM.Enum.FieldType.ft_Guid:
        case BM.Enum.FieldType.ft_Text:
        case BM.Enum.FieldType.ft_Picture:
        case BM.Enum.FieldType.ft_TimeStamp:
            {
                linkObj.setEnumExpression(null);
                linkObj.setEditMode(BM.Enum.DataLinkEditMode.dl_Default);
                linkObj.setEnumStyle(BM.Enum.DataLinkEnumStyle.de_Code);
            }
            break;
    }
};

//移除坐标对应的单元格所设置的枚举引用.
BM.FormObject.prototype.setEnum = function (enumExpression, col, row) {
    var linkObj = this.getLinkByPos(col, row);
    if (linkObj) {
        linkObj.setEnumExpression(enumExpression);
        linkObj.setEditMode(BM.Enum.DataLinkEditMode.dl_DropDown);
        linkObj.setEnumStyle(BM.Enum.DataLinkEnumStyle.de_Title);

        //枚举类型指标设置不汇总.
        var fieldObj = this.getRegionByID(linkObj.DataRegionID).getFieldById(linkObj.LinkExpression);
        fieldObj.setGatherType(BM.Enum.FieldGatherType.fg_None);
    }
};

//移除坐标对应的单元格所设置的枚举引用.
BM.FormObject.prototype.removeEnum = function (col, row) {
    var linkObj = this.getLinkByPos(col, row);
    if (linkObj) {
        linkObj.setEnumExpression(null);
        linkObj.setEditMode(BM.Enum.DataLinkEditMode.dl_Default);
        linkObj.setEnumStyle(BM.Enum.DataLinkEnumStyle.de_Code);
        linkObj.setDisplayMode(BM.Enum.DataLinkDisplayMode.dm_Default);
    }
};

//移除当前表单内链接引用的指定的枚举ID.
BM.FormObject.prototype.removeEnumByEnumExpression = function (enumExpression) {
    for (var regId in this.Regions) {
        var regObj = this.Regions.get(regId);
        for (var linkId in regObj.Links) {
            var linkObj = regObj.Links.get(linkId);
            if (linkObj.EnumExpression == enumExpression) {
                linkObj.setEnumExpression(null);
                linkObj.setEditMode(BM.Enum.DataLinkEditMode.dl_Default);
                linkObj.setEnumStyle(BM.Enum.DataLinkEnumStyle.de_Code);
            }
        }
    }
};

//设置指定单元格为表样单元格.
//note 区域进行调整时，为了保证数据不丢失，进行浮动区域的向下移动一行，并将当前行的所有的指标设置到固定区域中.
BM.FormObject.prototype.setAsFormStyleByPos = function (col, row) {
    var regionByCell = this.getRegionByPos(col, row);
    if (!regionByCell) {
        return false;
    }
    var result = false;
    var linkInCell = regionByCell.getLinkByPos(col, row);
    if (linkInCell) {
        regionByCell.deleteLinkByPos(col, row);

        var fieldObj = regionByCell.getFieldById(linkInCell.LinkExpression);
        if (fieldObj) {
            var ownerTable = regionByCell.getTableById(fieldObj.OwnerTableID);
            ownerTable.delFieldById(fieldObj.ID);

            //删除只剩下业务主键和UpdateTime指标的存储表.
            var tableCanDelete = regionByCell.__checkTableCanDelete(ownerTable.ID);
            if (tableCanDelete == true) {
                regionByCell.deleteTableByID(ownerTable.ID);
            }
        }
        result = true;
    }

    //删除单元格公式
    var formula = this.getFormulaByPos(col, row);
    if (formula) {
        this.deleteFormula(formula.ID);
    }
    if (result === true)
        this.setNeedReAnalysis(true);

    return result;
};

//减小数据精度。
BM.FormObject.prototype.increaseDecimal = function (col, row) {
    var regionByCell = this.getRegionByPos(col, row);
    if (!regionByCell) {
        return null;
    }
    var linkInCell = this.getLinkByPos(col, row);
    if (linkInCell) {
        var fieldId = linkInCell.LinkExpression;
        var fieldObj = regionByCell.getFieldById(fieldId);
        if (fieldObj) {
            switch (fieldObj.FieldType) {
                case BM.Enum.FieldType.ft_String:
                    {
                        if (fieldObj.Size < 500) {
                            fieldObj.setSize(fieldObj.Size + 1);
                        }
                    }
                    break;
                case BM.Enum.FieldType.ft_Decimal:
                    {
                        if (fieldObj.FractionDigits < 10) {
                            fieldObj.setFractionDigits(fieldObj.FractionDigits + 1);
                            //fieldObj.setSize(fieldObj.Size + 1);
                        }
                    }
                    break;
                case BM.Enum.FieldType.ft_Float:
                    {
                        if (fieldObj.FractionDigits < 10) {
                            fieldObj.setFractionDigits(fieldObj.FractionDigits + 1);
                        }
                    }
                    break;
                default:
                    {

                    } break;
            }
            return fieldObj;
        }
    }

    return null;
};

//减小数据精度。
BM.FormObject.prototype.decreaseDecimal = function (col, row) {
    var regionByCell = this.getRegionByPos(col, row);
    if (!regionByCell) {
        return null;
    }
    var linkInCell = this.getLinkByPos(col, row);
    if (linkInCell) {
        var fieldId = linkInCell.LinkExpression;
        var fieldObj = regionByCell.getFieldById(fieldId);
        if (fieldObj) {
            switch (fieldObj.Type) {
                case BM.Enum.FieldType.ft_String:
                    {
                        if (fieldObj.Size >= 2) {
                            fieldObj.setSize(fieldObj.Size - 1);
                        }
                    }
                    break;
                case BM.Enum.FieldType.ft_Decimal:
                    {
                        if (fieldObj.FractionDigits >= 2) {
                            fieldObj.setFractionDigits(fieldObj.FractionDigits - 1);
                            //fieldObj.setSize(fieldObj.Size - 1);
                        }
                    }
                    break;
                case BM.Enum.FieldType.ft_Float:
                    {
                        if (fieldObj.FractionDigits >= 2) {
                            fieldObj.setFractionDigits(fieldObj.FractionDigits - 1);
                        }
                    }
                    break;
                default:
                    {

                    } break;
            }
            return fieldObj;
        }
    }

    return null;
};

//根据指标ID获取对应的指标，不存在则返回null.
BM.FormObject.prototype.getFieldById = function (fieldId) {
    for (var regId in this.Regions) {
        var fieldObj = this.Regions.get(regId).getFieldById(fieldId);
        if (fieldObj)
            return fieldObj;
    }

    return null;
};

/*
* 根据链接ID获取当前数据区域下的指定链接
* @param linkId {String} 链接的ID
* @returns {LinkObject} 链接
*/
BM.FormObject.prototype.getLinkById = function (linkId) {
    for (var regId in this.Regions) {
        var linkObj = this.Regions.get(regId).getLinkById(linkId);
        if (linkObj)
            return linkObj;
    }

    return null;
};

//表格删除列之后，进行对应的参数缓存处理。
BM.FormObject.prototype.deleteCol = function (colIndex) {
    if (colIndex < 0) return;

    for (var regionId1 in this.Regions) {
        var regionObj1 = this.Regions.get(regionId1);
        regionObj1.deleteCol(colIndex);
    }

    //同时更新保留的浮动区域相关信息.
    for (var resRegionId in this.ReservedFloatRegion) {
        var resRegionObj = this.ReservedFloatRegion.get(resRegionId);
        resRegionObj.deleteCol(colIndex);
    }

    var delRegionIds = [];
    for (var regionId2 in this.Regions) {
        var regionObj2 = this.Regions.get(regionId2);
        switch (regionObj2.RegionKind) {
            case BM.Enum.DataRegionKind.dr_Simple:
                {
                    if (regionObj2.RegionRight >= colIndex && regionObj2.RegionLeft <= colIndex) {
                        regionObj2.setRegionRight(regionObj2.RegionRight - 1);
                        if (regionObj2.RegionRight < regionObj2.RegionLeft) {
                            delRegionIds.push(regionObj2.ID);
                        } else {
                            regionObj2.markDirty();
                        }
                    } else {
                        if (regionObj2.RegionRight > colIndex && regionObj2.RegionLeft >= colIndex) {
                            regionObj2.setRegionLeft(regionObj2.RegionLeft - 1);
                            regionObj2.setRegionRight(regionObj2.RegionRight - 1);
                        }
                    }
                }
                break;
            case BM.Enum.DataRegionKind.dr_RowList: {
                regionObj2.setRegionRight(regionObj2.RegionRight - 1);
            }
                break;
            case BM.Enum.DataRegionKind.dr_ColList:
                {
                    if (regionObj2.RegionLeft > colIndex) {
                        regionObj2.setRegionLeft(regionObj2.RegionLeft - 1);
                        regionObj2.setRegionRight(regionObj2.RegionRight - 1);
                    } else {

                        if (regionObj2.RegionLeft == colIndex
                            && regionObj2.RegionRight == colIndex) {
                            delRegionIds.push(regionObj2.ID);
                        } else {

                            if (regionObj2.RegionLeft <= colIndex
                                && regionObj2.RegionRight >= colIndex) {
                                regionObj2.setRegionLeft(regionObj2.RegionRight - 1);
                            }
                        }
                    }
                }
                break;
        }
    }

    for (var delRegionIdIndex = 0 ; delRegionIdIndex < delRegionIds.length; delRegionIdIndex++) {
        var delRegionId = delRegionIds[delRegionIdIndex];
        this.deleteRegion(delRegionId);
    }

    //删列处理公式
    var delFormulaIds = [];
    for (var fmId in this.Formulas) {
        var tempFormula = this.Formulas.get(fmId);
        if (tempFormula.PosX == colIndex && tempFormula.PosY != 0) {
            delFormulaIds.push(tempFormula.ID);
        } else if (tempFormula.PosX > colIndex) {
            //修改坐标
            tempFormula.setPosX(tempFormula.PosX - 1);
        }
    }
    for (var delFormulaIndex = 0; delFormulaIndex < delFormulaIds.length; delFormulaIndex++) {
        var delFormulaId = delFormulaIds[delFormulaIndex];
        this.deleteFormula(delFormulaId);
    }

    for (var rowIdx in this.RowExtensions) {
        var tempRowExtension = this.RowExtensions.get(rowIdx);
        if (tempRowExtension.OrderIndex == colIndex) {
            tempRowExtension.setOrderIndex(-1);
        }
        if (tempRowExtension.EntityNameIndex == colIndex) {
            tempRowExtension.setEntityNameIndex(-1);
        }
    }

    this.setBigDataChanged(true);
    this.setNeedReAnalysis(true);
};

//表格删除行之后，进行对应的参数缓存处理。done.
BM.FormObject.prototype.deleteRow = function (rowIndex) {
    if (rowIndex < 0) return;

    var delLinkIds = [];
    for (var regionId1 in this.Regions) {
        var regionObj1 = this.Regions.get(regionId1);
        for (var linkId1 in regionObj1.Links) {
            var linkObj1 = regionObj1.__getLinkById(linkId1);//已经删除的数据链接也进行相应的变化，方便以后进行重用.
            if (linkObj1.PosY == rowIndex && linkObj1.PosX != 0) {
                delLinkIds.push(linkId1);

                var fieldObj1 = regionObj1.getFieldById(linkObj1.LinkExpression);
                if (fieldObj1) {
                    var tableObj1 = regionObj1.getTableById(fieldObj1.OwnerTableID);
                    if (tableObj1.isBizField(fieldObj1.ID)) {
                        regionObj1.removePrimaryKeyByPos(linkObj1.PosX, linkObj1.PosY);
                    }

                    tableObj1.delFieldById(fieldObj1.ID);

                    //删除只剩下业务主键和UpdateTime指标的存储表.
                    var tableCanDelete = regionObj1.__checkTableCanDelete(tableObj1.ID);
                    if (tableCanDelete == true) {
                        regionObj1.deleteTableByID(tableObj1.ID);
                    }
                }
            }
            else if (linkObj1.PosY > rowIndex) {//该行对应的数据链接上移.
                linkObj1.setPosY(linkObj1.PosY - 1);
            }
        }

        for (var delLinkIdIndex = 0 ; delLinkIdIndex < delLinkIds.length; delLinkIdIndex++) {
            var delLinkId = delLinkIds[delLinkIdIndex];
            regionObj1.deleteLinkByID(delLinkId);
        }

        delLinkIds = [];
    }

    var delRegionIds = [];
    for (var regionId2 in this.Regions) {
        var regionObj2 = this.Regions.get(regionId2);
        switch (regionObj2.RegionKind) {
            case BM.Enum.DataRegionKind.dr_Simple:
                {
                    if (regionObj2.RegionBottom >= rowIndex && regionObj2.RegionTop <= rowIndex) {
                        regionObj2.setRegionBottom(regionObj2.RegionBottom - 1);

                        if (regionObj2.RegionBottom < regionObj2.RegionTop) {
                            delRegionIds.push(regionObj2.ID);
                        }
                    } else {
                        if (regionObj2.RegionBottom > rowIndex && regionObj2.RegionTop > rowIndex) {
                            regionObj2.setRegionTop(regionObj2.RegionTop - 1);
                            regionObj2.setRegionBottom(regionObj2.RegionBottom - 1);
                        }
                    }
                }
                break;
            case BM.Enum.DataRegionKind.dr_RowList:
                {
                    if (regionObj2.RegionTop > rowIndex) {
                        regionObj2.setRegionTop(regionObj2.RegionTop - 1);
                        regionObj2.setRegionBottom(regionObj2.RegionBottom - 1);
                    } else {
                        if (regionObj2.RegionTop <= rowIndex && regionObj2.RegionBottom >= rowIndex) {

                            if (regionObj2.RegionTop == regionObj2.RegionBottom) {
                                delRegionIds.push(regionObj2.ID);
                            } else {
                                regionObj2.setRegionBottom(regionObj2.RegionBottom - 1);
                            }

                        }
                    }
                }
                break;
            case BM.Enum.DataRegionKind.dr_ColList: {
                regionObj2.setRegionBottom(regionObj2.RegionBottom - 1);
            }
                break;
        }
    }

    for (var delRegionIdIndex = 0 ; delRegionIdIndex < delRegionIds.length; delRegionIdIndex++) {
        var delRegionId = delRegionIds[delRegionIdIndex];
        this.deleteRegion(delRegionId);
    }

    //删行处理公式
    var delFormulaIds = [];
    for (var fmId in this.Formulas) {
        var tempFormula = this.Formulas.get(fmId);
        if (tempFormula.PosY == rowIndex && tempFormula.PosX != 0) {
            delFormulaIds.push(tempFormula.ID);
        } else if (tempFormula.PosY > rowIndex) {

            //修改坐标
            tempFormula.setPosY(tempFormula.PosY - 1);
        }
    }
    for (var delFormulaIndex = 0; delFormulaIndex < delFormulaIds.length; delFormulaIndex++) {
        var delFormulaId = delFormulaIds[delFormulaIndex];
        this.deleteFormula(delFormulaId);
    }

    this.RowExtensions.remove(rowIndex);
    for (var rowIdx in this.RowExtensions) {
        if (rowIdx > rowIndex) {
            //修改坐标
            var rowExtension = this.RowExtensions.get(rowIdx);
            this.RowExtensions.remove(rowIdx);
            var newIndex = parseInt(rowIdx) - 1;
            this.RowExtensions.add(newIndex, rowExtension);
        }
    }

    this.setBigDataChanged(true);
    this.setNeedReAnalysis(true);
};

//表格增加列之后，进行对应的参数缓存处理。done.
BM.FormObject.prototype.insertCol = function (colIndex) {
    if (colIndex < 0) return;

    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        regionObj.insertCol(colIndex);
    }

    //同时更新保留的浮动区域相关信息.
    for (var resRegionId in this.ReservedFloatRegion) {
        var resRegionObj = this.ReservedFloatRegion.get(resRegionId);
        resRegionObj.insertCol(colIndex);
    }

    //增列处理公式
    for (var fmId in this.Formulas) {
        var tempFormula = this.Formulas.get(fmId);
        if (tempFormula.PosX >= colIndex) {
            //修改坐标
            tempFormula.setPosX(tempFormula.PosX + 1);
        }
    }

    for (var rowIdx in this.RowExtensions) {
        var tempRowExtension = this.RowExtensions.get(rowIdx);
        if (tempRowExtension.OrderIndex >= colIndex) {
            tempRowExtension.setOrderIndex(tempRowExtension.OrderIndex + 1);
        }
        if (tempRowExtension.EntityNameIndex >= colIndex) {
            tempRowExtension.setEntityNameIndex(tempRowExtension.EntityNameIndex + 1);
        }
    }

    this.setBigDataChanged(true);
    this.setNeedReAnalysis(true);
};

//表格追加列之后，进行对应的参数缓存处理。done.
BM.FormObject.prototype.appendCol = function (colIndex, appendedCount) {
    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        regionObj.appendCol(colIndex, appendedCount);
    }
    this.setBigDataChanged(true);
    this.setNeedReAnalysis(true);
};

//表格增加行之后，进行对应的参数缓存处理。done.
BM.FormObject.prototype.insertRow = function (rowIndex) {
    if (rowIndex < 0) return;

    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        regionObj.insertRow(rowIndex);
    }

    //同时更新保留的浮动区域相关信息.
    for (var resRegionId in this.ReservedFloatRegion) {
        var resRegionObj = this.ReservedFloatRegion.get(resRegionId);
        resRegionObj.insertRow(rowIndex);
    }

    //增行处理公式
    for (var fmId in this.Formulas) {
        var tempFormula = this.Formulas.get(fmId);
        if (tempFormula.PosY >= rowIndex) {
            //修改坐标
            tempFormula.setPosY(tempFormula.PosY + 1);
        }
    }

    var tmpRowExtensions = new BM.BusinessList("临时行扩展数组");
    for (var rowIdx in this.RowExtensions) {
        if (rowIdx >= rowIndex) {
            //修改坐标
            var rowExtension = this.RowExtensions.get(rowIdx);
            this.RowExtensions.remove(rowIdx);
            var newRowIndex = parseInt(rowIdx) + 1;
            tmpRowExtensions.add(newRowIndex, rowExtension);
        }
    }

    for (var tmpRowIdx in tmpRowExtensions) {
        var rmpRowExtension = tmpRowExtensions.get(tmpRowIdx);
        this.RowExtensions.add(tmpRowIdx, rmpRowExtension);
    }

    var newRowExtension = new BM.RegionExtension("");
    this.RowExtensions.add(rowIndex, newRowExtension);

    this.setBigDataChanged(true);
    this.setNeedReAnalysis(true);
};

//表格追加行之后，进行对应的参数缓存处理。done.
BM.FormObject.prototype.appendRow = function (rowIndex, appendedCount) {
    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        regionObj.appendRow(rowIndex, appendedCount);
    }

    for (var count = 0; count < appendedCount; count++) {
        var newRowExtension = new BM.RegionExtension("");
        this.RowExtensions.add(parseInt(rowIndex) + count, newRowExtension);
    }
    this.setBigDataChanged(true);
    this.setNeedReAnalysis(true);
};

BM.FormObject.prototype.setLinkVisibility = function (colVisibility, rowVisibility) {
    for (var regId in this.Regions) {
        var regionObj = this.Regions.get(regId);
        regionObj.setLinkVisibility(colVisibility, rowVisibility);
    }
};

//对表单进行重命名.
BM.FormObject.prototype.setTitle = function (newTitle) {
    if (newTitle == null || newTitle.trim() === "")
        return null;
    newTitle = newTitle.trim();

    var mainFrame = $.m("BM_MainFrameModel");
    
    var formHelper = mainFrame.getService(BM_ServiceName_FormHelper);
    var newTitleLength = formHelper.getStringCodeLength(newTitle);
    if (newTitleLength > formHelper.FormTitleMaxLength) {
        return null;
    }

    var currentTask = mainFrame.getCurrentTask();
    if (currentTask.isExistFormByTitle(this.ID, newTitle))
        return null;

    if (this.Title != newTitle) {
        this.beforePropertyChanged("Title", this.Title, newTitle);
        this.Title = newTitle;
        this.markDirty();
        this.propertyHasChanged("Title");
    }
    return this;
};

BM.FormObject.prototype.setFormType = function (newFormType) {
    if (newFormType == null || newFormType === "")
        return null;

    if (this.FormType != newFormType) {
        this.beforePropertyChanged("FormType", this.FormType, newFormType);
        this.FormType = newFormType;
        this.markDirty();
        this.propertyHasChanged("FormType");
    }
    return this;
};

//表样修改时调用.
BM.FormObject.prototype.setBigDataChanged = function (newBigDataChanged) {
    if (newBigDataChanged == null || newBigDataChanged === "")
        return null;

    if (this.BigDataChanged != newBigDataChanged) {
        this.beforePropertyChanged("BigDataChanged", this.BigDataChanged, newBigDataChanged);
        this.BigDataChanged = newBigDataChanged;
        this.markDirty();
        this.propertyHasChanged("BigDataChanged");
    }

    if (this.FormStyleChanged !== true) {
        this.beforePropertyChanged("FormStyleChanged", this.FormStyleChanged, true);
        this.FormStyleChanged = true;
        this.markDirty();
        this.propertyHasChanged("FormStyleChanged");
    }
    return this;
};

BM.FormObject.prototype.setOwnGroupId = function (newOwnGroupId) {
    if (this.OwnGroupId != newOwnGroupId) {
        this.beforePropertyChanged("OwnGroupId", this.OwnGroupId, newOwnGroupId);
        this.OwnGroupId = newOwnGroupId;
        this.markDirty();
        this.propertyHasChanged("OwnGroupId");
    }
    return this;
};

//设置是否需要重新进行浮动行分析.
BM.FormObject.prototype.setNeedReAnalysis = function (newNeedReAnalysis) {
    if (newNeedReAnalysis == null || newNeedReAnalysis === "")
        return null;

    if (this.onInitializingReservedCount === true)
        return this;

    if (this.NeedReAnalysis != newNeedReAnalysis) {
        this.beforePropertyChanged("NeedReAnalysis", this.NeedReAnalysis, newNeedReAnalysis);
        this.NeedReAnalysis = newNeedReAnalysis;
        this.markDirty();
        this.propertyHasChanged("NeedReAnalysis");
    }
    return this;
};

BM.FormObject.prototype.setOrder = function (newOrder) {
    if (newOrder == null || newOrder === "")
        return null;

    if (this.onInitializingReservedCount === true)
        return this;

    if (this.Order != newOrder) {
        this.beforePropertyChanged("Order", this.Order, newOrder);
        this.Order = newOrder;
        this.markDirty();
        this.propertyHasChanged("Order");
    }
    return this;
};

//重置表单为干净状态，除了新建./?
BM.FormObject.prototype.markClean = function () {

    this.GlobalChanged = false;

    if (!this.IsNew)
        this.IsDirty = false;

    this.BigDataChanged = false;
    this.FormStyleChanged = false;
    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        regionObj.IsDirty = false;

        for (var linkId in regionObj.Links) {
            var linkObj = regionObj.Links.get(linkId);
            linkObj.IsDirty = false;
        }

        for (var tableId in regionObj.Tables) {
            var tableObj = regionObj.Tables.get(tableId);
            tableObj.IsDirty = false;

            for (var fieldId in tableObj.Fields) {
                var fieldObj = tableObj.Fields.get(fieldId);
                fieldObj.IsDirty = false;
            }
        }
    }

    for (var fmId in this.Formulas) {
        var formulaObj = this.Formulas.get(fmId);
        formulaObj.IsDirty = false;
    }

    return this;
},

BM.FormObject.prototype.markDirty = function () {
    if (this.onInitializingReservedCount == true)
        return this;

    if (this.IsNew == false) {
        if (this.IsDirty != true) {
            this.beforePropertyChanged("IsDirty", this.IsDirty, true);
            this.IsDirty = true;
            this.propertyHasChanged("IsDirty");
        }
    }
    return this;
},

BM.FormObject.prototype.markNew = function () {
    if (this.IsNew != true) {
        this.beforePropertyChanged("IsNew", this.IsNew, true);
        this.IsNew = true;
        this.propertyHasChanged("IsNew");
    }
    if (this.IsDirty != false) {
        this.beforePropertyChanged("IsDirty", this.IsDirty, false);
        this.IsDirty = false;
        this.propertyHasChanged("IsDirty");
    }
    if (this.IsDeleted != false) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, false);
        this.IsDeleted = false;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.FormObject.prototype.markDeleted = function () {
    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        if (isNaN(regionObj.ID) && regionObj.ID.indexOf("RegionObject") >= 0) {
            //如果要删除的区域本来就是新建的，则直接删除。   
            this.Regions.remove(regionObj.ID);
        } else {
            regionObj.markDeleted();
            this.DelRegions.add(regionId, regionObj);
        }
    }
    this.Regions.clear();

    if (this.IsNew != false) {
        this.beforePropertyChanged("IsNew", this.IsNew, false);
        this.IsNew = false;
        this.propertyHasChanged("IsNew");
    }
    if (this.IsDirty != false) {
        this.beforePropertyChanged("IsDirty", this.IsDirty, false);
        this.IsDirty = false;
        this.propertyHasChanged("IsDirty");
    }
    if (this.IsDeleted != true) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, true);
        this.IsDeleted = true;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.FormObject.prototype.toString = function () {
    return "表单【" + this.Title + "】Code【" + this.Code + "】";
};

/*
* 根据数据链接唯一标识获取对应的链接.
* @param linkUniqueCode {String} 数据链接唯一标识
* @returns 存在则返回一个LinkObject，不存在返回NULL.
*/
BM.FormObject.prototype.getLinkByUniqueCode = function (linkUniqueCode) {
    for (var regId in this.Regions) {
        var linkObj = this.Regions.get(regId).getLinkByUniqueCode(linkUniqueCode);
        if (linkObj)
            return linkObj;
    }

    return null;
};

//工具栏中删除了枚举之后，需要更新缓存中所有引用了该枚举的链接及指标.
BM.FormObject.prototype.updateFieldLinkByDeleteEnum = function (enumExpression) {
    var changedObjects = new BM.BusinessList();
    for (var regId in this.Regions) {
        var regionObj = this.Regions.get(regId);
        for (var linkId in regionObj.Links) {
            var linkObj = regionObj.Links.get(linkId);
            if (linkObj != null && linkObj.EnumExpression == enumExpression) {
                linkObj.setEnumExpression(null);
                linkObj.setEditMode(BM.Enum.DataLinkEditMode.dl_Default);
                linkObj.setEnumStyle(BM.Enum.DataLinkEnumStyle.de_Code);

                changedObjects.add("Link" + linkObj.ID, linkObj);
            }
        }
    }
    try {
        var mainFrameModel = $.m("BM_MainFrameModel");
        var currentForm = mainFrameModel.getCurrentForm();
        if (currentForm.ID == this.ID) {
            //note 删除操作步骤，以便撤销的时候不会恢复对该枚举的引用.
            var undoUnit = mainFrameModel.getUndoUnit();
            if (undoUnit == null) return;
            for (var i = 0; i < undoUnit._undoStack.length; i++) {
                var undoAction = undoUnit._undoStack[i];
                for (var j = 0; j < undoAction._events.length; j++) {
                    var undoEvent = undoAction._events[j];
                    if (undoEvent instanceof BM.ChangedUndoEvent) {
                        if (undoEvent.Item != null) {
                            if (undoEvent.Item instanceof BM.LinkObject) {
                                if (undoEvent.Member == 'EnumExpression'
                                    && (undoEvent.OldValue == enumExpression || undoEvent.NewValue == enumExpression)
                                    && changedObjects.get("Link" + undoEvent.Item.ID) != null)
                                    undoAction._events.splice(j, 1);
                            }
                        }
                    } else if (undoEvent instanceof BM.AddRemoveUndoEvent) {
                        if (undoEvent.Item != null) {
                            if (undoEvent.Item instanceof BM.LinkObject) {
                                if (undoEvent.Item.EnumExpression == enumExpression
                                    && changedObjects.get("Link" + undoEvent.Item.ID) != null)
                                    undoAction._events.splice(j, 1);
                            }
                        }
                    }
                }
            }

            for (var i = 0; i < undoUnit._redoStack.length; i++) {
                var undoAction = undoUnit._redoStack[i];
                for (var j = 0; j < undoAction._events.length; j++) {
                    var undoEvent = undoAction._events[j];
                    if (undoEvent instanceof BM.ChangedUndoEvent) {
                        if (undoEvent.Item != null) {
                            if (undoEvent.Item instanceof BM.LinkObject) {
                                if (undoEvent.Member == 'EnumExpression'
                                    && (undoEvent.OldValue == enumExpression || undoEvent.NewValue == enumExpression)
                                    && changedObjects.get("Link" + undoEvent.Item.ID) != null)
                                    undoAction._events.splice(j, 1);
                            }
                        }
                    } else if (undoEvent instanceof BM.AddRemoveUndoEvent) {
                        if (undoEvent.Item != null) {
                            if (undoEvent.Item instanceof BM.LinkObject) {
                                if (undoEvent.Item.EnumExpression == enumExpression
                                    && changedObjects.get("Link" + undoEvent.Item.ID) != null)
                                    undoAction._events.splice(j, 1);
                            }
                        }
                    }
                }
            }

        }
    } catch (e) {
        window.console && window.console.error(e.message);
    }
};

//重新切换页签之后，对撤销缓存中的Grid进行更新.
BM.FormObject.prototype.updateUndoActionsOnLoaded = function () {
    var grid = $.m("BM_GridViewModel")._grid;

    try {
        var mainFrameModel = $.m("BM_MainFrameModel");
        var currentForm = mainFrameModel.getCurrentForm();
        if (currentForm.ID == this.ID) {
            //note 删除操作步骤，以便撤销的时候不会恢复对该枚举的引用.
            var undoUnit = mainFrameModel.getUndoUnit(this.ID);
            if (undoUnit == null) return;
            for (var i = 0; i < undoUnit._undoStack.length; i++) {
                var undoAction = undoUnit._undoStack[i];
                for (var j = 0; j < undoAction._events.length; j++) {
                    var undoEvent = undoAction._events[j];
                    if (undoEvent.Grid != null) {
                        undoEvent.Grid = grid;
                    }
                }
            }

            for (var i = 0; i < undoUnit._redoStack.length; i++) {
                var undoAction = undoUnit._redoStack[i];
                for (var j = 0; j < undoAction._events.length; j++) {
                    var undoEvent = undoAction._events[j];
                    if (undoEvent.Grid != null) {
                        undoEvent.Grid = grid;
                    }
                }
            }

        }
    } catch (e) {
        window.console && window.console.error(e.message);
    }
};

//获取该行对应的编号及名称设置
BM.FormObject.prototype.getRowExtension = function (row) {
    var rowExtension = this.RowExtensions.get(row);
    if (rowExtension != null)
        return rowExtension;
    return null;
};

//当表单为新表单时，保存时，对固定区域存储表进行指标分布优化.
BM.FormObject.prototype.reAssignFieldsIfFormIsNew = function () {
    if (this.IsNew === false)
        return;

    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);

    for (var regId in this.Regions) {
        var regionObj = this.Regions.get(regId);
        if (regionObj.RegionKind != BM.Enum.DataRegionKind.dr_RowList) {
            var tableIds = [];

            for (var tableId in regionObj.Tables) {
                var tableObj = regionObj.Tables.get(tableId);
                var fieldsCountInTable = tableObj.Fields.getCount();
                if (fieldsCountInTable < formHelper.FieldsMaxCount) {
                    //note 说明当前的存储表内指标个数少于指标个数的上限，此时应该从别的存储表移动相应的指标过来以优化存储.
                    tableIds.push(tableId);
                }

            }

            for (var idx1 = 0; idx1 < tableIds.length; idx1++) {
                for (var idx2 = idx1 + 1; idx2 < tableIds.length; idx2++) {
                    var table1 = regionObj.Tables.get(tableIds[idx1]);
                    var table2 = regionObj.Tables.get(tableIds[idx2]);

                    //从table2移动到table1中的指标个数.
                    if (table1 && table2) {
                        for (var fieldId in table2.Fields) {
                            if (table2.isBizField(fieldId) == false) {//非主键指标才可以进行移动.
                                var fieldObj = table2.Fields.get(fieldId);
                                regionObj.__moveFieldToNewTableNDelete(fieldObj, table2, table1);
                                if (table1.Fields.getCount() >= formHelper.FieldsMaxCount)
                                    break;
                            }
                        }
                    }
                }
            }
        }
    }

};

//设置表单填报指南.
BM.FormObject.prototype.setFillGuide = function (guide) {
    if (null == guide) {
        return null;
    }

    this.beforePropertyChanged("FillingGuide", this.FillingGuide, guide);
    this.FillingGuide = guide;
    this.markDirty();
    this.propertyHasChanged("FillingGuide");
    return this;
};

//设置表单可见条件.
BM.FormObject.prototype.setActiveCondition = function (condition) {
    var isCheck = false;
    if (null == condition) {
        return null;
    }
    if (condition == "1" || condition == "0")
        isCheck = true;

    if (this.ActiveCondition && this.ActiveCondition.indexOf(";") > -1) {
        var aConditions = this.ActiveCondition.split(";");
        if (isCheck) {
            condition = condition + ";" + aConditions[1];
        } else {
            condition = aConditions[0] + ";" + condition;
        }
    } else if (this.ActiveCondition == "1" || this.ActiveCondition == "0") {
        if (!isCheck) {
            condition = this.ActiveCondition + ";" + condition;
        }
    } else {
        if (isCheck && this.ActiveCondition) {
            condition = condition + ";" + this.ActiveCondition;
        }
    }

    if (this.ActiveCondition != condition) {
        this.beforePropertyChanged("ActiveCondition", this.ActiveCondition, condition);
        this.ActiveCondition = condition;
        this.markDirty();
        this.propertyHasChanged("ActiveCondition");
    }
    return this;
};

//设置表单是否汇总.
BM.FormObject.prototype.setUnGather = function (gather) {
    if (null == gather) {
        return null;
    }

    this.beforePropertyChanged("UnGather", this.UnGather, gather);
    this.UnGather = !gather;
    this.markDirty();
    this.propertyHasChanged("UnGather");
    return this;
};

//判断表单是否进行初始化过，初始化过的话，区域上的保留行信息为0.
BM.FormObject.prototype.isFormInitializeReservedRows = function () {
    for (var regionId in this.Regions) {
        var regionObj = this.Regions.get(regionId);
        if (regionObj.RowsInFloatRegion > 0)
            return false;
    }

    return true;
};

BM.FormObject.prototype.isPrimaryKeyByPos = function (colIndex, rowIndex) {
    var regionObj = this.getRegionByPos(colIndex, rowIndex);
    if (regionObj == null)
        throw new Error("坐标X:" + colIndex + ",Y:" + rowIndex + "不存在任务数据区域");

    return regionObj.isPrimaryKeyByPos(colIndex, rowIndex);
};

BM.FormObject.prototype.setAsPrimaryKeyByPos = function (colIndex, rowIndex) {
    var regionObj = this.getRegionByPos(colIndex, rowIndex);
    if (regionObj == null)
        throw new Error("坐标X:" + colIndex + ",Y:" + rowIndex + "不存在任务数据区域");

    regionObj.setAsPrimaryKeyByPos(colIndex, rowIndex);
};

BM.FormObject.prototype.removePrimaryKeyByPos = function (colIndex, rowIndex) {
    var regionObj = this.getRegionByPos(colIndex, rowIndex);
    if (regionObj == null)
        throw new Error("坐标X:" + colIndex + ",Y:" + rowIndex + "不存在任务数据区域");

    regionObj.removePrimaryKeyByPos(colIndex, rowIndex);
};

//dt_EnumDefine: 0,     // 枚举表数据源
//dt_CurrentForm: 1,    // 本表枚举数据源
//dt_AcrossForm: 2,     // 跨表枚举数据源
//dt_FixedData: 3,      // 固定数据数据源
//dt_ErrorEmum: 4       // 错误枚举
BM.FormObject.prototype.getEnumExpressionType = function (linkObject) {
    var formHelper = $.m("BM_MainFrameModel").getService(BM_ServiceName_FormHelper);
    if (formHelper.isNullOrEmpty(linkObject.EnumExpression))
        return null;

    if (linkObject.EnumSourceType == BM.Enum.EnumSourceType.st_Immutable)
        return BM.Enum.EnumDataType.dt_EnumDefine;

    var enumExpression = linkObject.EnumExpression;
    if (enumExpression.trim().startWith('=') == false)
        return BM.Enum.EnumDataType.dt_FixedData;

    try {
        var showFormula = formulaParser.getShowFormula(enumExpression);
        var enumExpressionNode = formulaParser.parseExpression(showFormula);
        var mainNode = formulaParser.getMainNode(enumExpressionNode.MainNode);
        if (formHelper.isNullOrEmpty(mainNode.SheetName)
            || mainNode.SheetName == this.Code) {
            return BM.Enum.EnumDataType.dt_CurrentForm;
        } else {
            return BM.Enum.EnumDataType.dt_AcrossForm;
        }
    } catch (ex) {
        return BM.Enum.EnumDataType.dt_ErrorEmum;
    }
};

//fixme to be opitimized.
BM.FormObject.prototype.getEnumDataParams = function (enumSourceType, enumExpression) {
    var mainFrame = $.m("BM_MainFrameModel");
    var formHelper = mainFrame.getService(BM_ServiceName_FormHelper);
    if (formHelper.isNullOrEmpty(enumExpression))
        return [null, null];

    var taskObject = mainFrame.getCurrentTask();
    if (enumSourceType == BM.Enum.EnumSourceType.st_Immutable) {
        var enumNodes = taskObject.getEnumNodesById(enumExpression);
        var enumDatas = [];
        for (var key in enumNodes) {
            var enumData = {};
            var enumNodeObj = enumNodes[key];
            enumData["Code"] = enumNodeObj.Code;
            enumData["Title"] = enumNodeObj.Title;
            enumData["ParentCode"] = enumNodeObj.ParentCode;
            enumData["Order"] = enumNodeObj.Order;
            enumData["Mnemonic"] = enumNodeObj.Mnemonic;
            enumDatas.push(enumData);
        }
        var newEnumData = JSON.stringify(enumDatas);

        return [BM.Enum.EnumDataType.dt_EnumDefine, newEnumData];
    }

    if (enumExpression.trim().startWith('=') == false)
        return [BM.Enum.EnumDataType.dt_FixedData, enumExpression];

    try {
        var showFormula = formulaParser.getShowFormula(enumExpression);
        var enumExpressionNode = formulaParser.parseExpression(showFormula);
        var mainNode = formulaParser.getMainNode(enumExpressionNode.MainNode);

        var cellCoordinate = null;
        if (mainNode instanceof CellNode) {
            //单个单元格的
            var left = mainNode.Position.PosX;
            var top = mainNode.Position.PosY;
            var right = mainNode.Position.PosX;
            var bottom = mainNode.Position.PosY;
        } else if (mainNode instanceof RegionNode) {
            var left, top, right, bottom;
            var cellNodes = mainNode.CellNodes;
            for (var index = 0; index < cellNodes.length; index++) {
                var cellNode = cellNodes[index];
                var pos = cellNode.Position;
                if (left == null) {
                    left = pos.PosX;
                    top = pos.PosY;
                    right = pos.PosX;
                    bottom = pos.PosY;
                    continue;
                }
                if (left > pos.PosX) {
                    left = pos.PosX;
                }
                if (top > pos.PosY) {
                    top = pos.PosY;
                }
                if (right < pos.PosX) {
                    right = pos.PosX;
                }
                if (bottom < pos.PosY) {
                    bottom = pos.PosY;
                }
            }
            if (left != right && top != bottom) {
                throw new Error("必须是单列或者单行");
            }
        }

        if (formHelper.isNullOrEmpty(mainNode.SheetName)
            || mainNode.SheetName == this.Code) {

            var TL_Link = this.getLinkByPos(left, top);
            var BR_Link = this.getLinkByPos(right, bottom);
            cellCoordinate = [TL_Link.ID, BR_Link.ID];
            return [BM.Enum.EnumDataType.dt_CurrentForm, cellCoordinate];
        } else {
            var otherForm = taskObject.getFormByCode(mainNode.SheetName);
            if (otherForm.FormStatus < BM.Enum.FormStatus.fs_Queried) {
                mainFrame.action("/BusinessModel/FormService/RequestForm", { formId: otherForm.ID }, function (returnData) {
                    if (returnData) {
                        var formObj = returnData.data.formObject;
                        if (!formObj) {
                            throw "未找到表单：" + otherForm.Title;
                        }
                        var formObject = new BM.FormObject(formObj);
                        formObject.FormStatus = BM.Enum.FormStatus.fs_Queried;
                        this._taskObject.addFormObject(formObject);
                    } else {
                        throw "未找到表单：" + otherForm.Title;
                    }
                }, false);
            }

            otherForm = taskObject.getFormByCode(mainNode.SheetName);
            var cellTitles = [];
            var otherLink;
            var otherField;
            for (var col = left; col <= right; col++) {
                for (var row = top; row <= bottom; row++) {
                    otherLink = otherForm.getLinkByPos(col, row);
                    if (otherLink) {
                        otherField = otherForm.getFieldById(otherLink.LinkExpression);
                        if (otherField) {
                            cellTitles.push(otherField.DefaultValue);
                        }
                    }
                }
            }
            return [BM.Enum.EnumDataType.dt_AcrossForm, cellTitles];
        }
    } catch (ex) {
        return [BM.Enum.EnumDataType.dt_ErrorEmum, ex];
    }
};