﻿BM.FormulaObject = function (formulaObj) {
    BM.UndoBase.call(this);

    this.ID = formulaObj.ID;
    this.Description = formulaObj.Description || "";
    this.Order = formulaObj.Order || "";
    this.Code = formulaObj.Code || "";
    this.AdjustItems = formulaObj.AdjustItems || "";
    this.CheckType = formulaObj.CheckType ? formulaObj.CheckType : BM.Enum.FormulaCheckKind.fc_None;
    this.FormulaLevelCode = formulaObj.FormulaLevelCode || "";
    this.SubmitCheckStrategy = formulaObj.SubmitCheckStrategy ? formulaObj.SubmitCheckStrategy : BM.Enum.SubmitCheckStrategy.Default;
    this.Expression = formulaObj.Expression;
    this.FormID = formulaObj.FormID;
    this.SchemeID = formulaObj.SchemeID;
    this.AutoExecute = formulaObj.AutoExecute || false;
    this.UseCheck = formulaObj.UseCheck || false;
    this.UseBalance = formulaObj.UseBalance || false;
    this.UseCalculate = formulaObj.UseCalculate || false;
    this.PosX = formulaObj.PosX || 0;
    this.PosY = formulaObj.PosY || 0;

    this.IsNew = formulaObj.IsNew || false;
    this.IsDirty = formulaObj.IsDirty || false;
    this.IsDeleted = formulaObj.IsDeleted || false;
};

BM.FormulaObject.prototype = new BM.UndoBase();

BM.FormulaObject.prototype.isDataChanged = function () {
    if (this.IsNew || this.IsDirty || this.IsDirty)
        return true;

    return false;
};

//保存时，重新复制一份数据，以免原有数据状态被破坏.
BM.FormulaObject.prototype.getCopyToSave = function () {
    var result = {};

    result.ID = this.ID;
    result.IsNew = this.IsNew;
    result.IsDirty = this.IsDirty;
    result.IsDeleted = this.IsDeleted;

    if (this.IsNew === true || this.IsDeleted === true || this.IsDirty === true) {
        result.Description = this.Description;
        result.Order = this.Order;
        result.Code = this.Code;
        result.AdjustItems = this.AdjustItems;
        result.CheckType = this.CheckType;
        result.FormulaLevelCode = this.FormulaLevelCode;
        result.SubmitCheckStrategy = this.SubmitCheckStrategy;
        result.Expression = this.Expression;
        result.FormID = this.FormID;
        result.SchemeID = this.SchemeID;
        result.AutoExecute = this.AutoExecute;
        result.UseCheck = this.UseCheck;
        result.UseBalance = this.UseBalance;
        result.UseCalculate = this.UseCalculate;
        result.PosX = this.PosX;
        result.PosY = this.PosY;
    }

    return result;
};

BM.FormulaObject.prototype.markNew = function () {
    if (this.IsNew != true) {
        this.beforePropertyChanged("IsNew", this.IsNew, true);
        this.IsNew = true;
        this.propertyHasChanged("IsNew");
    }
    if (this.IsDirty != false) {
        this.beforePropertyChanged("IsDirty", this.IsDirty, false);
        this.IsDirty = false;
        this.propertyHasChanged("IsDirty");
    }
    if (this.IsDeleted != false) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, false);
        this.IsDeleted = false;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.FormulaObject.prototype.markDirty = function () {
    if (this.IsNew == false) {
        if (this.IsDirty != true) {
            this.beforePropertyChanged("IsDirty", this.IsDirty, true);
            this.IsDirty = true;
            this.propertyHasChanged("IsDirty");
        }
    }
    return this;
};

BM.FormulaObject.prototype.markDeleted = function () {
    if (this.IsNew != false) {
        this.beforePropertyChanged("IsNew", this.IsNew, false);
        this.IsNew = false;
        this.propertyHasChanged("IsNew");
    }
    if (this.IsDirty != false) {
        this.beforePropertyChanged("IsDirty", this.IsDirty, false);
        this.IsDirty = false;
        this.propertyHasChanged("IsDirty");
    }
    if (this.IsDeleted != true) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, true);
        this.IsDeleted = true;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.FormulaObject.prototype.markUnDeleted = function () {
    if (this.IsDeleted != false) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, false);
        this.IsDeleted = false;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.FormulaObject.prototype.setDescription = function (newDescription) {
    if (this.Description != newDescription) {
        this.beforePropertyChanged("Description", this.Description, newDescription);
        this.Description = newDescription;
        this.markDirty();
        this.propertyHasChanged("Description");
    }
    return this;
};

BM.FormulaObject.prototype.setOrder = function (newOrder) {
    if (this.Order != newOrder) {
        this.beforePropertyChanged("Order", this.Order, newOrder);
        this.Order = newOrder;
        this.markDirty();
        this.propertyHasChanged("Order");
    }
    return this;
};

BM.FormulaObject.prototype.setCode = function (newCode) {
    if (newCode == null || newCode === "")
        return null;
    newCode = newCode.trim();

    if (this.Code != newCode) {
        this.beforePropertyChanged("Code", this.Code, newCode);
        this.Code = newCode;
        this.markDirty();
        this.propertyHasChanged("Code");
    }
    return this;
};

BM.FormulaObject.prototype.setAdjustItems = function (newAdjustItems) {
    if (this.AdjustItems != newAdjustItems) {
        this.beforePropertyChanged("AdjustItems", this.AdjustItems, newAdjustItems);
        this.AdjustItems = newAdjustItems;
        this.markDirty();
        this.propertyHasChanged("AdjustItems");
    }
    return this;
};

BM.FormulaObject.prototype.setCheckType = function (newCheckType) {
    if (newCheckType == null || newCheckType === "")
        return null;

    if (this.CheckType != newCheckType) {
        this.beforePropertyChanged("CheckType", this.CheckType, newCheckType);
        this.CheckType = newCheckType;
        this.markDirty();
        this.propertyHasChanged("CheckType");
    }
    return this;
};

BM.FormulaObject.prototype.setFormulaLevelCode = function (newFormulaLevelCode) {
    newFormulaLevelCode = newFormulaLevelCode || "";

    if (this.FormulaLevelCode != newFormulaLevelCode) {
        this.beforePropertyChanged("FormulaLevelCode", this.FormulaLevelCode, newFormulaLevelCode);
        this.FormulaLevelCode = newFormulaLevelCode;
        this.markDirty();
        this.propertyHasChanged("FormulaLevelCode");
    }
    return this;
};

BM.FormulaObject.prototype.setSubmitCheckStrategy = function (newSubmitCheckStrategy) {
    if (newSubmitCheckStrategy == null || newSubmitCheckStrategy === "")
        return null;

    if (this.SubmitCheckStrategy != newSubmitCheckStrategy) {
        this.beforePropertyChanged("SubmitCheckStrategy", this.SubmitCheckStrategy, newSubmitCheckStrategy);
        this.SubmitCheckStrategy = newSubmitCheckStrategy;
        this.markDirty();
        this.propertyHasChanged("SubmitCheckStrategy");
    }
    return this;
};

BM.FormulaObject.prototype.setExpression = function (newExpression) {
    newExpression = newExpression || "";

    if (this.Expression != newExpression) {
        this.beforePropertyChanged("Expression", this.Expression, newExpression);
        this.Expression = newExpression;
        this.markDirty();
        this.propertyHasChanged("Expression");
    }
    return this;
};

BM.FormulaObject.prototype.setFormID = function (newFormID) {
    if (newFormID == null || newFormID === "")
        return null;

    if (this.FormID != newFormID) {
        this.beforePropertyChanged("FormID", this.FormID, newFormID);
        this.FormID = newFormID;
        this.markDirty();
        this.propertyHasChanged("FormID");
    }
    return this;
};

BM.FormulaObject.prototype.setSchemeID = function (newSchemeID) {
    if (newSchemeID == null || newSchemeID === "")
        return null;

    if (this.SchemeID != newSchemeID) {
        this.beforePropertyChanged("SchemeID", this.SchemeID, newSchemeID);
        this.SchemeID = newSchemeID;
        this.markDirty();
        this.propertyHasChanged("SchemeID");
    }
    return this;
};

BM.FormulaObject.prototype.setAutoExecute = function (newAutoExecute) {
    if (newAutoExecute == null || newAutoExecute === "")
        return null;

    if (this.AutoExecute != newAutoExecute) {
        this.beforePropertyChanged("AutoExecute", this.AutoExecute, newAutoExecute);
        this.AutoExecute = newAutoExecute;
        this.markDirty();
        this.propertyHasChanged("AutoExecute");
    }
    return this;
};

BM.FormulaObject.prototype.setUseCheck = function (newUseCheck) {
    if (newUseCheck == null || newUseCheck === "")
        return null;

    if (this.UseCheck != newUseCheck) {
        this.beforePropertyChanged("UseCheck", this.UseCheck, newUseCheck);
        this.UseCheck = newUseCheck;
        this.markDirty();
        this.propertyHasChanged("UseCheck");
    }
    return this;
};

BM.FormulaObject.prototype.setUseBalance = function (newUseBalance) {
    if (newUseBalance == null || newUseBalance === "")
        return null;

    if (this.UseBalance != newUseBalance) {
        this.beforePropertyChanged("UseBalance", this.UseBalance, newUseBalance);
        this.UseBalance = newUseBalance;
        this.markDirty();
        this.propertyHasChanged("UseBalance");
    }
    return this;
};

BM.FormulaObject.prototype.setUseCalculate = function (newUseCalculate) {
    if (newUseCalculate == null || newUseCalculate === "")
        return null;

    if (this.UseCalculate != newUseCalculate) {
        this.beforePropertyChanged("UseCalculate", this.UseCalculate, newUseCalculate);
        this.UseCalculate = newUseCalculate;
        this.markDirty();
        this.propertyHasChanged("UseCalculate");
    }
    return this;
};

BM.FormulaObject.prototype.setPosX = function (newPosX) {
    if (newPosX == null || newPosX === "")
        return null;

    if (this.PosX != newPosX) {
        this.beforePropertyChanged("PosX", this.PosX, newPosX);
        this.PosX = newPosX;
        this.markDirty();
        this.propertyHasChanged("PosX");
    }
    return this;
};

BM.FormulaObject.prototype.setPosY = function (newPosY) {
    if (newPosY == null || newPosY === "")
        return null;

    if (this.PosY != newPosY) {
        this.beforePropertyChanged("PosY", this.PosY, newPosY);
        this.PosY = newPosY;
        this.markDirty();
        this.propertyHasChanged("PosY");
    }
    return this;
};

BM.FormulaObject.prototype.toString = function () {
    return "公式ID【" + this.ID + "】X【" + this.PosX + "】Y【" + this.PosY + "】表达式【" + this.Expression + "】";
};