﻿
//-------------------------------------------------------------------------------------------------------------------------------存储表.
BM.TableObject = function (tableObj) {
    BM.UndoBase.call(this);

    this.ID = tableObj.ID;
    this.Title = tableObj.Title ? tableObj.Title : "";
    this.Order = tableObj.Order;

    this.IsNew = tableObj.IsNew;
    this.IsDirty = tableObj.IsDirty;
    this.IsDeleted = tableObj.IsDeleted;

    this.Code = tableObj.Code ? tableObj.Code : "";
    this.Kind = tableObj.Kind ? tableObj.Kind : BM.Enum.TableKind.tk_BizData;
    this.OwnerGroupID = tableObj.OwnerGroupID ? tableObj.OwnerGroupID : 0;
    this.PeriodFieldID = tableObj.PeriodFieldID ? tableObj.PeriodFieldID : 0;
    this.BizKeyFieldsStr = tableObj.BizKeyFieldsStr ? tableObj.BizKeyFieldsStr : "";
    this.Fields = new BM.UndoList("指标");
    for (var fieldId in tableObj.Fields) {
        var tmp = tableObj.Fields[fieldId];
        var fieldObj = new BM.FieldObject(tmp);
        this.Fields.add(fieldObj.ID, fieldObj);
    }

    this.DelFields = new BM.UndoList("删除指标");

    this.BizkeyFields = new BM.UndoList("业务主键");
    for (var bizFieldId in tableObj.BizkeyFields) {
        var tmpBizObj = tableObj.BizkeyFields[bizFieldId];
        var bizFieldObj = new BM.FieldObject(tmpBizObj);
        this.BizkeyFields.add(bizFieldObj.ID, bizFieldObj);
    }

    this.Views = new BM.UndoList("视图");
    for (var viewId in tableObj.Views) {
        var tmpView = tableObj.Views[viewId];
        var viewObj = new BM.EntityViewObject(tmpView);
        this.Views.add(viewObj.ID, viewObj);
    }
};

BM.TableObject.prototype = new BM.UndoBase();

//当前存储表是否已经修改.
BM.TableObject.prototype.isDataChanged = function () {
    if (this.IsNew || this.IsDirty || this.IsDirty)
        return true;

    if (this.DelFields.getCount() > 0)
        return true;

    for (var fieldId in this.Fields) {
        var fieldObj = this.Fields.get(fieldId);
        if (fieldObj.IsNew || fieldObj.IsDirty || fieldObj.IsDirty)
            return true;
    }

    for (var viewId in this.Views) {
        var tmpView = this.Views.get(viewId);
        if (tmpView.IsNew || tmpView.IsDirty || tmpView.IsDirty)
            return true;
    }

    return false;
};

//保存时，重新复制一份数据，以免原有数据状态被破坏.
BM.TableObject.prototype.getCopyToSave = function () {
    var result = {};

    result.ID = this.ID;
    result.IsNew = this.IsNew;
    result.IsDirty = this.IsDirty;
    result.IsDeleted = this.IsDeleted;

    result.OwnerGroupID = this.OwnerGroupID;
    result.Code = this.Code;
    result.PeriodFieldID = this.PeriodFieldID;
    result.BizKeyFieldsStr = this.BizKeyFieldsStr;
    if (this.IsNew === true || this.IsDeleted === true || this.IsDirty === true) {
        result.Title = this.Title;
        result.Order = this.Order;
        result.Kind = this.Kind;
    }
    result.Fields = new BM.BusinessList("指标");
    for (var fieldId in this.Fields) {
        var fieldObj = this.Fields.get(fieldId);
        if (fieldObj.IsDirty === true
            || fieldObj.IsDeleted === true
            || fieldObj.IsNew === true) {
            result.Fields.add(fieldId, fieldObj.getCopyToSave());
        }
    }

    for (var deletedFieldId in this.DelFields) {
        var delObj = {};
        delObj.ID = deletedFieldId;
        delObj.IsDeleted = true;
        result.Fields.add(deletedFieldId, delObj);
    }

    return result;
};

BM.TableObject.prototype.beforeSave = function () {
    //note 优化只传输修改的参数到后台，减少传输量.
    var fieldsToSave = new BM.BusinessList();
    for (var fieldId in this.Fields) {
        var fieldObj = this.Fields.get(fieldId);
        if (fieldObj.IsDirty === true
            || fieldObj.IsDeleted === true
            || fieldObj.IsNew === true) {
            fieldsToSave.add(fieldId, fieldObj);
        }
    }
    for (var deletedFieldId in this.DelFields) {
        //var deletedFieldObj = this.DelFields.get(deletedFieldId);
        //this.Fields.add(deletedFieldObj.ID, deletedFieldObj);
        var delObj = {};
        delObj.ID = deletedFieldId;
        delObj.IsDeleted = true;
        fieldsToSave.add(deletedFieldId, delObj);
        this.DelFields.remove(deletedFieldId);
    }
    this.Fields = fieldsToSave;
    this.BizkeyFields = null;
};

BM.TableObject.prototype.createFieldObject = function () {
    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);

    var newFieldId = formHelper.JCPGuid("FieldObject-");
    var tmpField = {};
    tmpField.ID = newFieldId;
    tmpField.IsNew = true;
    tmpField.IsDirty = false;
    tmpField.IsDeleted = false;

    var newFieldObj = new BM.FieldObject(tmpField);
    newFieldObj.OwnerTableID = this.ID;

    this.Fields.add(newFieldObj.ID, newFieldObj);

    return newFieldObj;
};

BM.TableObject.prototype.setCode = function (newCode) {
    if (newCode == null || newCode === "")
        return null;

    newCode = newCode.trim();

    if (newCode.length > 50)
        newCode = newCode.substring(0, 50);//指标标识不可超过50.

    if (this.Code != newCode) {
        this.beforePropertyChanged("Code", this.Code, newCode);
        this.Code = newCode;
        this.markDirty();
        this.propertyHasChanged("Code");
    }
    return this;
};

BM.TableObject.prototype.setTitle = function (newTitle) {
    if (newTitle == null || newTitle.trim() === "")
        return null;

    newTitle = newTitle.trim();

    if (this.Title != newTitle) {
        this.beforePropertyChanged("Title", this.Title, newTitle);
        this.Title = newTitle;
        this.markDirty();
        this.propertyHasChanged("Title");
    }
    return this;
};

BM.TableObject.prototype.setBizKeyFieldsStr = function (newBizKeyFieldsStr) {
    if (newBizKeyFieldsStr == null || newBizKeyFieldsStr.trim() === "")
        return null;

    newBizKeyFieldsStr = newBizKeyFieldsStr.trim();
    if (this.BizKeyFieldsStr != newBizKeyFieldsStr) {
        this.beforePropertyChanged("BizKeyFieldsStr", this.BizKeyFieldsStr, newBizKeyFieldsStr);
        this.BizKeyFieldsStr = newBizKeyFieldsStr;
        this.markDirty();
        this.propertyHasChanged("BizKeyFieldsStr");
    }
    return this;
};

//存储表是否已经存在指定标识的指标.
BM.TableObject.prototype.isExistFieldCode = function (fieldCode) {
    if (fieldCode == null || fieldCode === "")
        return false;

    for (var fieldId in this.Fields) {
        var fieldObj = this.Fields.get(fieldId);
        if (fieldObj.Code == fieldCode)
            return true;
    }

    return false;
};

//存储表是否已经存在指定标识的指标.
BM.TableObject.prototype.isExistFieldCodeExceptField = function (exceptFieldId, fieldCode) {
    if (fieldCode == null || fieldCode === "")
        return false;

    for (var fieldId in this.Fields) {
        if (fieldId == exceptFieldId) continue;
        var fieldObj = this.Fields.get(fieldId);
        if (fieldObj.Code == fieldCode)
            return true;
    }

    return false;
};

BM.TableObject.prototype.addField = function (fieldObject) {
    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);

    while (this.__getFieldByCode(fieldObject.Code) != null) {
        var newFieleCode = formHelper.generateNewStringByTemplet(fieldObject.Code);
        fieldObject.setCode(newFieleCode);
    }
    this.markDirty();
    this.Fields.add(fieldObject.ID, fieldObject);
};

BM.TableObject.prototype.getFieldByCode = function (fieldCode) {
    if (fieldCode == null || fieldCode === "")
        return null;
    for (var fieldId in this.Fields) {
        var fieldObj = this.Fields[fieldId];
        if (fieldObj.Code == fieldCode)
            return fieldObj;
    }
    return null;
};

BM.TableObject.prototype.__getFieldByCode = function (fieldCode) {
    if (fieldCode == null || fieldCode === "")
        return null;
    for (var fieldId in this.Fields) {
        var fieldObj = this.Fields[fieldId];
        if (fieldObj.Code == fieldCode)
            return fieldObj;
    }
    //note 如果当前指标列表中找不到该指标，则从删除列表里面查找，并重复利用.
    for (var delFieldId in this.DelFields) {
        var delFieldObj = this.DelFields[delFieldId];
        if (delFieldObj.Code == fieldCode)
            return delFieldObj;
    }

    return null;
};

BM.TableObject.prototype.getFieldById = function (fieldId) {
    return this.Fields.get(fieldId);
};

BM.TableObject.prototype.__getFieldById = function (fieldId) {
    var fieldObj = this.Fields.get(fieldId);

    //note 如果当前指标列表中找不到该指标，则从删除列表里面查找，并重复利用.
    if (!fieldObj)
        return this.DelFields.get(fieldId);

    return fieldObj;
};

//通过存储表来删除指标。
BM.TableObject.prototype.delFieldById = function (fieldId) {
    var field = this.Fields.get(fieldId);
    if (field) {
        this.removeBizFieldById(field.ID);

        if (!this.IsNew)
            this.markDirty();
        if (isNaN(field.ID) && field.ID.indexOf("FieldObject") >= 0) {
            //如果要删除的指标本来就是新建的，则直接删除。   
            this.Fields.remove(fieldId);
        } else {
            //如果带删除的指标是数据库中记录，则标记为删除.
            field.markDeleted();
            field.setDefaultValue(null);//note 删掉指标后，清除其默认值，要不会导致指标重用后，在界面上显示出来.
            this.Fields.remove(fieldId);
            this.DelFields.add(field.ID, field);
        }
    }
};

//判断指标ID是否属于业务主键指标.
BM.TableObject.prototype.__isFieldIDBizKey = function (fieldId) {
    if ((this.BizKeyFieldsStr == null || this.BizKeyFieldsStr === "") && fieldId == null || fieldId === "")
        return false;

    var bizKeys = this.BizKeyFieldsStr.split(';');
    for (var bizKeyIndex = 0; bizKeyIndex < bizKeys.length; bizKeyIndex++) {
        if (bizKeys[bizKeyIndex] == fieldId)
            return true;
    }

    return false;
};

BM.TableObject.prototype.unDeleteFieldByID = function (fieldId) {
    var fieldObj = this.DelFields.get(fieldId);
    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);

    if (fieldObj) {
        fieldObj.markUnDeleted();
        this.DelFields.remove(fieldObj.ID);

        while (this.isExistFieldCode(fieldObj.Code)) {
            var newFieleCode = formHelper.generateNewStringByTemplet(fieldObj.Code);
            fieldObj.setCode(newFieleCode);
        }
        this.Fields.add(fieldObj.ID, fieldObj);
    }
};

BM.TableObject.prototype.getBizFields = function () {
    var fieldIds = this.BizKeyFieldsStr.split(';');
    var result = new BM.BusinessList();
    for (var bizFieldId = 0; bizFieldId < fieldIds.length; bizFieldId++) {
        var fieldId = fieldIds[bizFieldId];
        var fieldObj = this.getFieldById(fieldId);
        if (fieldObj)
            result.add(fieldId, fieldObj);
    }
    return result;
};

BM.TableObject.prototype.getDefaultView = function () {
    for (var viewId in this.Views) {
        var viewObj = this.Views[viewId];
        if (viewObj.IsDefault)
            return viewObj;
    }
    return null;
};

BM.TableObject.prototype.markDirty = function () {
    if (this.IsNew == false) {
        if (this.IsDirty != true) {
            this.beforePropertyChanged("IsDirty", this.IsDirty, true);
            this.IsDirty = true;
            this.propertyHasChanged("IsDirty");
        }
    }
    return this;
};

BM.TableObject.prototype.markNew = function () {
    if (this.IsNew != true) {
        this.beforePropertyChanged("IsNew", this.IsNew, true);
        this.IsNew = true;
        this.propertyHasChanged("IsNew");
    }
    if (this.IsDirty != false) {
        this.beforePropertyChanged("IsDirty", this.IsDirty, false);
        this.IsDirty = false;
        this.propertyHasChanged("IsDirty");
    }
    if (this.IsDeleted != false) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, false);
        this.IsDeleted = false;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.TableObject.prototype.markDeleted = function () {
    for (var fieldId in this.Fields) {
        var fieldObj = this.Fields.get(fieldId);
        if (isNaN(fieldObj.ID) && fieldObj.ID.indexOf("FieldObject") >= 0) {
            //note 循环体外进行清除.
        } else {
            fieldObj.markDeleted();
            this.DelFields.add(fieldObj.ID, fieldObj);
        }
    }
    this.Fields.clear();

    for (var viewId in this.Views) {
        var viewObj = this.Views.get(viewId);
        if (isNaN(viewObj.ID) && viewObj.ID.indexOf("ViewObject") >= 0) {

        } else {
            viewObj.markDeleted();
            this.DelViews.add(viewObj.ID, viewObj);
        }
    }
    this.Views.clear();

    if (this.IsNew != false) {
        this.beforePropertyChanged("IsNew", this.IsNew, false);
        this.IsNew = false;
        this.propertyHasChanged("IsNew");
    }
    if (this.IsDirty != false) {
        this.beforePropertyChanged("IsDirty", this.IsDirty, false);
        this.IsDirty = false;
        this.propertyHasChanged("IsDirty");
    }
    if (this.IsDeleted != true) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, true);
        this.IsDeleted = true;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

//设置为未删除存储表后，存储表只剩下业务主键，其余指标先删除后按照需要进行恢复。
BM.TableObject.prototype.markUnDeleted = function () {
    for (var fieldId in this.Fields) {
        var fieldObj = this.Fields.get(fieldId);
        fieldObj.markDeleted();
        this.DelFields.add(fieldId, fieldObj);
    }
    this.Fields.clear();

    var keiFieldIDs = this.BizKeyFieldsStr.split(';');
    for (var IDIndex = 0; IDIndex < keiFieldIDs.length; IDIndex++) {
        var fieldObj1 = this.Fields.get(keiFieldIDs[IDIndex]);
        if (fieldObj1) {
            fieldObj1.markUnDeleted();
        }
        var fieldObj2 = this.DelFields.get(keiFieldIDs[IDIndex]);
        if (fieldObj2) {
            fieldObj2.markUnDeleted();
            this.Fields.add(fieldObj2.ID, fieldObj2);
        }

        this.DelFields.remove(keiFieldIDs[IDIndex]);
    }

    if (this.IsDeleted != false) {
        this.beforePropertyChanged("IsDeleted", this.IsDeleted, false);
        this.IsDeleted = false;
        this.propertyHasChanged("IsDeleted");
    }
    return this;
};

BM.TableObject.prototype.getFieldsCount = function () {
    return this.Fields.getCount();
};

//存储表中新增指定的视图.
BM.TableObject.prototype.addEntityView = function (entityViewId, entityViewObj) {
    if (entityViewObj.TableID != this.ID)
        entityViewObj.TableID = this.ID;
    return this.Views.add(entityViewId, entityViewObj);
};

BM.TableObject.prototype.toString = function () {
    return "存储表ID【" + this.ID + "】标题【" + this.Title + "】Code【" + this.Code + "】";
};

BM.TableObject.prototype.copyFrom = function (sourceTable) {
    if (sourceTable == null || !(sourceTable instanceof BM.TableObject))
        return;

    this.setCode(sourceTable.Code);

    var orderField = null;
    var floatOrderField = null;
    for (var fieldId in this.Fields) {
        if (orderField != null && floatOrderField != null)
            break;//查找到之后，跳出循环.

        var field = this.Fields.get(fieldId);
        if (field.Code.toLowerCase().indexOf('order') >= 0 && field.ValueType == BM.Enum.FieldValueType.fv_BizKeyOrder) {
            orderField = field;
        }
        else if (field.Code.toLowerCase().indexOf('floatorder') >= 0 && field.ValueType == BM.Enum.FieldValueType.fv_InputOrder) {
            floatOrderField = field;
        }
    }

    var srcOrderField = null;
    var srcFloatOrderField = null;
    for (var srcFieldId in sourceTable.Fields) {
        if (srcOrderField != null && srcFloatOrderField != null)
            break;//查找到之后，跳出循环.

        var srcField = sourceTable.Fields.get(srcFieldId);
        if (srcField.Code.toLowerCase().indexOf('order') >= 0 && srcField.ValueType == BM.Enum.FieldValueType.fv_BizKeyOrder) {
            srcOrderField = srcField;
        }
        else if (srcField.Code.toLowerCase().indexOf('floatorder') >= 0 && srcField.ValueType == BM.Enum.FieldValueType.fv_InputOrder) {
            srcFloatOrderField = srcField;
        }
    }

    if (orderField != null && srcOrderField != null) {
        orderField.copyFrom(srcOrderField);
        orderField.setOwnerTableID(this.ID);
    }

    if (floatOrderField != null && srcFloatOrderField != null) {
        floatOrderField.copyFrom(srcFloatOrderField);
        floatOrderField.setOwnerTableID(this.ID);
    }
};

//从当前指标列表中查找到FloatOrder指标，查找不到，则返回null.
BM.TableObject.prototype.getFloatOrderField = function () {
    for (var fieldId in this.Fields) {
        var field = this.Fields.get(fieldId);
        if (field.Code.toUpperCase() == 'FLOATORDER'
            && field.ValueType == BM.Enum.FieldValueType.fv_InputOrder) {
            return field;
        }
    }
    return null;
};

//note 内部方法，外部切勿使用.
//从当前指标列表中查找到FloatOrder指标，查找不到，则从删除列表中查找.
BM.TableObject.prototype.__getFloatOrderField = function () {
    for (var fieldId in this.Fields) {
        var field = this.Fields.get(fieldId);
        if (field.Code.toUpperCase() == 'FLOATORDER'
            && field.ValueType == BM.Enum.FieldValueType.fv_InputOrder) {
            return field;
        }
    }

    for (var delFieldId in this.DelFields) {
        var delField = this.DelFields.get(delFieldId);
        if (delField.Code.toUpperCase() == 'FLOATORDER'
            && delField.ValueType == BM.Enum.FieldValueType.fv_InputOrder) {
            return delField;
        }
    }
    return null;
};

//从当前指标列表中查找到Order指标，查找不到，则返回null.
BM.TableObject.prototype.getOrderField = function () {
    for (var fieldId in this.Fields) {
        var field = this.Fields.get(fieldId);
        if (field.Code.toUpperCase().indexOf('_ORDER') >= 0
            && field.ValueType == BM.Enum.FieldValueType.fv_BizKeyOrder) {
            return field;
        }
    }
    return null;
};

//note 内部方法，外部切勿使用.
//从当前指标列表中查找到Order指标，查找不到，则从删除列表中查找.
BM.TableObject.prototype.__getOrderField = function () {
    for (var fieldId in this.Fields) {
        var field = this.Fields.get(fieldId);
        if (field.Code.toUpperCase().indexOf('_ORDER') >= 0
            && field.ValueType == BM.Enum.FieldValueType.fv_BizKeyOrder) {
            return field;
        }
    }
    for (var delFieldId in this.DelFields) {
        var delField = this.DelFields.get(delFieldId);
        if (delField.Code.toUpperCase().indexOf('_ORDER') >= 0
            && delField.ValueType == BM.Enum.FieldValueType.fv_BizKeyOrder) {
            return delField;
        }
    }
    return null;
};

BM.TableObject.prototype.isBizField = function (fieldIdToCheck) {
    var fieldIds = this.BizKeyFieldsStr.split(';');
    for (var bizFieldId = 0; bizFieldId < fieldIds.length; bizFieldId++) {
        var fieldId = fieldIds[bizFieldId];
        if (fieldId == fieldIdToCheck)
            return true;
    }
    return false;
};

BM.TableObject.prototype.isSystemBizField = function (fieldIdToCheck) {
    var fieldObj = this.Fields.get(fieldIdToCheck);
    if (fieldObj == null) return false;

    var systemCodes = ['ENTITYKEY', 'CODE', 'FLOATORDER', '_ORDER'];
    if (systemCodes.indexOf(fieldObj.Code.toUpperCase()) < 0) {
        if (fieldObj.Code.toUpperCase().indexOf(systemCodes[3]) >= 0)
            return true;
        return false;
    }
    return true;
};

BM.TableObject.prototype.addBizField = function (fieldToAdd) {
    var fieldTmp = this.getFieldById(fieldToAdd.ID);
    if (fieldTmp == null)
        return false;

    var fieldIds = this.BizKeyFieldsStr.split(';');
    for (var bizFieldIdx = 0; bizFieldIdx < fieldIds.length; bizFieldIdx++) {
        var fieldId = fieldIds[bizFieldIdx];
        if (fieldId == fieldTmp.ID)
            return true;
    }

    fieldTmp.setNullable(false);//主键设置不允许为空.
    if (fieldTmp.Type != BM.Enum.FieldType.ft_String
        && fieldTmp.Type != BM.Enum.FieldType.ft_General
        && fieldTmp.Type != BM.Enum.FieldType.ft_Integer
        && fieldTmp.Type != BM.Enum.FieldType.ft_Guid
        && fieldTmp.Type != BM.Enum.FieldType.ft_Date
        && fieldTmp.Type != BM.Enum.FieldType.ft_DateTime) {

        fieldTmp.setType(BM.Enum.FieldType.ft_General);
        fieldTmp.setSize(501);
        fieldTmp.setFractionDigits(0);
        fieldTmp.setGatherType(BM.Enum.FieldGatherType.fg_Count);
    }
    var newBizFieldsStr = this.BizKeyFieldsStr + ";" + fieldTmp.ID;
    this.setBizKeyFieldsStr(newBizFieldsStr);
    this.BizkeyFields.add(fieldTmp.ID, fieldTmp);
    return true;
};

BM.TableObject.prototype.removeBizFieldById = function (fieldIdToRemove) {
    var fieldTmp = this.getFieldById(fieldIdToRemove);
    if (fieldTmp == null)
        return false;

    var changed = false;
    var fieldIds = this.BizKeyFieldsStr.split(';');
    for (var bizFieldIdx = fieldIds.length - 1; bizFieldIdx >= 0 ; bizFieldIdx--) {
        var fieldId = fieldIds[bizFieldIdx];
        if (fieldId == fieldIdToRemove) {
            fieldIds.splice(bizFieldIdx, 1);
            changed = true;
        }
    }
    if (changed) {
        var newBizFieldsStr = fieldIds.join(';');
        this.setBizKeyFieldsStr(newBizFieldsStr);
    }

    this.BizkeyFields.remove(fieldIdToRemove);
    return changed;
};

BM.TableObject.prototype.isAllSystemFields = function () {
    var fieldIds = this.BizKeyFieldsStr.split(';');
    for (var bizFieldIdx = fieldIds.length - 1; bizFieldIdx >= 0 ; bizFieldIdx--) {
        var fieldId = fieldIds[bizFieldIdx];
        var isSysField = this.isSystemBizField(fieldId);
        if (isSysField == false)
            return false;
    }
    return true;
};

BM.TableObject.prototype.createFloatOrderField = function () {
    var mainFrameModel = $.m("BM_MainFrameModel");
    var formHelper = mainFrameModel.getService(BM_ServiceName_FormHelper);

    //创建顺序号指标
    var orderFieldId = formHelper.JCPGuid("FieldObject-");
    var orderField = {};
    orderField.ID = orderFieldId;
    orderField.IsNew = true;
    orderField.IsDirty = false;
    orderField.IsDeleted = false;

    //Code 由"DK3Z_MTJ46WQ2AF_F7Or" 改成 "DK3Z_MTJ46WQ_F7Order".
    var tmpFieldCode = this.Code;
    if (tmpFieldCode.length > 14)
        tmpFieldCode = tmpFieldCode.substring(0, 14);


    tmpFieldCode = tmpFieldCode + "_" + formHelper.ORDERCODE;

    orderField.Code = tmpFieldCode;
    orderField.Title = formHelper.ORDERTITLE;
    orderField.Type = BM.Enum.FieldType.ft_Integer;
    orderField.Size = 10;
    orderField.OwnerTableID = this.ID;

    orderField.Description = formHelper.ORDERDESC;
    //orderField.FieldName = tmpFieldCode;//tmpTable.Code + formHelper.ORDERCODE;
    //orderField.TableName = tmpTable.Code;

    orderField.FixedSize = false;
    orderField.FractionDigits = 0;
    orderField.Nullable = false;
    orderField.Order = "";
    orderField.ReferField = 0;
    orderField.ShowFormat = "";
    orderField.ValueType = BM.Enum.FieldValueType.fv_BizKeyOrder;

    var orderFieldObj = new BM.FieldObject(orderField);

    this.Fields.add(orderFieldObj.ID, orderFieldObj);

    return orderFieldObj;

    if (this.BizKeyFieldsStr == "")
        this.BizKeyFieldsStr = orderFieldObj.ID;
    else
        this.BizKeyFieldsStr = this.BizKeyFieldsStr + ";" + orderFieldObj.ID;

    this.BizkeyFields.add(orderFieldObj.ID, orderFieldObj);
};