﻿BM.ValidationObject = function (validationObj) {
    BM.UndoBase.call(this);

    if (validationObj == null
        || validationObj == ""
        || $.trim(validationObj) == "") {

        //throw new Error("not a valid Validation String.");
        this.ID = BM.Enum.ValidationType.vt_LengthValidate;
        this.FormulaID = "";
        this.ValidationType = BM.Enum.ValidationType.vt_LengthValidate;
        this.DataValidationType = BM.Enum.DataValidationType.dvt_Between;
        this.NumberOne = "";
        this.NumberTwo = "";
        return this;
    }

    try {
        //var tmpObj = JSON.parse(validationStr);

        this.IsNew = validationObj.IsNew;
        this.IsDirty = validationObj.IsDirty;
        this.IsDeleted = validationObj.IsDeleted;

        this.ID = validationObj.ValidationType;
        this.FormulaID = validationObj.FormulaID;
        this.ValidationType = validationObj.ValidationType;
        this.DataValidationType = validationObj.DataValidationType;
        this.NumberOne = validationObj.NumberOne == null ? 0 : validationObj.NumberOne;
        this.NumberTwo = validationObj.NumberTwo == null ? 0 : validationObj.NumberTwo;
    } catch (e) {
        throw new Error("not a valid Validation String.");
    }
    return this;
};

BM.ValidationObject.prototype = new BM.UndoBase();

BM.ValidationObject.prototype.isDataChanged = function () {
    return this.IsNew === true || this.IsDirty === true;
};

//保存时，重新复制一份数据，以免原有数据状态被破坏.
BM.ValidationObject.prototype.getCopyToSave = function () {
};

BM.ValidationObject.prototype.setFormulaID = function (newFormulaID) {
    if (newFormulaID == null || $.trim(newFormulaID) == "")
        newFormulaID = "";

    if (this.FormulaID != newFormulaID) {
        this.FormulaID = newFormulaID;
        this.markDirty();
        this.beforePropertyChanged("FormulaID", this.FormulaID, newFormulaID);
        this.propertyHasChanged("FormulaID");
    }
    return this;
};

BM.ValidationObject.prototype.setValidationType = function (newValidationType) {
    if (newValidationType == null || $.trim(newValidationType) == "")
        newValidationType = BM.Enum.ValidationType.vt_LengthValidate;

    if (this.ValidationType != newValidationType) {
        this.ValidationType = newValidationType;
        this.markDirty();
        this.beforePropertyChanged("ValidationType", this.ValidationType, newValidationType);
        this.propertyHasChanged("ValidationType");
    }

    if (this.ID != newValidationType) {
        this.ID = newValidationType;
        this.markDirty();
        this.beforePropertyChanged("ID", this.ID, newValidationType);
        this.propertyHasChanged("ID");
    }
    return this;
};

BM.ValidationObject.prototype.setDataValidationType = function (newDataValidationType) {
    if (newDataValidationType == null || $.trim(newDataValidationType) == "")
        newDataValidationType = BM.Enum.DataValidationType.dvt_Between;

    if (this.DataValidationType != newDataValidationType) {
        this.beforePropertyChanged("DataValidationType", this.DataValidationType, newDataValidationType);
        this.DataValidationType = newDataValidationType;
        this.markDirty();
        this.propertyHasChanged("DataValidationType");
    }
    return this;
};

BM.ValidationObject.prototype.setNumberOne = function (newNumberOne) {
    if (newNumberOne == null || $.trim(newNumberOne) == "")
        newNumberOne = 0;

    if (this.NumberOne !== newNumberOne) {
        this.beforePropertyChanged("NumberOne", this.NumberOne, newNumberOne);
        this.NumberOne = newNumberOne;
        this.markDirty();
        this.propertyHasChanged("NumberOne");
    }
    return this;
};

BM.ValidationObject.prototype.setNumberTwo = function (newNumberTwo) {
    if (newNumberTwo == null || $.trim(newNumberTwo) == "")
        newNumberTwo = 0;

    if (this.NumberTwo !== newNumberTwo) {
        this.beforePropertyChanged("NumberTwo", this.NumberTwo, newNumberTwo);
        this.NumberTwo = newNumberTwo;
        this.markDirty();
        this.propertyHasChanged("NumberTwo");
    }
    return this;
};

BM.ValidationObject.prototype.markDirty = function () {
    if (!this.IsNew)
        this.IsDirty = true;

    return this;
};

BM.ValidationObject.prototype.markNew = function () {
    this.IsNew = true;
    this.IsDirty = false;
    this.IsDeleted = false;
    return this;
};

BM.ValidationObject.prototype.markDeleted = function () {
    this.IsNew = false;
    this.IsDirty = false;
    this.IsDeleted = true;
    return this;
};

BM.ValidationObject.prototype.toString = function () {
    return JSON.stringify(this);
};

BM.ValidationObject.prototype.toFormulaString = function (linkUniqueCode) {
    var numberCheck = "IsNumber([" + linkUniqueCode + "])";
    var numberOne = 0;
    var numberTwo = 0;
    switch (this.ValidationType) {
        case BM.Enum.ValidationType.vt_LengthValidate:
            {
                numberOne = parseInt(this.NumberOne, 10);
                numberTwo = parseInt(this.NumberTwo, 10);
            }
            break;
        case BM.Enum.ValidationType.vt_RangeValidate:
            {
                numberOne = parseFloat(this.NumberOne);
                numberTwo = parseFloat(this.NumberTwo);
                
                numberOne = Math.round(numberOne * 10000) / 10000;//取小数位4位.
                numberTwo = Math.round(numberTwo * 10000) / 10000;//取小数位4位.
            }
            break;
    }

    switch (this.ValidationType) {
        case BM.Enum.ValidationType.vt_LengthValidate:
            {
                switch (this.DataValidationType) {
                    case BM.Enum.DataValidationType.dvt_Between:
                        {
                            var patternOne = "Len([" + linkUniqueCode + "])>=" + numberOne;
                            var patternTwo = "Len([" + linkUniqueCode + "])<=" + numberTwo;
                            return "AND(" + patternOne + "," + patternTwo + ")";
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_NotBetween:
                        {
                            var patternOne = "Len([" + linkUniqueCode + "])<" + numberOne;
                            var patternTwo = "Len([" + linkUniqueCode + "])>=" + numberTwo;
                            return "OR(" + patternOne + "," + patternTwo + ")";
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_EqualTo:
                        {
                            return "Len([" + linkUniqueCode + "])=" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_NotEqualTo:
                        {
                            return "Len([" + linkUniqueCode + "])<>" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_GreaterThan:
                        {
                            return "Len([" + linkUniqueCode + "])>" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_LessThan:
                        {
                            return "Len([" + linkUniqueCode + "])<" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_GreaterThanOrEqualTo:
                        {
                            return "Len([" + linkUniqueCode + "])>=" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_LessThanOrEqualTo:
                        {
                            return "Len([" + linkUniqueCode + "])<=" + numberOne;
                        }
                        break;
                }
            }
            break;
        case BM.Enum.ValidationType.vt_RangeValidate:
            {
                switch (this.DataValidationType) {
                    case BM.Enum.DataValidationType.dvt_Between:
                        {
                            var patternOne = "[" + linkUniqueCode + "]>=" + numberOne;
                            var patternTwo = "[" + linkUniqueCode + "]<=" + numberTwo;
                            return "AND(" + numberCheck + "," + patternOne + "," + patternTwo + ")";
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_NotBetween:
                        {
                            var patternOne = "[" + linkUniqueCode + "]<" + numberOne;
                            var patternTwo = "[" + linkUniqueCode + "]>=" + numberTwo;
                            return "AND(" + numberCheck + ",OR(" + patternOne + "," + patternTwo + "))";
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_EqualTo:
                        {
                            return "[" + linkUniqueCode + "]=" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_NotEqualTo:
                        {
                            return "[" + linkUniqueCode + "]<>" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_GreaterThan:
                        {
                            return "[" + linkUniqueCode + "]>" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_LessThan:
                        {
                            return "[" + linkUniqueCode + "]<" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_GreaterThanOrEqualTo:
                        {
                            return "[" + linkUniqueCode + "]>=" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_LessThanOrEqualTo:
                        {
                            return "[" + linkUniqueCode + "]<=" + numberOne;
                        }
                        break;
                }
            }
            break;
    }
    return "";
};

BM.ValidationObject.prototype.toFormulaDescription = function () {
    var numberOne = this.NumberOne;
    var numberTwo = this.NumberTwo;
    switch (this.ValidationType) {
        case BM.Enum.ValidationType.vt_LengthValidate:
            {
                switch (this.DataValidationType) {
                    case BM.Enum.DataValidationType.dvt_Between:
                        {
                            return "文本长度应该在 " + numberOne + " ~ " + numberTwo+" 之间";
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_NotBetween:
                        {
                            return "文本长度应该小于等于 " + numberOne + " 或者大于等于 " + numberTwo;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_EqualTo:
                        {
                            return "文本长度应该等于 " + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_NotEqualTo:
                        {
                            return "文本长度应该不等于 " + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_GreaterThan:
                        {
                            return "文本长度应该大于 " + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_LessThan:
                        {
                            return "文本长度应该小于 " + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_GreaterThanOrEqualTo:
                        {
                            return "文本长度应该大于等于 " + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_LessThanOrEqualTo:
                        {
                            return "文本长度应该小于等于 " + numberOne;
                        }
                        break;
                }
            }
            break;
        case BM.Enum.ValidationType.vt_RangeValidate:
            {
                switch (this.DataValidationType) {
                    case BM.Enum.DataValidationType.dvt_Between:
                        {
                            return "数值范围应该在 " + numberOne + " ~ " + numberTwo+" 之间";
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_NotBetween:
                        {
                            return "数值范围应该小于等于 " + numberOne + " 或者大于等于 " + numberTwo;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_EqualTo:
                        {
                            return "数值应该等于 " + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_NotEqualTo:
                        {
                            return "数值应该不等于：" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_GreaterThan:
                        {
                            return "数值应该大于：" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_LessThan:
                        {
                            return "数值应该小于：" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_GreaterThanOrEqualTo:
                        {
                            return "数值应该大于等于：" + numberOne;
                        }
                        break;
                    case BM.Enum.DataValidationType.dvt_LessThanOrEqualTo:
                        {
                            return "数值应该小于等于：" + numberOne;
                        }
                        break;
                }
            }
            break;
    }
    return "";
};