
$.Class("FormEditor", MvcBase, {
	construct: function () {
		this._super().construct.call(this);
		
		this.DataInput = $.m("DataInput");
		this.GridForm = this.DataInput.GridForm;
	},

	_isAutoFocus: false,
	_isAutoSave: false,
	_errorCells: null,
	_isDataSaving: false,
	
	saveFormData: function (avoidAlert, callback) {
		var self = this;
		var gridForm = self.GridForm;
		var grid = gridForm._grid;
		//保存时单元格未触发cellChanging事件，无法调用审核，假如单元格数据修改的情况下，需要人为调用一次审核
		var isCellChanged = grid.isCellValueChanged();
		if (isCellChanged && (self.DataInput._checkErrorCells == null || self.DataInput._checkErrorCells.length <= 0)) {        	
        	var grid = gridForm._grid;
        	var curCell = grid.getCurrentCell(); 
        	if (curCell){
        		self.DataInput.excuteCheckExpression(curCell.getColIndex(), curCell.getRowIndex());
        	}        	
       	}	
       	//先调用下面的getData()方法会导致gridModel中维护的_cellValueChanged被重置为false，导致无法判断值修改。
       	//因此调整到前面先检查前台审核错误  2016-08-08 lvzhanmeng
        if (self.DataInput._checkErrorCells != null && self.DataInput._checkErrorCells.length > 0) {	
            if (!self._isAutoSave) {
            	//默认定位第一个出错的单元格
                var errorCellInfo = self.DataInput._checkErrorCells[0];
                var cellInfos = errorCellInfo.split("-");
                if (cellInfos.length == 2) {
                    var cellCol = parseInt(cellInfos[0], 10), cellRow = parseInt(cellInfos[1], 10);
                    if (cellCol > 0 && cellRow > 0) {
                        var grid = gridForm._grid;
                        var curCell = grid.getCell(cellCol, cellRow);
                        if (curCell) {
                            var errorMsg = self.DataInput.excuteCheckExpression(curCell.getColIndex(), curCell.getRowIndex());
                            if (errorMsg != null && errorMsg.length > 0) {
                            	mui.toast(errorMsg);                            	
								callback && callback();
					            return;
                            }
                        }
                    }
                }                
            	mui.toast("单元格存在数据校验错误，请修改后再保存。");
            }
			callback && callback();
            return;
        }        		
		
        var formData = gridForm.getData();   
        if (!self._saveCheck(!!formData)) {
            callback && callback();
            return;
        }
        
        formData["LastUpdateTime"] = self.DataInput.Context.LastUpdateTime;
        var formDataFixed = formData.Fixed;
        formData = JSON.stringify(formData);
        var needCalcReload = formulaCache.NeedCalcReload;
        //self.DataInput.Context.IsTempPeriod = self.DataInput.PeriodSelector.isTempPeriod();
	    
        self._action(self._getPath("DataInput/Home/SaveFormData", true), {
        	isFilterSort:false,
        	shareToken: self.DataInput.Context.ShareToken, 
        	taskId: self.DataInput.Context.TaskId, 
        	period: self.DataInput.Context.Period, 
        	entityKey: self.DataInput.Context.EntityKey, 
        	formId: self.DataInput.Context.FormId, 
        	isLeafNode: self.DataInput.Context.IsLeafNode, 
        	jsonData: formData,
        	isTempPeriod: self.DataInput.Context.IsTempPeriod,
        	isMobile: true,
        	needCalcReload: needCalcReload
        }, function(data){
        	try {
        		self.setEditable(true); //后面的resetStatus中调用的表格接口updateAfterStoreData方法会去修改表格单元格的状态，涉及到修改表格了，所以如果表格还是只读状态的话会修改不成功。
                if (data.BoolResult.Result) {
                    if (!avoidAlert && !data.ReloadData) mui.toast(data.BoolResult.Message);

                    self.GridForm.resetStatus();
                    self.GridForm.updateRowAttrs(data.RowKeysMap, data.FloatOrdersMap);
                    
                    self.DataInput.Context.LastUpdateTime = data.LastUpdateTime;
                    if (data.ReloadData) {
                        self.DataInput._loadFormData();
                    }
                    if (needCalcReload && data.FormData) {   //保存后需要同步运算并加载数据
                        self.DataInput.setFormData(data.FormData);
                    }
                    
                    if (data.ReloadData) mui.toast("保存成功，但数据已被其它用户修改，正重新加载……");
                    if (self._isAutoSave && !data.ReloadData) mui.toast("数据已自动保存。");
                    window.sendMessage(MSG_FormEditor_AfterDataSave, null, this);
                    
                    self.DataInput.GridForm.closeEditor();
                    $("#grid-edit-wrapper").hide();
                    $("#grid-view-wrapper").show();
                } else {
                    if (!avoidAlert) mui.alert(data.BoolResult.Message, "系统提示");
                    if (data.ErrorInfo) self._locateCells(data.ErrorInfo.Type, data.ErrorInfo.RowKey, data.ErrorInfo.LinkIds);
                }
//              if (self._getUseFormIsSortFilter()) {
//                  self.loadFormData();
//              }
            } catch (e) {
                mui.alert(e.message, "系统提示");
                console.log(e.stack);
            } finally {
                //self._setSaveInterval();
                self._isDataSaving = false;
                self.setEditable(true);
                callback && callback(data.BoolResult.Result);
            }
        });      
    },

    setEditable: function (canEdit) {
        if (this.DataInput.Status.ReadOnly.Result) return;
        this.GridForm.setCanEdit(canEdit);
    },
    
    emptyFormData: function () {
        var self = this;
        self._action(self._getPath("DataInput/Home/EmptyFormData", true), {
        	shareToken: self.DataInput.Context.ShareToken, 
        	taskId: self.DataInput.Context.TaskId, 
        	period: self.DataInput.Context.Period, 
        	entityKey: self.DataInput.Context.EntityKey, 
        	formId: self.DataInput.Context.FormId,
        	fromMobile: true
        }, function(data){
            if (data.BoolResult.Result && data.BoolResult.InfoType == "success") {
                self.DataInput.refreshMarkedFilledPeriods();
        		self.DataInput.setFormData(data.FormData);
        		self.DataInput.ToolBar.hideBottomBtns();
        		mui.toast("操作成功！");
            } else {
                mui.alert(data.BoolResult.Message, "系统提示");
            }
        });
    },
    
	isReadOnly: function () {
		if (this.DataInput.Status.ReadOnly.Result) return true;
		return false;
//      var entityKey = this.DataInput.Context.EntityKey;
//      var isStopped = this.DataInput.EntitySelector.getStopState(entityKey);
//      return isStopped;
    },
    
    isGathering: function () {//整个表单下的某个区域是否存在汇总
        var regGatherTypes = this.DataInput.Context.RegGatherTypes;
        if (!regGatherTypes) return false;
        
        var isGathering = false;
        $.each(regGatherTypes, function(key, value) {
            if (value !== 0 && !isGathering) isGathering = true;
        });

        return isGathering;
    },
    
    getGatherType: function () {//聚焦单元格所在的区域是哪种汇总类型
        var regGatherTypes = this.DataInput.Context.RegGatherTypes;
        if (!regGatherTypes) return 0;
        var focusCell = this.GridForm.getFocusCell();
        if (!focusCell) return 0;
        var index = focusCell.getRowIndex();
        var regId = this.GridForm.getRowAttr("reg", index);
        return regGatherTypes[regId] ? regGatherTypes[regId] : 0;
    },

    isTotalRow: function(index) {//某个表单如果存在汇总，那么它可以有明细行、合计行，以此做标记
        var state = this.GridForm.getRowAttr("state", index);
        return state === "9";
    },
    
    hasTotalRow: function (selections) {
        if (!selections) selections = this.GridForm.getSelections();
        if (!selections || !selections.length) return false;

        for (var i = 0, count = selections.length; i < count; i++) {
            var area = selections[i];
            var index;
            if (area.selectedRange === "row" || area.selectedRange === "col") {
                var top = area.y, bottom = top + area.height;
                for (index = top; index < bottom; index++) {
                    if (this.isTotalRow(index)) return true;
                }
            }
        }

        return false;
    },
     
    hasUnfloatReg: function (selections) {
        var formKind = this.GridForm.getFormKind();
        if (formKind === "fixed") return true;

        if (!selections) selections = this.GridForm.getSelections();
        if (!selections || selections.length !== 1) return false;

        for (var i = 0, count = selections.length; i < count; i++) {
            var area = selections[i];
            var index, regId, dataReg;
            if (formKind === "col") {
                var left = area.x, right = left + area.width;
                for (index = left; index < right; index++) {
                    regId = this.GridForm.getRowAttr("reg", index);
                    dataReg = this.GridForm.getDataReg(regId);
                    if (!dataReg || dataReg.RegionKind === 0) return true;
                }
            }
            if (formKind === "row") {
                var top = area.y, bottom = top + area.height;
                for (index = top; index < bottom; index++) {
                    regId = this.GridForm.getRowAttr("reg", index);
                    dataReg = this.GridForm.getDataReg(regId);
                    if (!dataReg || dataReg.RegionKind === 0) return true;
                }
            }
        }

        return false;
    },
		
    gridInsert: function (count, isInsertAfter, isColumn) {
        var selections = this.GridForm.getSelections();
        var hasTotalRow = this.hasTotalRow(selections);
        if (hasTotalRow) {
            jNoty("合计行不允许插入。");
            return;
        }

        var hasUnfloatReg = this.hasUnfloatReg(selections);
        if (hasUnfloatReg) {
            jNoty("非可插行（列）区域不允许插入。");
            return;
        }

        this._afterRowOps = true;
        this.GridForm.onInsert(count, isInsertAfter, isColumn);
    },
    
    onValueChange: function (sender, args) {
        if (!this._setDefaultValue(args)) return;
        
        this._autoAppend();
        this.DataInput.Status.IsModified = true;
        window.sendMessage(MSG_DataInput_DataStateChanged, null, this);
    },

    onDataChange: function (sender, ops) {
        if (ops.state === "delete") {
            this.DataInput.Status.IsModified = true;
        }

        window.sendMessage(MSG_DataInput_DataStateChanged, null, this);
    },
	
    onCellFocus: function () {
        var cell = this.GridForm.getFocusCell();
        if (!cell) return;
        //if (!this._afterRowOps && !this._isAutoFocus) this._autoAppend();
        //this._afterRowOps = false;
        this._isAutoFocus = false;
        this._setCurrentCell();

        window.sendMessage(MSG_DataInput_DataStateChanged, null, this);
    },

    onCellReFocus: function () {
        this._initFocusCell();
        this.onCellFocus();
    },

	onFormDataLoaded: function () {
        this._initFocusCell();
        var formKind = this.GridForm.getFormKind();
        this.GridForm.setReturnDir(formKind === "row" ? "right" : "down");

        var isReadOnly = this.isReadOnly();
        this.setEditable(!isReadOnly);

        if (isReadOnly || this.isGathering())
            this.GridForm.setCanInsert(false);

        window.sendMessage(MSG_FormEditor_AfterDataLoad, null, this);
    },
    
    onEditReadOnlyCell: function(){
        var focusCell = this.GridForm.getFocusCell();
        if (!focusCell) return;

        var readOnly = this.DataInput.Status.ReadOnly;
        if (readOnly.Result) {
            mui.toast(readOnly.Message);
            return;
        }

        if (this.isTotalRow()) {
            mui.toast("该行数据为合计数据，不允许编辑。");
            return;
        }

        if (focusCell.attr("acl") === "1" || focusCell.attr("cf") === "1") {
            mui.toast("该单元格由公式计算得出，无需填写。");
            return;
        }

        var calcCols = this._getAutoCalcCols();
        var colIndex = focusCell.getColIndex();
        if (calcCols.contains(colIndex)) {
            mui.toast("该单元格由系统自动计算得出，无需填写。");
            return;
        }

        if (!focusCell.getCanEdit()) {
            mui.toast("该单元格被设置为只读，不允许编辑。");
            return;
        }

        if (this._isGatherField(focusCell)) {
            mui.toast("当前正根据该列进行汇总，不允许编辑。");
            return;
        }

        var dataReg = this.GridForm.getDataReg(), regKind = dataReg ? dataReg.RegionKind : 0;
        if (regKind === 0) return;

        var isGathering = this.getGatherType() !== 0;
        if (isGathering) {
            //var entityshowtext = $("#datainput").attr("entityshowtext");
            //mui.toast("该" + entityshowtext + "的数据是从下级汇总的，不允许编辑。");
            mui.toast("从下级自动汇总，不允许编辑。");
            return;
        }
    },
	
	setFocusCell: function (cell) {
	    if (!cell) return;
	    this._isAutoFocus = true;
	    this.GridForm.setFocus(cell);
	},
	
	_action: function(path, _data, callback, async, onError, headers) {
		$.doAction(path, _data, callback, async, onError, headers);
	},

	_getPath: function(action, withTaskId) {
		var path = $.BaseUrl + action;
		if(withTaskId){
			path += "?_taskId_=" + this.DataInput.Context.TaskId;
		}
		return path;
	},
	
	_initFocusCell: function () {
	    var focusCell = this._getCurrentCell();
	    
    	if(this.DataInput.Context && this.DataInput.Context.LocateRowKeys && this.DataInput.Context.LocateLinkIds){
    		if(focusCell){
    			var focusRowKey = this.GridForm.getRowKey(focusCell);
    			if(focusRowKey !== this.DataInput.Context.LocateRowKeys){
    				focusCell = null;
    			}
    		}
    		if(!focusCell){
    			var cells = this.GridForm.queryCells(this.DataInput.Context.LocateRowKeys, this.DataInput.Context.LocateLinkIds);
    		    focusCell = (cells && cells.length > 0) ? cells[0] : null;
    		}
    		this.DataInput.Context.LocateRowKeys = null;
    		this.DataInput.Context.LocateLinkIds = null;
    	}
    	
	    if (!focusCell) focusCell = this.GridForm.getFirstCell();
	    this.setFocusCell(focusCell);
	},
	
	_setCurrentCell: function (cell) {
		if(!mui.os.plus)
		   return;
	    if (!cell) cell = this.GridForm.getFocusCell();
	    if (!cell) return;
	    if(!this.DataInput.Context || !this.DataInput.Context.UserId || this.DataInput.Context.TaskId <= 0) return;
	    
	    var tempKey = this.DataInput.Context.UserId + "_" + this.DataInput.Context.TaskId;
	    var rowIndex = cell.getRowIndex(), colIndex = cell.getColIndex();
	    var tempValue = {};
	    tempValue[this.DataInput.Context.FormId] = { row: rowIndex, col: colIndex };
	    if (localStorage.getItem(tempKey)) {
	        var oldValue = JSON.parse(localStorage.getItem(tempKey));
	        oldValue = $.extend(oldValue, tempValue);
	        localStorage.setItem(tempKey, JSON.stringify(oldValue));
	    } else {
	        localStorage.setItem(tempKey, JSON.stringify(tempValue));
	    }
	},
	
	_getCurrentCell: function () {
		if(!mui.os.plus)
		   return null;
		if(!this.DataInput.Context || !this.DataInput.Context.UserId || this.DataInput.Context.TaskId <= 0) return null;
	    var tempKey = this.DataInput.Context.UserId + "_" + this.DataInput.Context.TaskId;
	    if (localStorage.getItem(tempKey)) {//需判断当前表单对应的单元格是否有存储
	        var formId = this.DataInput.Context.FormId;
	        var oldValue = JSON.parse(localStorage.getItem(tempKey));
	        if (oldValue[formId]) {
	            var rowIndex = oldValue[formId].row, colIndex = oldValue[formId].col;
	            var rowCount = this.GridForm.getRowCount(), colCount = this.GridForm.getColCount();
	            if (rowIndex >= rowCount || colIndex >= colCount) return null;
	            return this.GridForm.getCell(colIndex, rowIndex);
	        }
	    }
	    return null;
	},
	
	_getColumnTitle: function (colIndex) {
	    var ABCStr = new Array(
            'Z', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L',
            'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'
        );
	    var colTitle = "";

	    if (colIndex < 0) {
	        return "NEG";
	    }

	    if (colIndex <= 26) {
	        colTitle = ABCStr[colIndex];
	    } else {
	        var SS = colIndex;
	        var YS = -1;
	        while (SS > 26) {
	            if (YS == 0) {
	                YS = SS % 26;
	                YS = YS - 1;
	            } else {
	                YS = SS % 26;
	            }
	            SS = Math.floor(SS / 26);
	            colTitle = ABCStr[YS] + colTitle;
	        }
	        if (YS == 0)
	            SS = SS - 1;
	        if (SS != 0)
	            colTitle = ABCStr[SS] + colTitle;
	    }

	    return colTitle;
	},
	
	_saveCheck: function (hasDataChanged) {
        if (!hasDataChanged) {
            this.DataInput.Status.IsModified = false;
            if (!this._isAutoSave) mui.toast("所有修改的数据都已保存！")
            return false;
        }

        if (this.isReadOnly())
            return false;

        if (!this.DataInput.Status.IsModified) {
            if (!this._isAutoSave) mui.toast("所有修改的数据都已保存！")
            return false;
        }
        
        if (this._isDataSaving) {
        	mui.toast("正在保存数据……")
            return false;
        }

        this._restoreCells();
        this._isDataSaving = true;
        this.setEditable(false);
        mui.toast("正在保存数据……")
        return true;
    },
	
	_setDefaultValue: function (args) {
        if (!args || !args.changeValues) return false;
        return true;

		//小伙伴名称重构后不再调用  lvzhanmeng 2016-08-16
        //var regExtAttrs = this._getRegExtAttrs();
        //if (!regExtAttrs || regExtAttrs.EntityNameIndex <= 0) return true;

        //var hasSetValue = false;
        //var cells = args.changeValues;
        //for (var i = 0, len = cells.length; i < len; i++) {
        //    var row = cells[i].row, col = cells[i].col, oldValue = cells[i].oldValue, newValue = cells[i].newValue;
        //    if (regExtAttrs.EntityNameIndex === col) {
        //        hasSetValue |= false;
        //        continue;
        //    }

        //    hasSetValue |= true;
        //    var cell = this.GridForm.getCell(regExtAttrs.EntityNameIndex, row);
        //    if (!cell) continue;

        //    if (!oldValue && newValue) {
        //        if (cell.getEditText()) continue;
        //        var entity = this.DataInput.EntitySelector.getCurEntity();
        //        cell.setValue(entity.Title, entity.Title, true);
        //    }

        //    if (oldValue && !newValue) {
        //        var isEmptyRow = this._isEmptyRow(row, [col]);
        //        if (isEmptyRow) cell.setValue("", "", true);
        //    }
        //}

        //return hasSetValue;
    },
    
    _getRegExtAttrs: function () {
        var dataReg = this.GridForm.getDataReg();
        if (!dataReg) return null;

        var regExtAttrs;
        try {
            regExtAttrs = $.parseJSON(dataReg.RegionExtension);
        } catch (e) {
            return null;
        }

        return regExtAttrs;
    },
    
    _getAutoCalcCols: function () {
        var excludeCol = [];

        var regExtAttrs = this._getRegExtAttrs();
        var orderIndex = regExtAttrs ? regExtAttrs.OrderIndex : 0;
        if (orderIndex > 0) excludeCol.push(orderIndex);

        var entityNameIndex = regExtAttrs ? regExtAttrs.EntityNameIndex : 0;
        if (entityNameIndex > 0) excludeCol.push(entityNameIndex);

        return excludeCol;
    },
     
    _locateCells: function (type, rowKey, linkIds) {
        if (type === 'row' && !rowKey) return;
        if (type === 'cell' && (!linkIds || !linkIds.length)) return;

        var cells = this._errorCells = this.GridForm.queryCells(rowKey, linkIds);
        if (!cells || !cells.length) return;
        for (var i = 0, count = cells.length; i < count; i++) {
            cells[i].setBackColor("#FF7871");
        }
    },
	
    _restoreCells: function () {
        var cells = this._errorCells;
        if (!cells || !cells.length) return;
        for (var i = 0, count = cells.length; i < count; i++) {
            cells[i].setBackColor();
        }
    },
   
    _autoAppend: function () {
        var isReadOnly = this.isReadOnly();
        if (isReadOnly || this.isGathering()) return;

        var selections = this.GridForm.getSelections();
        if (!selections || selections.length !== 1) return;
        var rangeType = selections[0].selectedRange;
        if (rangeType !== "cell") return;

        var focusCell = this.GridForm.getFocusCell();
        var dataReg = this.GridForm.getDataReg();
        var isFloatReg = dataReg && (dataReg.RegionKind === 2 || dataReg.RegionKind === 3);
        if (focusCell && isFloatReg) {
            var isColumn = dataReg.RegionKind === 2;
            var editIndex = isColumn ? focusCell.getColIndex() : focusCell.getRowIndex();
            var lastIndex = (isColumn ? dataReg.RegionLeft : dataReg.RegionTop) - 1 + dataReg.FloatDataCount;
            if (editIndex === lastIndex && !this._isEmptyRow(editIndex)) this.gridInsert(1, true, isColumn);
        }

        //不明白为何要增加图标  
        //this._updateIconToCell(this);
    },
           //更新附件上传url
    onUpdateAttachedFileUrlParams: function (sender, eventArgs) {
        var grid = this.DataInput.getGrid();
        var currentCell = grid.getCurrentCell();
        var fkey = grid.getDataRowAttrByName(RO.Enum.FloatRowKey, currentCell.getColIndex(), currentCell.getRowIndex());
        var curEntityKey = this.DataInput.Context.EntityKey;
        var urlParams = eventArgs.urlParams;
        try {
            var rowKeys = fkey ? eval('(' + eventArgs.downloadArgs.fkey + ')') : {};
            var entityKey = rowKeys && rowKeys.ENTITYKEY ? rowKeys.ENTITYKEY : curEntityKey;
            if (urlParams != null && urlParams.length > 0 && urlParams.indexOf(entityKey) < 0 && urlParams.indexOf(curEntityKey) >= 0 && entityKey != curEntityKey) {
                urlParams = urlParams.replace(curEntityKey, entityKey);
            }
        } catch (e) {
            //ignore 默认保留行或者新增的行fkey无法转换为json
            var rowEntityKeyset = this.DataInput.Context.RowEntityKeyset;
            if (rowEntityKeyset != null && rowEntityKeyset[fkey] != null && rowEntityKeyset[fkey].length > 0) {
                entityKey = rowEntityKeyset[fkey];
                if (urlParams != null && urlParams.length > 0 && urlParams.indexOf(entityKey) < 0 && urlParams.indexOf(curEntityKey) >= 0 && entityKey != curEntityKey) {
                    urlParams = urlParams.replace(curEntityKey, entityKey);
                }
            }
        }
        if (urlParams != null && urlParams.length > 0) {
            var taskIdIndex = urlParams.indexOf("_taskId_");
            if (taskIdIndex < 0) {
                if (urlParams.indexOf("?") >= 0) {
                    urlParams = urlParams + "&_taskId_=" + this.DataInput.Context.TaskId;
                } else {
                    urlParams = urlParams + "?_taskId_=" + this.DataInput.Context.TaskId;
                }
            }
        }
        eventArgs.urlParams = urlParams;
    },
    _isGatherField: function (cell) {
        if (!cell) cell = this.GridForm.getFocusCell();
        var linkId = cell.attr("lid");
        if (!linkId) return false;

        var dataReg = this.GridForm.getDataReg();
        if (!dataReg) return false;
        
        var regTotalItem = this.DataInput.Context.RegTotalItems && this.DataInput.Context.RegTotalItems[dataReg.RegionId];
        if (!regTotalItem) return false;

        var orderLinks = regTotalItem.OrderedLinks;
        return orderLinks && orderLinks.contains(linkId);
    },
    
    _isEmptyRow: function (rowIndex, excludeCol) {
        var cells = this.GridForm.getRowCells(rowIndex);
        if (!cells) return false;

        if (!excludeCol) excludeCol = [];
        excludeCol = $.merge(excludeCol, this._getAutoCalcCols());

        var isEmpty = true;
        for (var i = 0, len = cells.length; i < len; i++) {
            var cell = cells[i], colIndex = cell.getColIndex();
            if (colIndex <= 0 || excludeCol.contains(colIndex))
                continue;
            if (cell.attr("acl") === "1" || cell.attr("cf") === "1")
                continue;
            if (cell.getEditText()) {
                isEmpty = false;
                break;
            }
        }

        return isEmpty;
    }
    
	
});
