﻿//#region 公式执行器基类

function extend(subClass, superClass) {
    var F = function () { };
    F.prototype = superClass.prototype;
    subClass.prototype = new F();
    subClass.prototype.constructor = subClass;
    subClass.superclass = superClass.prototype; //加多了个属性指向父类本身以便调用父类函数
    if (superClass.prototype.constructor === Object.prototype.constructor) {
        superClass.prototype.constructor = superClass;
    }
}

function DynamicNodeFactory() {

};
DynamicNodeFactory.GetFieldDataMethod = "GetFieldData";
DynamicNodeFactory.SetFieldDataMethod = "SetFieldData";
DynamicNodeFactory.SetEnumTitleMethod = "SetEnumTitle";
DynamicNodeFactory.GetCellByDataLinkWithoutFkeyMethod = "GetCellByDataLinkWithoutFkey";
DynamicNodeFactory.GetStatisticCellsMethod = "GetStatisticCells";
DynamicNodeFactory.GetUnitTitleMethod = "GetUnitTitle";
DynamicNodeFactory.GetValuesByLidsMethod = "GetValuesByLids";
DynamicNodeFactory.methodJson = new Array();
DynamicNodeFactory.registerMethod = function (name, method) {
    delete this.methodJson[name];
    this.methodJson[name] = method;
};
DynamicNodeFactory.getMethod = function (name) {
    if (this.methodJson[name] != null) {
        return this.methodJson[name];
    }
    return null;
};

function AbstractData(value, dataType, isNull) {
    this.Value = value;
    this.dataType = dataType;
    this.isNull = isNull;
}

function ErrorData() {
    AbstractData.call(this);
    this.Value = "";
    this.dataType = FMR.ConstDataTypes.Error;
}

// Node （类似于公式解析中的Node）
function Node() {
    this.ChildCount = 0;
}
Node.prototype.getChild = function () {
    return null;
};
Node.prototype.getResult = function () {
    return new ErrorData();
};
Node.prototype.getResultType = function () {
    return FMR.ConstDataTypes.Error;
};
Node.prototype.getShowString = function () {
    throw "Not Supported Exception";
};
Node.prototype.isPrevLinkChange = function() {
    return false;
}

// VoidNode（空Node）
function VoidNode() {
    Node.call(this);
}
extend(VoidNode, Node);
VoidNode.prototype.getChild = function () {
    return null;
};
VoidNode.prototype.getResult = function () {
    return new AbstractData("", FMR.ConstDataTypes.Void, false);
};
VoidNode.prototype.getResultType = function () {
    return FMR.ConstDataTypes.Void;
};
VoidNode.prototype.getShowString = function() {
    return "";
};
VoidNode.prototype.isPrevLinkChange = function () {
    return false;
}

// FormulaFunction （公式函数基类）
function FormulaFunction(name, title, group) {
    this.Name = name;
    this.Title = title;
    this.Group = group;
}
FormulaFunction.prototype.callFunction = function () {
    return new ErrorData();
};
FormulaFunction.prototype.judgeResultType = function () {
    return FMR.ConstDataTypes.Error;
};
FormulaFunction.prototype.getShowString = function () {
    return this.Name.toUpperCase();
};
FormulaFunction.prototype.isPrevLinkChange = function (dataLink, nodes) {
    return false;
}
FormulaFunction.prototype.isGetPrevData = function () {
    return false;
}


// FunctionNode（函数Node）
function FunctionNode(func, parameters) {
    Node.call(this);
    this.Func = func;
    this.Parameters = parameters;
}
extend(FunctionNode, Node);
FunctionNode.prototype = new Node();
FunctionNode.prototype.SetParamter = function (nodes) {
    this.Parameters = nodes;
};
FunctionNode.prototype.getResult = function (fKey, dataLink, newValue) {
    return this.Func.callFunction(fKey, dataLink, newValue, this.Parameters);
};
FunctionNode.prototype.getResultType = function () {
    return this.Func.judgeResultType(this.Parameters);
};
FunctionNode.prototype.getShowString = function (fKey) {
    var showString = "";
    showString += this.Func.getShowString();
    var getPrevData = this.Func.isGetPrevData();
    showString += "(";
    for (var index = 0; index < this.Parameters.length; index++) {
        var paramNode = this.Parameters[index];
        if (paramNode instanceof VoidNode) {
            continue;
        }
        if (index > 0)
            showString += ",";
        showString += paramNode.getShowString(fKey, getPrevData);
    }
    showString += ")";
    return showString;
};
FunctionNode.prototype.isPrevLinkChange = function (dataLink) {
    var isPrevLink = this.Func.isPrevLinkChange(dataLink, this.Parameters);
    for (var index = 0; index < this.Parameters.length; index++) {
        var paramNode = this.Parameters[index];
        isPrevLink = isPrevLink || paramNode.isPrevLinkChange(dataLink);
    }
    return isPrevLink;
}

// StaticDataNode（静态常量Node）
function StaticDataNode(value, valueType, isNull) {
    Node.call(this);
    this.StaticValue = value;
    this.StaticValueType = valueType;
    this.IsNull = isNull;
}
extend(StaticDataNode, Node);
StaticDataNode.prototype.getResult = function () {
    return new AbstractData(this.StaticValue, this.StaticValueType, this.IsNull);
};
StaticDataNode.prototype.getResultType = function () {
    return this.StaticValueType;
};
StaticDataNode.prototype.getShowString = function () {
    if (this.StaticValueType === FMR.ConstDataTypes.String) {
        return "\"" + this.StaticValue + "\"";
    }
    if (this.StaticValueType === FMR.ConstDataTypes.Bool) {
        if (this.StaticValue == "0")
            return "否";
        else if (this.StaticValue == "1")
            return "是";
    }
    return this.StaticValue;
};
StaticDataNode.prototype.isPrevLinkChange = function () {
    return false;
}

// BracketNode（括号Node）
function BracketNode(mainNode) {
    Node.call(this);
    this.MainNode = mainNode;
}
extend(BracketNode, Node);
BracketNode.prototype.getResult = function (fKey, dataLink, newValue) {
    return this.MainNode.getResult(fKey, dataLink, newValue);
};
BracketNode.prototype.getResultType = function () {
    return this.MainNode.getResultType();
};
BracketNode.prototype.getShowString = function (fKey) {
    return "(" + this.MainNode.getShowString(fKey) + ")";
};
BracketNode.prototype.isPrevLinkChange = function (dataLink) {
    return this.MainNode.isPrevLinkChange(dataLink);
}

// Operate（操作符基类）
function Operator(sign, title) {
    this.Sign = sign;
    this.Title = title;
}
Operator.prototype.isAssign = function () {
    return false;
};
Operator.prototype.getResult = function () {
    return new ErrorData();
};
Operator.prototype.getResultType = function () {
    return FMR.ConstDataTypes.Error;
};
Operator.prototype.getShowString = function () {
    return this.Sign;
};
Operator.prototype.isPrevLinkChange = function () {
    return false;
}

// CompareOperator（比较操作符基类）
function CompareOperator() {

}
extend(CompareOperator, Operator);
CompareOperator.prototype.getResult = function () {
    return new ErrorData();
};
CompareOperator.prototype.getResultType = function (data1Type, data2Type) {
    switch (data1Type) {
        case FMR.ConstDataTypes.Bool:
            switch (data2Type) {
                case FMR.ConstDataTypes.Bool:
                case FMR.ConstDataTypes.String:
                case FMR.ConstDataTypes.Int:
                case FMR.ConstDataTypes.Float:
                case FMR.ConstDataTypes.Currency:
                case FMR.ConstDataTypes.General:
                    return FMR.ConstDataTypes.Bool;
                default:
                    return FMR.ConstDataTypes.Error;
            }
        case FMR.ConstDataTypes.Int:
        case FMR.ConstDataTypes.Float:
        case FMR.ConstDataTypes.Currency:
            switch (data2Type) {
                case FMR.ConstDataTypes.Int:
                case FMR.ConstDataTypes.Float:
                case FMR.ConstDataTypes.Currency:
                case FMR.ConstDataTypes.Bool:
                case FMR.ConstDataTypes.String:
                case FMR.ConstDataTypes.General:
                    return FMR.ConstDataTypes.Bool;
                default:
                    return FMR.ConstDataTypes.Error;
            }
        case FMR.ConstDataTypes.Date:
        case FMR.ConstDataTypes.DateTime:
            switch (data2Type) {
                case FMR.ConstDataTypes.Date:
                case FMR.ConstDataTypes.DateTime:
                case FMR.ConstDataTypes.General:
                    return FMR.ConstDataTypes.Bool;
                default:
                    return FMR.ConstDataTypes.Error;
            }
        case FMR.ConstDataTypes.String:
            switch (data2Type) {
                case FMR.ConstDataTypes.String:
                case FMR.ConstDataTypes.Bool:
                case FMR.ConstDataTypes.Int:
                case FMR.ConstDataTypes.Float:
                case FMR.ConstDataTypes.Currency:
                case FMR.ConstDataTypes.General:
                    return FMR.ConstDataTypes.Bool;
                default:
                    return FMR.ConstDataTypes.Error;
            }
        case FMR.ConstDataTypes.General:
            switch (data2Type) {
                case FMR.ConstDataTypes.String:
                case FMR.ConstDataTypes.Bool:
                case FMR.ConstDataTypes.Int:
                case FMR.ConstDataTypes.Float:
                case FMR.ConstDataTypes.Currency:
                case FMR.ConstDataTypes.Date:
                case FMR.ConstDataTypes.DateTime:
                case FMR.ConstDataTypes.General:
                    return FMR.ConstDataTypes.Bool;
                default:
                    return FMR.ConstDataTypes.Error;
            }
        default:
            return FMR.ConstDataTypes.Error;
    }
};

// OperatorNode （操作符Node）
function OperatorNode(oper, operNode1, operNode2) {
    Node.call(this);
    this.oper = oper;
    this.operNode1 = operNode1;
    this.operNode2 = operNode2;
}
extend(OperatorNode, Node);
OperatorNode.prototype.getResult = function (fKey, dataLink, newValue) {
    if (this.oper.isAssign()) {
        var assignData = this.operNode2.getResult(fKey, dataLink, newValue);
        if (assignData.dataType == FMR.ConstDataTypes.Void)
            return true;
        return this.operNode1.assignData(fKey, dataLink, newValue, assignData);
    }
    else {
        return this.oper.getResult(this.operNode1.getResult(fKey, dataLink, newValue), this.operNode2.getResult(fKey, dataLink, newValue));
    }
};
OperatorNode.prototype.getResultType = function () {
    return this.oper.getResultType(this.operNode1.getResultType(), this.operNode2.getResultType());
};
OperatorNode.prototype.getShowString = function (fKey) {
    var showString = "";
    showString += this.operNode1.getShowString(fKey);
    showString += this.oper.getShowString();
    showString += this.operNode2.getShowString(fKey);
    return showString;
};
OperatorNode.prototype.isPrevLinkChange = function (dataLink) {
    var isPrevLink = this.operNode1.isPrevLinkChange(dataLink);
    isPrevLink = isPrevLink || this.oper.isPrevLinkChange();
    isPrevLink = isPrevLink || this.operNode2.isPrevLinkChange(dataLink);
    return isPrevLink;
}

// DynamicDataNode（动态数据Node基类）
function DynamicDataNode() {
    Node.call(this);
}
extend(DynamicDataNode, Node);
DynamicDataNode.prototype.assignData = function () {
    return new ErrorData();
};
DynamicDataNode.prototype.getResult = function () {
    return new ErrorData();
};
DynamicDataNode.prototype.getResultType = function () {
    return FMR.ConstDataTypes.Error;
};
DynamicDataNode.prototype.getShowString = function () {
    throw "Not Supported Exception";
};
DynamicDataNode.prototype.isPrevLinkChange = function () {
    return false;
}

// VariableDataNode（变量Node）
function VariableDataNode(varType, varValue) {
    DynamicDataNode.call(this);
    this.VarValue = varValue;
    this.DataType = varType;
}
extend(VariableDataNode, DynamicDataNode);
VariableDataNode.prototype.assignData = function (fKey, dataLink, newValue, setData) {
    if (setData == null || setData.Value == null) {
        return new ErrorData();
    }
    this.VarValue = setData.Value;
    return new AbstractData("", FMR.ConstDataTypes.Void, false);
};
VariableDataNode.prototype.getResult = function () {
    return new AbstractData(this.VarValue, this.DataType, false);
};
VariableDataNode.prototype.getResultType = function () {
    return this.DataType;
};
VariableDataNode.prototype.getShowString = function () {
    throw "Not Supported Exception";
};
DynamicDataNode.prototype.isPrevLinkChange = function () {
    return false;
}

// Expression（类似于后台的Expression类，用于执行公式）
function Expression(mainNode, formulaId, formulaText, description) {
    Node.call(this);
    this.MainNode = mainNode;
    //便于分析问题使用，假如FormulaId和FormulaText为null，可以考虑重新发布试试。假如还是null的话，应该不是FormulaDefine解析的公式
    this.FormulaId = formulaId;   
    this.FormulaText = formulaText;
    this.Description = description;
}
extend(Expression, Node);
Expression.prototype.getResultType = function () {
    return this.MainNode.getResultType();
};
Expression.prototype.getResult = function (fKey, dataLink, newValue) {
    return this.MainNode.getResult(fKey, dataLink, newValue);
};
Expression.prototype.getShowString = function (fKey) {
    try {
        var expString = this.MainNode.getShowString(fKey);
        //界面展现由html()方法改为text()方法，。因此不需要再对特殊字符进行转义
        //expString = expString.replace(/&/g, "&amp;");//替换字符串s中的&为&amp; 
        //expString = expString.replace(/</g, "&lt;");//替换字符串中的<为&lt;
        //expString = expString.replace(/>/g, "&gt;");//替换字符串中的>为&gt; 
        if (expString.indexOf("=") >= 0) {
            return expString.substring(expString.indexOf("="));
        } else
            return expString;
    } catch (e) {
        return "";
    } 
};
Expression.prototype.isPrevLinkChange = function(dataLink) {
    try {
        var isPrevLink = this.MainNode.isPrevLinkChange(dataLink);
        return isPrevLink;
    } catch (e) {
        return false;
    }
}

//#endregion

//#region 公共枚举及方法

FMR = {};
FMR.ConstDataTypes = {
    Void : 0,
    Float : 1,
    String : 2,
    Int : 3,
    Bool : 4,
    Date : 5,
    DateTime: 6,
    General: 8,
    Currency : 14,
    Object : 32,
    Error : 100,
    AnyType : 127,
    Max : 128,
    Object_Unknown : 32,
    Object_Guid : 33,
    Object_Text : 34,
    Object_Picture : 35,
    Object_Attachment : 36,
    Object_TimeStamp : 37,
    Object_Array : 64,
    Object_List : 65
};
FMR.getValueByType = function (value, dataType, isNull, toType) {
    var convertType = dataType;
    if (dataType === FMR.ConstDataTypes.General) {
        convertType = toType;
    }
    switch (convertType) {
        case FMR.ConstDataTypes.String:
        case FMR.ConstDataTypes.Date:
        case FMR.ConstDataTypes.DateTime:
            return "" + value;
        case FMR.ConstDataTypes.Int:
            if (isNull)
                return 0;
            if (value == null || value === "")
                return NaN;
            var localIntValue = Number(value);
            if (isNaN(localIntValue))
                return NaN;
            return parseInt(value, 10);
        case FMR.ConstDataTypes.Float:
        case FMR.ConstDataTypes.Currency:
            if (isNull)
                return 0;
            if (value == null || value === "")
                return NaN;
            var localFloatValue = Number(value);
            if (isNaN(localFloatValue))
                return NaN;
            return parseFloat(value);
        case FMR.ConstDataTypes.Bool:
            if (value == null || value === "")
                return null;
            value = String(value);
            if (value === "1") 
                return true;
            else if (value === "0")
                return false;
            return null;
        default:
    }
    return value;
};
FMR.CheckNumberValid = function (value) {
    if (value == null)
        return value;
    var strData = "" + value;
    if (strData.indexOf(",") > 0) {
        var intStr, dotStr = "";
        if (strData.indexOf(".") > 0) {
            intStr = strData.substr(0, strData.indexOf("."));
            dotStr = strData.substr(strData.indexOf(".") + 1);
        } else {
            intStr = strData;
        }
        if (dotStr.indexOf(",") > 0) {
            return null;
        }
        if (intStr != null && intStr.indexOf(",") > 0) {
            var thsIndex = 4, intLen = intStr.length;
            if (thsIndex >= intLen)
                return null;
            while (thsIndex < intLen) {
                if (intStr.substr(intLen - thsIndex, 1) !== ",") {
                    return null;
                }
                thsIndex = thsIndex + 4;
            }
            strData = strData.replace(new RegExp(/,/g), '');
        }
    }
    return strData;
};

FMR.getStringByType = function (value, dataType) {
    switch (dataType) {
        case FMR.ConstDataTypes.Int:
            if (value == null || value === "")
                return "";
            var intData = parseInt(value, 10);
            if (isNaN(intData))
                return "";
            return "" + intData;
        case FMR.ConstDataTypes.Float:
        case FMR.ConstDataTypes.Currency:
            if (value == null || value === "")
                return "";
            var floatData = parseFloat(value);
            if (isNaN(floatData))
                return "";
            return "" + floatData;
        case FMR.ConstDataTypes.String:
        case FMR.ConstDataTypes.Date:
        case FMR.ConstDataTypes.DateTime:
            return "" + value;
        case FMR.ConstDataTypes.Bool:
            if (value == null || value === "")
                return "";
            value = String(value);
            if (value === "1") {
                return "1";
            }
            return "0";
        case FMR.ConstDataTypes.General:
            return "" + value;
        default:
    }
    return value;
};

FMR.StatEnum = {
    STAT_KIND_AVG: 0x03, // 统计方式：平均
    STAT_KIND_COUNT: 0x02, // 统计方式：计数
    STAT_KIND_MAX: 0x04, // 统计方式：最大值
    STAT_KIND_MIN: 0x05, // 统计方式：最小值
    STAT_KIND_NONE: 0x00, // 统计方式：无
    STAT_KIND_SUM: 0x01, // 统计方式：求和
    STAT_KIND_LIST: 0x08 //统计方式：罗列
}

FMR.StatisticsEnum = {
    Sum: 0,
    Avg: 1,
    Count: 2,
    Max: 3,
    Min: 4,
    Median: 5,
    Mode: 6
}

FMR.getShowCellStringByInterface = function (getCellMethod, fKey, dataLinkId, isLastRow, formTitle, position, getPrevData) {
    if (getCellMethod == null)
        throw "GetCellByDataLinkWithoutFkey Method isn't registered.";
    var cell = getCellMethod(fKey, dataLinkId, isLastRow, getPrevData);
    if (cell == null)
        throw "Can't find cell by dataLinkId.";
    var rowIndex = cell.getRowIndex();
    var colIndex = cell.getColIndex();
    return FMR.getShowCellString(colIndex, rowIndex, position == null ? 0 : position.Options, formTitle);
}

FMR.hasSpecialLetter = function (formTitle) {
    var reg = /^[A-Za-z0-9_\u4E00-\u9FA5]+$/;
    if (reg.test(formTitle))
        return false;
    return true;
};

FMR.getShowCellString = function (colIndex, rowIndex, options, formTitle) {
    var cellString = "";
    if (formTitle != null && formTitle.length > 0) {
        if (FMR.hasSpecialLetter(formTitle)) {
            formTitle = "'" + formTitle + "'";
        }
        cellString += formTitle;
        cellString += "!";
    }
    if (options === 1 || options === 3) {
        cellString += "$";
    }
    cellString += FMR.getColumnTitle(colIndex);
    if (options === 2 || options === 3) {
        cellString += "$";
    }
    cellString += rowIndex;
    return cellString;
};

FMR.getColumnTitle = function (colIndex) {
    var ABCStr = new Array(
        'Z', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L',
        'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'
    );
    var colTitle = "";

    if (colIndex < 0) {
        return "NEG";
    }

    if (colIndex <= 26) {
        colTitle = ABCStr[colIndex];
    } else {
        var SS = colIndex;
        var YS = -1;
        while (SS > 26) {
            if (YS === 0) {
                YS = SS % 26;
                YS = YS - 1;
            } else {
                YS = SS % 26;
            }
            SS = Math.floor(SS / 26);
            colTitle = ABCStr[YS] + colTitle;
        }
        if (YS === 0)
            SS = SS - 1;
        if (SS !== 0)
            colTitle = ABCStr[SS] + colTitle;
    }

    return colTitle;
};

FMR.getStatisticsData = function (fKey, dataLink, newValue, nodes) {
    var addDataToList = function (arrayData, valueData) {
        if (valueData == null || valueData.Value === "" || valueData.isNull)
            return;
        var curNum;
        if (valueData.dataType === FMR.ConstDataTypes.Int || valueData.dataType === FMR.ConstDataTypes.Float || valueData.dataType === FMR.ConstDataTypes.Currency) {
            curNum = FMR.getValueByType(valueData.Value, valueData.dataType, valueData.isNull);
            arrayData.push(curNum);
        }
        else if (valueData.dataType === FMR.ConstDataTypes.Bool) {
            curNum = String(valueData.Value);
            if (curNum === "1")
                arrayData.push(1);
            else 
                arrayData.push(0);
        }
        else if (valueData.dataType === FMR.ConstDataTypes.General) {
            curNum = FMR.getValueByType(valueData.Value, valueData.dataType, valueData.isNull, FMR.ConstDataTypes.Float);
            if (isNaN(curNum))
                return;
            arrayData.push(curNum);
        }
        else if (valueData.dataType === FMR.ConstDataTypes.String) {
            var stringValue = valueData.Value;
            var regex = new RegExp("^-?\\d+$");
            if (regex.test(stringValue)) {
                curNum = Number(valueData.Value);
                if (isNaN(curNum))
                    return;
                arrayData.push(curNum);
            } else {
                var floatRegex = new RegExp("^(-?\\d+)(\\.\\d+)?$");
                if (floatRegex.test(stringValue)) {
                    curNum = Number(valueData.Value);
                    if (isNaN(curNum))
                        return;
                    arrayData.push(curNum);
                }
            }
        }
    };
    var listDatas = new Array();
    var listIndex, curValue;
    for (var index = 0; index < nodes.length; index++) {
        var node = nodes[index];
        var curData = node.getResult(fKey, dataLink, newValue);
        if (curData == null || curData.dataType === FMR.ConstDataTypes.Error)
            return null;
        addDataToList(listDatas, curData);
        if (curData.dataType === FMR.ConstDataTypes.Object_Array) {
            var regionValues = curData.Value;
            for (var row = 0; row < regionValues.length; row++) {
                var rowValues = regionValues[row];
                for (var col = 0; col < rowValues.length; col++) {
                    var cellValue = rowValues[col];
                    if (cellValue == null || cellValue.Value === "" || cellValue.isNull)
                        continue;
                    addDataToList(listDatas, cellValue);
                    if (cellValue.dataType === FMR.ConstDataTypes.Object_List) {
                        var listValues = cellValue.Value;
                        for (listIndex = 0; listIndex < listValues.length; listIndex++) {
                            curValue = listValues[listIndex];
                            addDataToList(listDatas, curValue);
                        }
                    }
                }
            }
        }
        else if (curData.dataType === FMR.ConstDataTypes.Object_List) {
            var listData = curData.Value;
            for (listIndex = 0; listIndex < listData.length; listIndex++) {
                curValue = listData[listIndex];
                addDataToList(listDatas, curValue);
            }
        }
    }
    return listDatas;
};

FMR.getStatResult = function (fKey, dataLink, newValue, nodes, statEnum) {
    var listDatas = FMR.getStatisticsData(fKey, dataLink, newValue, nodes);
    if (listDatas.length <= 0) {
        return null;
    }
    var count = listDatas.length;
    var sumData = 0.0, maxData = null, minData = null, medianData, modeData = null;
    var modeDict = {};
    for (var index = 0; index < count; index++) {
        var listData = listDatas[index];
        sumData += listData;
        if (maxData == null || listData > maxData)
            maxData = listData;
        if (minData == null || listData < minData)
            minData = listData;
        if (modeDict[listData] != undefined) {
            modeDict[listData] = modeDict[listData] + 1;
        }
        else
            modeDict[listData] = 1;
    }
    switch (statEnum) {
        case FMR.StatisticsEnum.Sum:
            return sumData;
        case FMR.StatisticsEnum.Avg:
            return sumData / count;
        case FMR.StatisticsEnum.Count:
            return count;
        case FMR.StatisticsEnum.Max:
            return maxData;
        case FMR.StatisticsEnum.Median:
            listDatas.sort(function compare(x, y) { return x - y; });
            if (count % 2 === 1) {
                medianData = listDatas[Math.floor(count / 2)];
            }
            else {
                var leftData = listDatas[count / 2 - 1];
                var rightData = listDatas[count / 2];
                medianData = (leftData + rightData) / 2.0;
            }
            return medianData;
        case FMR.StatisticsEnum.Min:
            return minData;
        case FMR.StatisticsEnum.Mode:
            var maxCount = 0;
            for (var resultData in modeDict) {
                var curData = parseFloat(resultData);
                var resultCount = modeDict[resultData];
                if (modeData == null) {
                    modeData = curData;
                    maxCount = resultCount;
                } else {
                    if (resultCount > maxCount) {
                        modeData = curData;
                        maxCount = resultCount;
                    } else if (resultCount === maxCount) {
                        if (curData > modeData)
                            modeData = curData;
                    }
                }
            }
            return modeData;
        default:
            return null;
    }
};

FMR.compare = function(data1, data2) {
    if (data1.isNull && data2.isNull) 
        return 0;
    if (data1.isNull)
        return -1;
    if (data2.isNull)
        return 1;
    var curData1, curData2;
    var dataType1 = data1.dataType;
    var dataType2 = data2.dataType;
    if (dataType1 === dataType2) {
        if (dataType1 === FMR.ConstDataTypes.General) {
            curData1 = FMR.getValueByType(data1.Value, dataType1, data1.isNull, FMR.ConstDataTypes.Bool);
            curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull, FMR.ConstDataTypes.Bool);
            if (curData1 != null && curData2 != null)
                return FMR.compareWithBool(curData1, curData2);
            else if (curData1 != null)
                return 1;
            else if (curData2 != null)
                return -1;
            else {
                curData1 = FMR.getValueByType(data1.Value, dataType1, data1.isNull, FMR.ConstDataTypes.Float);
                curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull, FMR.ConstDataTypes.Float);
                if (isNaN(curData1) && isNaN(curData2)) {
                    curData1 = FMR.getValueByType(data1.Value, dataType1, data1.isNull, FMR.ConstDataTypes.String);
                    curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull, FMR.ConstDataTypes.String);
                    return curData1.localeCompare(curData2);
                }
                return FMR.compareWithNumber(curData1, curData2);
            }
        } else {
            curData1 = FMR.getValueByType(data1.Value, dataType1, data1.isNull);
            curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull);
            if (dataType1 === FMR.ConstDataTypes.Int || dataType1 === FMR.ConstDataTypes.Currency || dataType1 === FMR.ConstDataTypes.Float)
                return FMR.compareWithNumber(curData1, curData2);
            else if (dataType1 === FMR.ConstDataTypes.String || dataType1 === FMR.ConstDataTypes.Date || dataType1 === FMR.ConstDataTypes.DateTime)
                return curData1.localeCompare(curData2);
            else if (dataType1 === FMR.ConstDataTypes.Bool)
                return FMR.compareWithBool(curData1, curData2);
        }
    } else {
        if (dataType1 === FMR.ConstDataTypes.General) 
            return FMR.compareWithGeneral(data1, data2);
        var resultData;
        if (dataType2 === FMR.ConstDataTypes.General) {
            resultData = FMR.compareWithGeneral(data2, data1);
            return -resultData;
        }
        if (dataType1 === FMR.ConstDataTypes.Bool) {
            return FMR.compareWithBool(data1, data2);
        }
        if (dataType2 === FMR.ConstDataTypes.Bool) {
            resultData = FMR.compareWithBool(data2, data1);
            return -resultData;
        }
        if (dataType1 === FMR.ConstDataTypes.Int || dataType1 === FMR.ConstDataTypes.Currency || dataType1 === FMR.ConstDataTypes.Float) {
            if (dataType2 === FMR.ConstDataTypes.Int || dataType2 === FMR.ConstDataTypes.Currency || dataType2 === FMR.ConstDataTypes.Float) {
                curData1 = FMR.getValueByType(data1.Value, dataType1, data1.isNull);
                curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull);
                if (isNaN(curData1) && isNaN(curData2)) {
                    curData1 = FMR.getValueByType(data1.Value, dataType1, data1.isNull, FMR.ConstDataTypes.String);
                    curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull, FMR.ConstDataTypes.String);
                    return curData1.localeCompare(curData2);
                }
                return FMR.compareWithNumber(curData1, curData2);
            } else {
                return -1;
            }
        }
        if (dataType2 === FMR.ConstDataTypes.Int || dataType2 === FMR.ConstDataTypes.Currency || dataType2 === FMR.ConstDataTypes.Float)
            return 1;
        curData1 = FMR.getValueByType(data1.Value, dataType1, data1.isNull, FMR.ConstDataTypes.String);
        curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull, FMR.ConstDataTypes.String);
        return curData1.localeCompare(curData2);
    }
    curData1 = FMR.getValueByType(data1.Value, dataType1, data1.isNull, FMR.ConstDataTypes.String);
    curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull, FMR.ConstDataTypes.String);
    return curData1.localeCompare(curData2);
};

FMR.compareWithBool = function (boolData, data2) {
    var dataType2 = data2.dataType;
    var curData1 = FMR.getValueByType(boolData.Value, boolData.dataType, boolData.isNull);
    var curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull);
    if (dataType2 === FMR.ConstDataTypes.Bool) {
        if (curData1 === true) {
            if (curData2 === true) {
                return 0;
            }
            return 1;
        } else {
            if (curData2 === true) {
                return -1;
            }
            return 0;
        }
    }
    if (data2.dataType === FMR.ConstDataTypes.General) {
        curData2 = FMR.getValueByType(data2.Value, dataType2, data2.isNull, FMR.ConstDataTypes.Float);
        if (isNaN(curData2)) {
            return -1;
        }
        if (curData1 === true) {
            return curData2 > 1.0 ? -1 : (curData2 === 1.0 ? 0 : 1);
        } else {
            return curData2 > 0.0 ? -1 : (curData2 === 0.0 ? 0 : 1);
        }
    }
    if (dataType2 === FMR.ConstDataTypes.Int || dataType2 === FMR.ConstDataTypes.Currency || dataType2 === FMR.ConstDataTypes.Float) {
        if (curData1 === true) {
            return curData2 > 1.0 ? -1 : (curData2 === 1.0 ? 0 : 1);
        } else {
            return curData2 > 0.0 ? -1 : (curData2 === 0.0 ? 0 : 1);
        }
    }
    if (dataType2 === FMR.ConstDataTypes.String) {
        return -1;
    }
    return -1;
}

FMR.compareWithGeneral = function (generalData, data) {
    var dataType = data.dataType;
    if (dataType === FMR.ConstDataTypes.Bool) {
        return FMR.compareWithBool(data, generalData);
    } else {
        var curData2 = FMR.getValueByType(data.Value, dataType, data.isNull);
        var curData1 = FMR.getValueByType(generalData.Value, FMR.ConstDataTypes.General, generalData.isNull, FMR.ConstDataTypes.Float);
        if (dataType === FMR.ConstDataTypes.Int || dataType === FMR.ConstDataTypes.Currency || dataType === FMR.ConstDataTypes.Float) {
            return FMR.compareWithNumber(curData1, curData2);
        } else {
            curData1 = FMR.getValueByType(generalData.Value, FMR.ConstDataTypes.General, generalData.isNull, FMR.ConstDataTypes.String);
            return curData1.localeCompare(curData2);
        }
    }
};

FMR.compareWithNumber = function (curData1, curData2) {
    if (isNaN(curData1))
        return 1;
    else if (isNaN(curData2))
        return -1;
    if (curData1 > curData2)
        return 1;
    else if (curData1 === curData2)
        return 0;
    return -1;
}

FMR.FieldGatherType = {
    /// <summary>
    /// 不汇总
    /// </summary>
    fg_None: 0,
    /// <summary>
    /// 累加汇总
    /// </summary>
    fg_Sum: 1,
    /// <summary>
    /// 计数汇总
    /// </summary>
    fg_Count: 2,
    /// <summary>
    /// 平均值汇总
    /// </summary>
    fg_Avg: 3,
    /// <summary>
    /// 最小值汇总
    /// </summary>
    fg_Min: 4,
    /// <summary>
    /// 最大值汇总
    /// </summary>
    fg_Max: 5,
    /// <summary>
    /// 重算指标计算表达式
    /// </summary>
    fg_ReCalculate: 6,
};

FMR.ValidationType = {
    /// <summary>
    /// 长度校验.
    /// </summary>
    vt_LengthValidate : 0,

    /// <summary>
    /// 范围校验.
    /// </summary>
    vt_RangeValidate: 1,

    /// <summary>
    /// 唯一性校验.
    /// </summary>
    vt_DuplicateValidate: 2,
    
    /// <summary>
    /// 身份证校验
    /// </summary>
    vt_IDCardValidate: 3
};

FMR.getValue = function(newValue, getCode) {
    var curValue;
    if (getCode && newValue.isEnum) {
        var editText = newValue.editText;
        var showText = newValue.showText;
        if (editText != null && editText.length > 0 && showText != null && showText.length > 0) {
            if (editText == '00000000000000000000000000000000')     //自定义枚举
                return showText + "|" + showText;
            curValue = showText + "|" + editText;
            return curValue;
        }
    }
    curValue = newValue.isEnum ? newValue.showText : newValue.editText;
    return curValue;
};

FMR.isNumber = function(s) {
    if (s == null || s.length <= 0)
        return false;
    for (var index = 0; index < s.length; index++) {
        var c = s[index];
        if (c < '0' || c > '9')
            return false;
    }
    return true;
};

FMR.isLastIDCardChar = function(c) {
    if ((c < '0' || c > '9') && c != 'X' && c != 'x')
        return false;
    return true;
};

FMR.isIDCard18Date = function(idCard) {
    if (idCard == null || idCard.length != 18)
        return false;
    var birthDate = idCard.substr(6, 8);
    if (FMR.isNumber(birthDate)) {
        var year = parseInt(birthDate.substr(0, 4), 10);
        var month = parseInt(birthDate.substr(4, 2), 10);
        var day = parseInt(birthDate.substr(6, 2), 10);
        if (year < 1900 || year >= 3000 || month < 1 || month > 12 || day < 1 || day > 31)
            return false;
        var temp = new Date(year, month, 0);
        var maxDay = temp.getDate();
        if (day > maxDay)
            return false;
        return true;
    }
    return false;
};

FMR.isIDCard15Date = function (idCard) {
    if (idCard == null || idCard.length != 15)
        return false;
    var birthDate = idCard.substr(6, 6);
    if (FMR.isNumber(birthDate)) {
        var year = parseInt(birthDate.substr(0, 2), 10) + 1900;
        var month = parseInt(birthDate.substr(2, 2), 10);
        var day = parseInt(birthDate.substr(4, 2), 10);
        if (year < 1900 || year >= 2000 || month < 1 || month > 12 || day < 1 || day > 31)
            return false;
        var temp = new Date(year, month, 0);
        var maxDay = temp.getDate();
        if (day > maxDay)
            return false;
        return true;
    }
    return false;
};

FMR.getLastIDCardChar = function (s) {
    var resultChar = "";
    if (s == null || s.length != 17)
        return resultChar;
    var idcs = ["1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2"];
    var weighs = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2, 1];
    var sumValue = 0;
    for (var index = 0; index < s.length; index++) {
        sumValue = sumValue + (s[index] - "0") * weighs[index];
    }
    var idcIndex = sumValue % 11;
    var result = idcs[idcIndex];
    return result;
};

FMR.isIDCard = function(idCard) {
    if (idCard == null || idCard.length <= 0 || (idCard.length != 18 && idCard.length != 15))
        return false;
    var result = false;
    var address = "11x22x35x44x53x12x23x36x45x54x13x31x37x46x61x14x32x41x50x62x15x33x42x51x63x21x34x43x52x64x65x71x81x82x91";
    if (idCard.length == 18) {
        if (FMR.isNumber(idCard.substr(0, 17)) && FMR.isLastIDCardChar(idCard[17]))
            result = FMR.isIDCard18Date(idCard);    //生日验证
        if (result) {
            if (address.indexOf(idCard.substr(0, 2)) < 0)
                result = false;     //省份验证
            else {
                var lastChar = idCard[17].toUpperCase();
                var validateChar = FMR.getLastIDCardChar(idCard.substr(0, 17));
                if (lastChar != validateChar)
                    result = false;
            }
        }
    } else if (idCard.length == 15) {
        if (FMR.isNumber(idCard))
            result = FMR.isIDCard15Date(idCard);    //生日验证
        if (result && address.indexOf(idCard.substr(0, 2)) < 0)
            result = false;     //省份验证
    }
    return result;
};

FMR.getIDCard18 = function(idCard) {
    if (FMR.isIDCard(idCard)) {
        if (idCard.length == 15) {
            var resultValue = idCard.substr(0, 6) + "19" + idCard.substring(6);
            var lastChar = FMR.getLastIDCardChar(resultValue);
            if (lastChar != "")
                return (resultValue + lastChar).toUpperCase();
        } else if (idCard.length == 18)
            return idCard.toUpperCase();
    }
    return "";
};

//#endregion

//#region 扩展Number方法

//除法函数，用来得到精确的除法结果
//说明：javascript的除法结果会有误差，在两个浮点数相除的时候会比较明显。这个函数返回较为精确的除法结果。
//调用：accDivide(arg1,arg2)
//返回值：arg1除以arg2的精确结果
function accDivide(arg1, arg2) {
    var t, r1, r2, t1 = 0, t2 = 0;
    var s1 = arg1.toString();
    var s2 = arg2.toString();

    t = s1.split(".");
    // 判断有没有小数位，避免出错
    if (t[1]) {
        t1 += t[1].length;
    }

    t = s2.split(".");
    if (t[1]) {
        t2 += t[1].length;
    }
    with (Math) {
        r1 = Number(arg1.toString().replace(".", ""));
        r2 = Number(arg2.toString().replace(".", ""));
        var resultData = (r1 / r2) * pow(10, t2 - t1);
        return resultData;
    }
}
//给Number类型增加一个divide方法，调用起来更加 方便。
Number.prototype.divide = function(arg) {
    return accDivide(this, arg);
};
//乘法函数，用来得到精确的乘法结果
//说明：javascript的乘法结果会有误差，在两个浮点数相乘的时候会比较明显。这个函数返回较为精确的乘法结果。
//调用：accMultiply(arg1,arg2)
//返回值：arg1乘以 arg2的精确结果
function accMultiply(arg1, arg2) {
    var t, m = 0;
    var s1 = arg1.toString();
    var s2 = arg2.toString();
    t = s1.split(".");
    // 判断有没有小数位，避免出错
    if (t[1]) {
        m += t[1].length;
    }

    t = s2.split(".");
    if (t[1]) {
        m += t[1].length;
    }
    var resultData = Number(s1.replace(".", "")) * Number(s2.replace(".", "")) / Math.pow(10, m);
    return resultData;
}
// 给Number类型增加一个multiply方法，调用起来更加方便。
Number.prototype.multiply = function(arg) {
    return accMultiply(this, arg);
};
//加法函数，用来得到精确的加法结果
//说明：javascript的加法结果会有误差，在两个浮点数相加的时候会比较明显。这个函数返回较为精确的加法结果。
//调用：accAdd(arg1,arg2)
// 返回值：arg1加上arg2的精确结果
function accAdd(arg1, arg2) {
    var t, r1=0,r2=0;
    var s1 = arg1.toString();
    var s2 = arg2.toString();
    t = s1.split(".");
    // 判断有没有小数位，避免出错
    if (t[1]) {
        r1 = t[1].length;
    }

    t = s2.split(".");
    if (t[1]) {
        r2 = t[1].length;
    }
    var m = Math.pow(10, Math.max(r1, r2));
    var resultData = (arg1 * m + arg2 * m) / m;
    return resultData;
}
//给Number类型增加一个add方法，调用起来更加方便。
Number.prototype.add = function(arg) {
    return accAdd(this, arg);
};

//减法函数，用来得到精确的加法结果
//说明：javascript的加法结果会有误差，在两个浮点数相加的时候会比较明显。这个函数返回较为精确的减法结果。
//调用：accSubtract(arg1,arg2)
// 返回值：arg1减去arg2的精确结果
function accSubtract(arg1, arg2) {
    var t, r1 = 0, r2 = 0;
    var s1 = arg1.toString();
    var s2 = arg2.toString();
    t = s1.split(".");
    // 判断有没有小数位，避免出错
    if (t[1]) {
        r1 = t[1].length;
    }

    t = s2.split(".");
    if (t[1]) {
        r2 = t[1].length;
    }
    var n = (r1 >= r2) ? r1 : r2;
    var m = Math.pow(10, Math.max(r1, r2));
    var resultData = (arg1 * m - arg2 * m) / m;
    return resultData.toFixed(n);
}
//给Number类型增加一个subtract方法，调用起来更加方便。
Number.prototype.subtract = function (arg) {
    return accSubtract(this, arg);
};

//乘方函数，用来得到精确的加法结果
//说明：javascript的加法结果会有误差，在两个浮点数相加的时候会比较明显。这个函数返回较为精确的乘方结果。
//调用：accPower(arg1,arg2)
// 返回值：arg1乘方arg2的精确结果
function accPower(arg1, arg2) {
    var t, r1 = 0;
    var s1 = arg1.toString();
    t = s1.split(".");
    // 判断有没有小数位，避免出错
    if (t[1]) {
        r1 = t[1].length;
    }
    
    var m = Math.pow(10, r1);
    var resultData = Math.pow(arg1 * m, arg2) / (Math.pow(m, arg2));
    return resultData;
}
//给Number类型增加一个power方法，调用起来更加方便。
Number.prototype.power = function (arg) {
    return accPower(this, arg);
};

//取模函数，用来得到精确的加法结果
//说明：javascript的加法结果会有误差，在两个浮点数相加的时候会比较明显。这个函数返回较为精确的取模结果。
//调用：accReminder(arg1,arg2)
// 返回值：arg1取模arg2的精确结果
function accReminder(arg1, arg2) {
    var t, r1 = 0, r2 = 0;
    var s1 = arg1.toString();
    var s2 = arg2.toString();
    t = s1.split(".");
    // 判断有没有小数位，避免出错
    if (t[1]) {
        r1 = t[1].length;
    }

    t = s2.split(".");
    if (t[1]) {
        r2 = t[1].length;
    }
    var m = Math.pow(10, Math.max(r1, r2));
    var resultData = (arg1 * m % arg2 * m) / m;
    return resultData;
}
//给Number类型增加一个reminder方法，调用起来更加方便。
Number.prototype.reminder = function (arg) {
    return accReminder(this, arg);
};

//#endregion

//#region 定义公式解析结果的前台缓存类
function FormulaCache() {
    //运算公式及汇总计算缓存使用
    this.expressionDic = { };
    this.dataLinkExpDic = { };
    this.dataLinkCache = { };
    this.dataValueCache = { };
    this.dataLinksBackColor = { };
    this.dataLinksReadOnly = { };
    this.interfaceExpDic = new Array();
    this.enumLinkDic = { };
    this.writeDataLinkExpDic = {};
    this.writeDataLinkInterfaceExpDic = {};
    this.otherLinksCache = { };
    this.gatherTypeDic = {};
    this.prevLineExpDic = {};
    this.prevLineWriteDataLinks = new Array();
    this.NeedCalcReload = false;
    //审核公式缓存使用
    this.checkExpressionDic = {};
    this.dataLinkCheckExpDic = {};
    this.dataLinkValidationDic = {};
};

//#region 运算公式相关方法
FormulaCache.prototype.AddEnumExpression = function (dataLinkId, enumTitleLinkId) {
    this.enumLinkDic[dataLinkId] = enumTitleLinkId;
};

FormulaCache.prototype.GetEnumExpressionByDataLink = function (dataLinkId) {
    return this.enumLinkDic[dataLinkId];
};

FormulaCache.prototype.AddInterfaceExpression = function (varName) {
    this.interfaceExpDic.push(varName);
};

FormulaCache.prototype.GetAllInterfaceExpressions = function () {
    return this.interfaceExpDic;
};

FormulaCache.prototype.AddDataLinkBackColor = function (dataLinkId, backColor) {
    this.dataLinksBackColor[dataLinkId] = backColor;
};

FormulaCache.prototype.GetDataLinkBackColor = function (dataLinkId) {
    var backColor = this.dataLinksBackColor[dataLinkId];
    return backColor;
};

FormulaCache.prototype.GetAllDataLinkBackColor = function () {
    return this.dataLinksBackColor;
};

FormulaCache.prototype.AddDataLinkReadOnly = function (dataLinkId, readOnly) {
    this.dataLinksReadOnly[dataLinkId] = readOnly;
};

FormulaCache.prototype.GetDataLinkReadOnly = function (dataLinkId) {
    var readOnly = this.dataLinksReadOnly[dataLinkId];
    return readOnly;
};

FormulaCache.prototype.GetAllDataLinkReadOnly = function () {
    return this.dataLinksReadOnly;
};

FormulaCache.prototype.AddExpression = function (varName, formulaExp) {
    this.expressionDic[varName] = formulaExp;
};

FormulaCache.prototype.AddDataLinkExpression = function (dataLinkId, varName) {
    var expressionArray = this.dataLinkExpDic[dataLinkId];
    if (expressionArray) {
        expressionArray.push(varName);
    }
    else {
        expressionArray = new Array();
        expressionArray.push(varName);
        this.dataLinkExpDic[dataLinkId] = expressionArray;
    }
};

FormulaCache.prototype.AddDataLinkByFieldCode = function (dataLinkId, tableName, fieldCode) {
    var fieldKey = tableName + "[" + fieldCode + "]";
    this.dataLinkCache[fieldKey] = dataLinkId;
};

FormulaCache.prototype.AddOtherDataLinkByFieldCode = function (dataLinkId, tableName, fieldCode) {
    var fieldKey = tableName + "[" + fieldCode + "]";
    this.otherLinksCache[fieldKey] = dataLinkId;
};

FormulaCache.prototype.GetOtherDataLinkByFieldCode = function (tableName, fieldCode) {
    var fieldKey = tableName + "[" + fieldCode + "]";
    return this.otherLinksCache[fieldKey];
};

FormulaCache.prototype.AddDataValueByFieldCode = function(dataValue, tableName, fieldCode) {
    var fieldKey = tableName + "[" + fieldCode + "]";
    this.dataValueCache[fieldKey] = dataValue;
};

FormulaCache.prototype.GetDataLinkByFieldCode = function (tableName, fieldCode) {
    var fieldKey = tableName + "[" + fieldCode + "]";
    return this.dataLinkCache[fieldKey];
};

FormulaCache.prototype.GetDataValueByFieldCode = function (tableName, fieldCode) {
    var fieldKey = tableName + "[" + fieldCode + "]";
    return this.dataValueCache[fieldKey];
};

FormulaCache.prototype.GetExpressionNamesByDataLink = function (dataLinkId) {
    var varNames = this.dataLinkExpDic[dataLinkId];
    return varNames;
};

FormulaCache.prototype.GetExpressionsByDataLink = function (dataLinkId) {
    var varNames = this.dataLinkExpDic[dataLinkId];
    if (varNames != null) {
        var expressions = new Array();
        for (var i = 0; i < varNames.length; i++) {
            var varName = varNames[i];
            var expression = this.expressionDic[varName];
            if (expression != null)
                expressions.push(expression);
        }
        return expressions;
    }
    else
        return null;
};

FormulaCache.prototype.GetExpressionByName = function (varName) {
    return this.expressionDic[varName];
};

FormulaCache.prototype.AddWriteDataLinkExpression = function (dataLinkId, formulaExp) {
    this.writeDataLinkExpDic[dataLinkId] = formulaExp;
};

FormulaCache.prototype.GetExpressionByWriteDataLink = function (dataLinkId) {
    return this.writeDataLinkExpDic[dataLinkId];
};

FormulaCache.prototype.GetWriteDataLinkExpressions = function () {
    return this.writeDataLinkExpDic;
};

FormulaCache.prototype.AddGatherTypeByExpressionName = function(varName, gatherType) {
    this.gatherTypeDic[varName] = gatherType;
};

FormulaCache.prototype.GetGatherTypeByExpressionName = function (varName) {
    return this.gatherTypeDic[varName];
};

FormulaCache.prototype.AddPrevLineExpression = function (varName, formulaExp) {
    this.prevLineExpDic[varName] = formulaExp;
};

FormulaCache.prototype.GetPrevExpressionsByDataLink = function (dataLinkId) {
    var varNames = this.dataLinkExpDic[dataLinkId];
    if (varNames != null) {
        var expressions = new Array();
        for (var i = 0; i < varNames.length; i++) {
            var varName = varNames[i];
            var expression = this.prevLineExpDic[varName];
            if (expression != null)
                expressions.push(expression);
        }
        return expressions;
    }
    else
        return null;
};

FormulaCache.prototype.AddPrevLineWriteDataLink = function (writeLinkId) {
    if (this.prevLineWriteDataLinks.indexOf(writeLinkId) >= 0)
        return;
    this.prevLineWriteDataLinks.push(writeLinkId);
}

FormulaCache.prototype.GetAllPrevLineWriteDataLinks = function() {
    return this.prevLineWriteDataLinks;
}

FormulaCache.prototype.SetNeedCalcReload = function () {
    this.NeedCalcReload = true;
}

FormulaCache.prototype.AddWriteDataLinkInterfaceExpression = function (dataLinkId, formulaExp) {
    this.writeDataLinkInterfaceExpDic[dataLinkId] = formulaExp;
};

FormulaCache.prototype.GetWriteDataLinkInterfaceExpressions = function () {
    return this.writeDataLinkInterfaceExpDic;
};

FormulaCache.prototype.GetInterfaceExpressionByLink = function (dataLinkId) {
    return this.writeDataLinkInterfaceExpDic[dataLinkId];
};

//#endregion

//#region 审核公式相关方法
FormulaCache.prototype.AddCheckExpression = function (varName, formulaExp) {
    this.checkExpressionDic[varName] = formulaExp;
};

FormulaCache.prototype.AddDataLinkCheckExpression = function (dataLinkId, varName) {
    var expressionArray = this.dataLinkCheckExpDic[dataLinkId];
    if (expressionArray) {
        expressionArray.push(varName);
    }
    else {
        expressionArray = new Array();
        expressionArray.push(varName);
        this.dataLinkCheckExpDic[dataLinkId] = expressionArray;
    }
};

FormulaCache.prototype.GetCheckExpressionsByDataLink = function (dataLinkId) {
    var varNames = this.dataLinkCheckExpDic[dataLinkId];
    if (varNames != null) {
        var expressions = new Array();
        for (var i = 0; i < varNames.length; i++) {
            var varName = varNames[i];
            var expression = this.checkExpressionDic[varName];
            if (expression != null)
                expressions.push(expression);
        }
        return expressions;
    }
    else
        return null;
};

FormulaCache.prototype.GetAllCheckExpressions = function () {
    return this.checkExpressionDic;
};

FormulaCache.prototype.GetAllCheckExpressionLinks = function () {
    var linkIds = new Array();
    for (var dataLinkId in this.dataLinkCheckExpDic) {
        linkIds.push(dataLinkId);
    }
    return linkIds;
};

FormulaCache.prototype.GetRelatedCheckExpressionLinks = function (dataLinkId) {
    var varNames = this.dataLinkCheckExpDic[dataLinkId];
    if (varNames == null || varNames.length <= 1)
        return null;
    var linkIds = new Array();
    for (var linkId in this.dataLinkCheckExpDic) {
        if (linkId == "" || linkId == dataLinkId)
            continue;
        var subVarNames = this.dataLinkCheckExpDic[linkId];
        if (subVarNames == null || subVarNames.length <= 1)
            continue;
        for (var i = 0; i < varNames.length; i++) {
            if ($.inArray(varNames[i], subVarNames) >= 0) {
                linkIds.push(linkId);
                break;
            }
        }
    }
    return linkIds;
};

FormulaCache.prototype.AddDataLinkValidation = function(dataLinkId, formulaId, validationType) {
    var validationObjs = this.dataLinkValidationDic[dataLinkId];
    if (validationObjs) {
        validationObjs[formulaId] = validationType;
    }
    else {
        validationObjs = {};
        validationObjs[formulaId] = validationType;
        this.dataLinkValidationDic[dataLinkId] = validationObjs;
    }
};

FormulaCache.prototype.GetValidationByDataLink = function (dataLinkId) {
    var validations = this.dataLinkValidationDic[dataLinkId];
    return validations;
};
//#endregion

FormulaCache.prototype.Clear = function () {
    this.expressionDic = null;
    this.dataLinkExpDic = null;
    this.dataLinkCache = null;
    this.dataValueCache = null;
    this.dataLinksBackColor = null;
    this.dataLinksReadOnly = null;
    this.interfaceExpDic = null;
    this.writeDataLinkExpDic = null;
    this.otherLinksCache = null;
    this.gatherTypeDic = null;
    this.prevLineWriteDataLinks = null;
};
//#endregion

//#region 调用方法将js添加到前台
var formulaCache = new FormulaCache();

function reloadJSByContent(formulaId, jsText) { 
    var oldjs = window.document.getElementById(formulaId);
    if (oldjs) oldjs.parentNode.removeChild(oldjs);
    if (window.formulaCache) {
        window.formulaCache.Clear();
    }
    window.formulaCache = new FormulaCache();
    var scriptObj = window.document.createElement("script");
    scriptObj.text = jsText;
    scriptObj.type = "text/javascript";
    scriptObj.id = formulaId;
    window.document.getElementsByTagName("head")[0].appendChild(scriptObj);
};
//#endregion