﻿
$.Class("GridForm", MvcBase, {
    construct: function() {
        this._super().construct.call(this);
    },
    
    _grid: null,
    
    setGrid: function (grid) {
    	this._grid = grid;
    },
	
    asynLoadData: function (asynFormData, begin, callback, callbackArgs) {
        if (this._grid) {
            this._grid.asynLoadData(asynFormData, begin, callback, callbackArgs);
        }
    },
    
    getData: function () {
        if (!this._grid) return null;
        this._grid.closeEditor();
        var saveData = this._grid.saveFormData(false, true);
        return saveData;
    },

    resetStatus: function() {
        if (!this._grid) return;
        this._grid.updateAfterStoreData();
    },

    updateRowAttrs: function(keysMap, ordersMap) {
        if (!this._grid) return;
        if (keysMap && !$.isEmptyObject(keysMap))
            this._grid.refrashFkey(keysMap);
        if (ordersMap && !$.isEmptyObject(ordersMap))
            this._grid.refrashFloatOrder(ordersMap);
    },

    getFirstCell: function() {
        if (!this._grid) return null;
        var firstCell = this._grid.getCell(1, 1);//注：工作表中至少有一行一列
        //var regKind = this._grid.getFormRegionKind();
        //var firstCell = this._grid.getFirstEditableCell(regKind === "col");
        return firstCell;
    },

    getFocusCell: function() {
        if (!this._grid) return null;
        return this._grid.getCurrentCell();
    },

    getFrozenPos: function () {
        if (!this._grid) return null;
        return this._grid.getHeaderLockInfo();
    },

    getCell: function(col, row) {
        if (!this._grid) return null;
        return this._grid.getCell(col, row);
    },

    queryCells: function(rowKey, linkIds) {
        if (!this._grid) return null;
        var cells = this._grid.findByDataLinks(linkIds, rowKey);
        return cells;
    },

    getCellPos: function(cell) {
        if (!cell) return null;
        var position = cell.getPosition();
        if (!this._grid || !position) return null;
        var rowIndex = cell.getRowIndex();
        var rowHeight = this._grid.getRowHeight(rowIndex);
        var colIndex = cell.getColIndex();
        var colWidth = this._grid.getColWidth(colIndex);
        position.width = colWidth;
        position.height = rowHeight;
        return position;
    },

    getHeaderLockInfo: function () {
        if (!this._grid) return null;
        return this._grid.getHeaderLockInfo();
    },

    getRowAttr: function(attrName, index) {
        if (!this._grid) return null;
        var formKind = this.getFormKind();

        if (!index) {
            var curCell = this._grid.getCurrentCell();
            if (!curCell) return null;
            var isColumn = formKind === "col";
            index = isColumn ? curCell.getColIndex() : curCell.getRowIndex();
        }

        var value = null;
        if (formKind === "row")
            value = this._grid.getDataRowAttrByName(attrName, 0, index);
        if (formKind === "col")
            value = this._grid.getDataRowAttrByName(attrName, index, 0);
        return value;
    },
	
	getCurRowKey: function() {		
        if (!this._grid) return 0;
        var curCell = this._grid.getCurrentCell();
        if (!curCell) return null;
        var rowIndex = curCell.getRowIndex();
        var colIndex = curCell.getColIndex();
        var rowKey = this._grid.getDataRowAttrByName("fkey", colIndex, rowIndex);
        return rowKey;
	},
	
	getRowKey: function(cell){
		if (!this._grid || !cell) return null;
        var colIndex = cell.getColIndex();
        var rowIndex = cell.getRowIndex();
		return this._grid.getDataRowAttrByName("fkey", colIndex, rowIndex);
	},
    
	getCurRowState: function () {
	    if (!this._grid) return "";
	    var curCell = this._grid.getCurrentCell();
	    if (!curCell) return "";
	    var rowIndex = curCell.getRowIndex();
	    var colIndex = curCell.getColIndex();
	    return this._grid.getDataRowAttrByName("state", colIndex, rowIndex);
	},

    getRowCount: function() {
        if (!this._grid) return 0;
        return this._grid.getRowCount();
    },

    getColCount: function () {
        if (!this._grid) return 0;
        return this._grid.getColCount();
    },

    getFormKind: function() {
        if (!this._grid) return null;
        return this._grid.getFormRegionKind();
    },

    getDataReg: function(regId) {
        if (!this._grid) return null;

        if (!regId) {
            try {
                regId = parseInt(this.getRowAttr("reg"));
            } catch (e) {
                return null;
            }
        }

        if (typeof regId === "string") {
            try {
                regId = parseInt(regId);
            } catch (e) {
                return null;
            }
        }

        if (!regId) return null;
        var dataReg = this._grid.getDataRegion(regId);
        return dataReg;
    },
    
    getDrawAreaInfo: function () {
        if (!this._grid) return null;
        return this._grid.getDrawAreaInfo();
    },

    getDataRegs:function(){
        if (!this._grid) return null;
        return this._grid.getDataRegions();
    },
    
    getRowCells:function(rowIndex) {
        if (!this._grid) return null;
        return this._grid.getRowCells(rowIndex);
    },

    getColCells:function(colIndex){
        if (!this._grid) return null;
        return this._grid.getColCells(colIndex);
    },

    getSelectedCells: function() {
        if (!this._grid) return null;
        return this._grid.getSelectedCells();
    },
    
    getRowHeight: function (rowIndex) {
        if (!this._grid) return null;
        return this._grid.getRowHeight(rowIndex);
    },
    
    getColWidth: function(colIndex){
        if (!this._grid) return null;
        return this._grid.getColWidth(colIndex);
    },

    getSelections: function() {
        if (!this._grid) return null;
        return this._grid.getSelections();
    },

    checkMultiSelections: function() {
        if (!this._grid) return false;
        var selections = this._grid.getSelections();
        if (selections && selections.length > 1) {
            jNoty("该操作暂不支持多选区。", "warning");
            return false;
        }
        return true;
    },

    onDataCut: function() {
        if (!this._grid) return;
        if (!this.checkMultiSelections()) return;
        this._grid.cut();
    },

    onDataCopy: function() {
        if (!this._grid) return;
        if (!this.checkMultiSelections()) return;
        this._grid.copy();
    },

    onDataPaste: function() {
        if (!this._grid) return;
        if (!this.checkMultiSelections()) return;
        this._grid.paste();
    },

    setCanEdit: function(editable) {
        if (!this._grid) return;
        this._grid.setCanEdit(editable);
    },

    setCanInsert: function(enable) {
        if (!this._grid) return;
        this._grid.denyInsertByPaste();
    },
    
    setFocus:function(cell) {
        if (!this._grid) return;
        this._grid.setFocus(cell);
    },

    onInsert: function(count, isInsertAfter, isColumn) {
        if (!this._grid) return;
        if (!this.checkMultiSelections()) return;

        var selections = this.getSelections();
        if (!selections || !selections.length) return;

        var index = isColumn ? selections[0].x : selections[0].y;
        if (isInsertAfter)
            index += isColumn ? selections[0].width : selections[0].height;

        this._grid.insert(count, index);
    },

    onDelete: function(count, index) {
        if (!this._grid) return;
        if (!this.checkMultiSelections()) return;

        this._grid.delete(count, index);
    },
    
    closeEditor: function () {
        if (!this._grid) return;
        this._grid.closeEditor();
    },

    onFreeze: function(type) {
        if (!this._grid) return;

        if (this.isFrozen()) {
            this._grid.headerUnlock();
            return;
        }

        if (!this.checkMultiSelections()) return;
        var curCell = this._grid.getCurrentCell();
        if (!curCell) return;

        var row = 1, col = 1;
        switch (type) {
        case "both":
            row = curCell.getRowIndex();
            col = curCell.getColIndex();
            break;
        case "firstrow":
            row = 1;
            break;
        case "firstcol":
            col = 1;
            break;
        }

        this._grid.headerLock(col, row);
    },

    isFrozen: function() {
        return this._grid && this._grid.isHeaderLocked();
    },
    
    isEditing: function(){
    	return this._grid && this._grid.isEditing();
    },

    getReturnDir: function() {
        if (!this._grid) return null;
        return this._grid.getEnterDirection();
    },

    setReturnDir: function(enterDir) {
        if (!this._grid) return;
        this._grid.setEnterDirection(enterDir);
    },

    requestEnumNodes: function() {
        if (!this._grid)
            return;
        this._grid.requestEnumNodes();
    },
    
    openEditor: function(){    	
    	if (!this._grid) return null;
        var curCell = this._grid.getCurrentCell();
        if(curCell){
        	var colIndex = curCell.getColIndex();
        	var rowIndex = curCell.getRowIndex();
        	this._grid.openEditor(colIndex, rowIndex, true);
        }
    },

    onDestroy: function () {
        this._super().onDestroy.call(this);
    }
});
