﻿/**********************GridHtml5业务处理类********************************************/
var CopyRegion = {};
RO.GridHtml5 = function (shell, formData, widgetManager, designMode, scrollBar) {
    this.gridModel = new RO.GridHtml5.GridModel(shell, formData, widgetManager, scrollBar, designMode);

    this.designMode = designMode;
};
RO.GridHtml5.prototype.asynLoadData = function (asynFormData, begin, callback, callbackArgs) {
    if (!asynFormData || asynFormData.guid != this.gridModel.guid) return;

    var rowCount = asynFormData.rowCount - 1,
        gridModel = this.gridModel,
        gridData = gridModel.gridData,
        index = begin || 1,
        copyIndex = index - 1,
        depth = gridData.getCell(1, copyIndex).getDepth() + 1,
        regId = this.getDataRowAttrByName(RO.Enum.RegId, 0, copyIndex),
        regionTemplate = gridModel.dataRegions[regId].RegionTemplate,
        indexList = gridModel.rowList,
        regData = asynFormData.options.DataRowsAttr[regId];

    var copyFuncForRow = function(gridSourceCell) {
        return COM.widget.Grid.GridModel.copyCell((regionTemplate && regionTemplate.length > 0 && gridSourceCell.colIndex > 0) ? regionTemplate[0][gridSourceCell.colIndex] : gridSourceCell, COM.widget.Grid.COPY_STYLE);
    };

    var setRowHeights = function(start, count, model, self, minHeight) {
        var colCount = self.getColCount(), rowHeights = [], cols = [], rows = [];
        for (var col = 0; col < colCount; col++) {
            cols.push(self.getColWidth(col));
        }

        for (var r = 0; r < count; r++) {
            var height = minHeight;

            for (var c = 1; c < colCount; c++) {
                var cell = model.gridData.getCell(c, start + r).cell,
                    colSpan = cell.colSpan || 1,
                    width = cols[c];

                for (var i = 1; i < colSpan; i++) {
                    width += cols[c + i];
                }
                width -= 4;

                var newHeight = model.getCellHeight(cell, width);

                height = Math.max(height, newHeight);


                if (height > COM.widget.Grid.GridFigure.MAXROWHEIGHT) {
                    height = COM.widget.Grid.GridFigure.MAXROWHEIGHT;
                }
            }

            rowHeights.push(height);
            rows.push(r + start);
        }
        model.gridData.setRowsHeight(rows, rowHeights, true);
    };

    var setValues = function (cells, newRow, grid, model) {
        var gridCells = [];

        for (var r = 0, rc = cells.length; r < rc; r++) {
            var rowData = cells[r];
            for (var c = 1, cc = cells[0].length; c < cc; c++) {
                var cell = grid.getCell(c, newRow + r),
                    data = rowData[c];

                for (var prop in data) {
                    var val = data[prop];

                    switch (prop) {
                        case "backColor":
                            cell.setBackColor(val);
                            break;
                        case "borderColor":
                            cell.setBottomBorderColor(val[1]);
                            cell.setRightBorderColor(val[0]);
                            break;
                        case "border":
                            cell.setBottomBorderStyle(val[1]);
                            cell.setRightBorderStyle(val[0]);
                            break;
                        case "title":
                            cell.setTitle(val);
                            break;
                        case "selectable":
                            cell.setSelectable(val);
                            break;
                        case "editable":
                            cell.setEditable(val);
                            break;
                        case "checkable":
                            cell.setCheckable(val);
                            break;
                        case "checked":
                            cell.setChecked(val);
                            break;
                        case "fontName":
                            cell.setFontName(val);
                            break;
                        case "fontSize":
                            cell.setFontSize(val);
                            break;
                        case "fontBold":
                            cell.setFontBold(val);
                            break;
                        case "fontItalic":
                            cell.setFontItalic(val);
                            break;
                        case "fontColor":
                            cell.setFontColor(val);
                            break;
                        case "wrapLine":
                            cell.setWrapLine(val);
                            break;
                        case "fitFontSize":
                            cell.setFitFontSize(val);
                            break;
                        case "clientData":
                            cell.setClientData($.extend(cell.getClientData(), val));
                            break;
                        case "decoration":
                            cell.setDecoration(val);
                            break;
                        case "depth":
                            cell.setDepth(val);
                            break;
                        case "cellMode":
                            cell.setCellMode(val);
                            break;
                        case "expandable":
                            cell.setExpandable(val);
                            break;
                    }
                }

                var curCell = new RO.GridHtml5.Cell(cell, model);
                var filter = { "editText": data.editText, "showText": data.showText };
                var formatValue = model.filterData(curCell, filter);
                cell.setEditText(formatValue.editText, true);
                cell.setShowText(formatValue.showText, true);
                gridCells.push(curCell);
            }
        }

        return gridCells;
    };

    var addTreeMode = function(cells) {
        for (var i = cells.length - 1; i >= 0; i--) {
            var cell = cells[i][1];
            cell.cellMode = COM.widget.Grid.Cell.Cell_MODE_TREE;
            cell.expandable = false;
            cell.depth = depth;
        }
    };

    var initRowAttrs = function() {
        for (var i = 0; i < rowCount; i++) {
            var newIndex = index + i;
            var rowData = regData[i];

            if (rowData) {
                indexList[newIndex] = $.extend({}, indexList[newIndex], rowData);
                var fkey = rowData[RO.Enum.FloatRowKey];
                if (fkey) {
                    gridModel.extendData[fkey] = indexList[newIndex];
                }
                if (rowData[RO.Enum.State] === undefined) {
                    indexList[newIndex][RO.Enum.State] = "0";
                }
            }

            indexList[newIndex][RO.Enum.RegId] = regId;
        }
    };

    var adjustTreeRecord = function(self, start, count) {
        var parentRowIndexes,
            rCount = gridData.getRowCount() - count,
            regions = gridModel.dataRegions,
            curRegTop = regions[regId].RegionTop,
            regs = gridModel._floatRegList,
            cou = regs.length,
            i = cou - 1;
        
        parentRowIndexes = self.getRecordRelations(regs[i].id);

        for (var r = rCount - 1; r >= start; r--) {
            var parent = parentRowIndexes[r];
            if (parent !== undefined) {
                if (parent >= start) {
                    parent += count;
                }
                parentRowIndexes[r + count] = parent;
                delete parentRowIndexes[r];
            } else {
                for (var c = i; c >= 0; c--) {
                    if (regs[c].index < curRegTop) break;
                    
                    var newIndexes = self.getRecordRelations(regs[c].id);
                    var newParent = newIndexes[r];
                    if (newParent !== undefined) {
                        parent = newParent;
                        parentRowIndexes = newIndexes;
                        if (parent >= start) {
                            parent += count;
                        }
                        parentRowIndexes[r + count] = parent;
                        delete parentRowIndexes[r];
                        
                        i = c;
                        break;
                    }
                }
            }
        }

        var parentRowIndex = start - 1;
        parentRowIndexes = self.getRecordRelations(regId);
        if (parentRowIndexes == null)
            return;
        
        for (var row = 0; row < count; row++) {
            var rIndex = row + start;
            parentRowIndexes[rIndex] = parentRowIndex;
        }
    };

    var updateRowTitle = function(i) {
        for (var row = i, rowcount = gridData.getRowCount(); row < rowcount; row++) {
            var rowcell = gridData.getCell(0, row);
            rowcell.setShowText(row, true);
        }
    };
    
    var actionLock = gridModel.beginModifyDataRegions();
    if (actionLock) {
        return;
    }
    asynFormData.cells.splice(0, 1);
    var newCells = asynFormData.cells;
    addTreeMode(newCells);
    gridData.beginUpdate();
    gridModel.updateGridSize(index, rowCount, regId, false);
    gridData.insertRows(index, rowCount, copyIndex, copyFuncForRow);
    var gridCells = setValues(newCells, index, gridData, gridModel);
    setRowHeights(index, rowCount, gridModel, this, gridData.getDefaultRowHeight(copyIndex));
    gridModel.endModifyDataRegions(regId, rowCount);
    updateRowTitle(index);
    gridData.endUpdate();
    initRowAttrs();
    gridModel.initExtTypeCell(gridCells);
    adjustTreeRecord(this, index, rowCount);
    callback.apply(this, callbackArgs);
    var insertfalg = -1;
    if (RO.Enum.OperationMode.COL) {
        insertfalg = 1;
    } else if (RO.Enum.OperationMode.ROW) {
        insertfalg = 0;
    }
    
    var rowRegList=gridModel.rowRegList;
    var filterflag = false;
    var forindex = -1;
    var region = gridModel.dataRegions;
    for (var regid in region) {
           if (regid == regId) {
              if (rowRegList[regid].length > 0) {
                 for (var i = 0; i < rowRegList[regid].length; i++) {
                    if (rowRegList[regid][i].index == index) {
                        forindex = i;
                        break;
                    }
                 }
                if (forindex >= 0) {
                    rowRegList[regid][i].count = rowCount;
                    rowRegList[regid][i].countinit = 0;
                    for (var j = forindex + 1; j < rowRegList[regid].length; j++) {
                        rowRegList[regid][j].index += rowCount;
                    }
                    for (var regID in rowRegList) {
                        if (regID != regId && rowRegList[regID][0].index > index) {
                            for (var j = 0; j < rowRegList[regID].length; j++) {
                                rowRegList[regID][j].index += rowCount;
                            }
                        }
                    }
                } else {
                    rowRegList[regid].push({ "index": index, "count": rowCount, "countinit": 0 });
                    for (var j = 0; j < rowRegList[regid].length; j++) {
                        if (rowRegList[regid][j].index > index)
                            rowRegList[regid][j].index += rowCount;
                    }
                    for (var regID in rowRegList) {
                        if (regID != regId && rowRegList[regID][0].index > index) {
                            for (var j = 0; j < rowRegList[regID].length; j++) {
                                rowRegList[regID][j].index += rowCount;
                            }
                        }
                    }
                }
            } else {
                rowRegList[regid].push({ "index": index, "count": rowCount, "countinit": 0 });
                for (var regID in rowRegList) {
                    if (regID != regId && rowRegList[regID][0].index > index) {
                        for (var j = 0; j < rowRegList[regID].length; j++) {
                            rowRegList[regID][j].index += rowCount;
                        }
                    }
                }
            }
         }
      
           if (region[regid].RegionExtension && $.parseJSON(region[regid].RegionExtension).OrderIndex > 0) {
            filterflag = true;
       }
      
    }

    for (var regid in rowRegList) {
        rowRegList[regid].sort(function (x, y) { return x.index > y.index ? 1 : -1 });
    }
    /*
    var regRowListtemp = {};//对regRowList排序
    for (var regid in rowRegList) {
        regRowListtemp[regid] = [];
        var tempArr = [];
        for (var i = 0; i < rowRegList[regid].length; i++) {
            tempArr.push(rowRegList[regid][i].index);
        }
        if (tempArr.length > 0) {
            tempArr.sort(function (a, b) { return a > b ? 1 : -1 });
            for (var i = 0; i < rowRegList[regid].length; i++) {
                for (var j = 0; j < rowRegList[regid].length; j++) {
                    if (tempArr[i] == rowRegList[regid][j].index) {
                        regRowListtemp[regid].push({ "index": tempArr[i], "count": rowRegList[regid][j].count, "countinit": 0 });
                    }
                }
            }
        }
    }
    */
    gridModel.rowRegList = rowRegList;
   // gridModel.rowRegList = rowRegList;

    if (filterflag) {
        var autofilter = { "rowIndex": index, "rowCount": rowCount, "insertfalg": 0 };
        if (insertfalg >= 0) {
            gridModel.filterAutoNoData(autofilter);
        }
    }
};
RO.GridHtml5.prototype.getCell = function (colIndex, rowIndex, listMode) {
    var cell = this.gridModel.gridData.getCell(colIndex, rowIndex);
    var roCell = new RO.GridHtml5.Cell(cell, this.gridModel);
    return cell.cell ? listMode ? [roCell] : roCell : null;
};
RO.GridHtml5.prototype.getCurrentCell = function () {
    var curCell = this.gridModel.gridData.getCurrentCell();
    return curCell.cell ? new RO.GridHtml5.Cell(curCell, this.gridModel) : null;
};
RO.GridHtml5.prototype.setCurrentCell = function (colIndex, rowIndex, noEvent) {
    var model = this.gridModel,
        loading = model._loading,
        self = this;

    if (loading && rowIndex > model._firstLoadCount * 3 / 4) {
        window.setTimeout(function () {
            self.setCurrentCell(colIndex, rowIndex, noEvent);
        }, 2);
    } else {
        model.gridData.setCurrentCell(colIndex, rowIndex, null, noEvent);
        var cell = this.getCell(colIndex, rowIndex);
        if (!cell) return;
        model.gridData.select({ x: colIndex, y: rowIndex, width: 1, height: 1 }, false, noEvent);
    }
};
RO.GridHtml5.prototype.getSelectedCells = function () {
    var selections = this.gridModel.gridData.getSelections();
    if (selections) {
        var cellArray = [];
        for (var i = 0; i < selections.length; i++) {
            var selection = selections[i];
            var top = selection.y;
            var left = selection.x;
            var width = selection.width;
            var height = selection.height;
            for (var row = 0; row < height; row++) {
                for (var col = 0; col < width; col++) {
                    var cell = this.gridModel.gridData.getCell(left + col, top + row);
                    if (cell) {
                        cellArray.push(new RO.GridHtml5.Cell(cell, this.gridModel));
                    }
                }
            }
        }
        return cellArray;
    }
    return [];
};
RO.GridHtml5.prototype.getFirstEditableCell = function (colFirst) {
    if (colFirst) {
        var editMode = this.gridModel.gridData.getEditMode();

        var colCount = this.getColCount();
        var rowCount = this.getRowCount();
        for (var j = 1; j < colCount; j++) {
            for (var i = 1; i < rowCount; i++) {
                var cell = this.getCell(j, i);
                if ((editMode == COM.widget.Grid.EDIT_MODE_EDIT || cell.getCanEdit()) && !cell.cell.cell.merged)
                    return cell;
            }
        }
    } else {
        var cellInfo = this.gridModel.rowFirstEditableCellInfo;
        if (cellInfo)
            return this.getCell(cellInfo.col, cellInfo.row);
    }
    return null;
};
RO.GridHtml5.prototype.getSelections = function () {
    var grid = this.gridModel.gridData;
    var sels = grid.getSelections();
    var len = sels.length;
    var rowCount = grid.getRowCount();
    var colCount = grid.getColCount();

    for (var i = len - 1; i >= 0; i--) {
        var sel = sels[i];
        var rangeFlag = "";
        var width = sel.width;
        var height = sel.height;
        var x = sel.x;
        var y = sel.y;
        var curCell = grid.getCell(x, y);

        if (width == curCell.getColSpan() && height == curCell.getRowSpan()) {
            rangeFlag = "cell";
        } else if (sel.width == colCount - 1) {
            rangeFlag = "row";
        } else if (sel.height == rowCount - 1) {
            rangeFlag = "col";
        }
        sels[i].selectedRange = rangeFlag;
    }

    return sels;
};
RO.GridHtml5.prototype.setSelections = function (selections) {
    var newSelections = [];
    if (selections) {
        var rowCount = this.getRowCount();
        var colCount = this.getColCount();
        for (var i = selections.length - 1; i >= 0; i--) {
            if (selections[i].x > 0
                && selections[i].y > 0
                && selections[i].width >= 1
                && selections[i].height >= 1
                && selections[i].width + selections[i].x <= colCount
                && selections[i].height + selections[i].y <= rowCount) {
                newSelections.push({ 'x': selections[i].x, 'y': selections[i].y, 'width': selections[i].width, 'height': selections[i].height });
            }
        }
    }
    if (newSelections.length > 0) {
        this.gridModel.gridData.setCurrentCell(selections[0].x, selections[0].y);
    }
    this.gridModel.gridData.setSelections(newSelections, true);
};
RO.GridHtml5.prototype.getMergeSections = function () {
    var merges = this.gridModel.gridData.gridModel.data.mergeCells;
    var result = [];
    // 不直接返回引用的结果，避免业务模块更改其中的数据。
    if (merges) {
        for (var i = 0; i < merges.length; i++) {
            result.push({ "x": merges[i].col, "y": merges[i].row, "width": merges[i].width, "height": merges[i].height });
        }
    }
    return result;
};
RO.GridHtml5.prototype.setCanEdit = function (flag) {
    this.gridModel.gridData.setEditMode(flag ? COM.widget.Grid.EDIT_MODE_INPUT : COM.widget.Grid.EDIT_MODE_READ_ONLY);
};
RO.GridHtml5.prototype.getCanEdit = function () {
    var editMode = this.gridModel.gridData.getEditMode();
    return editMode !== COM.widget.Grid.EDIT_MODE_READ_ONLY;
};
RO.GridHtml5.prototype.setEnterDirection = function (direction) {
    switch (direction) {
        case RO.Enum.EnterDirection.UP:
            this.gridModel.gridData.setEnterNext(COM.widget.Grid.ENTER_NEXT_UP);
            break;
        case RO.Enum.EnterDirection.LEFT:
            this.gridModel.gridData.setEnterNext(COM.widget.Grid.ENTER_NEXT_LEFT);
            break;
        case RO.Enum.EnterDirection.DOWN:
            this.gridModel.gridData.setEnterNext(COM.widget.Grid.ENTER_NEXT_DOWN);
            break;
        case RO.Enum.EnterDirection.RIGHT:
            this.gridModel.gridData.setEnterNext(COM.widget.Grid.ENTER_NEXT_RIGHT);
            break;
        default:
            break;
    }
};
RO.GridHtml5.prototype.getEnterDirection = function () {
    var enterNext = this.gridModel.gridData.getEnterNext();
    var direction;
    switch (enterNext) {
        case COM.widget.Grid.ENTER_NEXT_UP:
            direction = RO.Enum.EnterDirection.UP;
            break;
        case COM.widget.Grid.ENTER_NEXT_LEFT:
            direction = RO.Enum.EnterDirection.LEFT;
            break;
        case COM.widget.Grid.ENTER_NEXT_DOWN:
            direction = RO.Enum.EnterDirection.DOWN;
            break;
        case COM.widget.Grid.ENTER_NEXT_RIGHT:
            direction = RO.Enum.EnterDirection.RIGHT;
            break;
        default:
            direction = RO.Enum.EnterDirection.RIGHT;
            break;
    }
    return direction;
};
RO.GridHtml5.prototype.doKeyDirection = function (direction, passReadonly) {
    var newDir;
    switch (direction) {
        case RO.Enum.EnterDirection.UP:
            newDir = COM.widget.Grid.ENTER_NEXT_UP;
            break;
        case RO.Enum.EnterDirection.LEFT:
            newDir = COM.widget.Grid.ENTER_NEXT_LEFT;
            break;
        case RO.Enum.EnterDirection.RIGHT:
            newDir = COM.widget.Grid.ENTER_NEXT_RIGHT;
            break;
        case RO.Enum.EnterDirection.DOWN:
        default:
            newDir = COM.widget.Grid.ENTER_NEXT_DOWN;
            break;
    }
    this.gridModel.gridData.keyDirection(newDir, passReadonly);
};
RO.GridHtml5.prototype.moveUp = function () {
    var grid = this.gridModel.gridData;
    var selection = grid.getSelection();
    var insertIndex = selection.y;
    if (insertIndex == 1) return;
    for (var i = 0, count = selection.height; i < count; i++) {
        grid.insertRows(insertIndex + i - 1, 1, insertIndex + i, COM.widget.Grid.COPY_BOTH);
        grid.deleteRows(insertIndex + i + 1, 1);
        var cell1 = grid.getCell(0, insertIndex + i - 1);
        var cell2 = grid.getCell(0, insertIndex + i);
        var showText2 = cell1.getShowText();
        cell1.setShowText(cell2.getShowText());
        cell2.setShowText(showText2);
    }
    selection.y = selection.y - 1;
    var curCell = grid.getCurrentCell();
    grid.setCurrentCell(curCell.getCol(), curCell.getRow() - 1);
    grid.select(selection);
};
RO.GridHtml5.prototype.moveDown = function () {
    var grid = this.gridModel.gridData;
    var selection = grid.getSelection();
    var insertIndex = selection.y + selection.height - 1;
    if (insertIndex == grid.getRowCount()) return;
    for (var i = 0, count = selection.height; i < count; i++) {
        grid.insertRows(insertIndex - i + 1, 1, insertIndex - i - 1, COM.widget.Grid.COPY_BOTH);
        var cell1 = grid.getCell(0, insertIndex - i);
        var cell2 = grid.getCell(0, insertIndex - i + 1);
        var showText1 = cell2.getShowText();
        cell2.setShowText(cell1.getShowText());
        cell1.setShowText(showText1);
        grid.deleteRows(insertIndex - i - 1, 1);
    }
    selection.y = selection.y + 1;
    var curCell = grid.getCurrentCell();
    grid.setCurrentCell(curCell.getCol(), curCell.getRow() + 1);
    grid.select(selection);
};
RO.GridHtml5.prototype.insert = function (count, index, operationMode, simpleMode, copyRowHeights, depth, isAuto) {
    if (!this.getCanEdit()) {
        return false;
    }

    var NewRowIndex;
    var insetfalg = -1;//初始为0，即插行，改为1则为插列
    if (RO.Enum.OperationMode.COL) {
        insertfalg = 1;
    } else if (RO.Enum.OperationMode.ROW) {
        insertfalg = 0;
    }

    var oldindex = index;
    var fkey, clientData, template, offset, showText, selection, dataRowAttr;
    var gridModel = this.gridModel;
    var gridData = gridModel.gridData;
    var colCount = gridData.getColCount(), rowCount = gridData.getRowCount();

    if (index == undefined || isNaN(index) || index < 0) {
        var focusCell = gridData.getCurrentCell();
        index = operationMode === RO.Enum.OperationMode.COL ? focusCell.getCol() : focusCell.getRow();
    }

    var updateOrderCache = function (insertPos, prevOrder, nextOrder, insertIndex, insertCount, model, rid, newCache, stopFlag) {

        var createFloatOrders = function (prev, next, orderCount, endflag) {
            var orders = [], gap, orderStep;
            prev = parseFloat(prev);
            next = parseFloat(next);
            var newPrev = Math.ceil(prev);

            var pushOrder = function (data, start, pushCount, pushStep) {
                for (var oIndex = 1; oIndex <= pushCount; oIndex++) {
                    data.push(start + oIndex * pushStep);
                }
            };

            if (next == -1) {
                pushOrder(orders, newPrev, orderCount, 1000);
            } else {
                var newNext = Math.floor(next);

                gap = Math.floor(newNext - newPrev);
                orderStep = Math.floor(gap / (orderCount + 1));
                if (orderStep > 0) {
                    pushOrder(orders, newPrev, orderCount, orderStep);
                } else {
                    gap = parseInt(next * 100000 - prev * 100000) / 100000;
                    var prevStr = prev.toString().split(".");
                    var nextStr = next.toString().split(".");
                    var floatLen = Math.max(prevStr.length == 1 ? 0 : prevStr[1].length, nextStr.length == 1 ? 0 : nextStr[1].length);

                    orderStep = gap / (orderCount + 1);
                    var stepStr = orderStep.toString();
                    var stepStrs = stepStr.split(".");
                    var stepFloatLen = stepStrs.length == 1 ? 0 : stepStrs[1].length;
                    if (stepFloatLen > 3) {
                        var newStep = stepStrs[1].substr(0, floatLen);
                        if (parseFloat(newStep) == 0) {
                            var newFloatLen = floatLen + 1;
                            newStep = stepStrs[1].substr(0, newFloatLen);
                            while (parseFloat(newStep) == 0 && newFloatLen < 3) {
                                newFloatLen++;
                                newStep = stepStrs[1].substr(0, newFloatLen);
                            }
                            if (parseFloat(newStep) > 0 && newFloatLen < 5) {
                                orderStep = parseFloat(stepStrs[0] + "." + newStep);
                            } else {
                                if (endflag) {
                                    orderStep = parseFloat(stepStrs[0] + "." + stepStrs[1].substr(0, 5));
                                } else {
                                    return null;
                                }
                            }
                        } else {
                            stepStr = stepStr.substr(0, stepStr.length - stepFloatLen + floatLen);
                            orderStep = parseFloat(stepStr);
                        }
                    }
                    pushOrder(orders, prev, orderCount, orderStep);
                }
            }

            return orders;
        };

        var newOrders = createFloatOrders(prevOrder, nextOrder, insertCount, stopFlag);

        if (newOrders == null) {
            var cache = model._insertDataCache[insertPos];
            var posCount = 0, newIndex = insertIndex, nex = nextOrder;

            if (!RO.BasicFun.isEmptyObject(cache)) {
                newCache = [];
                for (var item in cache) {
                    posCount++;
                    newCache.push({ "key": cache[item], "value": item });
                }
                newCache.sort(function (a, b) {
                    return a.key - b.key;
                });

                var regionData = model.dataRegions[rid];
                for (var fRow = 0, rc = regionData.FloatDataCount; fRow < rc; fRow++) {
                    var rI = fRow + regionData.RegionTop;
                    var key = model.rowList[rI][RO.Enum.FloatRowKey];
                    if (key == newCache[newCache.length - 1].value) {
                        var attr = model.getDataRowAttrs(null, rI + 1);
                        nex = attr["floatorder"] || -1;
                    }
                    if (key == newCache[0].value) {
                        newIndex = rI - 1;
                    }
                }
            }
            updateOrderCache(insertPos, insertPos, nex, newIndex, insertCount + posCount, model, rid, newCache, true);
        } else if (newCache) {
            model._insertDataCache[insertPos] = {};
            for (var nIndex = 0; nIndex < insertCount; nIndex++) {
                var nAttrs = model.getDataRowAttrs(null, insertIndex + nIndex);
                nAttrs.insertpos = insertPos;
                nAttrs.floatorder = newOrders[nIndex];
                var nRowKey = nAttrs[RO.Enum.FloatRowKey];
                model.setDataRowAttrs(null, insertIndex + nIndex, nAttrs);

                model._insertDataCache[insertPos][nRowKey] = newOrders[nIndex];
            }
        } else {
            for (var iIndex = 0; iIndex < insertCount; iIndex++) {
                var attrs = model.getDataRowAttrs(null, insertIndex + iIndex);
                attrs.insertpos = insertPos;
                attrs.floatorder = newOrders[iIndex];
                var rowKey = attrs[RO.Enum.FloatRowKey];
                model.setDataRowAttrs(null, insertIndex + iIndex, attrs);

                if (!model._insertDataCache[insertPos]) {
                    model._insertDataCache[insertPos] = {};
                }
                model._insertDataCache[insertPos][rowKey] = newOrders[iIndex];
            }
        }
    };

    selection = this.gridModel.gridData.getSelection();
    
    if (this.designMode) {
        gridData.clearSelection(); // 不clear的话表头上的选区会不对
        if (count == undefined || isNaN(count) || count <= 0) {
            count = operationMode == RO.Enum.OperationMode.COL ? selection.width : selection.height;
        }

        if (operationMode == RO.Enum.OperationMode.COL) {
            gridData.insertCols(index, count, index <= 1 ? 1 : index - 1, COM.widget.Grid.COPY_STYLE);
            for (var c0 = index, ccount = gridData.getColCount() ; c0 < ccount; c0++) {
                var ccell = gridData.getCell(c0, 0);
                ccell.setShowText(gridModel.getTitleColIndex(c0), true);
            }
        } else {
            gridData.insertRows(index, count, index <= 1 ? 1 : index - 1, COM.widget.Grid.COPY_STYLE);
            for (var r0 = index, rcount = gridData.getRowCount() ; r0 < rcount; r0++) {
                var rcell = gridData.getCell(0, r0);
                rcell.setShowText(r0, true);
            }
        }
        if (selection) {
            gridData.setCurrentCell(-1, -1, null, true);
            gridData.setCurrentCell(selection.x, selection.y, null, true);
            gridData.select(selection);
        }
        $(gridModel.entry).trigger(RO.Enum.Event.datachange, { state: 'insert', index: index, byRow: operationMode, count: count });
        return;
    }

    var regId = gridModel.options["currentRegId"];
    var region = null;
    if (regId) {
        region = gridModel.dataRegions[regId];
    } else {
        for (var reg in gridModel.dataRegions) {
            region = gridModel.dataRegions[reg];
            if (region.RegionKind === 3) {
                offset = region.RegionTop + region.FloatDataCount - 1;
                if (index === offset || (index < offset && index >= region.RegionTop) || index == offset + 1) {
                    regId = reg;
                    break;
                }
            }
        }
    }
    if (!regId || !region || region.RegionKind != 3) return;
    
    if (count == undefined || isNaN(count) || count <= 0) {
        count = selection.height;
    }
    
    if (region.FixedRecordCount) {
        !isAuto && mui.toast("当前区域设置了固定行数，不能增行。");
        return;
    } else if (region.MaxRecordCount > 0) {
        if (region.FloatDataCount + count > region.MaxRecordCount) {
            !isAuto && mui.toast("当前区域设置了最大行数为" + region.MaxRecordCount);
            count = region.MaxRecordCount - region.FloatDataCount;
            if (count <= 0) {
                return;
            }
        }
    }

    var actionLock = gridModel.beginModifyDataRegions();
    if (actionLock) {
        return;
    }
    gridData.clearSelection(); // 不clear的话表头上的选区会不对
    var regionTemplate = region.RegionTemplate;
    var regTop = region.RegionTop;

    gridModel._curRowIndexCache += count;

    var copyFuncForRow = function (gridSourceCell, sourceCopyIndex) {
        if (regionTemplate && regionTemplate.length > 0 && gridSourceCell.colIndex >= 0) {
            var templateIndex = sourceCopyIndex - regTop;

            // regionTemplate行浮动时第一维是行
            return COM.widget.Grid.GridModel.copyCell(regionTemplate[templateIndex][gridSourceCell.colIndex], COM.widget.Grid.COPY_STYLE);
        } else {
            return COM.widget.Grid.GridModel.copyCell(gridSourceCell, COM.widget.Grid.COPY_STYLE);
        }
    };
    var size = gridData.gridModel.data.rows[regTop].size;
    var sizes = [];
    gridModel.updateGridSize(index, count, regId, false);
    for (var k = 0; k < count; k++) {
        sizes.push(size);
    }
    gridData.insertRows(index, count, regTop, copyFuncForRow, copyRowHeights && copyRowHeights.length > 0 ? copyRowHeights.slice(0, count) : sizes.slice(0, count), depth);
    var copyIndex = index <= regTop ? regTop + count : regTop;
    for (var j = 0; j < count; j++) {
        var newRowIndex = index + j;
        if (j == 0) NewRowIndex = newRowIndex;
        gridModel.setSectionMerge(newRowIndex, copyIndex, region.RegionTemplate[0]);
        fkey = gridModel.newId();
        dataRowAttr = gridModel.getDataRowAttrs(null, newRowIndex);
        dataRowAttr.fkey = fkey;
        gridModel.setDataRowAttrs(null, newRowIndex, dataRowAttr);

        gridModel.ModifyDataState(fkey, 1, true, regId);

        for (var c = 1; c < colCount; c++) {
            var curCell = this.getCell(c, newRowIndex);
            showText = null,editText=null, enumArr=[];
            if (regionTemplate && regionTemplate.length > 0) {
                // template对应后台的GridHtml5Cell类的对象
                template = regionTemplate[0][c];
                clientData = template.clientData;
                 if (clientData["dv"]) {
                    showText = clientData["dv"] == "" ? "" : clientData["dv"].split("|")[0];
                    editText = clientData["dv"] == "" ? "" : clientData["dv"].split("|")[1];
                }
            } else {
                template = this.getCell(c, copyIndex);
                clientData = template.cell.getClientData();
                if (!clientData["lid"]) {
                    showText = template.getShowText();
                }
            }
             if((clientData[RO.Enum.EnumDisplayMode]=="1" || clientData[RO.Enum.EnumDisplayMode]=="dm_InCell")){
                if (clientData[RO.Enum.EnumID] && gridModel.enumData[clientData[RO.Enum.EnumID]]) {
                    enumArr = gridModel.enumData[clientData[RO.Enum.EnumID]];
                    for (var i = 0, enumArr; i < enumArr.length; i++) {
                        if (i == 0) {
                            enumdata = "{\"" + enumArr[i].id + "\":\"" + enumArr[i].name + "\"";
                        } else {
                            enumdata += ",\"" + enumArr[i].id + "\":\"" + enumArr[i].name + "\"";
                        }
                    }
                    if (enumdata != "") {
                        enumdata += "}";
                    }
                    clientData["enumdata"] = enumdata;
                }
           }
            curCell.cell.setClientData(gridModel.copyClientData(clientData));
            gridModel.InitCell(regId, curCell, showText, simpleMode);
//          if (enumArr.length > 0 && (curCell.getAttrByName("edm") == "1" || curCell.getAttrByName("edm") == "dm_InCell")) {
//              var options = {};
//              options.enumOperationType = curCell.getAttrByName("eot") ? curCell.getAttrByName("eot") : curCell.getAttrByName("ems");
//              options.enumDisplayMode = curCell.getAttrByName("edm");
//              options.enumData = enumArr;
//              curCell.setEnumIntoCell(options);
//          }
        }
    }

    var rowInsertPos;
    var preIndex = index - 1;
    if (preIndex == 0) {
        rowInsertPos = 0;
    } else {
        var prevAttrs = gridModel.getDataRowAttrs(null, preIndex);
        rowInsertPos = prevAttrs["insertpos"] !== undefined ? prevAttrs["insertpos"] : prevAttrs["floatorder"] !== undefined ? prevAttrs["floatorder"] : 0;
    }
    var pattr = gridModel.getDataRowAttrs(null, preIndex);
    var pOrder = pattr && pattr["floatorder"] || 0;
    var nattr = gridModel.getDataRowAttrs(null, index + count);
    var nOrder = nattr && nattr["floatorder"] || -1;
    var caches = null;
    updateOrderCache(rowInsertPos, pOrder, nOrder, index, count, gridModel, regId, caches);
    for (var row = index, rowcount = gridData.getRowCount() ; row < rowcount; row++) {
        var rowcell = gridData.getCell(0, row);
        rowcell.setShowText(row, true);
    }

    //修正上一行是否含折叠号,折叠状态可能会有问题！
    if (depth != null && index != 1) {
        var topCell = gridData.getCell(1, index - 1);
        var topDeep = topCell.getDepth();
        if (topDeep == null) {
            topDeep = 0;
        }
        if (topDeep < depth) {
            if (topCell.getDepth() == null) {
                topCell.cell.figure.setRowSpan(topCell.cell.rowSpan);
                topCell.cell.figure.setColSpan(topCell.cell.colSpan);
                if (topCell.getCellMode() == COM.widget.Grid.Cell.Cell_MODE_TREE) {
                    topCell.cell.figure.setDepth(topDeep);
                }
                topCell.cell.figure.setCellMode(COM.widget.Grid.Cell.Cell_MODE_TREE);
                topCell.setDepth(topDeep);
            }
            topCell.setExpandable(true);
            if (!topCell.cell.expanded) {
                gridModel.cellExpand(topCell, gridData);
            }
        }
    }
      var getRegonId = function (row, regiondata) {
        var regonId;
        for (var i in regiondata) {
            if (regiondata[i].RegionTop <= row && (regiondata[i].RegionTop + regiondata[i].FloatDataCount - 1) >= row) {
                regonId = i;
                break;
            }
        }
        return regonId;
    }
    var adjustTreeRecord = function (self, start, count, rid) {
        var rCount = gridModel.gridData.getRowCount();
        var parentIndexes = self.getRecordRelations(rid);
        var parentIndexesSize = 0;
        for (key in parentIndexes) {
            if (parentIndexes.hasOwnProperty(key)) parentIndexesSize++;
        }
        if (parentIndexesSize == 0) return;
        var pIndex = parentIndexes[start];
      
        if (!pIndex) pIndex = parentIndexes[start - 1];       
        for (var r = rCount - 1  ; r >= start + count; r--) {
            var regIdRow = getRegonId(r - count, gridModel.dataRegions);
            if (!regIdRow)continue;
            var parentIndexe = self.getRecordRelations(regIdRow);
            var parent = parentIndexe[r-count];
            if (parent != -1 && parent >= start) {
                parentIndexe[r] = parent + count;
            } else {
                parentIndexe[r] = parent;
            }

        }
        var expandable = false, detailFlag = false, hasRegId = false, hasCell = true;
        var compareCell = gridData.getCell(1, count + start);
        if (compareCell.cell) {
            expandable = compareCell.cell.expandable ? true : false;
            detailFlag = new RO.GridHtml5.Cell(compareCell, gridModel).getDataRowAttr(RO.Enum.extendAttrs) == "1" ? true : false;
            hasRegId = getRegonId(count + start, gridModel.dataRegions) ? true : false;
        } else {
            hasCell = false;
        }
        if (!hasCell || expandable || detailFlag || !hasRegId) {
            var rRelations = self.getRecordRelations(getRegonId(start - 1, gridModel.dataRegions));
            var pISize = 0;
            for (key in rRelations) {
                if (rRelations.hasOwnProperty(key)) pISize++;
            }
            if (pISize == 0) return;
            pIndex = rRelations[start - 1];
        }
        var pIndexCatch = {};
        var startCell = gridData.getCell(1, start - count)
        if (startCell.cell && new RO.GridHtml5.Cell(startCell, gridModel).getDataRowAttr(RO.Enum.extendAttrs) == "1") {
            pIndex = start - count;

        }
        for (var row = start; row < count + start; row++) {
            pIndexCatch[row] = parentIndexes[row];
            parentIndexes[row] = pIndex;
            
        }

    };
    var region = gridModel.dataRegions;
    adjustTreeRecord(this, index, count, regId);
    var filterflag = false;
    for (var regid in region) {
        if (region[regid].RegionExtension && $.parseJSON(region[regid].RegionExtension).OrderIndex > 0) {
            filterflag = true;
        }
    }
    if (filterflag) {
        var addindex = index, addcount = count;
        var autofilter = { "rowIndex": addindex, "rowCount": addcount,"filterType":3, regid: regId };
        if (insertfalg >= 0) {
                 gridModel.filterAutoNoData(autofilter);
        }
    }
  
    // gridModel.updateRegBorderStyle(index, count, region, false);
    gridModel.endModifyDataRegions(regId, count, selection);
    if (!simpleMode) {
        this.timeOutId = window.setTimeout(function () {
            $(gridModel.entry).trigger(RO.Enum.Event.datachange, { state: 'insert', index: index, byRow: operationMode, count: count });
        }, 1);
    }
    return true;
};
RO.GridHtml5.prototype.delete = function (actCount, actIndex, actOperationMode, simpleMode) {
    if (!this.getCanEdit()) {
        return false;
    }

    var delflag = -1;
    if (RO.Enum.OperationMode.COL) {
        delflag = 1;
    } else if (RO.Enum.OperationMode.ROW) {
        delflag = 0
    }
    
    var gridModel = this.gridModel;
    var regId = -1;
    if (!this.designMode) {
        actOperationMode = RO.Enum.OperationMode.ROW;
    }

    var selection = gridModel.gridData.getSelection(), selectionFlag = false;
    if (actIndex == undefined || isNaN(actIndex) || actIndex < 0) {
        actIndex = actOperationMode == RO.Enum.OperationMode.COL ? selection && selection.x : selection && selection.y;
        selectionFlag = true;
    }

    var clearRect = function (rect, self) {
        var changeValues = [];
        var fkey, editTextNew='';
        var editMode = gridModel.gridData.getEditMode();
        for (var i = 0; i < rect.height; i++) {
            for (var j = 0; j < rect.width; j++) {
                var roCell = self.getCell(rect.x + j, rect.y + i);
                if (roCell) {
                    if (roCell.cell.getCheckable()) {
                        roCell.cell.setChecked(false);
                    }
                    var oldValue = roCell.getEditText();
                    if (oldValue != undefined) {
                        var canEdit = roCell.getCanEdit();
                        roCell.setCanEdit(true);
                        fkey = roCell.getDataRowAttr(RO.Enum.FloatRowKey);
                       
                           if (roCell.getAttrByName("dt") == '11') {
                            var figureChildren = roCell.cell.getFigure().children;
                            if ( figureChildren.length) {
                               for (var k = 0; k < figureChildren.length; k++) {
                                    if (figureChildren[k].name == "attachmentImg") {                   
                                        var options = figureChildren[k].options;
                                        options.showText = "(0)";
                                        roCell.setImgIntoCell("attachmentImg", options, false);
                                   }
                               }
                           }
                             editTextNew=roCell.getEditText() && eval(roCell.getEditText());
                             if(editTextNew && editTextNew.length>0){
                             	for(var i=0;i<editTextNew.length;i++){
                             		editTextNew[i].FileState=3;
                             	}
                             }
                             var modified = self.gridModel.setValue(roCell, editTextNew, "",  false, true, null, true, true);
                        }else{
                        	 var modified = self.gridModel.setValue(roCell, "", "", true, true);
                        }
                        if (modified) {
                            changeValues.push({ row: roCell.getRowIndex(), col: roCell.getColIndex(), oldValue: oldValue, newValue: editTextNew, fKey: fkey });
                        }
                        roCell.setCanEdit(canEdit);
                    }
                }
            }
        }
        if (simpleMode) {
            $(self.gridModel.entry).trigger(RO.Enum.Event.valuechange, { "changeValues": changeValues });
        }
    };

    var delFromIndex = function (count, operationMode, index, self, linkIds, deleteDatas) {
        var tmpCell, region, gModel = self.gridModel, curSelection;
        var colCount = self.gridModel.gridData.getColCount();

        tmpCell = operationMode === RO.Enum.OperationMode.COL ? self.getCell(index, 1) : self.getCell(1, index);
        regId = tmpCell.getDataRowAttr(RO.Enum.RegId);
        if (!regId) {
            throw new Error("单元格上缺少reg属性！");
        }
        region = gModel.dataRegions[regId];
        if (region.RegionKind != 3) {
            return 0;
        }

        var actionLock = gModel.beginModifyDataRegions();
        if (actionLock) {
            return 0;
        }
        var regTop = region.RegionTop;
        curSelection = self.gridModel.gridData.getSelection();

        //修正待删除起始坐标、删除行数
        if (count == undefined || isNaN(count) || count <= 0) {
            count = curSelection.height;
        }

        for (var sr = 0; sr < count; sr++) {
            var rcell = self.getCell(1, index + sr);
            var rfkey = rcell.getDataRowAttr(RO.Enum.FloatRowKey, true);
            if (rfkey && rfkey != "0") {
                var rowInsertPos = rcell.getDataRowAttr("insertpos");
                if (rowInsertPos) {
                    var cacheRows = gModel._insertDataCache[rowInsertPos];
                    if (cacheRows && cacheRows[rfkey] !== undefined) {
                        delete cacheRows[rfkey];
                    }
                    if (RO.BasicFun.isEmptyObject(cacheRows)) {
                        delete gModel._insertDataCache[rowInsertPos];
                    }
                }
            }
            for (var colIndex = 1; colIndex < colCount ; colIndex++) {
                var cell = self.getCell(colIndex, index + sr);
                if (cell == null)
                    continue;
                var lid = cell.getAttrByName("lid");
                if (lid != null && linkIds.indexOf(lid) < 0) {
                    linkIds.push(lid);
                }

                curRowIndex = cell.getRowIndex();
                curColIndex = cell.getColIndex();
                if (deleteDatas[curRowIndex]) {
                    colDatas = deleteDatas[curRowIndex];
                    colDatas[curColIndex] = cell.getEditText();
                } else {
                    colDatas = new Array();
                    colDatas[curColIndex] = cell.getEditText();
                    deleteDatas[curRowIndex] = colDatas;
                }
            }
        }
        // 剩余行数不够保留的行数：
        var defaultCount = region.DefaultCount + 1;
        if (region.MaxRecordCount > 0 && region.MaxRecordCount < defaultCount) {
            defaultCount = region.MaxRecordCount;
        }
        var clearCount = defaultCount - (region.FloatDataCount - count);

        if (clearCount > 0) {
            var rRect = { x: 1, y: index, width: colCount - 1, height: clearCount };
            // 此情况需触发valuechange事件
            clearRect(rRect, self);


            var insertPos;
            if (index == regTop) {
                insertPos = 1000;
            } else {
                insertPos = gModel.rowList[index - 1]["floatorder"] || 1000;
            }
            if (!gModel._insertDataCache) {
                gModel._insertDataCache = {};
            }
            if (!gModel._insertDataCache[insertPos]) {
                gModel._insertDataCache[insertPos] = {};
            }
            for (var clearI = 0; clearI < clearCount; clearI++) {
                var attrs = gModel.getDataRowAttrs(null, index + clearI);
                attrs.insertpos = insertPos;
                attrs.floatorder = Math.ceil(insertPos) + 1000 * (clearI + 1);
                var rowKey = attrs["fkey"];
                gModel.setDataRowAttrs(null, index + clearI, attrs);
                gModel._insertDataCache[insertPos][rowKey] = attrs.floatorder;
            }

            index = index + clearCount;
            count -= clearCount;
        }

        if (count > 0) {
            // 由于存在保留行的场景，做删行操作时有的行可能只是执行了清空，因此删除状态的修改调整到真正做删行时进行
            for (var sr = 0; sr < count; sr++) {
                var c = self.getCell(1, index + sr);
                var key = c.getDataRowAttr("fkey", true);
                if (key && key != "0") {
                    gModel.ModifyDataState(key, 2, false, regId);
                }
            }
            $(self.gridModel.entry).trigger(RO.Enum.Event.beforeDataChange, { state: 'delete', index: index, byRow: operationMode, count: count });

            gModel.updateRegBorderStyle(index, count, region, true);
            gModel.gridData.deleteRows(index, count);
            for (var row = index, rCount = gModel.gridData.getRowCount() ; row < rCount; row++) {
                var rowcell = gModel.gridData.getCell(0, row);
                rowcell.setShowText(row, true);
            }
            gModel.updateGridSize(index, count, regId, true);
        }
        gModel.endModifyDataRegions(regId, 0 - count, curSelection);
        return count;
    };

    var delCount = actCount;
    var linkIds = new Array();
    var deleteDatas = new Array();
    var deleteRowIndexesByRow = new Array();
    if (this.designMode) {
        var cell;
        var grid = gridModel.gridData;

        if (actCount == undefined || isNaN(actCount) || actCount <= 0) {
            actCount = actOperationMode == RO.Enum.OperationMode.COL ? selection && selection.width : selection && selection.height;
            delCount = actCount;
        }

        if (actOperationMode == RO.Enum.OperationMode.COL) {
            var colCount = grid.getColCount();
            if (actIndex == 1 && actCount >= colCount - 1) {
                if (actCount == 1) {
                    return;
                } else {
                    actIndex++;
                    actCount--;
                }
            }
            grid.deleteCols(actIndex, actCount);
            for (var c0 = actIndex, ccount = grid.getColCount() ; c0 < ccount; c0++) {
                cell = grid.getCell(c0, 0);
                cell.setShowText(gridModel.getTitleColIndex(c0), true);
            }
        } else {
            var rowCount = grid.getRowCount();
            if (actIndex == 1 && actCount >= rowCount - 1) {
                if (actCount == 1) {
                    return;
                } else {
                    actIndex++;
                    actCount--;
                }
            }
            grid.deleteRows(actIndex, actCount);
            for (var r0 = actIndex, rcount = grid.getRowCount() ; r0 < rcount; r0++) {
                cell = grid.getCell(0, r0);
                cell.setShowText(r0, true);
            }
        }
        var selections = grid.getSelection();
        if (selections) {
            if (selections && selections.x < grid.getColCount() && selections.y < grid.getRowCount()) {
                this.setCurrentCell(selections.x, selections.y);
                grid.select(selections);
            } else {
                this.setCurrentCell(1, 1);
            }
        } else {
            this.setCurrentCell(1, 1);
        }

    } else {
        var dCount = actCount;
        if (dCount == null) {
            var selection = gridModel.gridData.getSelection();
            dCount = actOperationMode == RO.Enum.OperationMode.COL ? selection.width : selection.height;
        }

        //树形删除时特殊处理
        var topCell = this.getCell(1, actIndex).cell;
        var topDeep = topCell.getDepth();
        var hasSameLevel = false;
        if (actIndex != 1) {
            topCell = this.getCell(1, actIndex - 1).cell;
            topDeep = topCell.getDepth();
        }
        if (topDeep == null) {
            topDeep = 0;
        }
        var minDeep = 10000;
        for (var j = 0; j < dCount; j++) {
            var curcell = this.getCell(1, actIndex + j).cell;
            var curdeep = curcell.getDepth();
            if (curdeep == null) {
                minDeep = 0;
                break;
            }
            minDeep = Math.min(curdeep, minDeep);
        }
        var rowLength = this.getRowCount();
        var downCount = 0;
        for (var k = actIndex + dCount; k < rowLength; k++) {
            var downcell = this.getCell(1, k).cell;
            var downdeep = downcell.getDepth();
            if (downdeep == null) {
                downdeep = 0;
            }
            if (downdeep == minDeep) {
                downCount = k - actIndex - dCount;
                hasSameLevel = true;
                break;
            } else if (downdeep < minDeep) {
                downCount = k - actIndex - dCount;
                break;
            }
        }
        if (downCount > 0) {
            dCount += downCount;
        }

        actCount = dCount;
        if (actIndex != 1 && (topDeep >= minDeep || !hasSameLevel)) {
            //去掉可折叠号
            if (topCell.getDepth() != null) {
                topCell.setExpandable(false);
            }
        }

        //for (var i = 0; i < dCount; i++) {
        //    var row = actIndex + i;
        //    var rowIndex = this.getDataRowIndex(row);
        //    deleteRowIndexesByRow[row] = rowIndex;
        //}
        $(gridModel.entry).trigger(RO.Enum.Event.beforeDataChange, { state: 'delete', index: actIndex, byRow: actOperationMode, count: dCount });
        delCount = delFromIndex(actCount, actOperationMode, actIndex, this, linkIds, deleteDatas);
    }

    var deleteIndexes = new Array();
    if (!this.designMode) {
        //var parentRowIndexes = this.getRecordRelations(regId);
        //if (parentRowIndexes == null)
        //    return ;
        //var startIndex = deleteRowIndexesByRow[actIndex];
        //for (i = 0; i < delCount; i++) {
        //    var row = actIndex + i;
        //    var rowIndex = deleteRowIndexesByRow[row];
        //    if (parentRowIndexes[rowIndex] != null) {
        //        delete parentRowIndexes[rowIndex];
        //    }
        //}
        ////维护浮动行数据序号间父子关系
        //var endIndex = this.getDataRowIndex(actIndex);
        //for (var childIndex in parentRowIndexes) {
        //    var childRowIndex = Number(childIndex);
        //    if (isNaN(childRowIndex) || childRowIndex < startIndex)
        //        continue;
        //    else if (childRowIndex >= endIndex) {
        //        var parentIndex = parentRowIndexes[childIndex];
        //        if (parentIndex >= endIndex) {
        //            delete parentRowIndexes[childRowIndex];
        //            parentRowIndexes[childRowIndex - delCount] = parentIndex - delCount;
        //        } else {
        //            delete parentRowIndexes[childRowIndex];
        //            parentRowIndexes[childRowIndex - delCount] = parentIndex;
        //        }
        //    }
        //}

        var getDeleteIndexes = function (self, rid, start, count) {
            var recordRelations = self.getRecordRelations(rid);
            var deleteIndexesCache = new Array();
            if (recordRelations == null)
                return deleteIndexesCache;
            var deleteRows;
            var startRow = start;
            for (var i = 0; i < count; i++) {
                var row = startRow + i;
                if (recordRelations[row] != null) {
                    var parentIndex = recordRelations[row];
                    if (deleteIndexesCache[parentIndex]) {
                        deleteRows = deleteIndexesCache[parentIndex];
                        deleteRows.push(row);
                    } else {
                        deleteRows = new Array();
                        deleteRows.push(row);
                        deleteIndexesCache[parentIndex] = deleteRows;
                    }
                }
            }
            return deleteIndexesCache;
        };

        var adjustTreeRecord = function (self, start, count, rid) {
            var parentIndexes,
                rCount = gridModel.gridData.getRowCount() + count,
                regions = gridModel.dataRegions,
                curRegTop = regions[rid].RegionTop,
                regs = gridModel._floatRegList,
                cou = regs.length,
                regsI = 0;

            parentIndexes = self.getRecordRelations(rid);

            for (var row = 0; row < count; row++) {
                delete parentIndexes[row + start];
            }

            for (var r = start + count; r < rCount; r++) {
                var parent = parentIndexes[r];
                if (parent !== undefined) {
                    if (parent >= start) {
                        parent -= count;
                    }
                    parentIndexes[r - count] = parent;
                    delete parentIndexes[r];
                } else {
                    for (var c = regsI; c < cou; c++) {
                        if (regs[c].index < curRegTop) break;

                        var newIndexes = self.getRecordRelations(regs[c].id);
                        var newParent = newIndexes[r];
                        if (newParent !== undefined) {
                            parent = newParent;
                            parentIndexes = newIndexes;
                            if (parent >= start) {
                                parent -= count;
                            }
                            parentIndexes[r - count] = parent;
                            delete parentIndexes[r];

                            regsI = c;
                            break;
                        }
                    }
                }
            }
        };

        deleteIndexes = getDeleteIndexes(this, regId, actIndex, delCount);
        adjustTreeRecord(this, actIndex, delCount, regId);

        var filterflag = false;
        var region = gridModel.dataRegions;
        for (var regid in region) {
            if (region[regid].RegionExtension && $.parseJSON(region[regid].RegionExtension).OrderIndex > 0) {
                filterflag = true;
            }
        }
        if (filterflag) {
            var delcount = actCount;
            var autofilter = { "rowIndex": actIndex, "rowCount": delcount, "filterType":2};
            if (delflag >= 0) {
                gridModel.filterAutoNoData(autofilter);
            }
        }
    }

    $(gridModel.entry).trigger(RO.Enum.Event.datachange, { state: 'delete', index: actIndex, byRow: actOperationMode, count: delCount, linkIds: linkIds, deleteDatas: deleteDatas, regId: regId, deleteIndexes: deleteIndexes, actCount: actCount });

    return true;
};
RO.GridHtml5.prototype.blur = function () {
    this.gridModel.gridData.blur();
};
RO.GridHtml5.prototype.setSize = function (width, height) {
    this.gridModel.gridData.setSize(width, height);
};
RO.GridHtml5.prototype.getDataRegion = function (regId) {
    var region = this.gridModel.dataRegions[regId];
    return region;
};
RO.GridHtml5.prototype.getRecordRelations = function (regId) {
    var region = this.gridModel.dataRegions[regId];
    if (region == null || region.RecordRelations == null)
        return null;
    return region.RecordRelations;
};
RO.GridHtml5.prototype.getDataRegions = function () {
    return this.gridModel.dataRegions;
};
RO.GridHtml5.prototype.storeData = function (submitAllFlag, updateAfterSave, storeTreeData) {
    var formData = {}, fload = {}, fixed = {}, dataFlag = false, sIndex, count, fixedRegId = -1, fdls = {}, region, fkey, state, dls, floatOrder, insertPos;
    var model = this.gridModel;
    var grid = model.gridData;
    var savedCellCaches = model.savedCellCaches;

    var colCount = grid.getColCount(), rowCount = grid.getRowCount();
    var dataRegions = model.dataRegions;
    var appendFloatDataByLink = function (dataLinks, col, row, cCount, rCount, self) {
        var modifiedFlag = false, hasValue = false;
        for (var i = 0; i < rCount; i++) {
            for (var j = 0; j < cCount; j++) {
                var curCell = self.getCell(col + j, row + i);
                if (!curCell) continue;
                var curLid = curCell.getAttrByName(RO.Enum.DataLinkId);
                var curDf = curCell.getAttrByName(RO.Enum.EnumID);
                var isHide = grid.isRowHidden(row + i) && grid.isColHidden(col + j);
                if (!isHide && curLid) {
                    var modified = !!curCell.getAttrByName("modified", true);
                    if (updateAfterSave && modified) {
                        savedCellCaches.push({ "col": col + j, "row": row + i });
                    }

                    modifiedFlag = modifiedFlag || modified;
                    var editText = curCell.getEditText(), extEditText = curCell.getAttrByName(RO.Enum.ExtEditText);
                    var enumshowText = curCell.getShowText();
                    hasValue = hasValue || (extEditText !== undefined && extEditText !== null) || (!!editText && !curCell.getAttrByName(RO.Enum.EmptyCell)) || (!!enumshowText);
                    var val = extEditText === undefined || extEditText === null ? (editText === undefined || editText === null ? "" : editText) : extEditText;
                    if (curDf != null && curDf > 0) {
                        val = "";
                        if (editText != "" && editText != null) {
                            enumshowText = enumshowText.split(';');
                            editText = editText.split(';');
                            for (var k = 0; k < enumshowText.length; k++) {
                                val += editText[k] + "|" + enumshowText[k];
                                if (k <= enumshowText.length - 1) {
                                    val += ";";
                                }
                            }
                        } else {
                            val = enumshowText;
                        }
                    }
                    dataLinks[curLid] = { Value: val, Modified: modified };
                }
            }
        }

        return modifiedFlag && hasValue;
    };

    var checkHasValue = function (col, row, cCount, rCount, self) {
        var regs = self.gridModel.dataRegions;
        var hasValue = false, specialCell = false;
        for (var i = 0; i < rCount; i++) {
            for (var j = 0; j < cCount; j++) {
                var numFlag = false;
                for (var r in regs) {
                    if (i + row >= regs[r].RegionTop && row + i <= regs[r].RegionTop + regs[r].FloatDataCount-1) {
                        if (regs[r].RegionExtension && j+col == $.parseJSON(regs[r].RegionExtension).OrderIndex) {
                            var numFlag = true;
                        }
                           
                    }
                }
                if (!numFlag) {
                    var curCell = self.getCell(col + j, row + i);
                    if (!curCell) continue;
                    var curLid = curCell.getAttrByName(RO.Enum.DataLinkId);
                    var acl = curCell.getAttrByName(RO.Enum.AutoCalcLink);

                    if (acl) {
                        continue;
                    }

                    var isHide = grid.isRowHidden(row + i) && grid.isColHidden(col + j);
                    //var cellMode = curCell.cell.getCellMode();
                    //if (cellMode == COM.widget.Grid.Cell.Cell_MODE_TREE) {
                    //    specialCell = true;
                    //}

                    if (!isHide && curLid) {
                        var editText = curCell.getEditText(), extEditText = curCell.getAttrByName(RO.Enum.ExtEditText);
                        var showText = curCell.getShowText();
                        hasValue = hasValue || (curCell.cell.getCheckable() ? curCell.cell.isChecked() : (extEditText !== undefined && extEditText !== null && extEditText !== "") || (!!editText && !curCell.getAttrByName(RO.Enum.EmptyCell)) || (!!showText));
                        if (hasValue) {
                            break;
                        }
                    }
                }
            }
            if (hasValue || specialCell) {
                break;
            }
        }
        return hasValue || specialCell;
    };

    //处理浮动区域
    var states = model.TempDataStates || {};
    for (var regId in dataRegions) {
        var gapOverflow = false;
        region = dataRegions[regId];
        var regionKind = region.RegionKind;
        if (regionKind == 0) {
            fixedRegId = regId;
            continue;
        }
        var regionData = [];
        var regStates = states[regId];
        if (regStates) {
            for (var fk in regStates) {
                var delState = regStates[fk];
                if (delState && delState.State == 3) {
                    regionData.push({ Key: fk, State: 3, DataLinks: {}, FloatOrder: 0, InsertPos: 0 });
                }
            }
        }
        if (regionKind == 3) {
            sIndex = region.RegionTop;
            count = region.FloatDataCount;
            var treeNodes = {};
            for (var rIndex = 0; rIndex < count; rIndex++) {
                dls = {};
                var rStart = rIndex + sIndex;
                var rCell = this.getCell(1, rStart);
                if (!rCell) continue;
                //支持填报对象重构树形级次存储
                var deep = rCell.cell.getDepth();
                if (deep == null) {
                    deep = 0;
                }
                fkey = rCell.getDataRowAttr(RO.Enum.FloatRowKey);
                treeNodes[deep] = fkey;
                state = regStates && regStates[fkey];
                floatOrder = rCell.getDataRowAttr("floatorder");
                insertPos = rCell.getDataRowAttr("insertpos");
                if (state) {
                    if (state.State == 1) {
                        if (checkHasValue(1, rStart, colCount - 1, 1, this)) {
                            var rModifiedFlag = appendFloatDataByLink(dls, 1, rStart, colCount - 1, 1, this);
                            if (!rModifiedFlag) continue;
                            if (floatOrder.toString().indexOf(".") >= 0) {
                                gapOverflow = true;
                            }
                            if (storeTreeData != null && storeTreeData) {
                                //支持填报对象重构树形级次存储
                                var pk = null;
                                if (deep != 0) {
                                    pk = treeNodes[deep - 1];
                                }
                                regionData.push({ Key: fkey, State: 1, DataLinks: dls, FloatOrder: floatOrder, InsertPos: insertPos, Pk: pk });
                            } else {
                                regionData.push({ Key: fkey, State: 1, DataLinks: dls, FloatOrder: floatOrder, InsertPos: insertPos });
                            }
                        } else {
                            if (RO.BasicFun.isEmptyObject(model._insertCacheAfterSave[insertPos])) {
                                model._insertCacheAfterSave[insertPos] = [];
                            }
                            model._insertCacheAfterSave[insertPos].push({ "fkey": fkey, "index": rStart });
                        }
                    } else if (state.State == 2) {
                        if (checkHasValue(1, rStart, colCount - 1, 1, this)) {
                            appendFloatDataByLink(dls, 1, rStart, colCount - 1, 1, this);
                            regionData.push({ Key: fkey, State: 2, DataLinks: dls, FloatOrder: floatOrder, InsertPos: insertPos });
                        } else {
                            state.State = 3;
                            regionData.push({ Key: fkey, State: 3, DataLinks: {}, FloatOrder: 0, InsertPos: 0 });
                        }
                    }

                } else if (submitAllFlag) {
                    appendFloatDataByLink(dls, 1, rStart, colCount - 1, 1, this);
                    regionData.push({ Key: fkey, State: 0, DataLinks: dls, Index: rIndex });

                }
            }

        }
        if (regionData.length > 0) {
            if (gapOverflow) {
                regionData[0].GapOverflow = gapOverflow;
            }
            fload[regId] = regionData;
            if (!dataFlag) dataFlag = true;
        }
    }
    if (dataFlag) {
        formData["Fload"] = fload;
    }

    //处理固定区域
    dataFlag = false;
    for (var r = 1, c = 1; r < rowCount && c < colCount;) {
        var cell = this.getCell(c, r);
        if (cell) {
            var reg = cell.getDataRowAttr(RO.Enum.RegId);
            if (reg) {
                region = dataRegions[reg];
                if (region.RegionKind == 0) {
                    if (fixedRegId == -1) {
                        fixedRegId = reg;
                    }
                } else {
                    r = region.RegionBottom + region.FloatDataCount;
                    c = 1;
                    continue;
                }
                var lid = cell.getAttrByName(RO.Enum.DataLinkId, true);
                var curDf = cell.getAttrByName(RO.Enum.EnumID);
                if (lid) {
                    var extEditText = cell.getAttrByName(RO.Enum.ExtEditText);
                    var val = extEditText === undefined || extEditText === null ? cell.getEditText() : extEditText;
                    val = val == undefined ? "" : val;

                    if (curDf != null && curDf > 0) {
                        var showText = cell.getShowText();
                        if (val != "" && val != null) {
                            showText = showText.split(';');
                            var editText = val.split(';');
                            val = "";
                            for (var k = 0; k < showText.length; k++) {
                                val += editText[k] + "|" + showText[k];
                                if (k <= showText.length - 1) {
                                    val += ";";
                                }
                            }
                        } else {
                            val = showText;
                        }
                    }

                    var modif = !!cell.getAttrByName("modified", true);
                    if (modif) {
                        fdls[lid] = { Value: val, Modified: modif };
                    }
                    if (!dataFlag) dataFlag = true;

                    if (updateAfterSave && modif) {
                        savedCellCaches.push({ "col": cell.getColIndex(), "row": cell.getRowIndex() });
                    }
                }
            }
        }
        if (++c >= colCount) {
            c = 1;
            r++;
        }
    }
    if (dataFlag) {
        fixed[fixedRegId] = fdls;
        formData["Fixed"] = fixed;
    }

    if (formData["Fixed"] || formData["Fload"]) {
        formData["IsPaste"] = model.isPasted;
        return formData;
    }
    return null;
};
RO.GridHtml5.prototype.storeStyle = function (noCompressed) {
    var selectableFalse = new Array();
    var editableFalse = new Array();
    var fitFontSizeTrue = new Array();
    var expandedTure = new Array();
    var wrapLineFalse = new Array();
    var vertTextTure = new Array();
    var fontItalicTure = new Array();
    var fontBoldTure = new Array();

    var setFormData = function (src, des) {
        des.options = src.options;
        delete des.options.DataRegions;
        des.mergeCells = src.mergeCells;
        des.rows = src.rows;
        des.cols = src.cols;
        des.colHeaderCount = src.colHeaderCount;
        des.rowHeaderCount = src.rowHeaderCount;
        des.colFooterCount = src.colFooterCount;
        des.rowFooterCount = src.rowFooterCount;
        des.defaultEditorId = src.defaultEditorId;
        des.rowCount = src.rowCount;
        des.colCount = src.colCount;
        data.options.cellBoolAttrs = {};
        data.options.cellBoolAttrs["selectableFalse"] = selectableFalse;
        data.options.cellBoolAttrs["editableFalse"] = editableFalse;
        data.options.cellBoolAttrs["fitFontSizeTrue"] = fitFontSizeTrue;
        data.options.cellBoolAttrs["expandedTure"] = expandedTure;
        data.options.cellBoolAttrs["wrapLineFalse"] = wrapLineFalse;
        data.options.cellBoolAttrs["vertTextTure"] = vertTextTure;
        data.options.cellBoolAttrs["fontItalicTure"] = fontItalicTure;
        data.options.cellBoolAttrs["fontBoldTure"] = fontBoldTure;

    };

    var initCellData = function (srcCell, desCell, isCompr) {
        //列号
        desCell.colIndex = srcCell.colIndex;
        //行号
        desCell.rowIndex = srcCell.rowIndex;

        if (isCompr) {
            if (srcCell.editorId && srcCell.editorId != 1) {
                desCell.editorId = srcCell.editorId;
            }
            if (srcCell.title != "") {
                desCell.title = srcCell.title;
            }
            if (srcCell.showText != "") {
                desCell.showText = srcCell.showText;
            }
            if (srcCell.editText != "") {
                desCell.editText = srcCell.editText;
            }
            if (!srcCell.selectable) {
                selectableFalse.push(r * cc + c);
            }
            if (!srcCell.editable) {
                editableFalse.push(r * cc + c);
            }

            var ubcolor = RO.Enum.DefaultBorderColor.toUpperCase();
            var color1 = RO.Enum.DefaultBorderColor + "," + RO.Enum.DefaultBorderColor;
            var color2 = color1 + ",black,black";
            var color3 = ubcolor + "," + ubcolor;
            var color4 = color3 + ",black,black";
            var bColor = srcCell.borderColor && srcCell.borderColor.join();
            if (bColor && bColor != color1 && bColor != color2 && bColor != color3 && bColor != color4) {
                desCell.borderColor = srcCell.borderColor;
            }
            if (srcCell.decoration && srcCell.decoration != COM.widget.Grid.Cell.DECORATION.NONE) {
                desCell.decoration = srcCell.decoration;
            }
            if (srcCell.html != "") {
                desCell.html = srcCell.html;
            }
            //desCell.checked = srcCell.checked;

            //desCell.checkable = srcCell.checkable;

            //desCell.expanded = srcCell.expanded;

            //desCell.expandable = srcCell.expandable;


            //合并单元格相关
            if (srcCell.rowSpan && srcCell.rowSpan != 1) {
                desCell.rowSpan = srcCell.rowSpan;
            }
            if (srcCell.colSpan && srcCell.colSpan != 1) {
                desCell.colSpan = srcCell.colSpan;
            }

            //desCell.merged = srcCell.merged;

            //背景样式
            if (srcCell.backStyle !== undefined && srcCell.backStyle != 1) {
                desCell.backStyle = srcCell.backStyle;
            }

            //背景颜色
            if (srcCell.backColor && srcCell.backColor.toLowerCase() != "white" && srcCell.backColor.toLowerCase() != "#ffffff") {
                desCell.backColor = srcCell.backColor;
            }

            if (srcCell.foreColor && srcCell.foreColor.toLowerCase() != "white" && srcCell.foreColor.toLowerCase() != "#fff") {
                desCell.foreColor = srcCell.foreColor;
            }

            //边框相关
            //[右，下，对角线，反对角线]
            if (srcCell.border && srcCell.border.join() != "-1,-1,0,0") {
                desCell.border = srcCell.border;
            }

            if (!!srcCell.fontName && srcCell.fontName != "宋体") {
                desCell.fontName = srcCell.fontName;
            }
            if (srcCell.fontSize && srcCell.fontSize != 14) {
                desCell.fontSize = srcCell.fontSize;
            }

            if (srcCell.fontBold) {
                fontBoldTure.push(r * cc + c);
            }

            if (srcCell.fontItalic) {
                fontItalicTure.push(r * cc + c);
            }
            if (srcCell.fontColor && srcCell.fontColor != '#000000' && srcCell.fontColor != '#0' && srcCell.fontColor != 'black') {
                desCell.fontColor = srcCell.fontColor;
            }

            //描边
            //textStroke = srcCell.textStroke || null;
            //阴影
            //textShadow = srcCell.textShadow || null;


            if (!srcCell.wrapLine) {
                wrapLineFalse.push(r * cc + c);
            }
            if (srcCell.indent) {
                desCell.indent = srcCell.indent;
            }
            if (srcCell.horzAlign !== undefined && srcCell.horzAlign != 0) {
                desCell.horzAlign = srcCell.horzAlign;
            }
            if (srcCell.vertAlign !== undefined && srcCell.vertAlign != 0) {
                desCell.vertAlign = srcCell.vertAlign;
            }
            if (srcCell.vertText) {
                vertTextTure.push(r * cc + c);
            }

            if (srcCell.fitFontSize) {
                fitFontSizeTrue.push(r * cc + c);
            }

            //if (!srcCell.multiLine) {
            //    multiLineFalse.push(r * cc + c);
            //}

            if (srcCell.cellMode != undefined && srcCell.cellMode != COM.widget.Grid.Cell.Cell_MODE_NORMAL) {
                desCell.cellMode = srcCell.cellMode;
            }

            // desCell.padding = srcCell.padding || RO.Enum.DefaultCellPadding; // 默认不设置padding，否则单元格大小和字体大小接近时文字会有被遮挡的情况
            //其他数据存储点
            if (srcCell.clientData && srcCell.clientData != {}) {
                desCell.clientData = srcCell.clientData;
            }
        } else {
            desCell.colIndex = srcCell.colIndex;
            //行号
            desCell.rowIndex = srcCell.rowIndex;
            desCell.editorId = srcCell.editorId;
            desCell.title = srcCell.title;
            desCell.showText = srcCell.showText;
            desCell.editText = srcCell.editText;
            desCell.selectable = srcCell.selectable;
            desCell.editable = srcCell.editable;
            desCell.cellMode = srcCell.cellMode;
            desCell.html = srcCell.html;
            desCell.control = null;
            //合并单元格相关
            desCell.rowSpan = srcCell.rowSpan;
            desCell.colSpan = srcCell.colSpan;
            desCell.merged = !!srcCell.merged;
            desCell.mergeInfo = srcCell.mergeInfo;
            //背景样式
            desCell.backStyle = srcCell.backStyle;
            //背景颜色
            desCell.backColor = srcCell.backColor;
            desCell.foreColor = srcCell.foreColor;

            //边框相关
            //[右，下，对角线，反对角线]
            desCell.border = srcCell.border;
            desCell.borderColor = srcCell.borderColor;

            desCell.fontName = srcCell.fontName;
            desCell.fontSize = srcCell.fontSize;
            desCell.fontBold = !!srcCell.fontBold;
            desCell.fontItalic = !!srcCell.fontItalic;
            desCell.fontColor = srcCell.fontColor;
            //描边
            //textStroke = srcCell.textStroke || null;
            //阴影
            //textShadow = srcCell.textShadow || null;
            desCell.wrapLine = srcCell.wrapLine;
            desCell.indent = srcCell.indent;
            desCell.horzAlign = srcCell.horzAlign;
            desCell.vertAlign = srcCell.vertAlign;
            desCell.vertText = !!srcCell.vertText;
            desCell.fitFontSize = !!srcCell.fitFontSize;
            desCell.multiLine = true;
            //下划线和删除线，没有就设为null
            desCell.decoration = srcCell.decoration;
            desCell.padding = srcCell.padding; // 默认不设置padding，否则单元格大小和字体大小接近时文字会有被遮挡的情况
            //其他数据存储点
            desCell.clientData = srcCell.clientData || {};
        }
    };

    var data = {};
    var origdata = this.gridModel.gridData.gridModel.data;

    var rowList = [];
    var oldRowList = origdata.cells;

    for (var r = 0, rc = this.getRowCount() ; r < rc; r++) {
        var row = [];
        for (var c = 0, cc = this.getColCount() ; c < cc; c++) {
            var cell = {};
            initCellData(oldRowList[r][c], cell, !noCompressed);
            row.push(cell);
        }
        rowList.push(row);
    }
    data.cells = rowList;

    setFormData(origdata, data);


    return { formData: data };
};
RO.GridHtml5.prototype.setOptionByName = function (key, value) {
    if (!value && this.gridModel.options[key]) delete this.gridModel.options[key];
    this.gridModel.options[key] = value;
};
RO.GridHtml5.prototype.getOptionByName = function (key) {
    return this.gridModel.options[key];
};
RO.GridHtml5.prototype.headerLock = function (col, row) {
    var grid = this.gridModel.gridData;
    var drawArea = grid.getDrawAreaInfo();
    var rowbegin = drawArea.startRow;
    var colbegin = drawArea.startCol;

    if (this.isHeaderLocked()) {
        return false;
    }

    if (rowbegin > row || colbegin > col || drawArea.endRow < row || drawArea.endCol < col) {
        return false;
    }

    var colIndex = 1, rowIndex = 1, curCell;
    if (arguments.length === 0) {
        curCell = this.getCurrentCell();
        if (!curCell) {
            return false;
        } else {
            rowIndex = curCell.getRowIndex();
            colIndex = curCell.getColIndex();
        }
    } else if (arguments.length === 2) {
        if (isNaN(col) || isNaN(row)) {
            curCell = this.getCurrentCell();
            if (!curCell) {
                return false;
            }
            rowIndex = curCell.getRowIndex();
            colIndex = curCell.getColIndex();
        } else {
            rowIndex = row;
            colIndex = col;
        }
    }

    this.gridModel.lockHeaderRowCache = [];
    this.gridModel.lockHeaderColCache = [];
    var rowlist = this.gridModel.lockHeaderRowCache, collist = this.gridModel.lockHeaderColCache, cols = [], rows = [];

    for (var ii = 1; ii < rowbegin; ii++) {
        if (!grid.isRowHidden(ii)) {
            rowlist[ii] = true;
            rows.push(ii);
        }
    }
    for (var jj = 1; jj < colbegin; jj++) {
        if (!grid.isColHidden(jj)) {
            collist[jj] = true;
            cols.push(jj);
        }
    }
    grid.setScrollTop(0, true);
    grid.setScrollLeft(0, true);
    grid.setRowsVisible(rows, false, true);
    grid.setColsVisible(cols, false, true);
    grid.setRowHeaderCount(rowIndex, true);
    grid.setColHeaderCount(colIndex, true);
    grid.refresh();
    return true;
};
RO.GridHtml5.prototype.headerUnlock = function () {
    if (!this.isHeaderLocked()) {
        return false;
    }

    var grid = this.gridModel.gridData, rowlist = this.gridModel.lockHeaderRowCache, collist = this.gridModel.lockHeaderColCache, rows = [], cols = [];

    if (rowlist.length == 0) {
        var preRowHeaderCount = grid.getRowHeaderCount();
        for (var i = 1; i < preRowHeaderCount; i++) {
            if (grid.isRowHidden(i)) {
                rows.push(i);
            }
        }
    } else {
        for (var ii = 0; ii < rowlist.length; ii++) {
            if (rowlist[ii]) {
                rows.push(ii);
            }
        }
    }
    if (collist.length == 0) {
        var preColHeaderCount = grid.getColHeaderCount();
        for (var j = 1; j < preColHeaderCount; j++) {
            if (grid.isColHidden(j)) {
                cols.push(j);
            }
        }
    } else {
        for (var jj = 0; jj < collist.length; jj++) {
            if (collist[jj]) {
                cols.push(jj);
            }
        }
    }
    this.gridModel.lockHeaderRowCache = [];
    this.gridModel.lockHeaderColCache = [];
    grid.setScrollTop(0, true);
    grid.setScrollLeft(0, true);
    grid.setRowsVisible(rows, true, true);
    grid.setColsVisible(cols, true, true);
    grid.setRowHeaderCount(1, true);
    grid.setColHeaderCount(1, true);
    grid.refresh();
    return true;
};
RO.GridHtml5.prototype.isHeaderLocked = function () {
    var grid = this.gridModel.gridData;
    return grid.getColHeaderCount() > 1 || grid.getRowHeaderCount() > 1;
};
RO.GridHtml5.prototype.isRowHidden = function (rowIndex) {
    return this.gridModel.gridData.isRowHidden(rowIndex);
};
RO.GridHtml5.prototype.isColHidden = function (colIndex) {
    return this.gridModel.gridData.isColHidden(colIndex);
};
RO.GridHtml5.prototype.setRowVisible = function (row, visible) {
    if (!this.getCanEdit()) {
        return false;
    }
    this.gridModel.gridData.setRowVisible(row, visible);
};
RO.GridHtml5.prototype.setColVisible = function (col, visible) {
    if (!this.getCanEdit()) {
        return false;
    }
    this.gridModel.gridData.setColVisible(col, visible);
};
RO.GridHtml5.prototype.findByDataLinks = function (lids, fkey, fromCol, fromRow) {
    var cells = [], regForFkey = null, cell, lid;
    if (lids && !RO.BasicFun.isArray(lids)) {
        var lidArr = [];
        lids = lidArr.push(lids);
    }
    var rowCount = this.gridModel.gridData.getRowCount();
    var colCount = this.gridModel.gridData.getColCount();
    for (var r = 1; r < rowCount; r++) {
        for (var c = 1; c < colCount; c++) {
            cell = this.getCell(c, r);
            if (!cell) {
                var args = [];
                args.push(lids);
                args.push(fkey);
                args.push(c);
                args.push(r);
                this.gridModel.addAsynAction("findByDataLinks", args);
                this.center(c, r);
                return [];
            }
            lid = cell.getAttrByName("lid", true);
            if (!lid) {
                continue;
            }
            regForFkey = cell.getDataRowAttr(RO.Enum.FloatRowKey);
            if (regForFkey) {
                if (fkey === undefined && (jQuery.inArray(lid, lids) != -1 || jQuery.inArray(parseInt(lid), lids) != -1)) {
                    cells.push(cell);
                } else {
                    if (regForFkey === fkey && (jQuery.inArray(lid, lids) != -1 || jQuery.inArray(parseInt(lid), lids) != -1)) {
                        cells.push(cell);
                    }
                }
            } else {
                if ((jQuery.inArray(lid, lids) != -1) || (jQuery.inArray(parseInt(lid), lids) != -1)) {
                    cells.push(cell);
                }
            }

        }
    }
    if (arguments.length === 4) {
        $(this).trigger("findByDataLinksCallBack", { data: cells });
    }
    return cells;
};
RO.GridHtml5.prototype.getWidth = function () {
    return this.gridModel.gridData.getWidth();
};
RO.GridHtml5.prototype.getColCount = function () {
    return this.gridModel.gridData.getColCount();
};
RO.GridHtml5.prototype.getRowCount = function () {
    return this.gridModel.gridData.getRowCount();
};
RO.GridHtml5.prototype.getRowCells = function (rowIndex) {
    var cells = [];
    for (var i = 0, count = this.gridModel.gridData.getColCount() ; i < count; i++) {
        cells.push(this.getCell(i, rowIndex, true)[0]);
    }
    return cells;
};
RO.GridHtml5.prototype.getColCells = function (colIndex) {
    var cells = [];
    for (var i = 0, count = this.gridModel.gridData.getRowCount() ; i < count; i++) {
        cells.push(this.getCell(colIndex, i));
    }
    return cells;
};
RO.GridHtml5.prototype.getExtendData = function () {
    if (!RO.BasicFun.alertNullGrid(this.grid)) {
        return null;
    }
    return this.gridMode.options["extendInfo"];
};
RO.GridHtml5.prototype.paste = function () {
    //var t = new Date().getTime();
    var gridModel = this.gridModel;
    var me = this, autoRowHeights = [], rowNums = [], rowSizes = [], modified = false, cellModified = false, startInsert = 0;
    var tempvalue, flag = false, tempstart = 0, tempend = 0;

    var editMode = this.gridModel.gridData.getEditMode();

    if (!this.getCanEdit()) return;

    if (me.gridModel.gridData.isEditing()) {
        var myField = $(".gridEditor", me.gridModel.entry)[0];
        flag = true;
        if (document.selection) { //IE support  
            var sel = document.selection.createRange();
            var copy = sel.duplicate();
            copy.moveToElementText(myField);
            copy.setEndPoint('EndToEnd', sel);
            tempstart = copy.text.length - sel.text.length;
            tempend = myField.selectionStart + sel.text.length;
            tempvalue = $(".gridEditor", me.gridModel.entry).val();
        } else if (myField.selectionStart || myField.selectionStart == '0') {
            tempstart = myField.selectionStart;
            tempend = myField.selectionEnd;
            tempvalue = myField.value;
        }
    }

    var initAutoRowHeights = function (rowHeights, values, curRowIndex, curColIndex, dataColCount, dataRowCount, fixRegion) {
        var row, col;
        for (var r = 0; r < dataRowCount; r++) {
            var height = 0;
            var rowData = values[r];

            for (var c = 0; c < dataColCount; c++) {
                var colData = rowData[c];
                if (fixRegion) {
                    row = curRowIndex + r;
                    col = curColIndex + c;
                } else {
                    row = curRowIndex;
                    col = curColIndex + c;
                }
                var cell = me.getCell(col, row);
                height = Math.max(height, me.gridModel.getAutoRowHeight(cell, colData));
                if (height > COM.widget.Grid.GridFigure.MAXROWHEIGHT) {
                    height = COM.widget.Grid.GridFigure.MAXROWHEIGHT;
                }
            }

            rowHeights.push(height);
        }
    };

    var toArray = function (str) {
        var cCount = 0, rows = [];
        var newStr = String(str);
        if (newStr.lastIndexOf('\r\n') != -1 && (newStr.length - 2) == newStr.lastIndexOf('\r\n')) {
            newStr = newStr.substring(0, newStr.length - 2);
        }

        if (newStr.length > 0 && newStr.indexOf("\"") > -1) {
            var strArr = newStr.split("");
            for (var i = 0; i < strArr.length; i++) {
                if (strArr[i] == "\"") {
                    if (strArr[i + 1] == "\"") { //连续的双引号为正常显示的一个双引号，不作为回车判断的特殊字符
                        i++;
                        continue;
                    } else {  // 进一步处理单元格内文字回车, 双引号开头表示回车文字开始
                        strArr[i] = ""; // 清除起始双引号
                        var n = i;
                        while (++n < strArr.length) { // 向前搜索，遇到回车便替换
                            i = n;
                            if (strArr[n] == "\"") {
                                if (strArr[n + 1] == "\"") {
                                    n++;
                                    i++;
                                    continue;
                                } else {  // 双引号结尾表示回车文字结束
                                    strArr[n] = ""; // 清除结束双引号
                                    break;
                                }
                            }
                            if (strArr[n] == "\n") {
                                strArr[n] = "<br>";
                            }
                        }
                    }
                }
            }
            newStr = strArr.join("").replace(/\"\"/g, "\"");
        }
        if (flag) {
            var tempstartvalue = tempvalue.substring(0, tempstart);
            var tempendvalue = tempvalue.substring(tempend, tempvalue.length);
            newStr = tempstartvalue + newStr + tempendvalue;
            rows.push(newStr);
        } else {
            rows = newStr.split(/\n/g);
        }

        if (rows && rows.length > 0 && rows[rows.length - 1] !== '') {
            rows.push('');
        }

        var rCount = rows.length - 1;
        for (var rIndex = 0; rIndex < rCount; rIndex++) {
            var row = rows[rIndex].replace(/<br>/g, "\n");
            if (row != null && row != "\r") {
                row = row.split('\t');
                rows[rIndex] = row;
                cCount = Math.max(row.length, cCount);
            } else if (row == "\r") {
                rows[rIndex] = [];
            }
        }
        return { values: rows, colCount: cCount, rowCount: rCount };
    };

    var setData = function (dataColC, dataRowC, cColIndex, cRowIndex, datas) {
        var rowData, roCell, colData;
        var changeValues = [];
        var rowCount = me.gridModel.gridData.getRowCount();

        for (var r = 0; r < dataRowC; r++) {
            var rowIndex = cRowIndex + r;
            if (rowIndex >= rowCount) {
                break;
            }

            var heightOrgin = me.gridModel.gridData.getRowHeight(rowIndex);
            var height = heightOrgin;

            rowData = datas[r];
            if (rowData == null) continue;

            for (var c = 0; c < dataColC; c++) {
                roCell = me.getCell(cColIndex + c, cRowIndex + r);
                if (!roCell || (editMode != COM.widget.Grid.EDIT_MODE_EDIT && !roCell.getCanEdit())) continue;
                var oldVal = roCell.getEditText(),oldTitle=roCell.getShowText();
                var fkey = roCell.getDataRowAttr(RO.Enum.FloatRowKey);
                var dt = roCell.getAttrByName("dt");
                if (dt == "11" || dt == "6") continue;
                colData = c < rowData.length ? rowData[c] : "";
                if (colData.length > 0 && colData.charAt(colData.length - 1) == '\r') {
                    colData = colData.substring(0, colData.length - 1);
                }

                if (startInsert > r) {
                    height = Math.max(height, me.gridModel.getAutoRowHeight(roCell, colData));
                    if (height > COM.widget.Grid.GridFigure.MAXROWHEIGHT) {
                        height = COM.widget.Grid.GridFigure.MAXROWHEIGHT;
                    }
                }

                var eid = roCell.getAttrByName(RO.Enum.EnumID);
                if (eid != null && eid > 0) {
                    cellModified = me.gridModel.setValue(roCell, "", colData, true, false, null, true);
                } else {
                    cellModified = me.gridModel.setValue(roCell, colData, colData, true, false, null, true);
                }

                modified = modified || cellModified;
                if (cellModified) {
                    changeValues.push({ row: roCell.getRowIndex(), col: roCell.getColIndex(), oldValue: oldVal, newValue: roCell.getEditText(),oldTitle:oldTitle,newTitle:roCell.getShowText(), fKey: fkey });
                }
                if (!me.gridModel.isPasted && modified)
                    me.gridModel.isPasted = true;
            }

            if (height > 0 && rowIndex < rowCount && heightOrgin != height) {
                rowNums.push(rowIndex);
                rowSizes.push(height);
            }
        }

        if (modified)
            $(me.gridModel.entry).trigger(RO.Enum.Event.valuechange, { "changeValues": changeValues, "operationType": "paste" });
    };

    var needPasteStyle = function (dataArr) {
        // dataArr中最后一个元素为空字符串，长度比CopyRegion大1//已修改
        if (dataArr.rowCount == CopyRegion.height && dataArr.colCount == CopyRegion.width) {
            return true;
        } else {
            CopyRegion = {};
            return false;
        }
    };

    var pasteCell = function (dColIndex, dRowIndex, dataArr) {
        if (CopyRegion.cellRows && CopyRegion.cellRows.length > 0) {
            var sColCount = CopyRegion.width;
            var sRowCount = CopyRegion.height;
            var sColIndex = CopyRegion.x;
            var sRowIndex = CopyRegion.y;
            var maxRowIndex = me.gridModel.gridData.getRowCount() - 1;
            var maxColIndex = me.gridModel.gridData.getColCount() - 1;
            var dRowIndexBoundary = Math.min(dRowIndex + dataArr.rowCount - 1, maxRowIndex); //dataArr中最后一个元素为空字符串
            var dColIndexBoundary = Math.min(dColIndex + dataArr.colCount - 1, maxColIndex);

            // 判断粘贴到的目标区域中是否有合并单元格
            var mergeCells = me.gridModel.gridData.gridModel.data.mergeCells;
            var pasteRect = new Rectangle(dColIndex, dRowIndex, dColIndexBoundary - dColIndex + 1, dRowIndexBoundary - dRowIndex + 1);
            for (var i = 0; i < mergeCells.length; i++) {
                var mergeRect = new Rectangle(mergeCells[i].col, mergeCells[i].row, mergeCells[i].width, mergeCells[i].height);
                if (pasteRect.intersects(mergeRect)) {
                    alert("不能粘贴到合并单元格！");
                    return;
                }
            }
            // 粘贴单元格
            $(me.gridModel.entry).trigger(RO.Enum.Event.beginChangeCell, { "changeRegion": { x: dColIndex, y: dRowIndex, width: dColIndexBoundary - dColIndex + 1, height: dRowIndexBoundary - dRowIndex + 1 }, "copyRegion": CopyRegion, "operationType": "paste" });
            var changeValues = [];
            var rowCount = me.gridModel.gridData.getRowCount();
            for (var sr = 0, dr = dRowIndex; sr <= sRowCount - 1 && dr <= dRowIndexBoundary; sr++, dr++) {
                var height = 0;
                var rowHeight = me.getRowHeight(dr);

                for (var sc = 0, dc = dColIndex; sc <= sColCount - 1 && dc <= dColIndexBoundary; dc++, sc++) {
                    var sCell = CopyRegion.cellRows[sr][sc];
                    var dCell = me.gridModel.gridData.gridModel.getCell(dc, dr);
                    changeValues.push({ row: dr, col: dc, oldValue: dCell.showText, newValue: sCell.showText,oldTitle:dCell.showText,newTitle:sCell.showText, fKey: null });
                    me.gridModel.gridData.gridModel.pasteCell(sCell, dCell, COM.widget.Grid.PASTE_BOTH);

                    var cell = me.getCell(dc, dr);
                    if (sc == 0) {
                        var leftBorderStyle = parseInt(cell.getAttrByName("leftCopyBorderStyle"));
                        var leftBorderColor = cell.getAttrByName("leftCopyBorderColor");
                        var leftCell = me.getCell(dc - 1, dr);
                        if (leftBorderColor) {
                            var lcolors = leftCell.getBorderColor();
                            leftCell.setBorderColor(leftBorderColor, lcolors[1]);
                        }
                        if (!isNaN(leftBorderStyle)) {
                            var lstyles = leftCell.getBorderStyle();
                            leftCell.setBorderStyle(leftBorderStyle, lstyles[1]);
                        }
                    }
                    if (sr == 0) {
                        var topBorderStyle = parseInt(cell.getAttrByName("topCopyBorderStyle"));
                        var topBorderColor = cell.getAttrByName("topCopyBorderColor");
                        var topCell = me.getCell(dc, dr - 1);
                        if (topBorderColor) {
                            var tcolors = topCell.getBorderColor();
                            topCell.setBorderColor(tcolors[0], topBorderColor);
                        }
                        if (!isNaN(topBorderStyle)) {
                            var tstyles = topCell.getBorderStyle();
                            topCell.setBorderStyle(tstyles[0], topBorderStyle);
                        }
                    }

                    height = Math.max(height, me.gridModel.getAutoRowHeight(cell, cell.getShowText()));
                    if (height > COM.widget.Grid.GridFigure.MAXROWHEIGHT) {
                        height = COM.widget.Grid.GridFigure.MAXROWHEIGHT;
                    }
                }

                if (rowCount > dr && height > 0 && height > rowHeight) {
                    rowNums.push(dr);
                    rowSizes.push(height);
                }

            }

            // 处理合并单元格：源区域中的合并单元格对应的在目标区域中也做一次合并
            if (CopyRegion.merges && CopyRegion.merges.length > 0) {
                var merges = CopyRegion.merges;
                for (var i = 0; i < merges.length; i++) {
                    var dx = merges[i].col - sColIndex; // 合并单元格的横向偏移
                    var dy = merges[i].row - sRowIndex; // 合并单元格的纵向偏移

                    var dMergeRow = dRowIndex + dy;
                    var dMergeCol = dColIndex + dx;
                    if (dMergeRow > maxRowIndex || dMergeCol > maxColIndex)
                        continue;
                    var dMergeWidth = dMergeCol + merges[i].width - 1 <= maxColIndex ? merges[i].width : maxColIndex - dMergeCol + 1;
                    var dMergeHeight = dMergeRow + merges[i].height - 1 <= maxRowIndex ? merges[i].height : maxRowIndex - dMergeRow + 1;
                    me.mergeCells(dMergeCol, dMergeRow, dMergeWidth, dMergeHeight);
                }
            }
            $(me.gridModel.entry).trigger(RO.Enum.Event.valuechange, { "changeValues": changeValues, "operationType": "paste" });
        }
    };

    var curCell = this.getCurrentCell();
    var textAreaP = $(".clipTextArea", me.gridModel.entry)[0];
    textAreaP.focus();
    textAreaP.select();
    setTimeout(function () {
        var text = textAreaP.value;
        textAreaP.blur();
        if (text && curCell) {
            var curColIndex = curCell.getColIndex();
            var curRowIndex = curCell.getRowIndex();
            var regId = curCell.getDataRowAttr(RO.Enum.RegId);
            var dataArr = toArray(text);
            var values = dataArr.values;
            var dataRowCount = dataArr.rowCount;
            var dataColCount = dataArr.colCount;
            var isNotSelection = true;
            var isTreePaste = false;

            isTreePaste = $(me.gridModel.entry).triggerHandler(RO.Enum.Event.beforeOnPaste, text) || isTreePaste;
            if (isTreePaste) {
                return;
            }

            var selection = me.gridModel.gridData.getSelection();
            if (selection.height != 1 && selection.width != 1) {
                isNotSelection = false;
                dataRowCount = Math.min(dataRowCount, selection.height);
                dataColCount = Math.min(dataColCount, selection.width);
            }

            if (me.designMode) {
                var rCount = me.gridModel.gridData.getRowCount();
                var cCount = me.gridModel.gridData.getColCount();
                var insertColCount = dataColCount - (cCount - curColIndex);
                var insertRowCount = dataRowCount - (rCount - curRowIndex); //剪切板补了一个空行，这里减掉
                if (insertColCount > 0) {
                    me.insert(insertColCount, cCount, RO.Enum.OperationMode.COL, true);
                }
                if (insertRowCount > 0) {
                    me.insert(insertRowCount, rCount, RO.Enum.OperationMode.ROW, true);
                }
                startInsert = dataRowCount;
            } else if (regId && isNotSelection) {
                var region = me.gridModel.dataRegions[regId];
                if (region.RegionKind == 3) {
                    dataColCount = Math.min(dataColCount, me.gridModel.gridData.getColCount() - curColIndex);

                    initAutoRowHeights(autoRowHeights, values, curRowIndex, curColIndex, dataColCount, dataRowCount);

                    var regionBottom = region.RegionTop + region.FloatDataCount - 1;
                    var insertRowC = curRowIndex + dataRowCount - regionBottom;//dataRowCount没加“”
                    if (insertRowC > 0) {
                        if (me.gridModel.denyInsertByPaste) {
                            dataRowCount -= insertRowC;
                            if (insertRowC > 2) {//最后一行为空，为粘贴自增一行
                                setTimeout(function () {
                                    $(me.gridModel.entry).trigger(RO.Enum.Event.pasteDataPartLost);
                                }, 100);
                            }
                        } else {
                            startInsert = regionBottom + 1 - curRowIndex;
                            var prevCell = me.gridModel.gridData.getCell(1, regionBottom);
                            if (prevCell != null && prevCell.getCellMode() === COM.widget.Grid.Cell.Cell_MODE_TREE) {
                                me.insert(insertRowC, regionBottom + 1, RO.Enum.OperationMode.ROW, true, autoRowHeights.slice(startInsert), prevCell.getDepth());
                            } else {
                                me.insert(insertRowC, regionBottom + 1, RO.Enum.OperationMode.ROW, true, autoRowHeights.slice(startInsert));
                            }
                        }
                    } else {
                        startInsert = dataRowCount;
                    }
                } else {
                    dataRowCount = Math.min(dataRowCount, me.gridModel.gridData.getRowCount() - curRowIndex);
                    dataColCount = Math.min(dataColCount, me.gridModel.gridData.getColCount() - curColIndex);
                    startInsert = dataRowCount;
                    initAutoRowHeights(autoRowHeights, values, curRowIndex, curColIndex, dataColCount, dataRowCount, true);
                }
            } else {
                dataRowCount = Math.min(dataRowCount, me.gridModel.gridData.getRowCount() - curRowIndex);
                dataColCount = Math.min(dataColCount, me.gridModel.gridData.getColCount() - curColIndex);
                startInsert = dataRowCount;
                initAutoRowHeights(autoRowHeights, values, curRowIndex, curColIndex, dataColCount, dataRowCount, true);
            }
            if (me.designMode && needPasteStyle(dataArr)) {
                pasteCell(curColIndex, curRowIndex, dataArr);
            } else {
                setData(dataColCount, dataRowCount, curColIndex, curRowIndex, values);
            }
            if (rowNums.length > 0) {
                me.gridModel.gridData.setRowsHeight(rowNums, rowSizes, true);
            }
            me.gridModel.gridData.focus();
          
        }
    }, 0);
   
};
RO.GridHtml5.prototype.copy = function () {
    var me = this;
    var copyToBoard = function (content) {
        if (content) {
            var textAreaC = $(".clipTextArea", me.gridModel.entry)[0];
            textAreaC.value = content;
            textAreaC.focus();
            textAreaC.select();
            setTimeout(function () {
                textAreaC.blur();
                //textAreaC.value = "";
                me.gridModel.gridData.focus();
            }, 200);
        }
    };
    var text = [];
    var selections = this.getSelections();
    if (selections.length > 0) {
        var area = selections[0];
        var right = area.x + area.width - 1;
        var bottom = area.y + area.height - 1;
        var colspan = 0;
        var curCell = this.getCell(right, bottom);
        if (curCell) {
            colspan = curCell.getAttrByName("colspan");
        }
        if (colspan > 0) {
            right += colspan - 1;
        }
        var rows = [];
        for (var r = area.y; r <= bottom; r++) {
            var rowText = [];
            var rowCells = [];
            for (var c = area.x; c <= right; c++) {
                var selectedCell = this.getCell(c, r);
                var copyCell = this.gridModel.gridData.gridModel.copyCell(selectedCell.cell.cell, COM.widget.Grid.COPY_BOTH);

                if (!copyCell.clientData) {
                    copyCell.clientData = {};
                }

                if (r == area.y) {
                    var topCell = this.getCell(c, r - 1);
                    copyCell.clientData.topCopyBorderColor = topCell.getBorderColor()[1];
                    copyCell.clientData.topCopyBorderStyle = topCell.getBorderStyle()[1];
                }
                if (c == area.x) {
                    var leftCell = this.getCell(c - 1, r);
                    copyCell.clientData.leftCopyBorderColor = leftCell.getBorderColor()[0];
                    copyCell.clientData.leftCopyBorderStyle = leftCell.getBorderStyle()[0];
                }

                rowCells.push(copyCell);
                var dt = selectedCell.getAttrByName("dt");
                if (dt == "11" && dt == "6") {
                    rowText.push("");
                    continue;
                }
                if (selectedCell) {
                    var cellText = selectedCell.getShowText();
                    if (cellText != undefined) {
                        cellText = cellText.toString();
                        if (cellText.indexOf("&nbsp;") >= 0) cellText = cellText.replace(/&nbsp;/g, " ");
                        if (cellText.indexOf("<br />") >= 0) cellText = cellText.replace(/<br \/>/g, "\n");
                        if (cellText.indexOf("<br>") >= 0) cellText = cellText.replace(/<br>/g, "\n");
                        if (cellText.indexOf("<BR>") >= 0) cellText = cellText.replace(/<BR>/g, "\n");
                        if (cellText.indexOf("<BR />") >= 0) cellText = cellText.replace(/<BR \/>/g, "\n");
                    }
                    if (/\n/.test(cellText)) {
                        if (/\"/g.test(cellText)) {
                            cellText = cellText.replace(/\"/g, "\"\"");
                        }
                        cellText = "\"" + cellText + "\"";
                    }
                    rowText.push(cellText || " ");
                }
            }
            text.push(rowText.join("\t"));
            rows.push(rowCells);
        }
        // 找到复制区域中的合并区域
        var merges = this.gridModel.gridData.gridModel.data.mergeCells;
        var containMerges = [];
        var copyRect = new Rectangle(area.x, area.y, area.width, area.height);
        for (var i = 0; i < merges.length; i++) {
            var mergeRect = new Rectangle(merges[i].col, merges[i].row, merges[i].width, merges[i].height);
            if (copyRect.contains(mergeRect)) {
                containMerges.push(merges[i]);
            }
        }
        // 初始化复制区域
        CopyRegion = { cellRows: rows, merges: containMerges, x: area.x, y: area.y, width: area.width, height: area.height };

        //text.push("");
        text = text.join("\r\n");
        copyToBoard(text);
    }
};
RO.GridHtml5.prototype.cut = function (rect) {
    if (!rect) {
        rect = this.gridModel.gridData.getSelections()[0];
    }
    this.gridModel.cut(rect);
};
RO.GridHtml5.prototype.setCopyFunction = function (fn, obj) {
    this.gridModel.gridData.setCopyFunction(fn, obj);
};
RO.GridHtml5.prototype.setPasteFunction = function (fn, obj) {
    this.gridModel.gridData.setPasteFunction(fn, obj);
};
RO.GridHtml5.prototype.getRowHeight = function (rowIndex) {
    return this.gridModel.gridData.getRowHeight(rowIndex);
};
RO.GridHtml5.prototype.setRowHeight = function (rowIndex, height) {
    this.gridModel.gridData.setRowHeight(rowIndex, height);
};
RO.GridHtml5.prototype.getColWidth = function (colIndex) {
    return this.gridModel.gridData.getColWidth(colIndex);
};
RO.GridHtml5.prototype.setColWidth = function (colIndex, width) {
    this.gridModel.gridData.setColWidth(colIndex, width);
};
RO.GridHtml5.prototype.center = function () {
    var hWinWidth = this.gridModel.gridData.getWidth() / 2;
    var hWinHeight = this.gridModel.gridData.getHeight() / 2;
    var scrollWidth = this.gridModel.gridData.getScrollWidth();
    var scrollHeight = this.gridModel.gridData.getScrollHeight();
    var offsetX = 0, offsetY = 0;
    var rowIndex = this.getRowCount();
    var colIndex = this.getColCount();
    var rStart = this.gridModel.gridData.getRowHeaderCount() - 1;
    var cStart = this.gridModel.gridData.getColHeaderCount() - 1;
    if (arguments.length === 1) {
        rowIndex = arguments[0].getRowIndex();
        colIndex = arguments[0].getColIndex();
    } else if (arguments.length === 2) {
        rowIndex = arguments[1];
        colIndex = arguments[0];
    }
    for (var r = rStart; r < rowIndex; r++) {
        if (this.isRowHidden(r)) continue;
        offsetY += this.gridModel.gridData.getRowHeight(r);
    }
    for (var c = cStart; c < colIndex; c++) {
        if (this.isColHidden(c)) continue;
        offsetX += this.gridModel.gridData.getColWidth(c);
    }
    offsetX -= hWinWidth;
    offsetY -= hWinHeight;
    this.gridModel.gridData.setScrollLeft(Math.min(scrollWidth, offsetX < 0 ? 0 : offsetX));
    this.gridModel.gridData.setScrollTop(Math.min(scrollHeight, offsetY < 0 ? 0 : offsetY));
    var self = this;
    self.setCurrentCell(colIndex, rowIndex);
};
RO.GridHtml5.prototype.mergeCells = function (col, row, width, height) {
    if (!this.getCanEdit()) {
        return false;
    }

    var color, oldColor;

    if (width > 1 || height > 1) {

        if (width > 1) {
            for (var c = 0; c < width; c++) {
                var tCell = this.getCell(col + c, row);
                var bCell = this.getCell(col + c, row + height - 1);
                color = bCell.getBorderColor();
                oldColor = tCell.getBorderColor();
                tCell.setBorderColor(oldColor[0], color[1]);
            }
        }
        if (height > 1) {
            for (var r = 0; r < height; r++) {
                var lCell = this.getCell(col, row + r);
                var rCell = this.getCell(col + width - 1, row + r);
                color = rCell.getBorderColor();
                oldColor = lCell.getBorderColor();
                lCell.setBorderColor(color[0], oldColor[1]);
            }
        }

        var cell = this.getCell(col, row);
        if (cell) {
            var backColor = cell.getBackColor();
            for (var i = 0; i < width; i++) {
                for (var j = 0; j < height; j++) {
                    if (i == 0 && j == 0) {
                        continue;
                    }
                    var newCell = this.getCell(col + i, row + j);
                    newCell.setBackColor(backColor);
                }
            }
        }

        var returnValue = this.gridModel.gridData.mergeCell(col, row, width, height);
        //避免合并后，当前选择的单元格不是左上角单元格的话，显示出单元格分割线的问题，参见JQGG-510
        this.gridModel.gridData.setCurrentCell(col, row, null, true);
        return returnValue;
    }
    return false;
};
RO.GridHtml5.prototype.unMergeCell = function (col, row, force) {
    if (!this.getCanEdit()) {
        return false;
    }

    var color, oldColor, rCell, bCell, lCell, tCell, i;
    var cell = this.getCell(col, row);
    var rowSpan = cell.getRowSpan();
    var colSpan = cell.getColSpan();
    if (rowSpan == 1 && colSpan > 1) {
        rCell = this.getCell(col + colSpan - 1, row);
        lCell = this.getCell(col, row);
        color = lCell.getBorderColor();
        oldColor = rCell.getBorderColor();
        rCell.setBorderColor(color[0], oldColor[1]);

        for (i = 0; i < colSpan - 1; i++) {
            cell = this.getCell(col + i, row);
            oldColor = cell.getBorderColor();
            cell.setBorderColor(RO.Enum.DefaultBorderColor, oldColor[1]);
        }

    } else if (colSpan == 1 && rowSpan > 1) {
        bCell = this.getCell(col, row + rowSpan - 1);
        tCell = this.getCell(col, row);
        color = tCell.getBorderColor();
        oldColor = bCell.getBorderColor();
        bCell.setBorderColor(oldColor[0], color[1]);

        for (i = 0; i < rowSpan - 1; i++) {
            cell = this.getCell(col, row + i);
            oldColor = cell.getBorderColor();
            cell.setBorderColor(oldColor[0], RO.Enum.DefaultBorderColor);
        }

    } else if (colSpan > 1 && rowSpan > 1) {
        var lastTop = this.getCell(col + colSpan - 1, row);
        var lastLeft = this.getCell(col, row + rowSpan - 1);
        var last = this.getCell(col + colSpan - 1, row + rowSpan - 1);
        var rColor = lastLeft.getBorderColor()[0];
        var bColor = lastTop.getBorderColor()[1];
        last.setBorderColor(rColor, bColor);

        for (var r = 0; r < rowSpan - 1; r++) {
            lCell = this.getCell(col, row + r);
            color = lCell.getBorderColor();
            rCell = this.getCell(col + colSpan - 1, row + r);
            oldColor = rCell.getBorderColor();
            rCell.setBorderColor(color[0], RO.Enum.DefaultBorderColor);
        }
        for (var c = 0; c < colSpan - 1; c++) {
            tCell = this.getCell(col + c, row);
            color = tCell.getBorderColor();
            bCell = this.getCell(col + c, row + rowSpan - 1);
            oldColor = bCell.getBorderColor();
            bCell.setBorderColor(RO.Enum.DefaultBorderColor, color[1]);
        }

        for (r = 0; r < rowSpan - 1; r++) {
            for (c = 0; c < colSpan - 1; c++) {
                cell = this.getCell(col + c, row + r);
                cell.setBorderColor(RO.Enum.DefaultBorderColor, RO.Enum.DefaultBorderColor);
            }
        }

    }

    return this.gridModel.gridData.unMergeCell(col, row, force);
};
RO.GridHtml5.prototype.setGridTitleBar = function (showFlag) {
    var gridModel = this.gridModel;
    if (showFlag) {
        gridModel.gridData.setRowHeight(0, gridModel._titleRowHeight);
        gridModel.gridData.setColWidth(0, gridModel._titleColWidth);
    } else {
        gridModel.gridData.setRowHeight(0, 0);
        gridModel.gridData.setColWidth(0, 0);
    }
};
RO.GridHtml5.prototype.getMouseWheelHeight = function (step) {
    var grid = this.gridModel.gridData;
    var drawArea = grid.getDrawAreaInfo();
    var start, end, height = 0;

    if (step < 0) {
        end = Math.max(drawArea.startRow - 1, 1);
        start = Math.max(end + step + 1, 1);
    } else {
        start = drawArea.startRow;
        end = Math.min(start + step - 1, grid.getRowCount() - 1);
    }

    for (var i = start; i <= end; i++) {
        height += grid.getRowHeight(i);
    }

    return step < 0 ? 0 - height : height;

};
RO.GridHtml5.prototype.getMouseWheelWidth = function (step) {
    var grid = this.gridModel.gridData;
    var drawArea = grid.getDrawAreaInfo();
    var start, end, width = 0;

    if (step < 0) {
        start = Math.max(drawArea.startCol, 1);
        end = Math.min(start - step - 1, grid.getColCount() - 1);
    } else {
        end = drawArea.endCol;
        start = Math.max(end - step + 1, 1);
    }

    for (var i = start; i <= end; i++) {
        width += grid.getColWidth(i);
    }

    return step < 0 ? 0 - width : width;

};
RO.GridHtml5.prototype.getDataRowAttrByName = function (name, colIndex, rowIndex) {
    var flag = colIndex == undefined || isNaN(colIndex) || colIndex < 0 || rowIndex == undefined || isNaN(rowIndex) || rowIndex < 0;
    var cell = flag ? this.getCurrentCell() : this.getCell(colIndex, rowIndex);
    return cell && cell.getDataRowAttr(name);
};
RO.GridHtml5.prototype.setDataRowAttrByName = function (name, value, colIndex, rowIndex) {
    if (!this.getCanEdit()) return;
    var flag = colIndex == undefined || isNaN(colIndex) || colIndex < 0 || rowIndex == undefined || isNaN(rowIndex) || rowIndex < 0;
    var cell = flag ? this.getCurrentCell() : this.getCell(colIndex, rowIndex);
    return cell.setDataRowAttr(name, value);
};
RO.GridHtml5.prototype.getCellByPosition = function (clientX, clientY) {
    var model = this.gridModel;
    var offset = $(model.entry).offset();
    var x = clientX - offset.left;
    var y = clientY - offset.top;
    var cell = model.gridData.getCellByPoint(x, y);
    return this.getCell(cell.col, cell.row);
};
RO.GridHtml5.prototype.focusGrid = function () {
    try {
        this.gridModel.gridData.focus();
        return true;
    } catch (ex) {
        console.log(ex.message);
        return false;
    }
};
RO.GridHtml5.prototype.refrashFkey = function (fkeyMaps) {
    if (!this.getCanEdit()) return;
    var model = this.gridModel;
    var tmpStates = model.TempDataStates;

    for (var fkey in fkeyMaps) {
        var data = model.extendData[fkey];
        if (data) {
            var newKey = String(fkeyMaps[fkey]);
            var reg = data[RO.Enum.RegId];
            delete tmpStates[reg][fkey];
            data[RO.Enum.FloatRowKey] = newKey;
            data[RO.Enum.State] = 0;
            delete data["insertpos"];
            delete model.extendData[fkey];
            model.extendData[newKey] = data;
        }
    }
};
RO.GridHtml5.prototype.refrashFloatOrder = function (fkeyMaps) {
    if (!this.getCanEdit()) return;

    var model = this.gridModel;

    delete model._insertDataCache;
    model._insertDataCache = {};

    var getOrders = function (prev, next, orderCount) {
        var orders = [], gap, orderStep;
        prev = parseInt(prev);
        next = parseInt(next);

        var pushOrder = function (orderdata, begin, pushCount, pushStep) {
            for (var oIndex = 1; oIndex <= pushCount; oIndex++) {
                orderdata.push(begin + oIndex * pushStep);
            }
        };

        if (next == -1) {
            pushOrder(orders, prev, orderCount, 1000);
        } else {

            gap = next - prev;
            orderStep = Math.floor(gap / (orderCount + 1));
            if (orderStep > 0) {
                pushOrder(orders, prev, orderCount, orderStep);
            } else {
                orderStep = gap / (orderCount + 1);
                var stepStr = orderStep.toString();
                var stepStrs = stepStr.split(".");
                var len = 1;
                var newStep = stepStrs[1].substr(0, len);
                if (parseFloat(newStep) == 0) {
                    len++;
                    newStep = stepStrs[1].substr(0, len);
                    while (parseFloat(newStep) == 0 && len < 5) {
                        len++;
                        newStep = stepStrs[1].substr(0, len);
                    }
                    if (parseFloat(newStep) > 0 && len < 5) {
                        orderStep = parseFloat(stepStrs[0] + "." + newStep);
                    } else {
                        orderStep = parseFloat(stepStrs[0] + "." + stepStrs[1].substr(0, 5));
                    }
                }
                pushOrder(orders, prev, orderCount, orderStep);
            }
        }

        return orders;
    };

    for (var fkey in fkeyMaps) {
        var data = model.extendData[fkey];
        if (data) {
            var order = fkeyMaps[fkey];
            data["floatorder"] = order;
            delete data["insertpos"];
        }
    }

    var newCache = [];
    for (var insertpos in model._insertCacheAfterSave) {
        var insertposList = model._insertCacheAfterSave[insertpos];
        insertposList.sort(function (a, b) {
            return parseInt(a.index) - parseInt(b.index);
        });
        for (var num = insertposList.length - 1; num > 0; num--) {
            var posItem = insertposList[num];
            var preItem = insertposList[num - 1];
            if (posItem.fkey == preItem.fkey && posItem.index == preItem.index) {
                insertposList.splice(num, 1);
            }
        }

        newCache.push(insertposList);
    }


    for (var i = 0, count = newCache.length; i < count; i++) {
        var newData = newCache[i];
        var start = newData[0]["index"] - 1;
        var end = newData[newData.length - 1]["index"] + 1;
        var extData = model.rowList[start];
        var insertPos = extData["floatorder"] || 0;
        var endData = model.rowList[end];
        var nextPos = endData && endData["floatorder"] || -1;

        if (!model._insertDataCache[insertPos]) {
            model._insertDataCache[insertPos] = {};
        }

        var newOrder = getOrders(insertPos, nextPos, newData.length);

        for (var j = 0, cou = newData.length; j < cou; j++) {
            var item = newData[j];
            var exData = model.extendData[item.fkey];
            if (!exData) {
            } else {
                exData["insertpos"] = insertPos;
                exData["floatorder"] = newOrder[j];
                model._insertDataCache[insertPos][item.fkey] = newOrder[j];
            }
        }
    }

    delete model._insertCacheAfterSave;
    model._insertCacheAfterSave = {};
};
RO.GridHtml5.prototype.updateAfterStoreData = function () {
    if (!this.getCanEdit()) return;

    var model = this.gridModel;
    var cellCache = model.savedCellCaches;

    for (var i = cellCache.length - 1; i >= 0; i--) {
        var cell = this.getCell(cellCache[i].col, cellCache[i].row);
        cell && cell.setAttrByName("modified", null);
    }

    var tmpStates = model.TempDataStates;
    for (var rid in tmpStates) {
        var rStates = tmpStates[rid];
        if (rStates) {
            for (var key in rStates) {
                if (rStates[key].State == 3) {
                    delete rStates[key];
                    var data = model.extendData[key];
                    if (!data) continue;
                    data.state = 1;
                    data.fkey = model.newId();
                    delete model.extendData[key];
                    model.extendData[data.fkey] = data;
                } else if (rStates[key].State == 2) {
                    delete rStates[key];
                    model.extendData[key].state = 0;
                }
            }
        }
    }

    model.savedCellCaches = [];
};
RO.GridHtml5.prototype.getDataRowIndex = function (rowIndex) {
    //获取浮动区域的浮动编号 
    var rid = this.getDataRowAttrByName(RO.Enum.RegId, 1, rowIndex);
    var regions = this.gridModel.dataRegions;
    var region = regions[rid];
    if (region == undefined) {
        return rowIndex;
    }
    var top = region.RegionTop;
    var count = region.FloatDataCount;
    if (region.RegionKind == 0) {
        //固定区域不处理
    } else if (region.RegionKind == 3) {
        //处理行浮动
        var length = count + top - 1;
        if (rowIndex >= top && rowIndex <= length) {
            //取整即可
            rowIndex = rowIndex - top;
        }
    }
    //此时行位于固定区域中，因此只返回行号
    return rowIndex;

};
RO.GridHtml5.prototype.getRowIndexInRegion = function (rid, dataRowIndex) {
    //获取浮动区域浮动行首行RowNum
    var regions = this.gridModel.dataRegions;
    var isresult = false;
    var region = regions[rid];
    if (region == undefined) {
        return 0;
    }
    var top = region.RegionTop;
    var count = region.FloatDataCount;
    if (region.RegionKind == 0) {
        //固定区域不处理
    } else if (region.RegionKind == 2) {
        //列浮动暂不处理
    } else if (region.RegionKind == 3) {
        //处理行浮动
        if (dataRowIndex <= count) {
            dataRowIndex = dataRowIndex + top;
        } else {
            dataRowIndex = count + top;
        }
        isresult = true;
    }

    //若无浮动区域，此时返回第一行
    if (isresult) {
        return dataRowIndex;
    } else {
        return 0;
    }
};
RO.GridHtml5.prototype.isLastRow = function (rowIndex) {
    //获取浮动区域的浮动编号 
    var islast = false;
    var rid = this.getDataRowAttrByName(RO.Enum.RegId, 1, rowIndex);
    var regions = this.gridModel.dataRegions;
    var region = regions[rid];
    if (region == undefined) {
        return false;
    }
    var top = region.RegionTop;
    var count = region.FloatDataCount;
    if (region.RegionKind == 0) {
        //固定区域不处理
    } else if (region.RegionKind == 2) {
        //列浮动暂不处理
    } else if (region.RegionKind == 3) {
        //处理行浮动
        var length = count + top - 1;
        if (rowIndex == length) {
            islast = true;
        }
    }
    //此时行位于固定区域中，因此只返回行号
    return islast;

};
RO.GridHtml5.prototype.getNextRowIndex = function (rowIndex) {
    var regions = this.gridModel.dataRegions;
    var rid = this.getDataRowAttrByName(RO.Enum.RegId, 1, rowIndex);
    var region = regions[rid];
    if (region == undefined) {
        return 0;
    }
    var datarow = this.getDataRowIndex(rowIndex);
    return this.getRowIndexInRegion(rid, datarow + 1);
};
RO.GridHtml5.prototype.denyInsertByPaste = function () {
    this.gridModel.denyInsertByPaste = true;
};
RO.GridHtml5.prototype.formatTextByCell = function (cell, editText) {
    var formatValue = this.gridModel.formatTextByCell(cell.cell, editText);
    return formatValue;
};
RO.GridHtml5.prototype.beginUpdate = function () {
    this.gridModel.gridData.beginUpdate();
};
RO.GridHtml5.prototype.endUpdate = function () {
    this.gridModel.gridData.endUpdate();
};
RO.GridHtml5.prototype.upGradeTreeNode = function (rowList) {
    if (!this.getCanEdit()) return;

    var rows = [];
    var grid = this.gridModel.gridData;
    //判断是否传入参数
    if (rowList == null || rowList.length == 0) {
        //是否有选中数据
        var sels = grid.getSelections();
        var len = sels.length;
        for (var i = 0; i < len; i++) {
            var sel = sels[i];
            var height = sel.height;
            var y = sel.y;
            for (var j = 0; j < height; j++) {
                var row = y + j;
                if (!rows.contains(row)) {
                    rows.push(row);
                }
            }
        }
    } else {
        rows = rowList;
    }

    grid.beginUpdate();

    var uprow = [];
    var rowCount = grid.getRowCount();
    //处理所有升级行
    for (var k = 0; k < rows.length; k++) {
        var currow = rows[k];
        if (uprow.contains(currow)) {
            continue;
        }
        var curCell = grid.getCell(1, currow);
        var deep = curCell.getDepth();
        if (deep == null || deep == 0) {
            grid.endUpdate();
            return;
        }

        //判断上一行是否去掉折叠号
        var upcell = grid.getCell(1, currow - 1);
        var upcelldeep = upcell.getDepth();
        var deleteExpandable = false;
        if (upcelldeep < deep) {
            deleteExpandable = true;
        }
        //设置树形属性
        if (curCell.cell.figure) {
            curCell.cell.figure.setRowSpan(curCell.cell.rowSpan);
            curCell.cell.figure.setColSpan(curCell.cell.colSpan);
            if (curCell.getCellMode() == COM.widget.Grid.Cell.Cell_MODE_TREE) {
                curCell.cell.figure.setDepth(deep - 1);
            }
            curCell.cell.figure.setCellMode(COM.widget.Grid.Cell.Cell_MODE_TREE);
        }
        curCell.setDepth(deep - 1);

        uprow.push(currow);
        var hasSameLevel = false;
        //寻找该节点的所有下级
        for (var m = currow + 1; m < rowCount; m++) {
            if (uprow.contains(m)) {
                break;
            } else {
                var cell = grid.getCell(1, m);
                var celldeep = cell.getDepth();
                var canEdit = cell.cell.editable;
                if (celldeep == null || !canEdit) {
                    break;
                }
                if (celldeep > deep) {
                    if (cell.cell.figure) {
                        cell.cell.figure.setRowSpan(cell.cell.rowSpan);
                        cell.cell.figure.setColSpan(cell.cell.colSpan);
                        if (cell.getCellMode() == COM.widget.Grid.Cell.Cell_MODE_TREE) {
                            cell.cell.figure.setDepth(celldeep - 1);
                        }
                        cell.cell.figure.setCellMode(COM.widget.Grid.Cell.Cell_MODE_TREE);
                    }
                    cell.setDepth(celldeep - 1);
                    if (cell.getExpandable()) {
                        cell.setExpandable(true);
                    }
                    uprow.push(m);
                } else if (celldeep == deep) {
                    //是否有同级节点
                    if (!hasSameLevel)
                        hasSameLevel = true;
                    break;
                } else {
                    break;
                }
            }
        }
        //有折叠号或有同级，则添加折叠号
        if (curCell.getExpandable() || hasSameLevel) {
            curCell.setExpandable(true);
            curCell.setExpanded(true);
        }
        if (deleteExpandable) {
            upcell.setExpandable(false);
        }
    }
    grid.endUpdate();
};
RO.GridHtml5.prototype.downGradeTreeNode = function (rowList) {
    if (!this.getCanEdit()) return;

    var rows = [];
    var grid = this.gridModel.gridData;
    //判断是否传入参数
    if (rowList == null || rowList.length == 0) {
        //是否有选中数据
        var sels = grid.getSelections();
        var len = sels.length;
        for (var i = 0; i < len; i++) {
            var sel = sels[i];
            var height = sel.height;
            var y = sel.y;
            for (var j = 0; j < height; j++) {
                var row = y + j;
                if (!rows.contains(row)) {
                    rows.push(row);
                }
            }
        }
    } else {
        rows = rowList;
    }
    grid.beginUpdate();
    var downrow = [];
    var rowCount = grid.getRowCount();
    //处理所有降级行
    for (var k = 0; k < rows.length; k++) {
        var currow = rows[k];
        if (downrow.indexOf(currow) >= 0 || currow - 1 == 0) {
            grid.endUpdate();
            return;
        }

        var curCell = grid.getCell(1, currow);
        var deep = curCell.getDepth();
        if (deep == null) {
            deep = 0;
        }

        var upcell = grid.getCell(1, currow - 1);
        var upcelldeep = upcell.getDepth();
        var editable = upcell.cell.editable;
        if (upcelldeep == null) {
            upcelldeep = 0;
        }
        var addExpandable = false;

        if (upcelldeep < deep || !editable) {
            grid.endUpdate();
            return;
        }
        if (upcelldeep == deep) {
            addExpandable = true;
        }
        //设置树形属性
        if (curCell.cell.figure) {
            curCell.cell.figure.setRowSpan(curCell.cell.rowSpan);
            curCell.cell.figure.setColSpan(curCell.cell.colSpan);
            if (curCell.getCellMode() == COM.widget.Grid.Cell.Cell_MODE_TREE) {
                curCell.cell.figure.setDepth(deep + 1);
            }
            curCell.cell.figure.setCellMode(COM.widget.Grid.Cell.Cell_MODE_TREE);
        }
        curCell.setDepth(deep + 1);

        downrow.push(currow);
        //寻找该节点的所有下级
        for (var m = currow + 1; m < rowCount; m++) {
            if (downrow.indexOf(m) >= 0) {
                break;
            } else {
                var cell = grid.getCell(1, m);
                var celldeep = cell.getDepth();
                var canEdit = cell.cell.editable;
                if (!canEdit) {
                    break;
                }
                if (celldeep == null) {
                    celldeep = 0;
                }
                if (celldeep > deep) {
                    if (cell.cell.figure) {
                        cell.cell.figure.setRowSpan(cell.cell.rowSpan);
                        cell.cell.figure.setColSpan(cell.cell.colSpan);
                        if (cell.getCellMode() == COM.widget.Grid.Cell.Cell_MODE_TREE) {
                            cell.cell.figure.setDepth(celldeep + 1);
                        }
                        cell.cell.figure.setCellMode(COM.widget.Grid.Cell.Cell_MODE_TREE);
                    }
                    cell.setDepth(celldeep + 1);
                    if (cell.getExpandable()) {
                        cell.setExpandable(true);
                    }
                    downrow.push(m);
                } else {
                    break;
                }
            }
        }
        //有折叠号或有同级，则添加折叠号
        if (curCell.getExpandable()) {
            curCell.setExpandable(true);
        }
        if (addExpandable) {
            if (upcell.getDepth() == null) {
                if (upcell.cell.figure) {
                    upcell.cell.figure.setRowSpan(upcell.cell.rowSpan);
                    upcell.cell.figure.setColSpan(upcell.cell.colSpan);
                    if (upcell.getCellMode() == COM.widget.Grid.Cell.Cell_MODE_TREE) {
                        upcell.cell.figure.setDepth(0);
                    }
                    upcell.cell.figure.setCellMode(COM.widget.Grid.Cell.Cell_MODE_TREE);
                }
                upcell.setDepth(0);
            }
            upcell.setExpandable(true);
            upcell.setExpanded(true);
        }
    }
    grid.endUpdate();
};
RO.GridHtml5.prototype.getGrade = function (rowIndex) {
    var grid = this.gridModel.gridData;
    var rowCount = grid.getRowCount();
    if (rowIndex == null || rowIndex < 1 || rowIndex >= rowCount) {
        var sel = grid.getSelection();
        if (sel.height > 1) {
            return null;
        }
        rowIndex = sel.y;
    }
    var cell = grid.getCell(1, rowIndex);
    var celldeep = cell.getDepth();
    if (celldeep == null) {
        celldeep = 0;
    }
    return celldeep;
};
RO.GridHtml5.prototype.closeEditor = function () {
    var model = this.gridModel;
    if (model.simpleEditType || model.editorOpened) {
        model._cellValueChanged = false;
        model.gridData.closeActiveEditor();
    }
};
RO.GridHtml5.prototype.openEditor = function (col, row, simpleType) {
    this.gridModel.gridData.openEditor(col, row, simpleType);
};
RO.GridHtml5.prototype.isEditing = function () {
    return this.gridModel.gridData.isEditing();
}
RO.GridHtml5.prototype.setCustomGridCursor = function (cursor) {
    var model = this.gridModel;
    if (cursor) {
        if (cursor == "brush") {
            cursor = RO.Enum.gridBrushCursor;
        }
        model._customGridCursor = cursor;
        $("canvas", model.entry).css("cursor", cursor);
    } else {
        model._customGridCursor = null;
        $("canvas", model.entry).css("cursor", "");
    }

};
RO.GridHtml5.prototype.getCustomGridCursor = function () {
    var model = this.gridModel;
    return model._customGridCursor;
};

RO.GridHtml5.prototype.getDrawAreaInfo = function () {
    var model = this.gridModel;
    return model.gridData.getDrawAreaInfo();
};
RO.GridHtml5.prototype.setShowProportion = function (value) {
    Draw2D.setShowProportion(value);
    this.gridModel.gridData.refresh();
};
RO.GridHtml5.prototype.lockEditor = function (isLock) {
    var entry = this.gridModel.entry,
        editor = $(".gridEditor", entry),
        lock = editor.attr("lock");

    if (isLock && lock != "lock") {
        if (!this.gridModel.editorOpened && this.gridModel.gridData.isEditing()) {
            this.gridModel.editorOpened = true;
        }
        editor.attr("lock", "lock");
    } else if (!isLock && lock == "lock") {
        editor.attr("lock", null);
    }
};
RO.GridHtml5.prototype.isEditorLocked = function () {
    var entry = this.gridModel.entry,
        editor = $(".gridEditor", entry),
        lock = editor.attr("lock");
    if (lock === "lock") {
        return true;
    }
    return false;
};
RO.GridHtml5.prototype.setShowProportion = function (value) {
    Draw2D.setShowProportion(value);
    this.gridModel.gridData.refresh();
};
RO.GridHtml5.prototype.toImg = function (mineType) {
    return this.gridModel.gridData.toImg(mineType);
};
RO.GridHtml5.prototype.isCellValueChanged = function() {
    return this.gridModel._cellValueChanged;
};
RO.GridHtml5.prototype.destroy = function () {
    this.gridModel.destroy();
    delete this.gridModel;
    for (var fun in this) {
        delete this[fun];
    }
};


/********************GridHtml5Cell业务处理类******************************************/
RO.GridHtml5.Cell = function (cell, gridModel) {
    this.cell = cell;
    this.gridModel = gridModel;
};

RO.GridHtml5.Cell.prototype.getAttrByName = function (attrName, forCurLevel) {
    var cellAttrs = this.cell.getClientData();
    if (cellAttrs) {
        var attr = cellAttrs[attrName];
        if (attr !== undefined) {
            return attr;
        }
    }
    return null;
};
RO.GridHtml5.Cell.prototype.setAttrByName = function (attrName, value) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var attr = this.cell.getClientData() || {};
    if ((value === null || value === undefined) && attr[attrName]) {
        delete attr[attrName];
    } else if (value !== null && value !== undefined) {
        attr[attrName] = value;
    }
    this.cell.setClientData(attr);
};
RO.GridHtml5.Cell.prototype.getDataRowAttr = function (attrName) {
    var attr = this.gridModel.getDataRowAttrs(this.cell);
    if (attr && attr[attrName] !== undefined) {
        return attr[attrName];
    } else if (attrName == RO.Enum.RegId) {
        var dataRegions = this.gridModel.dataRegions;
        for (var reg in dataRegions) {
            var curReg = dataRegions[reg];
            if (curReg.RegionKind == 0) {
                return reg;
            }
        }
    }
    return "";
};
RO.GridHtml5.Cell.prototype.setDataRowAttr = function (attrName, value) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var attr = this.gridModel.getDataRowAttrs(this.cell);
    if (value !== undefined && attr[attrName]) {
        delete attr[attrName];
    } else {
        attr[attrName] = value;
    }
    this.gridModel.setDataRowAttrs(this.cell, attr);
};
RO.GridHtml5.Cell.prototype.removeAttr = function (attrName, value) {

};
RO.GridHtml5.Cell.prototype.setCanEdit = function (flag) {
    this.gridModel.gridCanEdit() && this.cell.setEditable(flag);
};
RO.GridHtml5.Cell.prototype.getCanEdit = function () {
    return this.gridModel.designMode ? this.cell.cell.editable : this.cell.isEditable();
};
RO.GridHtml5.Cell.prototype.getShowText = function () {
    var text = this.cell.getShowText();
    if (text === undefined || text === null) {
        return "";
    }
    return text;
};
RO.GridHtml5.Cell.prototype.getEditText = function () {
    var text = this.cell.getEditText();
    if (text === undefined || text === null) {
        return "";
    }
    return text;
};
RO.GridHtml5.Cell.prototype.getRowIndex = function () {
    return this.cell.getRow();
};
RO.GridHtml5.Cell.prototype.getColIndex = function () {
    return this.cell.getCol();
};
RO.GridHtml5.Cell.prototype.setEditorId = function (id) {
    return this.cell.setEditorId(id);
};
RO.GridHtml5.Cell.prototype.setValue = function (showText, editText, force, customEventData, silence,numflag) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    if (force) {
        var oldValue = this.cell.getEditText(), nShowText = String(showText), nEditText = String(editText);
        if (this.cell.getShowText() == nShowText && oldValue == nEditText) {
            return;
        }

        var fkey = this.getDataRowAttr("fkey");
        this.cell.setShowText(nShowText);
        this.cell.setEditText(nEditText);
        if (!silence) {
            var changeValues = [{ row: this.getRowIndex(), col: this.getColIndex(), oldValue: oldValue, newValue: editText, fKey: fkey }];
            $(this.gridModel.entry).trigger(RO.Enum.Event.valuechange, { "changeValues": changeValues, "customEventData": customEventData });
        }
        return;
    }
    this.gridModel.setValue(this, editText, showText, silence, false, customEventData,undefined,numflag);
};
RO.GridHtml5.Cell.prototype.getControl = function () {
    return this.cell.getControl();
};

RO.GridHtml5.Cell.prototype.setBackColor = function (color, noCache) {
    if (!this.gridModel ||  !this.gridModel.gridCanEdit || !this.gridModel.gridCanEdit()) {
        return null;
    }

    var bColor = this.getAttrByName("backColor", true);
    if (color) {
        if (!bColor && !noCache) {
            this.setAttrByName("backColor", this.cell.getBackColor());
        }
        this.cell.setBackColor(color);
    } else if (bColor) {
        this.cell.setBackColor(bColor);
        this.setAttrByName("backColor", null);
    }
};
RO.GridHtml5.Cell.prototype.getBackColor = function (fromTemplate) {
    if (!fromTemplate) {
        return this.cell.getBackColor();
    }

    var rid = this.getDataRowAttr(RO.Enum.RegId);
    var region = this.gridModel.dataRegions[rid];
    
    if (region && region.RegionTemplate) {
        var template = region.RegionTemplate[0][this.getColIndex()];
        return template && template.backColor || "white";
    } else {
        var bColor = this.getAttrByName("backColor", true);
        return bColor || this.cell.getBackColor();
    }
};
RO.GridHtml5.Cell.prototype.setBackStyle = function (bStyle) {
    this.gridModel.gridCanEdit() && this.cell.setBackStyle(bStyle);
};
RO.GridHtml5.Cell.prototype.getBackStyle = function () {
    return this.cell.getBackStyle();
};
RO.GridHtml5.Cell.prototype.setBorderColor = function (rColor, bColor, noCache) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var cell, clientData, figure;
    if (this.getRowIndex() == 0) {
        this.setAttrByName("bottombordercolor", bColor);
        var bb = this.getAttrByName("bottomborderstyle", true);
        cell = this.gridModel.gridData.getCell(this.getColIndex(), 1);
        clientData = cell.getClientData() || {};
        if (bb == COM.widget.Grid.Cell.BORDER_BOLD) {
            clientData.userUpBorder = true;
            clientData.topbordercolor = bColor;
            cell.setClientData(clientData);
            figure = cell.getFigure();
            figure && figure.repaint();
        } else {
            delete clientData.userUpBorder;
            cell.setClientData(clientData);
        }
        return;
    }
    if (this.getColIndex() == 0) {
        this.setAttrByName("rightbordercolor", rColor);
        var rb = this.getAttrByName("rightborderstyle", true);
        cell = this.gridModel.gridData.getCell(1, this.getRowIndex());
        clientData = cell.getClientData() || {};
        if (rb == COM.widget.Grid.Cell.BORDER_BOLD) {
            clientData.userLeftBorder = true;
            clientData.leftbordercolor = rColor;
            cell.setClientData(clientData);
            figure = cell.getFigure();
            figure && figure.repaint();
        } else {
            delete clientData.userLeftBorder;
            cell.setClientData(clientData);
        }
        return;
    }

    var rBColor = this.getAttrByName("rBColor", true);
    var bBColor = this.getAttrByName("bBColor", true);
    if (rColor) {
        if (!rBColor && !noCache)
            this.setAttrByName("rBColor", this.cell.getRightBorderColor());
        this.cell.setRightBorderColor(rColor);
    } else if (rBColor) {
        this.cell.setRightBorderColor(rBColor);
        this.setAttrByName("rBColor", null);
    }
    if (bColor) {
        if (!bBColor && !noCache)
            this.setAttrByName("bBColor", this.cell.getBottomBorderColor());
        this.cell.setBottomBorderColor(bColor);
    } else if (bBColor) {
        this.cell.setBottomBorderColor(bBColor);
        this.setAttrByName("bBColor", null);
    }
};
RO.GridHtml5.Cell.prototype.getBorderColor = function () {
    var rbc = this.getAttrByName("rightbordercolor", true);
    var bbc = this.getAttrByName("bottombordercolor", true);
    var rColor = rbc || this.cell.getRightBorderColor();
    var bColor = bbc || this.cell.getBottomBorderColor();
    return new Array(rColor, bColor);
};
RO.GridHtml5.Cell.prototype.setBorderStyle = function (rStyle, bStyle, noCache) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var cell, clientData, figure;
    if (this.getRowIndex() == 0 && bStyle != undefined) {
        this.setAttrByName("bottomborderstyle", bStyle);

        cell = this.gridModel.gridData.getCell(this.getColIndex(), 1);
        clientData = cell.getClientData() || {};
        if (bStyle == COM.widget.Grid.Cell.BORDER_BOLD) {
            clientData.userUpBorder = true;
            clientData.topbordercolor = this.getAttrByName("bottombordercolor", true);
            cell.setClientData(clientData);
            figure = cell.getFigure();
            figure && figure.repaint();
        } else {
            delete clientData.userLeftBorder;
            cell.setClientData(clientData);
        }
        return;
    }
    if (this.getColIndex() == 0 && rStyle != undefined) {
        this.setAttrByName("rightborderstyle", rStyle);

        cell = this.gridModel.gridData.getCell(1, this.getRowIndex());
        clientData = cell.getClientData() || {};
        if (rStyle == COM.widget.Grid.Cell.BORDER_BOLD) {
            clientData.userLeftBorder = true;
            clientData.leftbordercolor = this.getAttrByName("rightbordercolor", true);
            cell.setClientData(clientData);
            figure = cell.getFigure();
            figure && figure.repaint();
        } else {
            delete clientData.userLeftBorder;
            cell.setClientData(clientData);
        }
        return;
    }

    var rBStyle = this.getAttrByName("rBStyle", true);
    var bBStyle = this.getAttrByName("bBStyle", true);
    if (rStyle != undefined) {
        if (!rBStyle && rBStyle !== 0 && !noCache)
            this.setAttrByName("rBStyle", this.cell.getRightBorderStyle());
        this.cell.setRightBorderStyle(rStyle);
    } else if (rBStyle || rBStyle === 0) {
        this.cell.setRightBorderStyle(rBStyle);
        this.setAttrByName("rBStyle", null);
    }
    if (bStyle != undefined) {
        if (!bBStyle && !noCache)
            this.setAttrByName("bBStyle", this.cell.getBottomBorderStyle());
        this.cell.setBottomBorderStyle(bStyle);
    } else if (bBStyle || bBStyle === 0) {
        this.cell.setBottomBorderStyle(bBStyle);
        this.setAttrByName("bBStyle", null);
    }
};
RO.GridHtml5.Cell.prototype.getBorderStyle = function () {
    var rstyle = this.getAttrByName("rightborderstyle");
    var bstyle = this.getAttrByName("botomborderstyle");
    var rStyle = rstyle == undefined ? this.cell.getRightBorderStyle() : rstyle;
    var bStyle = bstyle == undefined ? this.cell.getBottomBorderStyle() : bstyle;
    return new Array(rStyle, bStyle);
};

RO.GridHtml5.Cell.prototype.setDiagonalBorderColor = function (color, noCache) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var bColor = this.getAttrByName("DiagonalBorderColor", true);
    if (color) {
        if (!bColor && !noCache) {
            this.setAttrByName("DiagonalBorderColor", this.cell.getDiagonalBorderColor());
        }
        this.cell.setDiagonalBorderColor(color);
    } else if (bColor) {
        this.cell.setDiagonalBorderColor(bColor);
        this.setAttrByName("DiagonalBorderColor", null);
    }
};
RO.GridHtml5.Cell.prototype.setInverseDiagonalBorderColor = function (color, noCache) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var bColor = this.getAttrByName("InverseDiagonalBorderColor", true);
    if (color) {
        if (!bColor && !noCache) {
            this.setAttrByName("InverseDiagonalBorderColor", this.cell.getInverseDiagonalBorderColor());
        }
        this.cell.setInverseDiagonalBorderColor(color);
    } else if (bColor) {
        this.cell.setInverseDiagonalBorderColor(bColor);
        this.setAttrByName("InverseDiagonalBorderColor", null);
    }
};
RO.GridHtml5.Cell.prototype.setDiagonalBorderStyle = function (style, noCache) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var bstyle = this.getAttrByName("DiagonalBorderStyle", true);
    if (style || style === 0) {
        if (!bstyle && bstyle !== 0 && !noCache) {
            this.setAttrByName("DiagonalBorderStyle", this.cell.getDiagonalBorderStyle());
        }
        this.cell.setDiagonalBorderStyle(style);
    } else if (bstyle || bstyle === 0) {
        this.cell.setDiagonalBorderStyle(bstyle);
        this.setAttrByName("DiagonalBorderStyle", null);
    }
};
RO.GridHtml5.Cell.prototype.setInverseDiagonalBorderStyle = function (style, noCache) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var bstyle = this.getAttrByName("InverseDiagonalBorderStyle", true);
    if (style || style === 0) {
        if (!bstyle && bstyle !== 0 && !noCache) {
            this.setAttrByName("InverseDiagonalBorderStyle", this.cell.getInverseDiagonalBorderStyle());
        }
        this.cell.setInverseDiagonalBorderStyle(style);
    } else if (bstyle || bstyle === 0) {
        this.cell.setInverseDiagonalBorderStyle(bstyle);
        this.setAttrByName("InverseDiagonalBorderStyle", null);
    }
};

RO.GridHtml5.Cell.prototype.setHorzAlign = function (align) {
    this.gridModel.gridCanEdit() && this.cell.setHorzAlign(align);
};
RO.GridHtml5.Cell.prototype.getHorzAlign = function () {
    return this.cell.getHorzAlign();
};
RO.GridHtml5.Cell.prototype.setVertAlign = function (align) {
    this.gridModel.gridCanEdit() && this.cell.setVertAlign(align);
};
RO.GridHtml5.Cell.prototype.getVertAlign = function () {
    return this.cell.getVertAlign();
};
RO.GridHtml5.Cell.prototype.setVertText = function (value) {
    this.gridModel.gridCanEdit() && this.cell.setVertText(value);
};
RO.GridHtml5.Cell.prototype.isVertText = function () {
    return this.cell.isVertText();
};
RO.GridHtml5.Cell.prototype.setFontSize = function (value) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    this.setAttrByName("fontSize", value);
    this.cell.setFontSize(value * 96 / 72);//pt=>px
    var height = this.gridModel.getAutoRowHeight(this, this.getShowText());
    if (height > COM.widget.Grid.GridFigure.MAXROWHEIGHT) {
        height = COM.widget.Grid.GridFigure.MAXROWHEIGHT;
    }
    this.gridModel.refrashRowHeight(this.getRowIndex(), height);
};
RO.GridHtml5.Cell.prototype.getFontSize = function () {
    //return this.cell.getFontSize();
    var size = parseFloat(this.getAttrByName("fontSize"));
    return size ? size : this.cell.getFontSize() * 72 / 96; //px=>pt
};
RO.GridHtml5.Cell.prototype.getRealFontSize = function () {
    return this.cell.getFontSize();
};
RO.GridHtml5.Cell.prototype.setFontName = function (value) {
    this.gridModel.gridCanEdit() && this.cell.setFontName(value);
};
RO.GridHtml5.Cell.prototype.getFontName = function () {
    return this.cell.getFontName();
};
RO.GridHtml5.Cell.prototype.setFontBold = function (value) {
    this.gridModel.gridCanEdit() && this.cell.setFontBold(value);
};
RO.GridHtml5.Cell.prototype.getFontBold = function () {
    return this.cell.isFontBold();
};
RO.GridHtml5.Cell.prototype.setFontItalic = function (value) {
    this.gridModel.gridCanEdit() && this.cell.setFontItalic(value);
};
RO.GridHtml5.Cell.prototype.getFontItalic = function () {
    return this.cell.isFontItalic();
};
RO.GridHtml5.Cell.prototype.setUnderline = function (value) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var decoration = this.cell.getDecoration();
    switch (decoration) {
        case COM.widget.Grid.Cell.DECORATION.NONE:
        case COM.widget.Grid.Cell.DECORATION.UNDERLINE:
            decoration = value ? COM.widget.Grid.Cell.DECORATION.UNDERLINE : COM.widget.Grid.Cell.DECORATION.NONE;
            break;
        case COM.widget.Grid.Cell.DECORATION.STRIKEOUT:
        case COM.widget.Grid.Cell.DECORATION.BOTH:
            decoration = value ? COM.widget.Grid.Cell.DECORATION.BOTH : COM.widget.Grid.Cell.DECORATION.STRIKEOUT;
            break;
    }
    this.cell.setDecoration(decoration);
};
RO.GridHtml5.Cell.prototype.isUnderline = function () {
    var decoration = this.cell.getDecoration();
    switch (decoration) {
        case COM.widget.Grid.Cell.DECORATION.UNDERLINE:
        case COM.widget.Grid.Cell.DECORATION.BOTH:
            return true;
    }
    return false;
};
RO.GridHtml5.Cell.prototype.setStrikeout = function (value) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    var decoration = this.cell.getDecoration();
    switch (decoration) {
        case COM.widget.Grid.Cell.DECORATION.NONE:
        case COM.widget.Grid.Cell.DECORATION.STRIKEOUT:
            decoration = value ? COM.widget.Grid.Cell.DECORATION.STRIKEOUT : COM.widget.Grid.Cell.DECORATION.NONE;
            break;
        case COM.widget.Grid.Cell.DECORATION.UNDERLINE:
        case COM.widget.Grid.Cell.DECORATION.BOTH:
            decoration = value ? COM.widget.Grid.Cell.DECORATION.BOTH : COM.widget.Grid.Cell.DECORATION.UNDERLINE;
            break;
    }
    this.cell.setDecoration(decoration);
};
RO.GridHtml5.Cell.prototype.isStrikeout = function () {
    var decoration = this.cell.getDecoration();
    switch (decoration) {
        case COM.widget.Grid.Cell.DECORATION.STRIKEOUT:
        case COM.widget.Grid.Cell.DECORATION.BOTH:
            return true;
    }
    return false;
};
RO.GridHtml5.Cell.prototype.setFontColor = function (value) {
    this.gridModel.gridCanEdit() && this.cell.setFontColor(value);
};
RO.GridHtml5.Cell.prototype.getFontColor = function () {
    return this.cell.getFontColor();
};
RO.GridHtml5.Cell.prototype.setFitFontSize = function (value) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    this.cell.setFitFontSize(value);
    if (value) {
        this.cell.setWrapLine(false);
    }
};
RO.GridHtml5.Cell.prototype.isFitFontSize = function () {
    return this.cell.isFitFontSize();
};
RO.GridHtml5.Cell.prototype.setWrapLine = function (value) {
    if (!this.gridModel.gridCanEdit()) {
        return null;
    }

    this.cell.setWrapLine(value);
    if (value) {
        this.cell.setFitFontSize(false);
    }
};
RO.GridHtml5.Cell.prototype.isWrapLine = function () {
    return this.cell.isWrapLine();
};
RO.GridHtml5.Cell.prototype.setMultiLine = function (value) {
    this.gridModel.gridCanEdit() && this.cell.setMultiLine(value);
};
RO.GridHtml5.Cell.prototype.isMultiLine = function () {
    return this.cell.isMultiLine();
};
RO.GridHtml5.Cell.prototype.setIndent = function (indent) {
    this.gridModel.gridCanEdit() && this.cell.setIndent(indent);
};
RO.GridHtml5.Cell.prototype.getIndent = function () {
    return this.cell.getIndent();
};
RO.GridHtml5.Cell.prototype.setIndentPx = function (indentPx) {
    this.gridModel.gridCanEdit() && this.cell.setIndentPx(indentPx);
};
RO.GridHtml5.Cell.prototype.getIndentPx = function () {
    return this.cell.getIndentPx();
};
RO.GridHtml5.Cell.prototype.setCellTip = function (value) {
    this.gridModel.gridCanEdit() && this.cell.setTitle(String(value));
};

RO.GridHtml5.Cell.prototype.getColSpan = function () {
    return this.cell.getColSpan();
};
RO.GridHtml5.Cell.prototype.getRowSpan = function () {
    return this.cell.getRowSpan();
};
RO.GridHtml5.Cell.prototype.getMergedInfo = function () {
    return this.cell.getMergedInfo();
};

RO.GridHtml5.Cell.prototype.getPosition = function () {
    var bounds = this.cell.getFigure().getBounds();
    var offset = $(this.gridModel.entry).offset();
    return { x: Draw2D.pixelRatio * offset.left + bounds.x, y: Draw2D.pixelRatio * offset.top + bounds.y };
};

RO.GridHtml5.Cell.prototype.setForeColor = function (foreColor) {
    this.gridModel.gridCanEdit() && this.cell.setForeColor(foreColor);
};
RO.GridHtml5.Cell.prototype.getForeColor = function () {
    return this.cell.getForeColor();
};

RO.GridHtml5.Cell.prototype.setImgIntoCell = function (name, options, fixRowHeight) {
    this.setAttrByName("tArr", []);
    this.setAttrByName(COM.widget.Grid.Cell.textCache,[]);
    var offSetWidth = options.bounds.width + options.margin.left + options.margin.right;
    if (options.bounds.horizontal == PositionConstants.LEFT && options.img.value!="") {
        this.setAttrByName(RO.Enum.ImgTextLeft, offSetWidth);
    } else if (options.bounds.horizontal == PositionConstants.LEFT && options.img.value == "") {
        this.setAttrByName(RO.Enum.ImgTextLeft, 0);
    }
    var valType = options.img.type;
    if (valType == 1) {//图片
       this.setAttrByName("examineName", name);
        var examineOptions = this.getAttrByName("examineOptions") || {};
        //examineOptions.img = options.img;
        //examineOptions.bounds = options.bounds;
        //examineOptions.margin = options.margin;
        this.setAttrByName("examineOptions", $.extend(examineOptions, options));
    }
  
    if (this.cell.getFigure()) {
        var figure=this.cell.getFigure();
	    if (figure) {
	        var hasEenumFigure = true;
	        while (hasEenumFigure) {
	            if (figure.children.length > 0) {
	                for (var i = figure.children.length - 1; i >= 0 ; i--) {
	                    if (figure.children[i] && figure.children[i].FigureName && figure.children[i].FigureName == options.FigureName) {
	                        figure.remove(figure.children[i]);
	                        break;
	                    }
	                    if (i == 0) {
	                        hasEenumFigure = false;
	                    }
	                }
	            }else{
	                hasEenumFigure = false;
	            }
	        }
	       
	    }
       figure.addImageFigure(name, options);
       figure.repaint();
    }
    if (!fixRowHeight) {
        var height = this.gridModel.getAutoRowHeight(this, this.getShowText());
        if (height > COM.widget.Grid.GridFigure.MAXROWHEIGHT) {
            height = COM.widget.Grid.GridFigure.MAXROWHEIGHT;
        }
        this.gridModel.refrashRowHeight(this.getRowIndex(), height);
    }
}
/********************RO.GridHtml5内部使用*******************************************/



RO.GridHtml5.CellFigureFactory = function (options) {
    this.options = options;
    this.create = function (model) {
        if (null == model) {
            return this.normalCreator.create(null);
        }
        if (0 == model.rowIndex || 0 == model.colIndex) {
            return this.headCreator.create(model);
        } else {
            return this.normalCreator.create(model);
        }
    };
    this.init = function (colorProvider, imageProvider, headStyle, rowsProvider) {
        this.normalCreator = new RO.GridHtml5.CellFigureCreator(colorProvider, imageProvider, rowsProvider, this.options);
        this.headCreator = new COM.widget.Grid.HeadFigureCreator(colorProvider, imageProvider, headStyle);
    };

};

RO.GridHtml5.CellFigureCreator = function (colorProvider, imageProvider, rowsProvider, options) {
    this.options = options;
    RO.GridHtml5.CellFigureCreator.superclass.constructor.call(this, colorProvider, imageProvider, rowsProvider);
};
RO.GridHtml5.CellFigureCreator.extend(COM.widget.Grid.NormalFigureCreator, {
    createFigure: function (model) {
        return new RO.GridHtml5.CellFigure(this.options);
    }
});

RO.GridHtml5.CellFigure = function (options) {
    this.options = options;
    RO.GridHtml5.CellFigure.superclass.constructor.call(this);
};
RO.GridHtml5.CellFigure.extend(COM.widget.Grid.CellFigure, {              //Figure
    paintBack: function (graphics) {
        var bounds = this.getBounds(),
            style = this.getBackStyle() || COM.widget.Grid.Cell.BACK_STYLE.FILL,
            backColor = this.calculateBackColor(),
            foreColor = this.getForeColor(),
            data = this.getGradientBackgound(),
            gradient,
            fillStyle;
        //set color
        if (data) {
            if (data.direction === COM.widget.Grid.Cell.DIRECTION_VERTI) {
                gradient = graphics.createLinearGradient(0, 0, 0, bounds.height);
            } else {
                gradient = graphics.createLinearGradient(0, 0, bounds.width, 0);
            }
            gradient.addColorStop(0, Draw2D.getSizeInCanvas(data.begin));
            gradient.addColorStop(1, Draw2D.getSizeInCanvas(data.end));
            fillStyle = gradient;
        } else if (backColor) {
            fillStyle = backColor;
        } else {
            fillStyle = '#fff';
        }

        //piant backStyle
        if (style == COM.widget.Grid.Cell.BACK_STYLE.FILL) {
            if ("buttonface" == fillStyle)
                graphics.setFillStyle("#F0F0F0");
            else
                graphics.setFillStyle(fillStyle);
            graphics.fillRect(0, 0, bounds.width, bounds.height);
        } else {
            COM.BackStyleHelper.drawBackStyle(graphics, 0, 0, bounds.width, bounds.height, style - 2, fillStyle, foreColor);
        }

        var clientData = this.model.clientData;
        if (clientData && clientData[RO.Enum.cellDataErrorFlag]) {
            var flagSize = 10;
            var flagColor = "red";
            graphics.pushState();
            graphics.beginPath();
            graphics.moveTo(0, 0);
            graphics.lineTo(flagSize, 0);
            graphics.lineTo(0, flagSize);
            graphics.closePath();
            graphics.setFillStyle(flagColor);
            graphics.fill();
            graphics.restoreState();
        }
        if (clientData && clientData[RO.Enum.cellErrorCheckType] != "" && clientData[RO.Enum.cellErrorCheckType] != undefined) {
            var cellErrorCheckType = clientData[RO.Enum.cellErrorCheckType];
            flagColor = cellErrorCheckType == '1' ? 'red':'green' ;
            var flagSize = 10;
            graphics.pushState();
            graphics.beginPath();
            graphics.moveTo(0, 0);
            graphics.lineTo(flagSize, 0);
            graphics.lineTo(0, flagSize);
            graphics.closePath();
            graphics.setFillStyle(flagColor);
            graphics.fill();
            graphics.restoreState();
        }
    },
    paintBorder: function (graphics) {
        if (!(this.getBorderStyle() && this.getBorderColor())) {
            return;
        }
        var userBorderColor = "#61b5f1", userBoderWidth = 1;
        if (this.options) {
            userBorderColor = this.options.userBorderColor || userBorderColor;
            userBoderWidth = this.options.userBorderWidth || userBoderWidth;
        }

        var bounds = this.getBounds();
        graphics.translate(bounds.x, bounds.y);
        var border, color;
        var borders = this.getBorderStyle();
        var colors = this.getBorderColor();
        var colorProvider = this.getColorProvider();
        var x = 0;
        var y = 0;

        var clientData = this.model.clientData;
        if (!clientData || !clientData.userBottomBorder) { //下边框
            border = borders[1];
            color = colors[1];
            if (color == null || color == "") {
                color = colorProvider.getDefaultBorderColor();
            }
            if (border == COM.widget.Grid.Cell.BORDER_AUTO) {
                border = colorProvider.getDefaultBorderStyle();
            }
            if (colorProvider.isShowMergeChildBorder() && this.isMergeHead()) {
                this.paintBottomMergedBorder(graphics);
            } else {
                this.paintBottomBorder(graphics, border, color, x, y, bounds.width, bounds.height);
            }
        }
        if (!clientData || !clientData.userRightBorder) { //右边框
            border = borders[0];
            color = colors[0];
            if (color == null || color == "") {
                color = colorProvider.getDefaultBorderColor();
            }
            if (border == COM.widget.Grid.Cell.BORDER_AUTO) {
                border = colorProvider.getDefaultBorderStyle();
            }

            if (colorProvider.isShowMergeChildBorder() && this.isMergeHead()) {
                this.paintRightMergedBorder(graphics);
            } else {
                this.paintRightBorder(graphics, border, color, x, y, bounds.width, bounds.height);
            }
        }
        if (clientData) {

            if (clientData.userUpBorder) { //上边框(设置0行下边框时调用)
                graphics.pushState();
                graphics.beginPath();
                graphics.moveTo(0, 0);
                graphics.lineTo(bounds.width, 0);
                graphics.setStrokeStyle(clientData.topbordercolor || "black");
                graphics.setLineWidth(1);
                graphics.stroke();
                graphics.restoreState();
            }
            if (clientData.userLeftBorder) { //左边框（设置0列右边框时调用）
                graphics.pushState();
                graphics.beginPath();
                graphics.moveTo(0, 0);
                graphics.lineTo(0, bounds.height);
                graphics.setStrokeStyle(clientData.leftbordercolor || "black");
                graphics.setLineWidth(1);
                graphics.stroke();
                graphics.restoreState();
            }
            if (clientData.userBottomBorder) { //下边框
                this.paintBottomBorder(graphics, userBoderWidth > 1 ? COM.widget.Grid.Cell.BORDER_BOLD : COM.widget.Grid.Cell.BORDER_SOLID, userBorderColor, x, y, bounds.width, bounds.height);
            }
            if (clientData.userRightBorder) { //右边框
                this.paintRightBorder(graphics, userBoderWidth > 1 ? COM.widget.Grid.Cell.BORDER_BOLD : COM.widget.Grid.Cell.BORDER_SOLID, userBorderColor, x, y, bounds.width, bounds.height);
            }
        }

        //对角线

        border = borders[2];
        color = colors[2];
        if (color == null || color == "") {
            color = colorProvider.getDefaultBorderColor();
        }
        if (border == COM.widget.Grid.Cell.BORDER_AUTO) {
            border = colorProvider.getDefaultBorderStyle();
        }
        if (border != null && border !== COM.widget.Grid.Cell.BORDER_NONE) {
            if (border === COM.widget.Grid.Cell.BORDER_SOLID) {
                graphics.pushState();
                graphics.beginPath();
                graphics.moveTo(x + bounds.width, y);
                graphics.lineTo(x, y + bounds.height);
                graphics.setStrokeStyle(color);
                graphics.stroke();
                graphics.restoreState();
            } else if (border === COM.widget.Grid.Cell.BORDER_DASH) {
                graphics.pushState();
                graphics.beginPath();
                graphics.dashedLine(x + bounds.width, y, x, y + bounds.height,
                        COM.widget.Grid.CellFigure.DASH_LEN, COM.widget.Grid.CellFigure.DASH_GAP_LEN, COM.widget.Grid.CellFigure.DASH_Array);
                graphics.setStrokeStyle(color);
                graphics.stroke();
                graphics.restoreState();
            } else if (border === COM.widget.Grid.Cell.BORDER_BOLD) {
                graphics.pushState();
                graphics.beginPath();
                graphics.setLineWidth(2);
                graphics.moveTo(x + bounds.width, y);
                graphics.lineTo(x, y + bounds.height);
                graphics.setStrokeStyle(color);
                graphics.stroke();
                graphics.restoreState();
            } else if (border === COM.widget.Grid.Cell.BORDER_DOUBLE) {
                graphics.pushState();
                graphics.beginPath();
                graphics.moveTo(x + bounds.width - 2, y);
                graphics.lineTo(x, y + bounds.height - 2);
                graphics.moveTo(x + bounds.width, y + 2);
                graphics.lineTo(x + 2, y + bounds.height);
                graphics.setStrokeStyle(color);
                graphics.stroke();
                graphics.restoreState();
            }
        }

        border = borders[3];
        color = colors[3];
        if (color == null || color == "") {
            color = colorProvider.getDefaultBorderColor();
        }
        if (border == COM.widget.Grid.Cell.BORDER_AUTO) {
            border = colorProvider.getDefaultBorderStyle();
        }

        //逆对角线
        if (border != null && border !== COM.widget.Grid.Cell.BORDER_NONE) {
            if (border === COM.widget.Grid.Cell.BORDER_SOLID) {
                graphics.pushState();
                graphics.beginPath();
                graphics.moveTo(x, y);
                graphics.lineTo(x + bounds.width, y + bounds.height);
                graphics.setStrokeStyle(color);
                graphics.stroke();
                graphics.restoreState();
            } else if (border === COM.widget.Grid.Cell.BORDER_DASH) {
                graphics.pushState();
                graphics.beginPath();
                graphics.dashedLine(x, y, x + bounds.width, y + bounds.height,
                        COM.widget.Grid.CellFigure.DASH_LEN, COM.widget.Grid.CellFigure.DASH_GAP_LEN, COM.widget.Grid.CellFigure.DASH_Array);
                graphics.setStrokeStyle(color);
                graphics.stroke();
                graphics.restoreState();
            } else if (border === COM.widget.Grid.Cell.BORDER_BOLD) {
                graphics.pushState();
                graphics.beginPath();
                graphics.setLineWidth(2);
                graphics.moveTo(x, y);
                graphics.lineTo(x + bounds.width, y + bounds.height);
                graphics.setStrokeStyle(color);
                graphics.stroke();
                graphics.restoreState();
            } else if (border === COM.widget.Grid.Cell.BORDER_DOUBLE) {
                graphics.pushState();
                graphics.beginPath();
                graphics.moveTo(x, y + 2);
                graphics.lineTo(x + bounds.width - 2, y + bounds.height);
                graphics.moveTo(x + 2, y);
                graphics.lineTo(x + bounds.width, y + bounds.height - 2);
                graphics.setStrokeStyle(color);
                graphics.stroke();
                graphics.restoreState();
            }
        }

    },
    dispose: function () {
        COM.widget.Grid.CellFigure.prototype.dispose.call(this);
        delete this.options;
    }
});


RO.GridHtml5.ScrollHelper = function ScrollHelper(scrollBar) {
    this.scrollBar = scrollBar || {};
    this.init();
};
RO.GridHtml5.ScrollHelper.extend(COM.widget.Grid.ScrollHelper, {
    init: function () {
        var verti = {};
        var scrollBar = this.scrollBar.verti;
        scrollBar.addClass("grid_customScrollBar");
        verti.scrollBar = scrollBar[0];
        verti.show = this.showFunc;
        verti.hide = this.hideFunc;

        this.verti = verti;
        verti.getSize = function () {
            return parseInt($(this.scrollBar).height());
        };
        verti.setContentSize = function (size) {
            var self = this;
            //  lasyCall("verti_setContentSize", function () {
            $(self.scrollBar).trigger("Event_SetScrollContentHeight", { size: size });
            //  });
        };
        verti.bindScrollListener = function (listen) {
            this.scrollBar.onscroll = listen;
        };
        verti.getScrollTop = function () {
            return parseInt($(this.scrollBar).attr("scrollTop"));
        };
        verti.setScrollTop = function (top) {
            var self = this;
            var scrollTop = parseInt($(this.scrollBar).attr("scrollTop"));

            if (top < 0) {
                if (scrollTop == 0) {
                    return;
                }
                top = 0;
            }

            var scrollT = parseInt($(this.scrollBar).attr("scrollSize"));
            if (scrollT && top > scrollT) {
                if (scrollT == scrollTop) {
                    return;
                }
                top = scrollT;
            }
            $(this.scrollBar).attr("scrollTop", top);
            // lasyCall("verti_setScrolltTop", function () {
            $(self.scrollBar).trigger("Event_SetScrollTop", { "size": top });
            // });
        };
        verti.getScrollHeight = function () {
            return parseInt($(this.scrollBar).attr("scrollSize"));
        };
        $(verti.scrollBar).attr("scrollTop", 0).attr("disableWheel", true);



        var horiz = {};
        scrollBar = this.scrollBar.horiz;
        scrollBar.addClass("grid_customScrollBar");
        horiz.scrollBar = scrollBar[0];
        horiz.show = this.showFunc;
        horiz.hide = this.hideFunc;
        this.horiz = horiz;

        horiz.getSize = function () {
            return parseInt($(this.scrollBar).width());
        };

        horiz.setContentSize = function (size) {
            var self = this;
            // lasyCall("horiz_setContentSize", function () {
            $(self.scrollBar).trigger("Event_SetScrollContentWidth", { "size": size });
            // });
        };
        horiz.bindScrollListener = function (listen) {
            this.scrollBar.onscroll = listen;
        };
        horiz.getScrollLeft = function () {
            var left = parseInt($(this.scrollBar).attr("scrollLeft"));
            return left;
        };
        horiz.setScrollLeft = function (left) {
            var self = this;
            var scrollLeft = parseInt($(this.scrollBar).attr("scrollLeft"));
            if (left < 0) {
                if (scrollLeft == 0) {
                    return;
                }
                left = 0;
            }

            var scrollW = parseInt($(this.scrollBar).attr("scrollSize"));
            if (scrollW && left > scrollW) {
                if (scrollLeft == scrollW) {
                    return;
                }
                left = scrollW;
            }
            $(this.scrollBar).attr("scrollLeft", left);
            //  lasyCall("horiz_setScrollLeft", function() {
            $(self.scrollBar).trigger("Event_SetScrollLeft", { "size": left });
            // });
        };
        horiz.getScrollWidth = function () {
            return parseInt($(this.scrollBar).attr("scrollSize"));
        };

        $(horiz.scrollBar).attr("scrollLeft", 0);

    },
    appendTo: function () {
    },
    getVertiScrollBar: function () {
        return this.verti.scrollBar;
    },
    getHorizScrollBar: function () {
        return this.horiz.scrollBar;
    },
    setSize: function (width, height) {
    },
    dispose: function () {
        COM.widget.Grid.ScrollHelper.prototype.dispose.call(this);
        delete this.scrollBar;
    }
});



RO.GridHtml5.GridModel = function (shell, formData, widgetManager, scrollBar, designMode) {
    this.entry = shell[0];
    this.dataRegions = formData.options.DataRegions;
    this.dataStates = {};
    this.guid = formData.guid;
    this.enumDataSourceUrl = formData.options.enumDataSourceUrl;
    this.enumDataBlobUrl = formData.options.enumDataBlobUrl;
    this.extendData = {};
    this.TempDataStates = {};
    this.editorOpened = false;
    this.gridData = null;
    this.options = {};
    this.rowList = [];
    this.colList = [];
    this.isPasted = false;
    this.timeOutId = "";
    this._actions = [];
    this._actionLock = false;
    this.asynLoadDataLock = false;
    this.lockHeaderRowCache = [];
    this.lockHeaderColCache = [];
    this._curRowIndexCache = -1;
    this._curColIndexCache = -1;
    this._titleRowHeight = 30;
    this._titleColWidth = 36;
    this.widgetManager = widgetManager || {};
    this._autoId = 1;
    this._sortedRegId = [];
    this.rowFirstEditableCellInfo = null;
    this.designMode = designMode;
    this.floatRegionBorderColor = formData.options.floatRegionBorderColor;
    this.floatRegionBorderWidth = formData.options.floatRegionBorderWidth;
    this._graphics = FigureUtilities.getGraphics();
    this._insertDataCache = {};
    this._insertCacheAfterSave = {};
    this._firstLoadCount = 80;
    this.hideFloatRegionBorder = formData.options.hideFloatRegionBorder;
    this._floatRegList = [];
    this._regAutoNo = formData.options.extendData && formData.options.extendData["regAutoNo"];
    this.urlParam = formData.options.extendData && formData.options.extendData.UploadParams;
    this.enumData = formData.options.extendData && formData.options.extendData[RO.Enum.EnumData];
    this.variableEnumData = formData.options.extendData && formData.options.extendData.mutableenumdata;
    this.rowRegList = {};
    this.rowRegnull = [];
    this._loading = true;
    //缓存待保存单元格坐标，用于保存后重置
    this.savedCellCaches = [];
    this._cellValueChanged = false;

    var floatRegionDefines = {};
    var origRegionBoundary = null;
    var regionBorderDirection = {
        RIGHT: 0,
        BOTTOM: 1,
        TOP: 2,
        LEFT: 3
    };

    //计算栏标题
    var startColCode = "A".charCodeAt(0);
    this.getTitleColIndex = function (index) {
        index--;
        var aTz = parseInt(index / 26);
        var last = index % 26;
        var str = String.fromCharCode(last + startColCode);
        while (aTz != 0) {
            last = aTz % 26;
            str = String.fromCharCode(last + startColCode - 1) + str;
            aTz = parseInt(aTz / 26);
        }
        return str;
    };
   //错误提示
    this.ilegalWarningData = function (cell, filter) {
        var warningData = {};
        var widget = this.widgetManager["GridEvent_ExtFunction"];
        if (widget) {
            var result;
            var formatFilter = { "type": "ExtFunction", "cell": cell, "filter": filter, "gridModel": this, "result": null };
            widget.execute(formatFilter);
        }
        return warningData;
    }
    //过滤数据
    this.filterData = function (cell, filter) {
        var formatData = {};
        var widget = this.widgetManager["GridEvent_doFilter"];
        if (widget) {
            var result;
            var formatFilter = { "type": "formatData", "cell": cell, "filter": filter, "gridModel": this, "result": null };
            widget.execute(formatFilter);
            result = formatFilter.result;
            if (result) {
                for (var j = 0, jcount = result.length; j < jcount; j++) {
                    formatData = $.extend({}, formatData, result[j]);
                }
            }
        }
        return formatData;
    };
    //自动序号数据
    this.filterAutoNoData = function (autofilter) {
        var AutoNoData = {};
        var widget = this.widgetManager["GridEvent_doFilter"];
        if (widget) {
            var autoNoFilter = { "type": "autoNo", "filter": autofilter, "gridModel": this, "result": null };
            widget.execute(autoNoFilter);
        }
    };
    //计算单元格文本高度
    var calcTextHeight = function (graphics, vertText, width, fontSize, fontName, showText, paddingHeight, wrapLine, multiLine, indent, indentPx, bold) {
        var wordSpacing = 1.25, defaultSize = 24;

        showText = new String(showText);
        //缩进
        if (indentPx != null && indentPx != 0) {
            width -= Math.abs(indentPx);
        } else if (indent != null && indent > 0) {
            width -= indent * 14;
        }

        if (vertText) {
            if (!wrapLine && !multiLine) {
                return Math.ceil(fontSize * wordSpacing * showText.length + paddingHeight);
            } else if (!wrapLine) {
                var lines = showText.split("\n");
                var len = 0;
                for (var i = lines.length - 1; i >= 0; i--) {
                    len = Math.max(len, lines[i].length);
                }
                return Math.ceil(fontSize * wordSpacing * len + paddingHeight);
            }

        } else {
            if (!wrapLine) {
                var newlines = showText.split("\n");
                return Math.ceil(newlines.length * fontSize * wordSpacing + paddingHeight);
            } else {
                var lineCount = TextHelper.getWrapLines(graphics, showText, fontSize, fontName, bold, width);
                //计算高度
                return Math.ceil(lineCount * Math.ceil(fontSize * wordSpacing) + paddingHeight);
            }

        }
        return defaultSize;
    };

    this.getCellHeight = function (cell, width) {
        var mergInfo = cell.mergeInfo;

        if (mergInfo != null) {
            return 0;
        }

        var indentPx = 0;
        var indent = cell.indent;
        var fit = cell.fitFontSize;
        var wrapLine = cell.wrapLine;
        var vertText = cell.vertText;
        var multiLine = cell.multiLine;
        var treeIndent = cell.cellMode == COM.widget.Grid.Cell.Cell_MODE_TREE ? 10 * parseInt(cell.depth) + 10 + 30 : 0;
        var dt = cell.clientData && cell.clientData["dt"];
        if (fit) {
            return 0;
        }

        if (!this.designMode && dt != "1" && dt != 0) {
            return 0;
        }

        if (vertText && wrapLine && multiLine) {
            return 0;
        } else if (!vertText && !wrapLine && !multiLine) {
            return 0;
        }

        var fontSize = Math.round(cell.fontSize);
        var showText = cell.showText;
        if (showText == "" || showText == null) {
            return 0;
        }

        var paddingWidth = RO.Enum.DefaultCellPadding[1] + RO.Enum.DefaultCellPadding[3] + (cell.border[0] == RO.Enum.BorderStyle.BOLD || cell.border[0] == RO.Enum.BorderStyle.DOUBLE ? 2 : 1);
        var paddingHeight = RO.Enum.DefaultCellPadding[0] + RO.Enum.DefaultCellPadding[2] + (cell.border[1] == RO.Enum.BorderStyle.BOLD || cell.border[1] == RO.Enum.BorderStyle.DOUBLE ? 2 : 1);

        width -= paddingWidth + treeIndent;

        return calcTextHeight(this._graphics, vertText, width, fontSize, cell.fontName, showText, paddingHeight, wrapLine, multiLine, indent, indentPx, cell.fontBold);
    };

    //初始化表格行高
    var initRowHeights = function (rowList, cols, rows, self){//, autoRows) {
        var rowCount = Math.min(rowList.length, self._firstLoadCount);
        for (var r = rowCount - 1; r > 0; r--) {
            var height = rows[r].clientSize;
            var row = rowList[r];
            for (var c = row.length - 1; c > 0; c--) {
                var cell = row[c];
                var merged = cell.merged;
                var rowSpan = cell.rowSpan || 1;
                var colSpan = cell.colSpan || 1;
                if (merged || rowSpan > 1 || colSpan > 1) {
                    continue;
                }

                var width = cols[c].clientSize;
                var newHeight = self.getCellHeight(cell, width);
                height = Math.max(height, newHeight);
            }

            rows[r].clientSize = height;
        }
    };

    //数据加载格式化
    var formatDataForLoad = function (dataCells, self) {
        var widget = self.widgetManager["GridEvent_doFilter"];
        if (widget) {
            var themeFilter = { "type": "theme", "cells": dataCells, "filter": { "from": 0, "mergeCells": formData.mergeCells }, "gridModel": self, "result": null };
            widget.execute(themeFilter);
            dataCells = themeFilter.result[0];
        }
    };
    //加载数据
    var formatLoadDatas = function (data, self) {
        var rowList = data.cells;
        if (!rowList) return data;

        formatDataForLoad(data.cells, self);

        var rowFirstEditFlag = false;
        for (var i = 1; i < rowList.length; i++) {
            var cellList = rowList[i];
            for (var j = 1; j < cellList.length; j++) {
                var cell = cellList[j];
                if (!self.rowFirstEditableCellInfo && cell.editable && !cell.merged) {
                    self.rowFirstEditableCellInfo = { row: i, col: j };
                    rowFirstEditFlag = true;
                    break;
                }
            }
            if (rowFirstEditFlag) {
                break;
            }
        }
        if (!self.designMode) {
            initRowHeights(data.cells, data.cols, data.rows, self);//, data.options.extendData && data.options.extendData["autoRows"] || {});
        }

        return data;
    };

    var initRegTemplate = function(region) {
        if (region.RegionTemplate && region.RegionTemplate[0]) {
            var template = region.RegionTemplate[0];
            
            for (var i = 0, count = template.length; i < count; i++) {
                template[i] = $.extend({}, RO.Enum.DefaultCell, template[i]);
            }
        }
    };
    //初始化数据区域
    var initDataRegions = function (gridModel) {
        var dataRegions = gridModel.dataRegions, floatRegList = gridModel._floatRegList, region, offset = 0, rows = formData.rows, extendData = formData.options.extendData, regKind = RO.Enum.FormRegionKind.Fixed;
        
        for (var reg in dataRegions) {
            region = dataRegions[reg];
            if (region.RegionKind != 0) {
                floatRegList.push({ id: reg, index: region.RegionTop });
                regKind = RO.Enum.FormRegionKind.ROW;
            }
        }
        gridModel.options["FormRegionKind"] = regKind;
        
        floatRegList.sort(function (a, b) {
            return parseInt(a.index - b.index);
        });
        
        for (var i = 0, count = floatRegList.length; i < count; i++) {
            region = dataRegions[floatRegList[i].id];
            var floatCount = region.FloatDataCount, newRows = [], top = region.RegionTop + offset;

            newRows.push(top + 1);
            newRows.push(0);
            for (var r = 1; r < floatCount; r++) {
                newRows.push($.extend({}, rows[top]));
            }

            rows.splice.apply(rows, newRows);
            region.RegionTop += offset;
            region.RegionBottom += offset;
            offset += floatCount - 1;

            initRegTemplate(region);

            var treeHidden = extendData["treeHidden_" + region.RegionId];
            if (treeHidden) {
                for (var index in treeHidden) {
                    for (var row = 0, rc = treeHidden[index]; row < rc; row++) {
                        rows[row + index + 1].hidden = true;
                    }
                }
            }
        }
    };
    //初始化RowKeyData（按fkey存储）
    var initDataRowAttrs = function (gridModel) {
        var rowCount = formData.rowCount, data = formData.options.DataRowsAttr, indexList = gridModel.rowList;
             indexList = gridModel.rowList, rowreglist = gridModel.rowRegList,
             rowRegnull = gridModel.rowRegnull, cells = formData.cells;
        for (var r = 0; r < rowCount; r++) {
            indexList[r] = { index: r };
        }
        
        for (var reg in data) {
            var region = gridModel.dataRegions[reg], kind = region.RegionKind, begin = 0;
            
            if (kind == 3) {
                begin = region.RegionTop;
            }

            var regData = data[reg];
            for (var i = 0, count = region.FloatDataCount; i < count; i++) {
                var index = begin + i;
                var rowData = regData[i];
                
                if (rowData) {
                    indexList[index] = $.extend({}, indexList[index], rowData);
                    var fkey = rowData[RO.Enum.FloatRowKey];
                    if (fkey) {
                        gridModel.extendData[fkey] = indexList[index];
                    }
                }

                indexList[index][RO.Enum.RegId] = reg;
                if (indexList[index][RO.Enum.State] === undefined) {
                    indexList[index][RO.Enum.State] = "0";
                }
            }
        }
        for (var regID in gridModel.dataRegions) {
            var region = gridModel.dataRegions[regID], kind = region.RegionKind, index = 0, rowcount = 0;
            var countinit = region.DefaultCount;
            if (kind == 3) {
                rowreglist[regID] = [];
                index = region.RegionTop;
                rowcount = region.FloatDataCount;
                for (var i = index, count = 0; i < index + rowcount; i++) {
                    if (i == index && indexList[i][RO.Enum.State] != '9') {
                       if (gridModel.rowList[i][RO.Enum.extendAttrs] & RO.Enum.ExtendAttrType.Classify) {
                            rowreglist[regID].push({ "index": i, "count": 1, "countinit": countinit });
                        } else {
                            indexrowIndex = index;
                            count++;
                        }
                        if (indexList[i][RO.Enum.State] == '1') {
                            rowRegnull.push(index);
                        }
                    } else if (i != index && count == 0 && indexList[i][RO.Enum.State] != '9') {
                        if (gridModel.rowList[i][RO.Enum.extendAttrs] & RO.Enum.ExtendAttrType.Classify) {
                            rowreglist[regID].push({ "index": i, "count": 1, "countinit": countinit });
                        } else {
                            indexrowIndex = i;
                            count++;
                        }
                        if (indexList[i][RO.Enum.State] == '1') {
                            rowRegnull.push(i);
                        }
                    } else if (i != index && count != 0 && indexList[i][RO.Enum.State] != '9') {
                        if (gridModel.rowList[i][RO.Enum.extendAttrs] & RO.Enum.ExtendAttrType.Classify) {
                            rowreglist[regID].push({ "index": indexrowIndex, "count": count, "countinit": countinit });
                            count = 0;
                            rowreglist[regID].push({ "index": i, "count": 1, "countinit": countinit });
                        }
                        else {
                            count++;
                        }
                        if (indexList[i][RO.Enum.State] == '1') {
                            rowRegnull.push(i);
                        }
                    } else if (i != index && count != 0 && indexList[i][RO.Enum.State] == '9') {
                        rowreglist[regID].push({ "index": indexrowIndex, "count": count, "countinit": countinit });
                        count = 0;
                    }
                    if (i == index + rowcount - 1 && count != 0) {
                        if (gridModel.rowList[i][RO.Enum.extendAttrs] & RO.Enum.ExtendAttrType.Classify) {
                            rowreglist[regID].push({ "index": i, "count": 1, "countinit": countinit });
                        } else {
                            rowreglist[regID].push({ "index": indexrowIndex, "count": count, "countinit": countinit });
                            count = 0;
                        }
                        
                    }
                }

            }
        }
       
       
        formData.options.DataRowsAttr = null;
    };
    //初始化浮动序号
    this.initOrder = function () {
        var dataRegions = this.dataRegions;

        for (var reg in dataRegions) {
            var dataRegion = dataRegions[reg];
            if (dataRegion.RegionKind == 3) {
                if (dataRegion.DefaultCount + 1 == dataRegion.FloatDataCount) {
                    var cacheCount = 0, cacheIndex = 0;
                    var regionTop = dataRegion.RegionTop;
                    for (var i = 0, count = dataRegion.FloatDataCount; i < count; i++) {
                        var state = this.rowList[regionTop + i][RO.Enum.State];
                        if (state == "1") {
                            cacheCount = count - i;
                            cacheIndex = i;
                            break;
                        }
                    }

                    if (cacheCount > 0) {
                        var insertPos;
                        var start = regionTop + cacheIndex;
                        if (cacheCount == dataRegion.FloatDataCount) {
                            insertPos = 0;
                        } else {
                            insertPos = this.rowList[start - 1]["floatorder"] || 0;
                        }

                        if (!this._insertDataCache) {
                            this._insertDataCache = {};
                            this._insertDataCache[insertPos] = {};
                        } else {
                            this._insertDataCache[insertPos] = {};
                        }

                        for (var cIndex = 0; cIndex < cacheCount; cIndex++) {
                            var dataRow = this.rowList[start + cIndex];
                            dataRow["insertpos"] = insertPos;
                            dataRow["floatorder"] = parseFloat(insertPos) + 1000 * (cIndex + 1);
                            this._insertDataCache[insertPos][dataRow[RO.Enum.FloatRowKey]] = dataRow["floatorder"];
                        }
                    }
                }
            }
        }
    };

    var formatRemainingData = function (self) {
        var guid = self.guid;
        try {
            var rowCount = self.gridData.getRowCount(), rowList = self.gridData.gridModel.data.cells, widget = self.widgetManager["GridEvent_doFilter"];

            if (widget) {
                var themeFilter = { "type": "theme", "cells": rowList, "filter": { "from": self._firstLoadCount, "mergeCells": formData.mergeCells, "guid": guid }, "gridModel": self, "result": null };
                widget.execute(themeFilter);
            }
            self._loading = false;
            var colCount = rowList[0].length;
            var cols = formData.cols;
            var rows = [], heights = [], colSize = [];//,
                //autoRows = formData.options.extendData && formData.options.extendData["autoRows"] || {};
            for (var i = 0; i < colCount; i++) {
                colSize.push(cols[i].clientSize);
            }
            for (var r = self._firstLoadCount; r < rowCount; r++) {
                var height = 0;
                var rowData = rowList[r];
                //if (!autoRows[r]) {
                //    continue;
                //}
                for (var c = 1; c < colCount; c++) {
                    var colSpan = rowData[c].colSpan || 1;
                    var width = colSize[c];

                    for (var j = 1; j < colSpan; j++) {
                        width += colSize[c + j];
                    }
                    width -= 4;
                    
                    var newHeight = self.getCellHeight(rowData[c], width);
                    height = Math.max(height, newHeight);
                }

                var rowHeight = self.gridData.getRowHeight(r);
                if (rowHeight < height) {
                    rows.push(r);
                    heights.push(height);
                }
            }
            if (guid == self.guid) {
                self.gridData.setRowsHeight(rows, heights, true);
            }
        } catch (ex) {
        }
    };

    this.filterExtFormData = function (cells) {
        var widget = this.widgetManager["GridEvent_doFilter"];
        widget && widget.execute({ "type": "extFormatData", "cells": cells, "gridModel": this });
    };

    this.initExtTypeCell = function () {
        var gridData = this.gridData,
            self = this,
            rowC = gridData.getRowCount(),
            colC = gridData.getColCount(),
            curRowC = Math.min(rowC, this._firstLoadCount),
            cells = [],
            r,
            c;

        for (r = 1; r < curRowC; r++) {
            for (c = 1; c < colC; c++) {
                cells.push(new RO.GridHtml5.Cell(gridData.getCell(c, r), this));
            }
        }
        this.filterExtFormData(cells);

        if (curRowC < rowC) {
            setTimeout(function () {
                cells = [];

                for (r = curRowC; r < rowC; r++) {
                    for (c = 1; c < colC; c++) {
                        cells.push(new RO.GridHtml5.Cell(gridData.getCell(c, r), self));
                    }
                }
                self.filterExtFormData(cells);
            }, 5);
        }
    };

    //GridModel初始化
    var initData = function (self) {
        //处理标题栏宽高
        var curW = formData.options.showGridTitle ? self._titleColWidth : 0;
        var curH = formData.options.showGridTitle ? self._titleRowHeight : 0;
        var firstRow = formData.rows[0], firstCol = formData.cols[0];
        firstRow.clientSize = firstRow.size = curH;
        firstCol.clientSize = firstCol.size = curW;
        formData.mergeCells = [];
        self.options["entry"] = self.entry;
        self.options["extendInfo"] = formData.options.extendInfo;

        formData.options.blankAfterRow = 10;
        formData.options.blankAfterCol = 10;

        initDataRegions(self);
        initDataRowAttrs(self);

        var figureOptions = { userBorderColor: self.floatRegionBorderColor, userBorderWidth: self.floatRegionBorderWidth };
        self.gridData = new COM.widget.Grid(self.entry, formatLoadDatas(formData, self), { "cellFigureFactory": new RO.GridHtml5.CellFigureFactory(figureOptions), "scrollHelper": scrollBar ? new RO.GridHtml5.ScrollHelper(scrollBar) : null });
        var filterflag = false;
        var region = self.dataRegions;
        for (var regid in region){
            if (region[regid].RegionExtension && $.parseJSON(region[regid].RegionExtension).OrderIndex > 0) {
                filterflag = true;
            }
        }
        if (filterflag) {
            var drawArea = self.gridData.getDrawAreaInfo();
            var RowRegList = self.rowRegList, RowRegNull = self.rowRegnull;
            var initindex = drawArea.startRow;
            var initcount = drawArea.endRow - drawArea.startRow + 1;
            var autofilter = { "rowIndex": initindex, "rowCount": initcount, "RowRegList": RowRegList, "RowRegNull": RowRegNull ,"Region":region,initFlag:true};
            self.filterAutoNoData(autofilter);
        }
        self.initOrder();
        $(self.entry).trigger("gridInitData");

        self.initExtTypeCell();

        var widget = self.widgetManager["GridEvent_ExtFunction"];
        if (widget) {
            var data = { "type": null, "cell": null, "gridModel": self, "filter": null, "result": null };
            widget.execute(data);
        }

        //if ($(".gridEditor", self.entry).length > 0) {
        //    $(".gridEditor", self.entry).remove();
        //}
        //getDefaultEditor(self);

        if ($(".clipTextArea", self.entry).length == 0) {
            $(self.entry).append($("<textarea class='clipTextArea' rows=0 cols=0 style='position:absolute;top:-1000px;'></textarea>"));
        }

        self.addLoadListener();
        self.addEditorListener();
        self.addOtherListener();
        self.addKeyEventListener();

        var rowCount = self.gridData.getRowCount();
        var remainingCount = rowCount - self._firstLoadCount;
        if (remainingCount <= 0) {
            self._loading = false;
            //$(self.entry).trigger(RO.Enum.Event.onFormDataLoaded);
            return;
        } else {
            self.timeOutId = setTimeout(function() {
                formatRemainingData(self);
            }, 1);
        }
    };
    //更新浮动数据状态
    var dataStateDef = function (key, state, isNew) {
        this.Key = key;
        this.State = state;//state：0初始化，1：新增，2：修改，3：删除
        this.IsNew = isNew;
        return this;
    };

    var preventDefault = function (e) {
        if (e.preventDefault) {
            e.preventDefault();
        } else {
            e.returnValue = false;
        }
    };

    this.getDefaultEditor = function () {
        var editor = $("<textarea class='gridEditor' spellcheck='false' style='overflow:hidden;resize:none;position:absolute;border:2px solid rgba(0,180,0,1);outline:none;margin:0;padding:0;vertical-align:top;z-index:-5;width:0;height:" + (COM.Util.browser.wk ? 7 : 0) + "px;top:0;left:0;'></textarea>").gridHtml5Editor();
        this.gridData.setDefaultEditor(editor[0]);
        return editor;
    };

    //监听器定义
    this.addEditorListener = function () {
        var self = this;
        var getControlById = function (id, obj) {
            var cell = new RO.GridHtml5.Cell(obj.cell, self);
            var editText = null;
            if (obj.cause.type == COM.widget.Grid.CAUSE_KEY) {
                editText = obj.cause.value;
            }
            var data = { "type": id, "cell": cell, "gridModel": self, "filter": { "cellSize": obj.cell.getShowSize(), "editText": editText }, "result": null };
            var widget = self.widgetManager["GridEvent_doEdit"];
            if (widget) widget.execute(data);
            return data.result;
        };
        this._editorOpenListener = function (e) {
            var cell = e.cell;
            var result = e.result;
            if (!cell.isEditable()) {
                result.doit = COM.widget.Grid.RESULT_FALSE;
                $(self.entry).trigger(RO.Enum.Event.tryEditReadOnlyCell);
                result.exit = true;
                return;
            }
            if (!e.grid.isEditing()) {
                $(self.entry).trigger(RO.Enum.Event.gridBeginEditing);
            }
            var clientData = cell.getClientData();
            var editorId = clientData && clientData["im"] || cell.getEditorId();
            if (editorId == "5") {
                //result.exit = true;
                return;
            } else {
                var editor = getControlById(editorId, e);
                if (e.simpleType) {
                    self.editorOpened = false;
                    self.simpleEditType = true;
                    return;
                }
                if (editor) {
                    cell.setEditor(editor);
                } else {
                    $(self.entry).trigger(RO.Enum.Event.tryEditReadOnlyCell);
                    result.exit = true;
                    return;
                }
            }
            result.doit = result.add = COM.widget.Grid.RESULT_FALSE;
        };
        this._editorShowListener = function (e) {
            var cell = e.cell;
            var editor = cell.getEditor();
            var result = e.result;

            var onEditorChange = function () {
                var dom = $(".gridEditor", self.entry);
                var scrollHeight = dom[0].scrollHeight;
                var minHeight = parseFloat(dom.attr("minHeight"));
                var maxHeight = Math.max(100, minHeight);
                if (minHeight > 0) {
                    dom.css("height", scrollHeight > maxHeight ? maxHeight : scrollHeight > minHeight ? scrollHeight : minHeight);
                }
            };

            if (cell.isEditable() && editor) {
                $(editor).keyup(onEditorChange).gridHtml5Editor("show");
                $(self.entry).trigger(RO.Enum.Event.EditCell);
                onEditorChange();
            }
            result.clearShow = COM.widget.Grid.RESULT_FALSE;
            result.doit = COM.widget.Grid.RESULT_FALSE;
        };
        this._editorCloseListener = function (e) {
            var cell = e.cell;
            var result = e.result;
            var editor = cell.getEditor();
            if (editor == null) {
                result.doit = COM.widget.Grid.RESULT_FALSE;
                return;
            }
            var editorObj = $(editor);
            if (self.editorOpened) {
                editorObj.gridHtml5Editor("hide");
            }

            cell.setEditor(null);
            if ($(".gridEditor", self.entry).length > 0) {
                $(".gridEditor", self.entry).gridHtml5Editor("destroy");
                $(".gridEditor", self.entry).remove();
            }
           
            var editorHeight = COM.Util.browser.wk ? 7 : 0;
            editorObj = self.getDefaultEditor();
            var cellfigure = cell.getFigure();
            var offset = cellfigure.getBounds().getCopy();
            cellfigure.translateToAbsolute(offset);
            // editorObj.css("height", editorHeight);
            editorObj.css("top", offset.y - 1 + offset.height - editorHeight);
            editorObj.css("left", offset.x - 1);

            self.gridData.focus();
            var curCell = self.gridData.getCurrentCell();
            if (curCell) {
                self.gridData.select({ x: curCell.getCol(), y: curCell.getRow(), width: 1, height: 1 });
                editrowindex = curCell.getRow();
            }
            result.doit = COM.widget.Grid.RESULT_FALSE;
          
        };

        this.gridData.addListener('editorOpen', this._editorOpenListener);
        this.gridData.addListener('editorLocate', function (e) {
            e.result.doit = COM.widget.Grid.RESULT_FALSE;
        });
        this.gridData.addListener('editorResize', function (e) {
            e.result.doit = COM.widget.Grid.RESULT_FALSE;
        });
        this.gridData.addListener('editorShow', this._editorShowListener);
        this.gridData.addListener('editorHide', function (e) {
            e.result.doit = COM.widget.Grid.RESULT_FALSE;
        });
        this.gridData.addListener('editorClose', this._editorCloseListener);


    };


    this.cellExpand = function (cell, grid) {
        grid.beginUpdate();
        grid.setCurrentCell(cell.getCol(), cell.getRow());
        var row = cell.getRow() + 1,
            depth = cell.getDepth(),
            fkey = new RO.GridHtml5.Cell(cell, this).getDataRowAttr(RO.Enum.FloatRowKey),
            nexCell,
            newDep,
            rows = [];
        
        cell.setExpanded(true);
        
        if (row >= grid.getRowCount()) {
            $(this.entry).trigger(RO.Enum.Event.gridAsynLoad, { "begin": row, "guid": this.guid, "fkey": fkey });
            return [];
        } else {
            nexCell = grid.getCell(1, row);
            newDep = nexCell.getDepth();
            if (newDep === undefined || newDep <= depth) {
                $(this.entry).trigger(RO.Enum.Event.gridAsynLoad, { "begin": row, "guid": this.guid, "fkey": fkey });
                return [];
            }
        }
        
        while (newDep >= depth + 1) {
            if (newDep == depth + 1) {
                rows.push(row);
            }
            row++;
            nexCell = grid.getCell(1, row);
            newDep = nexCell.getDepth();
        }
        grid.setRowsVisible(rows, true);
        this.onFocusFloatRegion();
        grid.endUpdate();
        return rows;
    };

    this.addLoadListener = function () {
        var self = this;
        this._cellExpandListener = function (e) {
            var cell = e.cell;
            var grid = e.grid;
            self.cellExpand(cell, grid);
            var filterflag = false;
             var region = self.dataRegions;
             for (var regid in region) { 
                 if (region[regid].RegionExtension && $.parseJSON(region[regid].RegionExtension).OrderIndex > 0) {
                     filterflag = true;
                 }
             }
             if (filterflag) {
                     var drawArea = self.gridData.getDrawAreaInfo();
                     var scrollindex = drawArea.startRow;
                     var colwidth = drawArea.endCol;
                     var scrollend = drawArea.endRow;
                     var autofilter = { "rowIndex": scrollindex, "rowCount": scrollend - scrollindex +1};
                     self.filterAutoNoData(autofilter);
                 }
        };

        this._cellCollapseListener = function (e) {
            var cell = e.cell;
            var grid = e.grid;

            grid.beginUpdate();
            grid.setCurrentCell(cell.getCol(), cell.getRow());
            var row = cell.getRow() + 1;
            var depth = cell.getDepth();
            cell.setExpanded(false);

            var nexCell = grid.getCell(1, row);
            var newDep = nexCell.getDepth();
            var rows = [];
            while (newDep >= depth + 1) {
                if (nexCell.getExpandable()) {
                    nexCell.setExpanded(false);
                }
                rows.push(row);
                row++;
                nexCell = grid.getCell(1, row);
                newDep = nexCell.getDepth();
            }
            grid.setRowsVisible(rows, false);
            self.onFocusFloatRegion();
            grid.endUpdate();
              var filterflag = false;
             var region = self.dataRegions;
             for (var regid in region) { 
                 if (region[regid].RegionExtension && $.parseJSON(region[regid].RegionExtension).OrderIndex > 0) {
                     filterflag = true;
                 }
             }
             if (filterflag) {
                     var drawArea = self.gridData.getDrawAreaInfo();
                     var scrollindex = drawArea.startRow;
                     var colwidth = drawArea.endCol;
                     var scrollend = drawArea.endRow;
                     var autofilter = { "rowIndex": scrollindex, "rowCount": scrollend - scrollindex + 1};
                      self.filterAutoNoData(autofilter);                 
                 }            
        };

        this.gridData.addListener('cellExpand', this._cellExpandListener);
        this.gridData.addListener('cellCollapse', this._cellCollapseListener);
    };
    this.addOtherListener = function () {
        var self = this;
        var grid = this.gridData;


        grid.addListener('currentCellChanged', function (e) {
            if (e.col != -1 && e.row != -1) {
                self._cellValueChanged = false;
                triggerCellFocus();
                $(".gridHtml5", shell).trigger("grid_closeEditor");
                self.onFocusFloatRegion({ "preventUpdate": true });
            }
        });
        grid.addListener(COM.widget.Grid.Event.CURRENT_CELL_CHANGING, function (e) {
            shell.trigger(RO.Enum.Event.CellChanging, e);
        });

        grid.addListener(COM.widget.Grid.Event.CELL_DOUBLE_CLICK, function (e) {
            shell.trigger(RO.Enum.Event.cellDblClick, { "row": e.cell.getRow(), "col": e.cell.getCol() });
        });

        grid.addListener('scrolled', function () {
            if (self.editorOpened) {
                self.gridData.closeActiveEditor();
            }
            self.onFocusFloatRegion();
            $(self.entry).trigger(RO.Enum.Event.onScrolled);

            var filterflag = false;
            var region = self.dataRegions;
            for (var regid in region) {
                if (region[regid].RegionExtension && $.parseJSON(region[regid].RegionExtension).OrderIndex > 0) {
                    filterflag = true;
                }
            }
            if (filterflag) {
                var drawArea = self.gridData.getDrawAreaInfo();
                var scrollindex = drawArea.startRow;
                var colwidth = drawArea.endCol;
                var scrollend = drawArea.endRow;
                var autofilter = { "rowIndex": scrollindex, "rowCount": scrollend-scrollindex+1};
                self.filterAutoNoData(autofilter);
             }
        });

        grid.addListener(COM.widget.Grid.Event.CELL_MOUSE_DOWN, function (e) {
            if (e && e.cell) {
                var sels = e.grid.getSelections();
                if (sels.length == 1 && sels[0].width == 1 && sels[0].height == 1) {
                    self._curColIndexCache = self._curRowIndexCache = -1;
                } else {
                   
                    var curCell = e.grid.getCurrentCell();
                }
            }
        });

        grid.addListener(COM.widget.Grid.Event.SELECTION, function (e) {
            var selections = self.gridData.getSelections(), selection;
            var len = selections.length;
            var totalRow = false, totalCol = false;

            if (len == 1) {
                selection = selections[0];
                var width = selection.width;
                var height = selection.height;
                var x = selection.x;
                var y = selection.y;
                var curCell = self.gridData.getCell(x, y);
                var cellFlag = width == curCell.getColSpan() && height == curCell.getRowSpan();

                if (cellFlag && !e.isChanged) {
                    if (self._curColIndexCache != -1 && self._curRowIndexCache != -1 && x == self._curColIndexCache && y == self._curRowIndexCache) {
                        triggerCellFocus();
                        self.onFocusFloatRegion();
                        $(".gridHtml5", shell).trigger("grid_closeEditor");
                        return;
                    }
                } else if (width == grid.getColCount() - 1) {
                    totalRow = true;
                } else if (height == grid.getRowCount() - 1) {
                    totalCol = true;
                }
            }
            if (e.isChanged)
                shell.trigger(RO.Enum.Event.selectionChange, { "selectedRange": totalRow ? "row" : totalCol ? "col" : "" });
        });

        grid.addListener(COM.widget.Grid.Event.COL_RESIZE, function (e) {
            var datas = [];
            for (var i = 0; i < e.col.length; i++) {
                datas.push({ col: e.col[i], oldSize: e.oldSize[i], newSize: e.width[i] });
            }
            shell.trigger(RO.Enum.Event.gridColResize, { cols: datas, autoSize: e.autoSize });
        });

        grid.addListener(COM.widget.Grid.Event.ROW_RESIZE, function (e) {
            var datas = [];
            for (var i = 0; i < e.row.length; i++) {
                datas.push({ row: e.row[i], oldSize: e.oldSize[i], newSize: e.height[i] });
            }
            shell.trigger(RO.Enum.Event.gridRowResize, { rows: datas, autoSize: e.autoSize });
        });

        grid.addListener(COM.widget.Grid.Event.CUT, function (e) {
            self.gridCanEdit() && self.cut(e);
        });

        grid.addListener(COM.widget.Grid.Event.CELL_CHECKED, function (e) {
            var curCell = new RO.GridHtml5.Cell(e.cell, self);
            self.setValue(curCell, e.checked ? 1 : 0, "", false, false, null, true);
        });

        grid.addListener(COM.widget.Grid.Event.CELL_MOUSE_IN, function (e) {
            $(self.entry).trigger(RO.Enum.Event.cellMouseIn, { "row": e.cell.getRow(), "col": e.cell.getCol() });
        });
        grid.addListener(COM.widget.Grid.Event.CELL_MOUSE_OUT, function (e) {
            $(self.entry).trigger(RO.Enum.Event.cellMouseOut, { "row": e.cell.getRow(), "col": e.cell.getCol() });
        });
        grid.addListener(COM.widget.Grid.Event.CELL_MOUSE_CLICK, function (e) {
            $(self.entry).trigger(RO.Enum.Event.cellMouseClick, { "row": e.cell.getRow(), "col": e.cell.getCol() });
        });

        //监听表格容器尺寸变化
        shell.parent().off("layoutonresizeall").on("layoutonresizeall", function () {
            if (self.editorOpened) {
                self.gridData.closeActiveEditor();
            }
            window.setTimeout(function () {
                var entry = $(self.entry);
                var width = entry.width();
                var height = entry.height();
                entry.trigger("Event_ResetScrollBar", { "width": width, "height": height });
                self.gridData.setSize && self.gridData.setSize(width, height);
                entry.trigger("gridInitData");
                self.onFocusFloatRegion();
            }, 100);
        });
        shell.off("scrollBar_resize").on("scrollBar_resize", function (e) {
            e.stopPropagation();
            var gridControllor = self.gridData.gridModel;
            gridControllor && gridControllor.editPart.freshScrollPlaceHolderSize();
        });
        shell.off("enterNext").on("enterNext", function (e, data) {
            if (self.editorOpened)
                self.gridData.closeActiveEditor();
            $(".gridHtml5", self.entry).trigger("grid_closeEditor");
            self.gridData.keyEnter();
        });
        function triggerCellFocus() {
            self.timeOutId = window.setTimeout(function () {
                $(self.entry).trigger(RO.Enum.Event.cellfocus);
            }, 0);
        }
    };
    this.addKeyEventListener = function () {
        var self = this;
        this._onKeyDownEvent = function (event) {
            var e = event.origin;
            var grid = event.grid;
            var curCell = grid.getCurrentCell();
            //if (e.keyCode == 8 || e.keyCode == 9) { //BackSpace键 Tab键
            if (e.keyCode == 8) { //BackSpace键
                if (curCell.cell) {
                    var col = curCell.getCol(), row = curCell.getRow();
                    if (self.editorOpened) {
                        return false;
                    }
                    preventDefault(e);

                    if (!self.gridCanEdit()) {
                        return false;
                    }

                    if (curCell.getClientData()["im"] == 4) {
                        $("td.imt4Btn", curCell.getControl()).click();
                        return false;
                    }
                    grid.openEditor(col, row);
                    $(".gridEditor", self.entry).val("");
                    $(".gridHtml5", self.entry).trigger("grid_backspaceEditor", { cell: curCell, gridModel: self, action: "dblclick" });
                    return false;
                }
                return true;
            }else if (e.keyCode == 113) { //F2键
                if (curCell.cell) {
                    var col = curCell.getCol(), row = curCell.getRow();
                    if (self.editorOpened) {
                        return false;
                    }

                    preventDefault(e);

                    if (!self.gridCanEdit()) {
                        return false;
                    }

                    if (curCell.getClientData()["im"] == 4) {
                        $("td.imt4Btn", curCell.getControl()).click();
                        return false;
                    }
                    grid.openEditor(col, row);
                  //  $(".gridEditor", self.entry).val("");
                    $(".gridHtml5", self.entry).trigger("grid_backspaceEditor", { cell: curCell, gridModel: self, action: "dblclick" });
                    return false;
                }
                return true;
            } else if (e.keyCode == 46) { //Del键
                if (curCell.cell && !self.editorOpened) {
                    preventDefault(e);

                    if (!self.gridCanEdit()) {
                        return false;
                    }

                    // 组织改变的单元格，发送valuechange事件
                    var selections = grid.getSelections();
                    var changeValues = [];
                    var modified = false;
                    var editMode = grid.getEditMode();

                    if (selections) {
                        $(self.entry).trigger(RO.Enum.Event.onDeletePressed, { "selections": selections });
                        for (var k = 0; k < selections.length; k++) {
                            var selection = selections[k];
                            for (var i = 0; i < selection.height; i++) {
                                for (var j = 0; j < selection.width; j++) {
                                    var c = grid.getCell(selection.x + j, selection.y + i);
                                    if (c) {
                                        if (c.getCheckable()) {
                                            c.setChecked(false);
                                        }
                                        var roCell = new RO.GridHtml5.Cell(c, self);
                                        var oldValue = roCell.getEditText();
                                        var oldTitle = roCell.getShowText();
                                        if ((editMode == COM.widget.Grid.EDIT_MODE_EDIT || roCell.getCanEdit()) && oldValue != undefined) {
                                            var fkey = roCell.getDataRowAttr(RO.Enum.FloatRowKey);
                                            modified = self.setValue(roCell, "", "", true, true);
                                            if (modified) {
                                                changeValues.push({ row: selection.y + i, col: selection.x + j, oldValue: oldValue, newValue: "",oldTitle:oldTitle,newTitle:"", fKey: fkey });
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if (changeValues.length > 0) {
                        $(self.entry).trigger(RO.Enum.Event.valuechange, { "changeValues": changeValues, "operationType": "delete" });                       
                    }

                }
                return true;
            } else if ((e.ctrlKey || e.metaKey) && e.keyCode == 65) { //Ctrl+A
                preventDefault(e);
                var rowCount = grid.getRowCount();
                var colCount = grid.getColCount();
                grid.select({ x: 1, y: 1, width: colCount, height: rowCount });
                grid.focus();
                return false;
            }
            return true;
        };

        this.gridData.addListener("KeyDownEvent", this._onKeyDownEvent);
    };

    //处理浮动区域边框样式
    this.onFocusFloatRegion = function (data) {
        /************浮动区域边框样式处理********************/
        if (this.hideFloatRegionBorder)
            return;
        var self = this;
        focusFloatRegion(data);

        function focusFloatRegion(e) {
            if (!!e && e.col != undefined && e.row != undefined) {
                self.gridData.setCurrentCell(e.col, e.row);
            }
            var cell = self.gridData.getCurrentCell();
            if (!cell || !cell.cell) { //表格没有焦点时，如果存在浮动区域样式，则清除
                delFloatRegionStyle();
                if (self.options["currentRegId"]) {
                    delete self.options["currentRegId"];
                }
                floatRegionDefines = {};
                return;
            }
            var curCell = new RO.GridHtml5.Cell(cell, self);
            var regId = null;
            var rid = curCell.getDataRowAttr(RO.Enum.RegId);
            if (!rid) {
                for (var reg in self.dataRegions) {
                    var curReg = self.dataRegions[reg];
                    if (curReg.RegionKind == 0) {
                        self.options["currentRegId"] = rid;
                    }
                }
                delFloatRegionStyle();//如果存在浮动区域样式，则清除
                if (self.options["currentRegId"]) {
                    delete self.options["currentRegId"];
                }
                floatRegionDefines = {};
            } else {
                if (e && e.preventUpdate == true && rid == self.options["currentRegId"]) {
                    return;
                }

                var region = self.dataRegions[rid];
                if (region != null && (region.RegionKind === 3)) { //行浮动区域 由DataRegionKind定义
                    regId = rid;
                }
                if (regId) {
                    var regionChanged = e && e.regionChanged;
                    delFloatRegionStyle();
                    addFloatRegionDefine(regId, regionChanged);
                    addFloatRegionStyle(floatRegionDefines[regId]);
                } else {
                    delFloatRegionStyle();
                }
                self.options["currentRegId"] = regId;
            }
        }

        //单元格原始边框样式定义
        function origCellBorder(col, row) {
            return col + "_" + row;
        }
        //区域定义
        function regionDefine(startColIndex, endColIndex, startRowIndex, endRowIndex, regionKind) {
            if (startRowIndex > endRowIndex || startColIndex > endColIndex) {
                alert("获取区域边界错误!");
                return null;
            }
            return { top: startRowIndex, bottom: endRowIndex, left: startColIndex, right: endColIndex, kind: regionKind };
        }

        //为区域添加边框样式
        function addFloatRegionStyle(regionDef) {
            var top = regionDef.top;
            var bottom = regionDef.bottom;
            var left = regionDef.left;
            var right = regionDef.right;
            origRegionBoundary = [];
            var curArea = self.gridData.getDrawAreaInfo();
            var headerColCount = self.gridData.getColHeaderCount();
            var headerRowCount = self.gridData.getRowHeaderCount();
            var drawL, drawR, drawT, drawB, newLeft, newTop, newRight, newBottom, r, c, headerL, headerT;
            drawB = drawL = drawR = drawT = true;
            newLeft = curArea.startCol;
            newTop = curArea.startRow;
            newRight = curArea.endCol;
            newBottom = curArea.endRow;
            headerL = headerT = false;

            if (curArea.startCol > left) {
                if (left <= headerColCount) {
                    headerL = true;
                } else {
                    drawL = false;
                }
            }
            if (curArea.endCol < right) {
                drawR = false;
            }
            if (curArea.startRow > top) {
                if (top <= headerRowCount) {
                    headerT = true;
                } else {
                    drawT = false;
                }
            }
            if (curArea.endRow < bottom) {
                drawB = false;
            }

            if (drawL || drawR) {
                newTop = Math.max(top, newTop);
                newBottom = Math.min(bottom, newBottom);
            }
            if (drawB || drawT) {
                newLeft = Math.max(newLeft, left);
                newRight = Math.min(newRight, right);
            }
            self.gridData.beginUpdate();
            if (drawL && left >= newLeft) {
                for (r = newTop; r <= newBottom; r++) {
                    setBorderStyle(origRegionBoundary, left - 1, r, regionBorderDirection.LEFT);
                }
            }
            if (drawR) {
                for (r = newTop; r <= newBottom; r++) {
                    setBorderStyle(origRegionBoundary, right, r, regionBorderDirection.RIGHT);
                }
            }
            if (drawT && top >= newTop) {
                for (c = newLeft; c <= newRight; c++) {
                    setBorderStyle(origRegionBoundary, c, top - 1, regionBorderDirection.TOP);
                }
            }
            if (drawB) {
                for (c = newLeft; c <= newRight; c++) {
                    setBorderStyle(origRegionBoundary, c, bottom, regionBorderDirection.BOTTOM);
                }
            }
            if (headerL) {
                for (c = left; c < headerColCount; c++) {
                    setBorderStyle(origRegionBoundary, c, bottom, regionBorderDirection.BOTTOM);
                    setBorderStyle(origRegionBoundary, c, top - 1, regionBorderDirection.TOP);
                }
                for (r = newTop; r <= newBottom; r++) {
                    setBorderStyle(origRegionBoundary, left - 1, r, regionBorderDirection.LEFT);
                }
            }
            if (headerT) {
                for (r = top; r < headerRowCount; r++) {
                    setBorderStyle(origRegionBoundary, right, r, regionBorderDirection.RIGHT);
                    setBorderStyle(origRegionBoundary, left - 1, r, regionBorderDirection.LEFT);
                }
                for (c = newLeft; c <= newRight; c++) {
                    setBorderStyle(origRegionBoundary, c, top - 1, regionBorderDirection.TOP);
                }
            }
            self.gridData.endUpdate();
        }

        function getAvailableData(col, row, borderDirection) {
            var hidden, changed = false;
            switch (borderDirection) {
                case regionBorderDirection.BOTTOM:
                case regionBorderDirection.TOP:
                    hidden = self.gridData.isRowHidden(row);
                    while (hidden && row >= 1) {
                        row--;
                        hidden = self.gridData.isRowHidden(row);
                        changed = !(changed && row == 0);
                    }
                    break;
                case regionBorderDirection.RIGHT:
                case regionBorderDirection.LEFT:
                    hidden = self.gridData.isColHidden(col);
                    while (hidden && col >= 1) {
                        col--;
                        hidden = self.gridData.isColHidden(col);
                        changed = !(changed && col == 0);
                    }
                    break;
            }

            return changed ? { "cell": self.gridData.getCell(col, row), "col": col, "row": row } : null;
        }

        function setBorderStyle(boundary, col, row, borderDirection) {
            var cell = self.gridData.getCell(col, row);
            if (!cell.cell) {
                return false;
            }

            var cellData = getAvailableData(col, row, borderDirection);
            if (cellData) {
                cell = cellData.cell;
                col = cellData.col;
                row = cellData.row;
            }

            var mergedInfo = cell.getMergedInfo();
            if (mergedInfo) {
                col = mergedInfo.col;
                row = mergedInfo.row;
                cell = self.gridData.getCell(col, row);
            }
            var clientData = cell.getClientData();
            if (!clientData) {
                clientData = {};
            }
            var origCell = origCellBorder(col, row);
            boundary.push(origCell);

            var figure;
            if (row == 0 && (borderDirection == regionBorderDirection.TOP)) {
                cell.setBottomBorderStyle(self.floatRegionBorderWidth > 1 ? RO.Enum.BorderStyle.BOLD : RO.Enum.BorderStyle.SOLID);
                cell.setBottomBorderColor(self.floatRegionBorderColor);
            } else if (col == 0 && borderDirection == regionBorderDirection.LEFT) {
                cell.setRightBorderStyle(self.floatRegionBorderWidth > 1 ? RO.Enum.BorderStyle.BOLD : RO.Enum.BorderStyle.SOLID);
                cell.setRightBorderColor(self.floatRegionBorderColor);
            } else if (borderDirection == regionBorderDirection.BOTTOM || borderDirection == regionBorderDirection.TOP) {
                clientData.userBottomBorder = true;
                figure = cell && cell.getFigure();
                figure && figure.repaint();
            } else if (borderDirection == regionBorderDirection.RIGHT || borderDirection == regionBorderDirection.LEFT) {
                clientData.userRightBorder = true;
                figure = cell && cell.getFigure();
                figure && figure.repaint();
            }
            return true;
        }

        //清除区域边框样式
        function delFloatRegionStyle() {
            if (origRegionBoundary == null) {
                return;
            }
            self.gridData.beginUpdate();
            for (var i = 0; i < origRegionBoundary.length; i++) {
                var origCell = origRegionBoundary[i].split("_");
                var curCell = self.gridData.getCell(origCell[0], origCell[1]);
                if (curCell) {
                    if (curCell.getRow() == 0) {
                        curCell.setBottomBorderStyle(RO.Enum.BorderStyle.SOLID);
                        curCell.setBottomBorderColor(RO.Enum.DefaultBorderColor);
                    } else if (curCell.getCol() == 0) {
                        curCell.setRightBorderStyle(RO.Enum.BorderStyle.SOLID);
                        curCell.setRightBorderColor(RO.Enum.DefaultBorderColor);
                    }

                    var clientData = curCell.getClientData();
                    if (clientData) {
                        delete clientData.userBottomBorder;
                        delete clientData.userRightBorder;
                        var figure = curCell.getFigure();
                        figure && figure.repaint();
                    }
                }

            }
            self.gridData.endUpdate();
            origRegionBoundary = null;
        }

        //添加浮动区域

        function addFloatRegionDefine(regId) {
            var region = self.dataRegions[regId];
            var regKind = region.RegionKind;
            var top = region.RegionTop;
            var bottom = region.RegionBottom;
            var left = region.RegionLeft;
            var right = region.RegionRight;
            if (regKind === 3) {
                bottom += region.FloatDataCount - 1;
            }
            floatRegionDefines[regId] = regionDefine(left, right, top, bottom, regKind);
        }

        /************end************************/
    };
    //异步处理浮动区域边框样式
    this.onAsyFocusFloatRegion = function (data) {
        var self = this;
        self.onFocusFloatRegion(data);
    };
    //异步加载数据格式化
    this.formatDataForAsyn = function (asynFormData) {
        var byRow = asynFormData.byRow;
        var asynData = asynFormData.list;
        var dataCount = asynData.length;
        for (var i = 0; i < dataCount; i++) {
            var dataItem = asynData[i];
            var begin = dataItem.begin;
            var end = dataItem.end;
            var data = dataItem.data;
            var colCount, rowCount;
            if (byRow) {
                colCount = data[0].length;
                rowCount = end - begin + 1;
            } else {
                colCount = end - begin + 1;
                rowCount = data[0].length;
            }
            for (var r = 0; r < rowCount; r++) {
                for (var c = 0; c < colCount; c++) {
                    var cell = data[r][c];
                    // data[r][c] = formatDataForLoad(cell, data, this);
                }
            }
        }
        $(this.entry).trigger("gridInitData");
        return asynFormData;
    };
    //开始修改数据区域
    this.beginModifyDataRegions = function () {
        if (this._actionLock) return true;
        this._actionLock = true;
        var selection = this.gridData.getSelection();
        this._curColIndexCache = !!selection ? selection.x : -1;
        this._curRowIndexCache = !!selection ? selection.y : -1;
        this.gridData.setCurrentCell(-1, -1, null, true);
        return false;
    };

    //结束修改数据区域
    this.endModifyDataRegions = function (regId, count, origSelection) {
        var curRegion;

        if (count != 0) {
            curRegion = this.dataRegions[regId];
            curRegion.FloatDataCount += count;
            if (curRegion.RegionKind === 3) {
                for (var rReg in this.dataRegions) {
                    var rRegion = this.dataRegions[rReg];
                    if (rRegion.RegionTop > curRegion.RegionTop) {
                        rRegion.RegionTop += count;
                        rRegion.RegionBottom += count;
                    }
                }
            }
        }
        this._actionLock = false;
        var oldRowIndex = this._curRowIndexCache;
        var lastCol = this.gridData.getColCount() - 1;
        var lastRow = this.gridData.getRowCount() - 1;
        if (this._curColIndexCache > lastCol) {
            this._curColIndexCache = lastCol;
        }
        if (this._curRowIndexCache > lastRow) {
            this._curRowIndexCache = lastRow;
        }
        if (this._curColIndexCache == -1 && this._curRowIndexCache == -1) {
            var selections = grid.getSelections();
            if (selections) {
                if (selections.length > 0) {
                    this._curRowIndexCache = selections[0].y > lastRow ? lastRow : selections[0].y;
                    this._curColIndexCache = selections[0].x > lastCol ? lastCol : selections[0].x;
                } else {
                    this._curRowIndexCache = 1;
                    this._curColIndexCache = 1;
                }
            }
        }

        var self = this;
        if (origSelection) {
            curRegion = this.dataRegions[regId];
            var top = curRegion.RegionTop;
            var floatDataCount = curRegion.FloatDataCount;
            var length = floatDataCount + top - 1;
            if (self._curColIndexCache == -1) {
                this._curColIndexCache = origSelection.x > lastCol ? lastCol : origSelection.x;
            }
            if (count < 0 && origSelection.y > length) {
                var curcellRow = length + oldRowIndex - origSelection.y;
                if (curcellRow > self.gridData.getRowCount() - 1) {
                    curcellRow = self.gridData.getRowCount() - 1;
                }
                origSelection.y = length;
                self.gridData.setCurrentCell(self._curColIndexCache, curcellRow, null, true);
                self.gridData.select(origSelection);
            } else {
                self.gridData.setCurrentCell(self._curColIndexCache, origSelection.y, null, true);
                self.gridData.select(origSelection);
            }
        } else {
            self.gridData.setCurrentCell(self._curColIndexCache, self._curRowIndexCache, null, true);
            if (self._curColIndexCache > 0 && self._curRowIndexCache > 0) {
                self.gridData.select({ x: self._curColIndexCache, y: self._curRowIndexCache, width: 1, height: 1 });
            }
        }
        self.onFocusFloatRegion();
    };
    //修改浮动数据状态
    this.ModifyDataState = function (key, state, isNew, regId) {
        var dataRow = this.extendData[key];
        if (!dataRow) {
            dataRow = this.extendData[key] = {};
        }
        var tmpDataState = this.TempDataStates[regId];
        if (!tmpDataState) {
            tmpDataState = this.TempDataStates[regId] = {};
        }
        var tmpData = tmpDataState && tmpDataState[key];
        if (state === 0 || state === 9) {
            dataRow[RO.Enum.State] = state;
            tmpDataState[key] = new dataStateDef(key, state, isNew);
        } else if (state === 1) {
            var curState;
            if (tmpData) {
                curState = tmpData.IsNew ? 1 : 2;
                tmpData.State = curState;
            } else if (dataRow[RO.Enum.State] == 1) {
                curState = 1;
                tmpDataState[key] = new dataStateDef(key, 1, true);
            } else {
                curState = isNew ? 1 : 2;
                tmpDataState[key] = new dataStateDef(key, curState, isNew);
            }
            dataRow[RO.Enum.State] = curState;
        } else if (state == 2) {
            dataRow[RO.Enum.State] = 3;
            if (tmpData) {
                if (tmpData.IsNew) {
                    delete tmpData;
                } else {
                    tmpData.State = 3;
                }
            } else if (dataRow[RO.Enum.State] != 1) {
                tmpDataState[key] = new dataStateDef(key, 3, false);
            }
        }
    };
    //初始化单元格对象
    this.InitCell = function (regId, curCell, showText, simpleInsert) {
        var self = this;
        curCell.cell.setPadding(RO.Enum.DefaultCellPadding[0], RO.Enum.DefaultCellPadding[1], RO.Enum.DefaultCellPadding[2], RO.Enum.DefaultCellPadding[3]);
        if (curCell.getAttrByName("emt") == "1") {
            curCell.cell.setEditText("  ");
            curCell.cell.setShowText("--");
            curCell.setCanEdit(false);
        }

        var dt = curCell.getAttrByName("dt");
        if (showText || dt == "4") {
            var filter = { "editText": showText, "showText": showText };
            var formatValue = self.filterData(curCell, filter);
            if (formatValue) {
                curCell.cell.setEditText(formatValue.editText);
                curCell.cell.setShowText(formatValue.showText);
            }
        }
        this.filterExtFormData([curCell]);
    };
    //深度拷贝ClientData
    this.copyClientData = function (clientData) {
        if (clientData == null) return null;
        var newData = {};
        for (var attr in clientData) {
            newData[attr] = clientData[attr];
        }
        return newData;
    };
    //单元格赋值
    this.setValue = function (cell, val, title, silence, noFormat, customEventData, fixRowHeight, isExtEditText) {
        cell = cell || new RO.GridHtml5.Cell(this.gridData.getCurrentCell(), this);
        if (cell.cell.cell) {
            var editMode = this.gridData.getEditMode();
            if (editMode != COM.widget.Grid.EDIT_MODE_EDIT && !cell.getCanEdit()) return false;
            var editText, showText, oldValue = isExtEditText ? cell.getAttrByName(RO.Enum.ExtEditText) : cell.getEditText();
            if (cell.getAttrByName(RO.Enum.EnumDisplayMode) == 1 && typeof val == "object") {
                editText = val[0] == undefined ? "" : val[0];
                showText = val[1];
            } else {
                editText = val == undefined ? "" : val;
                showText = title;
            }
            if (cell.cell.getCellMode() == COM.widget.Grid.Cell.Cell_MODE_TREE && cell.cell.getCheckable()) {
                var checked = editText == 1 ? 1 : 0;
                cell.cell.setChecked(checked);
            }

            var filter = { "editText": editText, "showText": showText };
            var formatValue = noFormat ? filter : this.filterData(cell, filter);

            if (!formatValue) {
                // 输入值非法
                return false;
            } else if (!isExtEditText && (oldValue == formatValue.editText && cell.getShowText() == formatValue.showText)) {
                // 没有修改值
                return false;
            }

            var fkey = null;
            var regId = cell.getDataRowAttr(RO.Enum.RegId);
            if (regId) {
                var region = this.dataRegions[regId];
                if (region.RegionKind != 0) {
                    fkey = cell.getDataRowAttr(RO.Enum.FloatRowKey);
                    if (fkey && fkey != "0") {
                        this.ModifyDataState(fkey, 1, false, regId);
                    } else {
                        fkey = this.newId() + "";
                        var attrs;
                        if (region.RegionKind == 3) {
                            var row = cell.getRowIndex();
                            attrs = this.getDataRowAttrs(null, row);
                            attrs[RO.Enum.FloatRowKey] = fkey;
                            this.setDataRowAttrs(null, row, attrs);
                        }
                        this.ModifyDataState(fkey, 1, true, regId);
                    }
                }
            }

            var oldValue = cell.getEditText(), text = !noFormat || !title ? formatValue.showText : title;
            var oldTitle = cell.getShowText();
            cell.cell.setEditText(formatValue.editText);
            cell.cell.setShowText(text, silence);

            if (!fixRowHeight) {
                var height = this.getAutoRowHeight(cell, text);
                if (height > COM.widget.Grid.GridFigure.MAXROWHEIGHT) {
                    height = COM.widget.Grid.GridFigure.MAXROWHEIGHT;
                }
                if (isExtEditText) {
                    cell.setAttrByName(RO.Enum.ExtEditText, formatValue.editText == "" ? null : formatValue.editText);
                    $(cell.getControl()).trigger("updateControl");
                } else {
                    cell.cell.setEditText(formatValue.editText);
                }
                cell.cell.setShowText(text, silence);
                this.refrashRowHeight(cell.getRowIndex(), height);
            } else {
                if (isExtEditText) {
                    cell.setAttrByName(RO.Enum.ExtEditText, formatValue.editText == "" ? null : formatValue.editText);
                    $(cell.getControl()).trigger("updateControl");
                } else {
                    cell.cell.setEditText(formatValue.editText);
                }
                cell.cell.setShowText(text, silence);
            }

            cell.setAttrByName("modified", 1);

            if (!silence) {
                this._cellValueChanged = true;
                var changeValues = [{ row: cell.getRowIndex(), col: cell.getColIndex(), oldValue: oldValue, newValue: formatValue.editText, oldTitle:oldTitle,newTitle:formatValue.showText, fKey: fkey }];
                $(this.entry).trigger(RO.Enum.Event.valuechange, { "changeValues": changeValues, "customEventData": customEventData });
            }
            return true;
        }
        return false;
    };

    //更新数据区域边框
    this.updateRegBorderStyle = function (index, count, region, isDelete) {
        var end, curIndex = index, endIndex = index + count - 1, preCell, oldStyle, newStyle, bCell, rCell, cell;
        var regTop = region.RegionTop;
        end = regTop + region.FloatDataCount - 1;
        if (curIndex >= end) {
            var bottomCell = isDelete ? this.gridData.getCell(1, index > 0 ? index - 1 : 0) : this.gridData.getCell(1, endIndex);
            preCell = this.gridData.getCell(1, end);
            oldStyle = preCell.getBottomBorderStyle();
            newStyle = preCell.getRightBorderStyle();
            var bStyle = bottomCell.getBottomBorderStyle();
            if (bStyle !== oldStyle || oldStyle != newStyle) {
                var colCount = this.gridData.getColCount();
                if (isDelete) {
                    for (var c = 0; c < colCount; c++) {
                        bCell = this.gridData.getCell(c, index > 0 ? index - 1 : 0);
                        bCell.setBottomBorderStyle(oldStyle);
                    }
                } else {
                    for (var c1 = 0; c1 < colCount; c1++) {
                        bCell = this.gridData.getCell(c1, endIndex);
                        bCell.setBottomBorderStyle(oldStyle);
                    }
                    for (var i = 0; i < count; i++) {
                        for (var c2 = 0; c2 < colCount; c2++) {
                            cell = this.gridData.getCell(c2, end + i);
                            cell.setBottomBorderStyle(newStyle);
                        }
                    }
                }
            }
        }
    };
    //更新grid行列数数组，用于异步加载
    this.updateGridSize = function (index, count, regId, isDelete) {
        if (isDelete) {
            count = Math.min(count, this.rowList.length - index);
            for (var r = 0; r < count; r++) {
                var rFkey = this.rowList[index + r].fkey;
                delete this.extendData[rFkey];
            }
            this.rowList.splice(index, count);
        } else {
            var newArray = new Array(count);
            for (var i = 0; i < count; i++) {
                newArray[i] = { index: -1, reg: regId, state: 1 };
            }
            newArray.splice(0, 0, index, 0);
            this.rowList.splice.apply(this.rowList, newArray);
        }
    };

    //异步待处理事件
    this.getAsynAction = function (funName) {
        for (var i = 0, count = this._actions.length; i < count; i++) {
            var actioni = this._actions[i];
            if (actioni.Name == funName) {
                return actioni;
            }
        }
        return null;
    };
    this.addAsynAction = function (funName, args) {
        var self = this;
        var curAction = {};
        curAction.Name = funName;
        curAction.Args = args;
        var flag = false;
        var interValId = "";

        var addAction = function (me, action) {
            if (!me._actionLock) {
                for (var i = 0, count = me._actions.length; i < count; i++) {
                    var actioni = me._actions[i];
                    if (actioni.Name == funName) {
                        me._actions.splice(i, 1, action);
                        flag = true;
                        break;
                    }
                }
                if (!flag) {
                    me._actions.push(action);
                }
                window.clearInterval(interValId);
            }
        };
        if (!this._actionLock) {
            addAction(self, curAction);
        }
        else {
            interValId = window.setInterval(function () {
                addAction(self, curAction);
            }, 1000);
        }
    };
    this.removeAsynAction = function (funName) {
        var interValId = "";
        var self = this;
        var removeAction = function (me, name) {
            if (!me._actionLock) {
                if (!me._actions) return;
                for (var i = 0, count = me._actions.length; i < count; i++) {
                    var action = me._actions[i];
                    if (action.Name == name) {
                        me._actions.splice(i, 1);
                    }
                }
                window.clearInterval(interValId);
            }
        };
        if (!this._actionLock) {
            removeAction(this, funName);
        }
        else {
            interValId = window.setInterval(function () {
                removeAction(self, funName);
            }, 1000);
        }
    };
    this.recoveryAsynAction = function () {
        var $this = this;
        var intervalId = "";
        var doRecovery = function (me) {
            if (me._actions.length > 0) {
                me._actionLock = true;
                var com = "";
                for (var i = 0, count = me._actions.length; i < count; i++) {
                    com += "obj." + me._actions[i].Name + "(";
                    var actionArgs = me._actions[i].Args;
                    var argLen = actionArgs.length;
                    if (actionArgs && argLen > 0) {
                        com += $.toJSON(actionArgs[0]);
                        for (var j = 1; j < argLen; j++) {
                            com += "," + $.toJSON(actionArgs[j]);
                        }
                    }
                    com += ");";
                }
                try {
                    eval(com);
                } catch (e) {
                    jAlert("GridHtml5异步加载异常：" + e.message, "系统提示");
                } finally {
                    me._actions = [];
                }
            } else {
                window.clearInterval(intervalId);
                me._actionLock = false;
            }
        };

        if (this._actions && !this._actionLock) {
            doRecovery(this);
        } else {
            intervalId = window.setInterval(function () {
                doRecovery($this);
            }, 1000);
        }
    };

    this.setSectionMerge = function (cIndex, copy, templateCells) {
        var merge = function (col, row, cellObj, self) {
            var rowSpan = cellObj.getRowSpan ? cellObj.getRowSpan() : cellObj.rowSpan;
            var colSpan = cellObj.getColSpan ? cellObj.getColSpan() : cellObj.colSpan;
            if (rowSpan > 1 || colSpan > 1) {
                self.gridData.mergeCell(col, row, colSpan, rowSpan);
            }
            return new Array(col + colSpan, row + rowSpan);
        };
        var gridData = this.gridData;
        var colCount = gridData.getColCount();
        var cCell, indexArray;
        for (var c = 1; c < colCount;) {
            if (templateCells && templateCells.length > 0) {
                cCell = templateCells[c];
            } else {
                cCell = gridData.getCell(c, copy);
            }
            indexArray = merge(c, cIndex, cCell, this);
            c = indexArray[0];
        }
    };
    //创建简单rowkey
    this.newId = function () {
        return "tmpkey_" + this._autoId++;
    };
    //获取单元格所在数据行扩展属性集合
    this.getDataRowAttrs = function (cell, index) {
        var curIndex;
        if (this.options["FormRegionKind"] == RO.Enum.FormRegionKind.ROW) {
            curIndex = cell && cell.getRow() || index;
            return this.rowList[curIndex];
        }
        return {};
    };
    //获取单元格所在数据行扩展属性集合
    this.setDataRowAttrs = function (cell, index, data) {
        var curIndex;
        if (this.options["FormRegionKind"] == RO.Enum.FormRegionKind.ROW) {
            curIndex = cell && cell.getRow() || index;
            this.rowList[curIndex] = data;
        }
        this.extendData[data[RO.Enum.FloatRowKey]] = data;
    };

    this.cut = function (rect) {
        var Data= this;
        var modify = false;
        var changeValues = [];
        for (var r = 0; r < rect.height; r++) {
            for (var c = 0; c < rect.width; c++) {
                var row = r + rect.y, col = c + rect.x;
                var cell = this.gridData.getCell(col, row);
                if (cell.getCheckable()) {
                    cell.setChecked(false);
                }
                var oldValue = cell.getEditText(),oldTitle=cell.getShowText();
                if (oldValue != "") {
                    var gridCell = new RO.GridHtml5.Cell(cell, this);
                    var fkey = gridCell.getDataRowAttr(RO.Enum.FloatRowKey);
                    modify = this.setValue(gridCell, "", "", true, true);
                    if (modify) {
                        changeValues.push({ "row": row, "col": col, oldValue: oldValue, newValue: "",oldTitle:oldTitle,newTitle:"", fKey: fkey });
                    }
                }
            }
        }
        if (modify) {
            $(this.entry).trigger(RO.Enum.Event.valuechange, { "changeValues": changeValues, "operationType": "cut" });
        }
    };

    //获取自动行高值
    this.getAutoRowHeight = function (cell, showText) {

        var fit = cell.isFitFontSize();
        var wrapLine = cell.isWrapLine();
        var multiLine = cell.isMultiLine();
        var dt = cell.getAttrByName("dt");
        var mergInfo = cell.getMergedInfo();
        var vertText = cell.isVertText();

        if (mergInfo != null || cell.getRowSpan() > 1 || cell.getColSpan() > 1) {
            return 0;
        }

        if (fit) {
            return 0;
        }

        if (!this.designMode && ((dt != "1" && dt != 0))) {
            return 0;
        }

        if (vertText && wrapLine && multiLine) {
            return 0;
        } else if (!vertText && !wrapLine && !multiLine) {
            return 0;
        }

        if (showText == "" || showText == null) {
            var rowcount = cell.getRowIndex();
            return 0;// formData.rows[rowcount].size;//默认行高
        }

        var fontSize = Math.round(cell.getRealFontSize());
        var indentPx = cell.getIndentPx();
        var indent = cell.getIndent();
        var bottomBorder = cell.cell.getBottomBorderStyle();
        var rightBorder = cell.cell.getRightBorderStyle();
        var paddingHeight = RO.Enum.DefaultCellPadding[0] + RO.Enum.DefaultCellPadding[2] + (bottomBorder == RO.Enum.BorderStyle.BOLD || bottomBorder == RO.Enum.BorderStyle.DOUBLE ? 2 : 1);
        var paddingWidth = RO.Enum.DefaultCellPadding[1] + RO.Enum.DefaultCellPadding[3] + (rightBorder == RO.Enum.BorderStyle.BOLD || rightBorder == RO.Enum.BorderStyle.DOUBLE ? 2 : 1);

        var figure = cell.cell.getFigure();
        var width = 0, colSpan = cell.getColSpan(), rowSpan = cell.getRowSpan(), col = cell.getColIndex();
        if (rowSpan == 1 && colSpan > 1) {
            for (var i = colSpan - 1; i >= 0; i--) {
                width += this.gridData.getColWidth(col + i);
            }
        } else {
            width = this.gridData.getColWidth(col);
        }
        width -= paddingWidth;
        return calcTextHeight(this._graphics, vertText, width, fontSize, cell.getFontName(), showText, paddingHeight, wrapLine, multiLine, indent, indentPx, cell.getFontBold());
    };

    this.refrashRowHeight = function (rowIndex, newHeight) {
        var colCount = this.gridData.getColCount();
        var rowHeight = this.gridData.getRowHeight(rowIndex);
        if (rowHeight < newHeight) {
            this.gridData.setRowHeight(rowIndex, newHeight, true);
        } else if (rowHeight > newHeight) {
            if (!this.designMode) {
                var size = formData.rows[rowIndex].size;
                var height = size;
                for (var c = 1; c < colCount; c++) {
                    var roCell;
                    var cell = this.gridData.getCell(c, rowIndex);
                    if (cell.cell) {
                        roCell = new RO.GridHtml5.Cell(cell, this);
                        height = Math.max(height, this.getAutoRowHeight(roCell, roCell.getShowText()));
                        if (height > COM.widget.Grid.GridFigure.MAXROWHEIGHT) {
                            height = COM.widget.Grid.GridFigure.MAXROWHEIGHT;
                        }
                    }
                }
                if (height < rowHeight) {
                    this.gridData.setRowHeight(rowIndex, height, true);
                }
            }
        }
    };

    this.formatAlignByCell = function (cell, dValue) {

        var autoHoriz = cell.horzAlign == COM.widget.Grid.Cell.ALIGN.AUTO;
        if (!autoHoriz) {
            if (cell.autohorzalign) {
                autoHoriz = true;
            }
        } else {
            cell.autohorzalign = 1;
        }

        if (cell.vertAlign == COM.widget.Grid.Cell.ALIGN.AUTO) {
            cell.vertAlign = COM.widget.Grid.Cell.ALIGN.CENTER;
        }

        var wrapLine = cell.wrapLine;
        if (wrapLine) {
            cell.defaultwrapLine = 1;
        }
        var fitFontSize = cell.fitFontSize;
        if (!fitFontSize) {
            cell.defaultfitFontSize = 1;
        }

        if (!isNaN(dValue) || dValue.match(/^(\d{1,4})(-|\/)(\d{1,2})\2(\d{1,2})$/)) {
            if (autoHoriz) {
                cell.horzAlign = COM.widget.Grid.Cell.ALIGN.BACK;
            }
            cell.wrapLine = false;
            cell.fitFontSize = true;

        } else {
            if (autoHoriz) {
                cell.horzAlign = COM.widget.Grid.Cell.ALIGN.FORE;
            }
            if (cell.defaultwrapLine) {
                cell.wrapLine = true;
            }
            if (cell.defaultfitFontSize) {
                cell.fitFontSize = false;
            }
        }
    };

    this.formatDataByCell = function (cell, dValue) {
        var celldata = cell.cell.cell;
        var autoHoriz = cell.getHorzAlign() == COM.widget.Grid.Cell.ALIGN.AUTO;
        if (!autoHoriz) {
            if (celldata.autohorzalign) {
                autoHoriz = true;
            }
        } else {
            celldata.autohorzalign = 1;
        }

        var wrapLine = celldata.wrapLine;
        if (wrapLine) {
            celldata.defaultwrapLine = 1;
        }
        var fitFontSize = celldata.fitFontSize;
        if (!fitFontSize) {
            celldata.defaultfitFontSize = 1;
        }

        if (cell.getVertAlign() == COM.widget.Grid.Cell.ALIGN.AUTO) {
            cell.setVertAlign(COM.widget.Grid.Cell.ALIGN.CENTER);
        }

        if (!isNaN(dValue) || dValue.match(/^(\d{1,4})(-|\/)(\d{1,2})\2(\d{1,2})$/)) {
            if (autoHoriz) {
                cell.setHorzAlign(COM.widget.Grid.Cell.ALIGN.BACK);
            }
            cell.setWrapLine(false);
            cell.setFitFontSize(true);
        } else {
            if (autoHoriz) {
                cell.setHorzAlign(COM.widget.Grid.Cell.ALIGN.FORE);
            }
            if (celldata.defaultwrapLine) {
                cell.setWrapLine(true);
            }
            if (celldata.defaultfitFontSize) {
                cell.setFitFontSize(false);
            }
        }
    };

    this.formatTextByCell = function (cell, editText) {
        var filter = { "editText": editText, "showText": null };
        var formatValue = this.filterData(cell, filter);
        if (!formatValue)
            return "";
        return formatValue.showText;
    }

    this.regularFloat = function (valStr) {
        if (valStr.match(/[eE]/)) {
            return valStr;
        }

        var str = new String(valStr);
        var strs = str.split(".");

        var re = /\d{1,3}(?=(\d{3})+$)/g;

        var newStr = strs[0].replace(re, "$&,");
        if (strs.length === 2) {
            newStr += "." + strs[1];
        }
        return newStr;
    };

    this.gridCanEdit = function () {
        var editMode = this.gridData.getEditMode();
        return editMode !== COM.widget.Grid.EDIT_MODE_READ_ONLY;
    };

    this.destroy = function () {
        clearTimeout(this.timeOutId);

        for (var region in floatRegionDefines) {
            delete floatRegionDefines[region];
        }
        delete floatRegionDefines;
        delete origRegionBoundary;
        delete this.widgetManager.GridEvent_doEdit;
        delete this.widgetManager.GridEvent_doFilter;
        delete this.widgetManager.GridEvent_ExtFunction;
        initRowHeights = null;
        formatDataForLoad = null;
        formatLoadDatas = null;
        this.gridData.setDefaultEditor(null);

        $(".gridEditor", this.entry).gridHtml5Editor("destroy");

        $(".gridHtml5", this.entry).off("keydown keyup grid_backspaceEditor grid_closeEditor");

        this.gridData.dispose();

        this._insertDataCache = null;
        this._insertCacheAfterSave = null;

        this.rowList.length = 0;
        this.colList.length = 0;
        delete this.TempDataStates;
        for (var item in this.extendData) {
            delete item;
        }

        for (var key in this) {
            delete this[key];
        }

        delete formData;
    };

    initData(this);
};


$.widget("ui.gridHtml5Editor", {
    _init: function () {
        this.element.on("keydown", function (e) {
            if (!(e.ctrlKey || e.metaKey) && !e.shiftKey && e.keyCode == 13) {
                e.stopPropagation();
                if (e.altKey) { //编辑器Alt Enter文本内容换行
                    var myField = this;
                    var enterChar = "\r\n";
                    if (document.selection) { //IE support  
                        var sel = document.selection.createRange();
                        sel.text = enterChar;
                        sel.select();
                        //IE9和IE10下Alt+Enter会导致浏览器全屏，参考网上的算法修改
                        window.event.keyCode = 8;
                        window.event.returnValue = false;
                        return false;
                    } else if (myField.selectionStart || myField.selectionStart == '0') { //MOZILLA/NETSCAPE support 
                        var startPos = myField.selectionStart;
                        var endPos = myField.selectionEnd;
                        myField.value = myField.value.substring(0, startPos) + enterChar + myField.value.substring(endPos, myField.value.length);
                        myField.selectionStart = startPos + enterChar.length - 1;
                        myField.selectionEnd = startPos + enterChar.length - 1;
                        myField.focus();
                        return false;
                    } else {
                        myField.value += "\r\n";
                        myField.focus();
                        return false;
                    }
                } else { //编辑器回车跳转下一单元格
                    if ($(this).attr("lock") != "lock") {
                        $(this).trigger("enterNext");
                        return false;
                    }
                }
            } else if ((e.ctrlKey || e.metaKey) && !e.altKey && !e.shiftKey && e.keyCode == 65) {
                //e.stopPropagation();
                this.focus();
                return true;
            }

            return true;
        }).on("contextmenu", function (e) {
            return false;
        });
    },
    destroy: function () {
        this.element.trigger("dispose");
        this.element.off("keydown mousedown blur contextmenu keyup");
        $.Widget.prototype.destroy.call(this);
    },
    show: function () {
        this.element.css("z-index", 5000);
        this.element.trigger("value");
        return true;
    },
    hide: function () {
        this.element.trigger("destroy");
    }
});


